﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Windows;
using System.Windows.Threading;
using ConnectorLib;
using EmoTracker.Core;
using EmoTracker.Data;
using EmoTracker.Data.JSON;
using EmoTracker.Data.Packages;
using EmoTracker.Data.Scripting;
using EmoTracker.Extensions.AutoTracker;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using WebSocketSharp;

namespace EmoTracker.Extensions.BontaMultiworld
{
	// Token: 0x02000034 RID: 52
	public class MultiWorldClientSession : ObservableObject
	{
		// Token: 0x17000076 RID: 118
		// (get) Token: 0x06000218 RID: 536 RVA: 0x0000A12D File Offset: 0x0000832D
		public IReadOnlyList<string> MessageLog
		{
			get
			{
				return this.mMessageLog;
			}
		}

		// Token: 0x06000219 RID: 537 RVA: 0x0000A135 File Offset: 0x00008335
		public void ClearMessageLog(object arg = null)
		{
			Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
			{
				this.mMessageLog.Clear();
			}));
		}

		// Token: 0x0600021A RID: 538 RVA: 0x0000A158 File Offset: 0x00008358
		public void Log(string format, params object[] tokens)
		{
			string formattedMsg = string.Format(format, tokens);
			Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
			{
				this.mMessageLog.Add(formattedMsg);
			}));
		}

		// Token: 0x17000077 RID: 119
		// (get) Token: 0x0600021B RID: 539 RVA: 0x0000A19D File Offset: 0x0000839D
		// (set) Token: 0x0600021C RID: 540 RVA: 0x0000A1A8 File Offset: 0x000083A8
		protected WebSocket Socket
		{
			get
			{
				return this.mSocket;
			}
			set
			{
				WebSocket existingSocket = this.mSocket;
				if (base.SetProperty<WebSocket>(ref this.mSocket, value, "Socket"))
				{
					if (existingSocket != null)
					{
						try
						{
							existingSocket.OnOpen -= this.Socket_OnOpen;
							existingSocket.OnClose -= this.Socket_OnClose;
							existingSocket.OnError -= this.Socket_OnError;
							existingSocket.OnMessage -= this.Socket_OnMessage;
							existingSocket.Close();
						}
						catch
						{
						}
					}
					if (this.mSocket != null)
					{
						this.mSocket.OnOpen += this.Socket_OnOpen;
						this.mSocket.OnClose += this.Socket_OnClose;
						this.mSocket.OnError += this.Socket_OnError;
						this.mSocket.OnMessage += this.Socket_OnMessage;
					}
				}
			}
		}

		// Token: 0x17000078 RID: 120
		// (get) Token: 0x0600021D RID: 541 RVA: 0x0000A2A0 File Offset: 0x000084A0
		// (set) Token: 0x0600021E RID: 542 RVA: 0x0000A2A8 File Offset: 0x000084A8
		public SessionLoginType SessionLoginType
		{
			get
			{
				return this.mSessionLoginType;
			}
			set
			{
				base.SetProperty<SessionLoginType>(ref this.mSessionLoginType, value, "SessionLoginType");
			}
		}

		// Token: 0x17000079 RID: 121
		// (get) Token: 0x0600021F RID: 543 RVA: 0x0000A2BD File Offset: 0x000084BD
		// (set) Token: 0x06000220 RID: 544 RVA: 0x0000A2C5 File Offset: 0x000084C5
		public SessionStatus SessionStatus
		{
			get
			{
				return this.mSessionStatus;
			}
			set
			{
				base.SetProperty<SessionStatus>(ref this.mSessionStatus, value, "SessionStatus");
			}
		}

		// Token: 0x1700007A RID: 122
		// (get) Token: 0x06000221 RID: 545 RVA: 0x0000A2DA File Offset: 0x000084DA
		// (set) Token: 0x06000222 RID: 546 RVA: 0x0000A2E2 File Offset: 0x000084E2
		public SessionError Error
		{
			get
			{
				return this.mSessionError;
			}
			set
			{
				base.SetProperty<SessionError>(ref this.mSessionError, value, "Error");
			}
		}

		// Token: 0x1700007B RID: 123
		// (get) Token: 0x06000223 RID: 547 RVA: 0x0000A2F7 File Offset: 0x000084F7
		// (set) Token: 0x06000224 RID: 548 RVA: 0x0000A2FF File Offset: 0x000084FF
		public AuthenticationFailure AuthenticationFailure
		{
			get
			{
				return this.mAuthFailure;
			}
			set
			{
				base.SetProperty<AuthenticationFailure>(ref this.mAuthFailure, value, "AuthenticationFailure");
			}
		}

		// Token: 0x1700007C RID: 124
		// (get) Token: 0x06000225 RID: 549 RVA: 0x0000A314 File Offset: 0x00008514
		// (set) Token: 0x06000226 RID: 550 RVA: 0x0000A31C File Offset: 0x0000851C
		public MultiWorldClientSession.SessionDescription Description
		{
			get
			{
				return this.mDescription;
			}
			protected set
			{
				base.SetProperty<MultiWorldClientSession.SessionDescription>(ref this.mDescription, value, "Description");
			}
		}

		// Token: 0x1700007D RID: 125
		// (get) Token: 0x06000227 RID: 551 RVA: 0x0000A331 File Offset: 0x00008531
		// (set) Token: 0x06000228 RID: 552 RVA: 0x0000A339 File Offset: 0x00008539
		public string HostUri
		{
			get
			{
				return this.mHostURI;
			}
			set
			{
				base.SetProperty<string>(ref this.mHostURI, value, "HostUri");
			}
		}

		// Token: 0x1700007E RID: 126
		// (get) Token: 0x06000229 RID: 553 RVA: 0x0000A34E File Offset: 0x0000854E
		// (set) Token: 0x0600022A RID: 554 RVA: 0x0000A356 File Offset: 0x00008556
		public string UserName
		{
			get
			{
				return this.mUserName;
			}
			set
			{
				base.SetProperty<string>(ref this.mUserName, value, "UserName");
			}
		}

		// Token: 0x1700007F RID: 127
		// (get) Token: 0x0600022B RID: 555 RVA: 0x0000A36B File Offset: 0x0000856B
		// (set) Token: 0x0600022C RID: 556 RVA: 0x0000A373 File Offset: 0x00008573
		public string Password
		{
			get
			{
				return this.mPassword;
			}
			set
			{
				base.SetProperty<string>(ref this.mPassword, value, "Password");
			}
		}

		// Token: 0x17000080 RID: 128
		// (get) Token: 0x0600022D RID: 557 RVA: 0x0000A388 File Offset: 0x00008588
		// (set) Token: 0x0600022E RID: 558 RVA: 0x0000A390 File Offset: 0x00008590
		public string UserTeam
		{
			get
			{
				return this.mUserTeam;
			}
			set
			{
				base.SetProperty<string>(ref this.mUserTeam, value, "UserTeam");
			}
		}

		// Token: 0x17000081 RID: 129
		// (get) Token: 0x0600022F RID: 559 RVA: 0x0000A3A5 File Offset: 0x000085A5
		// (set) Token: 0x06000230 RID: 560 RVA: 0x0000A3AD File Offset: 0x000085AD
		public MultiWorldClientSession.SessionDescription.Slot UserSlot
		{
			get
			{
				return this.mUserSlot;
			}
			set
			{
				base.SetProperty<MultiWorldClientSession.SessionDescription.Slot>(ref this.mUserSlot, value, "UserSlot");
			}
		}

		// Token: 0x06000231 RID: 561 RVA: 0x0000A3C4 File Offset: 0x000085C4
		// should be protected; just wacking compiler errors - viz
		public bool CanAuthenticate(object arg = null)
		{
			if (this.SessionStatus != SessionStatus.Connected)
			{
				return false;
			}
			if (this.Description == null)
			{
				return false;
			}
			if (this.Description.PasswordRequired && string.IsNullOrWhiteSpace(this.Password))
			{
				return false;
			}
			if (this.SessionLoginType == SessionLoginType.Legacy)
			{
				if (string.IsNullOrWhiteSpace(this.UserName))
				{
					return false;
				}
				if (this.UserSlot == null || !this.UserSlot.Available)
				{
					return false;
				}
			}
			return true;
		}

		// Token: 0x06000232 RID: 562 RVA: 0x0000A434 File Offset: 0x00008634
		// should be protected; just wacking compiler errors - viz
		public void Authenticate(object arg = null)
		{
			if (this.CanAuthenticate(null))
			{
				object abstractMessage;
				if (this.SessionLoginType == SessionLoginType.Legacy)
				{
					abstractMessage = new MultiWorldClientSession.LegacyConnectMsg
					{
						name = this.UserName,
						password = ((!string.IsNullOrWhiteSpace(this.Password)) ? this.Password : null),
						team = ((!string.IsNullOrWhiteSpace(this.UserTeam)) ? this.UserTeam : null),
						slot = this.UserSlot.Index
					};
				}
				else
				{
					MultiWorldClientSession.ConnectMsg connectMsg = new MultiWorldClientSession.ConnectMsg();
					this.mExpectedRom = this.GetRomHash(null);
					connectMsg.rom = new List<byte>(this.mExpectedRom);
					connectMsg.password = ((!string.IsNullOrWhiteSpace(this.Password)) ? this.Password : null);
					abstractMessage = connectMsg;
				}
				MultiWorldClientSession.Message message = new MultiWorldClientSession.Message
				{
					Command = "Connect",
					Data = JObject.FromObject(abstractMessage)
				};
				this.SendMessages(new Action<bool>(this.JoinSessionCompletionHandler), new MultiWorldClientSession.Message[]
				{
					message
				});
			}
		}

		// Token: 0x06000233 RID: 563 RVA: 0x0000A52C File Offset: 0x0000872C
		[Command("Connected")]
		private void OnAuthenticatedCmd(JToken data)
		{
			AutoTrackerExtension extension = ObservableSingleton<ExtensionManager>.Instance.FindExtension<AutoTrackerExtension>();
			if (extension != null)
			{
				if (this.SessionLoginType == SessionLoginType.Legacy)
				{
					JArray romContainer = data as JArray;
					if (romContainer != null && romContainer.Count <= 21)
					{
						this.mExpectedRom = new byte[romContainer.Count];
						for (int i = 0; i < romContainer.Count; i++)
						{
							this.mExpectedRom[i] = romContainer[i].GetValue<byte>(0);
						}
					}
					else
					{
						this.mExpectedRom = null;
					}
					if (!this.IsRomValid(null))
					{
						this.Disconnect(SessionError.RomValidationError);
						return;
					}
				}
				else
				{
					try
					{
						JArray root = data as JArray;
						if (root != null && root.Count >= 2)
						{
							JArray playerMap = root[1] as JArray;
							if (playerMap != null)
							{
								foreach (JToken jtoken in playerMap)
								{
									JArray entry = (JArray)jtoken;
									try
									{
										int playerIdx = entry[0].GetValue(0);
										string playerName = entry[1].GetValue<string>(null);
										this.mTeamPlayerNameMap[playerIdx] = playerName;
									}
									catch
									{
									}
								}
							}
						}
					}
					catch
					{
					}
				}
				this.mMemoryTimer = extension.AddMemoryTimer("Received Items Hook", new Func<IAddressableConnector, PackageManager.Game, bool>(this.WriteReceivedItems), 1000);
				MemorySegment.OnMemorySegmentUpdated += this.MemorySegment_OnMemorySegmentUpdated;
				this.SessionStatus = SessionStatus.Authenticated;
				this.SendLocationChecks(this.mCheckedLocations);
			}
		}

		// Token: 0x06000234 RID: 564 RVA: 0x0000A6B8 File Offset: 0x000088B8
		[Command("ConnectionRefused")]
		private void OnAuthenticationFailedCmd(JToken data)
		{
			if (data is JArray jarray && jarray.Count > 0)
			{
				AuthenticationFailure authenticationFailure = AuthenticationFailure.None;
				foreach (JToken token in jarray)
				{
					string str = token.GetValue<string>();
					if (!string.IsNullOrWhiteSpace(str))
					{
						switch (str)
						{
							case "InvalidName":
								authenticationFailure |= AuthenticationFailure.InvalidName;
								continue;
							case "InvalidPassword":
								authenticationFailure |= AuthenticationFailure.InvalidPassword;
								continue;
							case "InvalidRom":
								authenticationFailure |= AuthenticationFailure.InvalidRom;
								continue;
							case "InvalidSlot":
								authenticationFailure |= AuthenticationFailure.InvalidSlot;
								continue;
							case "InvalidTeam":
								authenticationFailure |= AuthenticationFailure.InvalidTeam;
								continue;
							case "NameAlreadyTaken":
								authenticationFailure |= AuthenticationFailure.NameAlreadyTaken;
								continue;
							case "SlotAlreadyTaken":
								authenticationFailure |= AuthenticationFailure.SlotAlreadyTaken;
								continue;
							default:
								continue;
						}
					}
				}
				this.AuthenticationFailure = authenticationFailure;
				if (authenticationFailure != AuthenticationFailure.None)
					this.Password = (string)null;
				if (!authenticationFailure.HasFlag((Enum)AuthenticationFailure.InvalidRom))
					return;
				this.Disconnect(SessionError.RomValidationError);
			}
			else
				this.Disconnect(SessionError.ProtocolError);
		}

		// Token: 0x06000235 RID: 565 RVA: 0x0000A870 File Offset: 0x00008A70
		private void InvokeCommand(string command, JToken data)
		{
			if (!string.IsNullOrWhiteSpace(command))
			{
				MethodInfo method = null;
				if (!this.mMethodCache.TryGetValue(command, out method))
				{
					bool bFound = false;
					Type currentType = base.GetType();
					while (currentType != null && !bFound)
					{
						foreach (MethodInfo candidateMethod in currentType.GetMethods(BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic))
						{
							CommandAttribute cmdAttr = candidateMethod.GetCustomAttribute<CommandAttribute>();
							if (cmdAttr != null && string.Equals(command, cmdAttr.Command, StringComparison.OrdinalIgnoreCase))
							{
								bFound = true;
								method = candidateMethod;
								this.mMethodCache[command] = method;
								break;
							}
						}
						currentType = currentType.BaseType;
					}
					if (!bFound)
					{
						this.mMethodCache[command] = null;
					}
				}
				if (method != null)
				{
					method.Invoke(this, new object[]
					{
						data
					});
					return;
				}
				this.Log("Unknown Command: {0} :: {1}", new object[]
				{
					command,
					(data != null) ? data.ToString() : "<no data>"
				});
			}
		}

		// Token: 0x06000236 RID: 566 RVA: 0x0000A964 File Offset: 0x00008B64
		private void Socket_OnMessage(object sender, MessageEventArgs e)
		{
			try
			{
				using (StringReader reader = new StringReader(e.Data))
				{
					JArray commandList = JToken.ReadFrom(new JsonTextReader(reader)) as JArray;
					if (commandList != null)
					{
						foreach (JToken jtoken in commandList)
						{
							JArray command = (JArray)jtoken;
							string cmd = null;
							JToken data = null;
							try
							{
								if (command.Count > 0)
								{
									cmd = command[0].Value<string>();
								}
								if (command.Count > 1)
								{
									data = command[1];
								}
								this.InvokeCommand(cmd, data);
							}
							catch (Exception ex)
							{
								this.Log("Exception occured while processing command: {0}", new object[]
								{
									e.Data
								});
								this.Log(ex.ToString(), Array.Empty<object>());
							}
						}
					}
				}
			}
			catch
			{
			}
		}

		// Token: 0x06000237 RID: 567 RVA: 0x0000AA70 File Offset: 0x00008C70
		private void Socket_OnClose(object sender, CloseEventArgs e)
		{
			this.Disconnect(SessionError.None);
		}

		// Token: 0x06000238 RID: 568 RVA: 0x0000AA79 File Offset: 0x00008C79
		private void Socket_OnError(object sender, WebSocketSharp.ErrorEventArgs e)
		{
			this.Disconnect(SessionError.ConnectionError);
		}

		// Token: 0x06000239 RID: 569 RVA: 0x00004FD9 File Offset: 0x000031D9
		private void Socket_OnOpen(object sender, EventArgs e)
		{
		}

		// Token: 0x0600023A RID: 570 RVA: 0x0000AA82 File Offset: 0x00008C82
		private void SendMessages(Action<bool> completionHandler, params MultiWorldClientSession.Message[] msgs)
		{
			this.SendMessages(msgs.AsEnumerable<MultiWorldClientSession.Message>(), completionHandler);
		}

		// Token: 0x0600023B RID: 571 RVA: 0x0000AA94 File Offset: 0x00008C94
		private void SendMessages(IEnumerable<MultiWorldClientSession.Message> msgs, Action<bool> completionHandler)
		{
			JArray batch = new JArray();
			foreach (MultiWorldClientSession.Message msg in msgs)
			{
				if (msg != null)
				{
					JArray msgArray = new JArray();
					msgArray.Add(JToken.FromObject(msg.Command));
					if (msg.Data != null)
					{
						msgArray.Add(msg.Data);
					}
					batch.Add(msgArray);
				}
			}
			if (batch.Count > 0)
			{
				using (StringWriter writer = new StringWriter())
				{
					using (JsonTextWriter jsonWriter = new JsonTextWriter(writer))
					{
						jsonWriter.AutoCompleteOnClose = true;
						jsonWriter.Formatting = Formatting.None;
						jsonWriter.WriteToken(batch.CreateReader());
						jsonWriter.Close();
					}
					string messageString = writer.ToString();
					if (this.Socket != null)
					{
						WebSocket socket = this.Socket;
						lock (socket)
						{
							this.Socket.SendAsync(messageString, completionHandler);
						}
					}
				}
			}
		}

		// Token: 0x0600023C RID: 572 RVA: 0x0000ABD4 File Offset: 0x00008DD4
		public bool CanConnect(object arg = null)
		{
			if (this.Socket != null)
			{
				return false;
			}
			if (string.IsNullOrWhiteSpace(this.HostUri))
			{
				return false;
			}
			AutoTrackerExtension autotracker = ObservableSingleton<ExtensionManager>.Instance.FindExtension<AutoTrackerExtension>();
			return autotracker != null && autotracker.ActiveConnector != null && autotracker.ActiveConnector.Connected;
		}

		// Token: 0x0600023D RID: 573 RVA: 0x0000AC20 File Offset: 0x00008E20
		public void Connect()
		{
			try
			{
				if (this.CanConnect(null))
				{
					string uri = this.HostUri;
					if (!uri.StartsWith("ws://") && !uri.StartsWith("wss://"))
					{
						uri = "ws://" + uri;
					}
					this.Socket = new WebSocket(uri, Array.Empty<string>());
					this.SessionStatus = SessionStatus.Connecting;
					this.Socket.ConnectAsync();
				}
			}
			catch
			{
				this.Disconnect(SessionError.ConnectionError);
			}
		}

		// Token: 0x0600023E RID: 574 RVA: 0x0000AA70 File Offset: 0x00008C70
		public void Disconnect(object arg)
		{
			this.Disconnect(SessionError.None);
		}

		// Token: 0x0600023F RID: 575 RVA: 0x0000ACA4 File Offset: 0x00008EA4
		public void Disconnect(SessionError error = SessionError.None)
		{
			bool bHadSocket = false;
			if (this.Socket != null)
			{
				bHadSocket = true;
				WebSocket socket = this.Socket;
				lock (socket)
				{
					this.Socket = null;
				}
			}
			this.ResetConnectionState();
			this.SessionLoginType = SessionLoginType.Unknown;
			this.AuthenticationFailure = AuthenticationFailure.None;
			this.SessionStatus = SessionStatus.Disconnected;
			this.Password = null;
			this.mTeamPlayerNameMap.Clear();
			this.Error = error;
			switch (this.Error)
			{
				case SessionError.None:
					if (bHadSocket)
					{
						Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
						{
							ObservableSingleton<ApplicationModel>.Instance.PushMarkdownNotification(NotificationType.Error, string.Format("You have been disconnected from the multi-world server.", Array.Empty<object>()), -1);
						}));
						return;
					}
					return;
				case SessionError.ProtocolError:
					Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
					{
						ObservableSingleton<ApplicationModel>.Instance.PushMarkdownNotification(NotificationType.Error, string.Format("You were disconnected from the multi-world server because the server responded to a request in an unexpected way.", Array.Empty<object>()), -1);
					}));
					return;
				case SessionError.RomValidationError:
					Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
					{
						ObservableSingleton<ApplicationModel>.Instance.PushMarkdownNotification(NotificationType.Error, string.Format("You were disconnected from the multi-world server because your ROM does not match the server's expectations.", Array.Empty<object>()), -1);
					}));
					return;
			}
			Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
			{
				ObservableSingleton<ApplicationModel>.Instance.PushMarkdownNotification(NotificationType.Error, string.Format("You have been disconnected from the multi-world server.", Array.Empty<object>()), -1);
			}));
		}

		// Token: 0x06000240 RID: 576 RVA: 0x0000AE14 File Offset: 0x00009014
		private void JoinSessionCompletionHandler(bool result)
		{
			if (!result)
			{
				this.Log("Failed to send connect message", Array.Empty<object>());
			}
		}

		// Token: 0x06000241 RID: 577 RVA: 0x0000AE29 File Offset: 0x00009029
		[Command("Print")]
		private void OnPrintCmd(JToken data)
		{
			if (data != null && !string.IsNullOrWhiteSpace(data.ToString()))
			{
				this.Log(data.ToString(), Array.Empty<object>());
			}
		}

		// Token: 0x06000242 RID: 578 RVA: 0x0000AE4C File Offset: 0x0000904C
		[Command("RoomInfo")]
		private void OnRoomInfoCmd(JToken data)
		{
			try
			{
				this.Description = new MultiWorldClientSession.SessionDescription(data, this);
				foreach (MultiWorldClientSession.SessionDescription.Slot slot in this.Description.Slots)
				{
					if (this.UserSlot == null && slot.Available)
					{
						this.UserSlot = slot;
					}
				}
				if (this.Description.Slots.Count == 0)
				{
					this.SessionLoginType = SessionLoginType.RomBased;
				}
				else
				{
					this.SessionLoginType = SessionLoginType.Legacy;
				}
				this.SessionStatus = SessionStatus.Connected;
				if (this.CanAuthenticate(null))
				{
					this.Authenticate(null);
				}
			}
			catch
			{
				this.Log("Invalid session description format", Array.Empty<object>());
				this.Disconnect(SessionError.ProtocolError);
			}
		}

		// Token: 0x06000243 RID: 579 RVA: 0x0000AF1C File Offset: 0x0000911C
		private void ResetConnectionState()
		{
			this.ClearMessageLog(null);
			this.mReceivedItems.Clear();
			this.mCheckedLocations.Clear();
			AutoTrackerExtension extension = ObservableSingleton<ExtensionManager>.Instance.FindExtension<AutoTrackerExtension>();
			if (extension != null && this.mMemoryTimer == null)
			{
				extension.RemoveMemoryTimer(this.mMemoryTimer);
				this.mMemoryTimer = null;
			}
			this.AuthenticationFailure = AuthenticationFailure.None;
			this.UserSlot = null;
			this.Description = null;
			this.mExpectedRom = null;
		}

		// Token: 0x17000082 RID: 130
		// (get) Token: 0x06000244 RID: 580 RVA: 0x0000AF8C File Offset: 0x0000918C
		private IAddressableConnector ActiveConnector
		{
			get
			{
				AutoTrackerExtension autotracker = ObservableSingleton<ExtensionManager>.Instance.FindExtension<AutoTrackerExtension>();
				if (autotracker == null || autotracker.ActiveConnector == null || !autotracker.ActiveConnector.Connected)
				{
					return null;
				}
				return autotracker.ActiveConnector;
			}
		}

		// Token: 0x06000245 RID: 581 RVA: 0x0000AFC4 File Offset: 0x000091C4
		private byte[] GetRomHash(IAddressableConnector connector = null)
		{
			connector = (connector ?? this.ActiveConnector);
			if (connector == null)
			{
				return null;
			}
			byte[] romHash = new byte[21];
			if (!connector.Read(7348224UL, romHash))
			{
				return null;
			}
			return romHash;
		}

		// Token: 0x06000246 RID: 582 RVA: 0x0000B000 File Offset: 0x00009200
		private bool IsRomValid(IAddressableConnector connector = null)
		{
			connector = (connector ?? this.ActiveConnector);
			if (this.mExpectedRom == null || this.mExpectedRom.Length > 21)
			{
				return false;
			}
			if (ObservableSingleton<ApplicationSettings>.Instance.IgnoreBontaMultiWorldRomCheck)
			{
				return true;
			}
			byte[] romHash = this.GetRomHash(null);
			if (romHash == null || romHash.Length < this.mExpectedRom.Length)
			{
				return false;
			}
			for (int i = 0; i < this.mExpectedRom.Length; i++)
			{
				if (romHash[i] != this.mExpectedRom[i])
				{
					return false;
				}
			}
			return true;
		}

		// Token: 0x06000247 RID: 583 RVA: 0x0000B07C File Offset: 0x0000927C
		private bool IsInGame(IAddressableConnector connector = null)
		{
			connector = (connector ?? this.ActiveConnector);
			if (!this.IsRomValid(connector))
			{
				return false;
			}
			I8BitConnector connector2 = connector as I8BitConnector;
			if (connector2 == null)
			{
				return false;
			}
			byte gameState;
			if (!connector2.Read8(8257552UL, out gameState))
			{
				return false;
			}
			switch (gameState)
			{
				case 7:
				case 9:
				case 11:
					return true;
			}
			return false;
		}

		// Token: 0x06000248 RID: 584 RVA: 0x0000B0E0 File Offset: 0x000092E0
		[Command("ItemSent")]
		private void OnItemSent(JToken data)
		{
			JArray container = data as JArray;
			if (container != null && container.Count == 4)
			{
				try
				{
					string userFrom = "Unknown Player";
					string userTo = "Unknown Player";
					int itemCode = -1;
					int locationCode = -1;
					if (this.SessionLoginType == SessionLoginType.Legacy)
					{
						userFrom = container[0].Value<string>();
						userTo = container[1].Value<string>();
						itemCode = container[2].Value<int>();
						locationCode = container[3].Value<int>();
					}
					else
					{
						int fromPlayerIdx = container[0].Value<int>();
						int toPlayerIdx = container[2].Value<int>();
						userFrom = this.GetPlayerNameForIndex(fromPlayerIdx);
						userTo = this.GetPlayerNameForIndex(toPlayerIdx);
						locationCode = container[1].Value<int>();
						itemCode = container[3].Value<int>();
					}
					this.Log("{0} sent {1} {2} ({3})", new object[]
					{
						userFrom,
						userTo,
						this.GetItemNameForID(itemCode),
						this.GetLocationNameForID(locationCode)
					});
					MultiworldNotificationLevel notificationLevel = ObservableSingleton<ApplicationSettings>.Instance.MultiworldNotificationLevel;
					if ((notificationLevel >= MultiworldNotificationLevel.Verbose && string.Equals(userFrom, this.mUserName)) || (notificationLevel >= MultiworldNotificationLevel.Verbose && string.Equals(userTo, this.mUserName)))
					{
						Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
						{
							ObservableSingleton<ApplicationModel>.Instance.PushMarkdownNotification(NotificationType.Message, string.Format("**{0}** sent **{1}** {2} *({3})*", new object[]
							{
								userFrom,
								userTo,
								this.GetItemNameForID(itemCode),
								this.GetLocationNameForID(locationCode)
							}), -1);
						}));
					}
				}
				catch
				{
				}
			}
		}

		// Token: 0x06000249 RID: 585 RVA: 0x0000B2A0 File Offset: 0x000094A0
		private string GetPlayerNameForIndex(int idx)
		{
			string playerName;
			if (!this.mTeamPlayerNameMap.TryGetValue(idx, out playerName))
			{
				playerName = "Unknown Player";
			}
			return playerName;
		}

		// Token: 0x0600024A RID: 586 RVA: 0x0000B2C4 File Offset: 0x000094C4
		[Command("ReceivedItems")]
		private void OnItemsReceived(JToken data)
		{
			List<MultiWorldClientSession.ReceivedItem> obj = this.mReceivedItems;
			lock (obj)
			{
				JArray jarray = (JArray)data;
				int start_idx = jarray[0].Value<int>();
				JArray items = jarray[1].Value<JArray>();
				if (start_idx == 0)
				{
					this.mReceivedItems.Clear();
				}
				else if (start_idx != this.mReceivedItems.Count)
				{
					this.mReceivedItems.Clear();
					this.SendResync();
					return;
				}
				if (start_idx == this.mReceivedItems.Count)
				{
					foreach (JToken jtoken in items)
					{
						JArray item = (JArray)jtoken;
						MultiWorldClientSession.ReceivedItem instance;
						if (this.SessionLoginType == SessionLoginType.Legacy)
						{
							instance = new MultiWorldClientSession.ReceivedItem
							{
								Item = item[0].Value<int>(),
								PlayerName = item[3].Value<string>()
							};
							try
							{
								instance.Location = this.GetLocationNameForID(item[1].Value<int>());
								goto IL_15F;
							}
							catch
							{
								instance.Location = item[1].Value<string>();
								goto IL_15F;
							}
							goto IL_F9;
						}
						goto IL_F9;
					IL_15F:
						this.mReceivedItems.Add(instance);
						continue;
					IL_F9:
						instance = new MultiWorldClientSession.ReceivedItem
						{
							Item = item[0].Value<int>(),
							PlayerName = this.GetPlayerNameForIndex(item[2].Value<int>())
						};
						try
						{
							instance.Location = this.GetLocationNameForID(item[1].Value<int>());
						}
						catch
						{
							instance.Location = item[1].Value<string>();
						}
						goto IL_15F;
					}
				}
			}
		}

		// Token: 0x0600024B RID: 587 RVA: 0x0000B4C8 File Offset: 0x000096C8
		private bool WriteReceivedItems(IAddressableConnector connector, PackageManager.Game game)
		{
			if (!this.IsInGame(connector))
			{
				return true;
			}
			try
			{
				List<MultiWorldClientSession.ReceivedItem> obj = this.mReceivedItems;
				lock (obj)
				{
					I16BitConnector connector2 = (I16BitConnector)connector;
					ushort recv_idx = 0;
					if (connector2.Read16(8320208UL, out recv_idx) && (int)recv_idx < this.mReceivedItems.Count)
					{
						byte pendingItemCode = 0;
						if (connector2.Read8(8320210UL, out pendingItemCode) && pendingItemCode == 0)
						{
							MultiWorldClientSession.ReceivedItem item = this.mReceivedItems[(int)recv_idx];
							if (ObservableSingleton<ApplicationSettings>.Instance.MultiworldNotificationLevel >= MultiworldNotificationLevel.Normal)
							{
								Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
								{
									this.Log("Received {0} from {1} ({2})", new object[]
									{
										this.GetItemNameForID(item.Item),
										item.PlayerName,
										item.Location
									});
									ObservableSingleton<ApplicationModel>.Instance.PushMarkdownNotification(NotificationType.Celebration, string.Format("Received **{0}** from **{1}** *({2})*", this.GetItemNameForID(item.Item), item.PlayerName, item.Location), -1);
								}));
							}
							using (connector2.GetBatchContext16())
							{
								connector2.Write16(8320208UL, recv_idx += 1);
								connector2.Write8(8320210UL, (byte)item.Item);
							}
						}
					}
				}
				return true;
			}
			catch
			{
			}
			return false;
		}

		// Token: 0x0600024C RID: 588 RVA: 0x0000B60C File Offset: 0x0000980C
		private void MemorySegment_OnMemorySegmentUpdated(MemorySegment segment, IAddressableConnector connector, PackageManager.Game game)
		{
			if (!(connector is I8BitConnector))
			{
				return;
			}
			List<MultiWorldClientSession.LocationData> newChecks = new List<MultiWorldClientSession.LocationData>();
			bool bHasCheckedInGameState = false;
			foreach (MultiWorldClientSession.LocationData location in MultiWorldClientSession.Locations)
			{
				if (!this.mCheckedLocations.Contains(location))
				{
					ulong locationAddress = 8318976UL + location.Offset;
					if (segment.ContainsAddress(locationAddress))
					{
						if (!bHasCheckedInGameState)
						{
							if (!this.IsInGame(connector))
							{
								return;
							}
							bHasCheckedInGameState = true;
						}
						if ((segment.ReadUInt8(locationAddress, false) & location.Mask) != 0)
						{
							newChecks.Add(location);
							this.mCheckedLocations.Add(location);
						}
					}
				}
			}
			this.SendLocationChecks(newChecks);
		}

		// Token: 0x0600024D RID: 589 RVA: 0x0000B6B0 File Offset: 0x000098B0
		private MultiWorldClientSession.Message BuildLocationChecksMessage(IEnumerable<MultiWorldClientSession.LocationData> locations)
		{
			if (locations == null)
			{
				return null;
			}
			if (!locations.Any<MultiWorldClientSession.LocationData>())
			{
				return null;
			}
			JArray locationIDArray = new JArray();
			foreach (MultiWorldClientSession.LocationData location in locations)
			{
				locationIDArray.Add(JToken.FromObject(location.ID));
			}
			return new MultiWorldClientSession.Message
			{
				Command = "LocationChecks",
				Data = locationIDArray
			};
		}

		// Token: 0x0600024E RID: 590 RVA: 0x0000B734 File Offset: 0x00009934
		private void SendLocationChecks(IEnumerable<MultiWorldClientSession.LocationData> locations)
		{
			this.SendMessages(delegate (bool x)
			{
			}, new MultiWorldClientSession.Message[]
			{
				this.BuildLocationChecksMessage(locations)
			});
		}

		// Token: 0x0600024F RID: 591 RVA: 0x0000B76C File Offset: 0x0000996C
		private void SendResync()
		{
			this.SendMessages(delegate (bool x)
			{
			}, new MultiWorldClientSession.Message[]
			{
				new MultiWorldClientSession.Message
				{
					Command = "Sync"
				},
				this.BuildLocationChecksMessage(this.mCheckedLocations)
			});
		}

		// Token: 0x06000250 RID: 592 RVA: 0x0000B7C8 File Offset: 0x000099C8
		public void Say(string text)
		{
			if (!string.IsNullOrWhiteSpace(text))
			{
				this.SendMessages(delegate (bool x)
				{
				}, new MultiWorldClientSession.Message[]
				{
					new MultiWorldClientSession.Message
					{
						Command = "Say",
						Data = JToken.FromObject(text)
					}
				});
			}
		}

		// Token: 0x06000251 RID: 593 RVA: 0x0000B828 File Offset: 0x00009A28
		public string GetLocationNameForID(int id)
		{
			if (id == -100)
			{
				return "Cheat Console";
			}
			foreach (MultiWorldClientSession.LocationData i in MultiWorldClientSession.Locations)
			{
				if (i.ID == id)
				{
					return i.Name;
				}
			}
			return "Unknown Location";
		}

		// Token: 0x06000252 RID: 594 RVA: 0x0000B874 File Offset: 0x00009A74
		public string GetItemNameForID(int id)
		{
			foreach (MultiWorldClientSession.ItemDescription item in MultiWorldClientSession.Items)
			{
				if (item.ID == id)
				{
					return item.Name;
				}
			}
			return "Unknown Item";
		}

		// Token: 0x06000253 RID: 595 RVA: 0x0000B8B4 File Offset: 0x00009AB4
		public MultiWorldClientSession()
		{
		}

		// Token: 0x06000254 RID: 596 RVA: 0x0000B90C File Offset: 0x00009B0C
		// Note: this type is marked as 'beforefieldinit'.
		static MultiWorldClientSession()
		{
		}

		// Token: 0x040000D6 RID: 214
		protected ObservableCollection<string> mMessageLog = new ObservableCollection<string>();

		// Token: 0x040000D7 RID: 215
		private WebSocket mSocket;

		// Token: 0x040000D8 RID: 216
		private SessionLoginType mSessionLoginType;

		// Token: 0x040000D9 RID: 217
		private SessionStatus mSessionStatus;

		// Token: 0x040000DA RID: 218
		private SessionError mSessionError;

		// Token: 0x040000DB RID: 219
		private AuthenticationFailure mAuthFailure;

		// Token: 0x040000DC RID: 220
		private MultiWorldClientSession.SessionDescription mDescription;

		// Token: 0x040000DD RID: 221
		private byte[] mExpectedRom;

		// Token: 0x040000DE RID: 222
		private string mHostURI = "127.0.0.1:38281";

		// Token: 0x040000DF RID: 223
		private string mUserName;

		// Token: 0x040000E0 RID: 224
		private string mPassword;

		// Token: 0x040000E1 RID: 225
		private string mUserTeam;

		// Token: 0x040000E2 RID: 226
		private MultiWorldClientSession.SessionDescription.Slot mUserSlot;

		// Token: 0x040000E3 RID: 227
		private Dictionary<int, string> mTeamPlayerNameMap = new Dictionary<int, string>();

		// Token: 0x040000E4 RID: 228
		private Dictionary<string, MethodInfo> mMethodCache = new Dictionary<string, MethodInfo>();

		// Token: 0x040000E5 RID: 229
		private MemoryTimer mMemoryTimer;

		// Token: 0x040000E6 RID: 230
		private List<MultiWorldClientSession.ReceivedItem> mReceivedItems = new List<MultiWorldClientSession.ReceivedItem>();

		// Token: 0x040000E7 RID: 231
		private HashSet<MultiWorldClientSession.LocationData> mCheckedLocations = new HashSet<MultiWorldClientSession.LocationData>();

		// Token: 0x040000E8 RID: 232
		private static MultiWorldClientSession.LocationData[] Locations = new MultiWorldClientSession.LocationData[]
		{
			new MultiWorldClientSession.LocationData("Mushroom", 1041UL, 16, 1572883),
			new MultiWorldClientSession.LocationData("Bottle Merchant", 969UL, 2, 191256),
			new MultiWorldClientSession.LocationData("Flute Spot", 682UL, 64, 1573194),
			new MultiWorldClientSession.LocationData("Sunken Treasure", 699UL, 64, 1573189),
			new MultiWorldClientSession.LocationData("Purple Chest", 969UL, 16, 212328),
			new MultiWorldClientSession.LocationData("Blind's Hideout - Top", 570UL, 16, 60175),
			new MultiWorldClientSession.LocationData("Blind's Hideout - Left", 570UL, 32, 60178),
			new MultiWorldClientSession.LocationData("Blind's Hideout - Right", 570UL, 64, 60181),
			new MultiWorldClientSession.LocationData("Blind's Hideout - Far Left", 570UL, 128, 60184),
			new MultiWorldClientSession.LocationData("Blind's Hideout - Far Right", 571UL, 1, 60187),
			new MultiWorldClientSession.LocationData("Link's Uncle", 966UL, 1, 188229),
			new MultiWorldClientSession.LocationData("Secret Passage", 170UL, 16, 59761),
			new MultiWorldClientSession.LocationData("King Zora", 1040UL, 2, 975299),
			new MultiWorldClientSession.LocationData("Zora's Ledge", 769UL, 64, 1573193),
			new MultiWorldClientSession.LocationData("Waterfall Fairy - Left", 552UL, 16, 59824),
			new MultiWorldClientSession.LocationData("Waterfall Fairy - Right", 552UL, 32, 59857),
			new MultiWorldClientSession.LocationData("King's Tomb", 550UL, 16, 59770),
			new MultiWorldClientSession.LocationData("Floodgate Chest", 534UL, 16, 59788),
			new MultiWorldClientSession.LocationData("Link's House", 520UL, 16, 59836),
			new MultiWorldClientSession.LocationData("Kakariko Tavern", 518UL, 16, 59854),
			new MultiWorldClientSession.LocationData("Chicken House", 528UL, 16, 59881),
			new MultiWorldClientSession.LocationData("Aginah's Cave", 532UL, 16, 59890),
			new MultiWorldClientSession.LocationData("Sahasrahla's Hut - Left", 522UL, 16, 60034),
			new MultiWorldClientSession.LocationData("Sahasrahla's Hut - Middle", 522UL, 32, 60037),
			new MultiWorldClientSession.LocationData("Sahasrahla's Hut - Right", 522UL, 64, 60040),
			new MultiWorldClientSession.LocationData("Sahasrahla", 1040UL, 16, 193020),
			new MultiWorldClientSession.LocationData("Kakariko Well - Top", 94UL, 16, 60046),
			new MultiWorldClientSession.LocationData("Kakariko Well - Left", 94UL, 32, 60049),
			new MultiWorldClientSession.LocationData("Kakariko Well - Middle", 94UL, 64, 60052),
			new MultiWorldClientSession.LocationData("Kakariko Well - Right", 94UL, 128, 60055),
			new MultiWorldClientSession.LocationData("Kakariko Well - Bottom", 95UL, 1, 60058),
			new MultiWorldClientSession.LocationData("Blacksmith", 1041UL, 4, 1572906),
			new MultiWorldClientSession.LocationData("Magic Bat", 1041UL, 128, 1572885),
			new MultiWorldClientSession.LocationData("Sick Kid", 1040UL, 4, 211407),
			new MultiWorldClientSession.LocationData("Hobo", 969UL, 1, 212605),
			new MultiWorldClientSession.LocationData("Lost Woods Hideout", 451UL, 2, 1572864),
			new MultiWorldClientSession.LocationData("Lumberjack Tree", 453UL, 2, 1572865),
			new MultiWorldClientSession.LocationData("Cave 45", 567UL, 4, 1572867),
			new MultiWorldClientSession.LocationData("Graveyard Cave", 567UL, 2, 1572868),
			new MultiWorldClientSession.LocationData("Checkerboard Cave", 589UL, 2, 1572869),
			new MultiWorldClientSession.LocationData("Mini Moldorm Cave - Far Left", 582UL, 16, 60226),
			new MultiWorldClientSession.LocationData("Mini Moldorm Cave - Left", 582UL, 32, 60229),
			new MultiWorldClientSession.LocationData("Mini Moldorm Cave - Right", 582UL, 64, 60232),
			new MultiWorldClientSession.LocationData("Mini Moldorm Cave - Far Right", 582UL, 128, 60235),
			new MultiWorldClientSession.LocationData("Mini Moldorm Cave - Generous Guy", 583UL, 4, 1572880),
			new MultiWorldClientSession.LocationData("Ice Rod Cave", 576UL, 16, 60238),
			new MultiWorldClientSession.LocationData("Bonk Rock Cave", 584UL, 16, 60223),
			new MultiWorldClientSession.LocationData("Library", 1040UL, 128, 1572882),
			new MultiWorldClientSession.LocationData("Potion Shop", 1041UL, 32, 1572884),
			new MultiWorldClientSession.LocationData("Lake Hylia Island", 693UL, 64, 1573188),
			new MultiWorldClientSession.LocationData("Maze Race", 680UL, 64, 1573186),
			new MultiWorldClientSession.LocationData("Desert Ledge", 688UL, 64, 1573187),
			new MultiWorldClientSession.LocationData("Desert Palace - Big Chest", 230UL, 16, 59791),
			new MultiWorldClientSession.LocationData("Desert Palace - Torch", 231UL, 4, 1573216),
			new MultiWorldClientSession.LocationData("Desert Palace - Map Chest", 232UL, 16, 59830),
			new MultiWorldClientSession.LocationData("Desert Palace - Compass Chest", 266UL, 16, 59851),
			new MultiWorldClientSession.LocationData("Desert Palace - Big Key Chest", 234UL, 16, 59842),
			new MultiWorldClientSession.LocationData("Desert Palace - Boss", 103UL, 8, 1573201),
			new MultiWorldClientSession.LocationData("Eastern Palace - Compass Chest", 336UL, 16, 59767),
			new MultiWorldClientSession.LocationData("Eastern Palace - Big Chest", 338UL, 16, 59773),
			new MultiWorldClientSession.LocationData("Eastern Palace - Cannonball Chest", 370UL, 16, 59827),
			new MultiWorldClientSession.LocationData("Eastern Palace - Big Key Chest", 368UL, 16, 59833),
			new MultiWorldClientSession.LocationData("Eastern Palace - Map Chest", 340UL, 16, 59893),
			new MultiWorldClientSession.LocationData("Eastern Palace - Boss", 401UL, 8, 1573200),
			new MultiWorldClientSession.LocationData("Master Sword Pedestal", 768UL, 64, 166320),
			new MultiWorldClientSession.LocationData("Hyrule Castle - Boomerang Chest", 226UL, 16, 59764),
			new MultiWorldClientSession.LocationData("Hyrule Castle - Map Chest", 228UL, 16, 60172),
			new MultiWorldClientSession.LocationData("Hyrule Castle - Zelda's Chest", 256UL, 16, 60169),
			new MultiWorldClientSession.LocationData("Sewers - Dark Cross", 100UL, 16, 59758),
			new MultiWorldClientSession.LocationData("Sewers - Secret Room - Left", 34UL, 16, 60253),
			new MultiWorldClientSession.LocationData("Sewers - Secret Room - Middle", 34UL, 32, 60256),
			new MultiWorldClientSession.LocationData("Sewers - Secret Room - Right", 34UL, 64, 60259),
			new MultiWorldClientSession.LocationData("Sanctuary", 36UL, 16, 60025),
			new MultiWorldClientSession.LocationData("Castle Tower - Room 03", 448UL, 16, 60085),
			new MultiWorldClientSession.LocationData("Castle Tower - Dark Maze", 416UL, 16, 60082),
			new MultiWorldClientSession.LocationData("Old Man", 1040UL, 1, 1010170),
			new MultiWorldClientSession.LocationData("Spectacle Rock Cave", 469UL, 4, 1572866),
			new MultiWorldClientSession.LocationData("Paradox Cave Lower - Far Left", 478UL, 16, 60202),
			new MultiWorldClientSession.LocationData("Paradox Cave Lower - Left", 478UL, 32, 60205),
			new MultiWorldClientSession.LocationData("Paradox Cave Lower - Right", 478UL, 64, 60208),
			new MultiWorldClientSession.LocationData("Paradox Cave Lower - Far Right", 478UL, 128, 60211),
			new MultiWorldClientSession.LocationData("Paradox Cave Lower - Middle", 479UL, 1, 60214),
			new MultiWorldClientSession.LocationData("Paradox Cave Upper - Left", 510UL, 16, 60217),
			new MultiWorldClientSession.LocationData("Paradox Cave Upper - Right", 510UL, 32, 60220),
			new MultiWorldClientSession.LocationData("Spiral Cave", 508UL, 16, 59839),
			new MultiWorldClientSession.LocationData("Ether Tablet", 1041UL, 1, 1572886),
			new MultiWorldClientSession.LocationData("Spectacle Rock", 643UL, 64, 1573184),
			new MultiWorldClientSession.LocationData("Tower of Hera - Basement Cage", 271UL, 4, 1573218),
			new MultiWorldClientSession.LocationData("Tower of Hera - Map Chest", 238UL, 16, 59821),
			new MultiWorldClientSession.LocationData("Tower of Hera - Big Key Chest", 270UL, 16, 59878),
			new MultiWorldClientSession.LocationData("Tower of Hera - Compass Chest", 78UL, 32, 59899),
			new MultiWorldClientSession.LocationData("Tower of Hera - Big Chest", 78UL, 16, 59896),
			new MultiWorldClientSession.LocationData("Tower of Hera - Boss", 15UL, 8, 1573202),
			new MultiWorldClientSession.LocationData("Pyramid", 731UL, 64, 1573191),
			new MultiWorldClientSession.LocationData("Catfish", 1040UL, 32, 975237),
			new MultiWorldClientSession.LocationData("Stumpy", 1040UL, 8, 209095),
			new MultiWorldClientSession.LocationData("Digging Game", 744UL, 64, 1573192),
			new MultiWorldClientSession.LocationData("Bombos Tablet", 1041UL, 2, 1572887),
			new MultiWorldClientSession.LocationData("Hype Cave - Top", 572UL, 16, 60190),
			new MultiWorldClientSession.LocationData("Hype Cave - Middle Right", 572UL, 32, 60193),
			new MultiWorldClientSession.LocationData("Hype Cave - Middle Left", 572UL, 64, 60196),
			new MultiWorldClientSession.LocationData("Hype Cave - Bottom", 572UL, 128, 60199),
			new MultiWorldClientSession.LocationData("Hype Cave - Generous Guy", 573UL, 4, 1572881),
			new MultiWorldClientSession.LocationData("Peg Cave", 591UL, 4, 1572870),
			new MultiWorldClientSession.LocationData("Pyramid Fairy - Left", 556UL, 16, 59776),
			new MultiWorldClientSession.LocationData("Pyramid Fairy - Right", 556UL, 32, 59779),
			new MultiWorldClientSession.LocationData("Brewery", 524UL, 16, 59884),
			new MultiWorldClientSession.LocationData("C-Shaped House", 568UL, 16, 59887),
			new MultiWorldClientSession.LocationData("Chest Game", 525UL, 4, 60840),
			new MultiWorldClientSession.LocationData("Bumper Cave Ledge", 714UL, 64, 1573190),
			new MultiWorldClientSession.LocationData("Mire Shed - Left", 538UL, 16, 60019),
			new MultiWorldClientSession.LocationData("Mire Shed - Right", 538UL, 32, 60022),
			new MultiWorldClientSession.LocationData("Superbunny Cave - Top", 496UL, 16, 60028),
			new MultiWorldClientSession.LocationData("Superbunny Cave - Bottom", 496UL, 32, 60031),
			new MultiWorldClientSession.LocationData("Spike Cave", 558UL, 16, 60043),
			new MultiWorldClientSession.LocationData("Hookshot Cave - Top Right", 120UL, 16, 60241),
			new MultiWorldClientSession.LocationData("Hookshot Cave - Top Left", 120UL, 32, 60244),
			new MultiWorldClientSession.LocationData("Hookshot Cave - Bottom Right", 120UL, 128, 60250),
			new MultiWorldClientSession.LocationData("Hookshot Cave - Bottom Left", 120UL, 64, 60247),
			new MultiWorldClientSession.LocationData("Floating Island", 645UL, 64, 1573185),
			new MultiWorldClientSession.LocationData("Mimic Cave", 536UL, 16, 59845),
			new MultiWorldClientSession.LocationData("Swamp Palace - Entrance", 80UL, 16, 60061),
			new MultiWorldClientSession.LocationData("Swamp Palace - Map Chest", 110UL, 16, 59782),
			new MultiWorldClientSession.LocationData("Swamp Palace - Big Chest", 108UL, 16, 59785),
			new MultiWorldClientSession.LocationData("Swamp Palace - Compass Chest", 140UL, 16, 60064),
			new MultiWorldClientSession.LocationData("Swamp Palace - Big Key Chest", 106UL, 16, 60070),
			new MultiWorldClientSession.LocationData("Swamp Palace - West Chest", 104UL, 16, 60067),
			new MultiWorldClientSession.LocationData("Swamp Palace - Flooded Room - Left", 236UL, 16, 60073),
			new MultiWorldClientSession.LocationData("Swamp Palace - Flooded Room - Right", 236UL, 32, 60076),
			new MultiWorldClientSession.LocationData("Swamp Palace - Waterfall Room", 204UL, 16, 60079),
			new MultiWorldClientSession.LocationData("Swamp Palace - Boss", 13UL, 8, 1573204),
			new MultiWorldClientSession.LocationData("Thieves' Town - Big Key Chest", 438UL, 32, 59908),
			new MultiWorldClientSession.LocationData("Thieves' Town - Map Chest", 438UL, 16, 59905),
			new MultiWorldClientSession.LocationData("Thieves' Town - Compass Chest", 440UL, 16, 59911),
			new MultiWorldClientSession.LocationData("Thieves' Town - Ambush Chest", 406UL, 16, 59914),
			new MultiWorldClientSession.LocationData("Thieves' Town - Attic", 202UL, 16, 59917),
			new MultiWorldClientSession.LocationData("Thieves' Town - Big Chest", 136UL, 16, 59920),
			new MultiWorldClientSession.LocationData("Thieves' Town - Blind's Cell", 138UL, 16, 59923),
			new MultiWorldClientSession.LocationData("Thieves' Town - Boss", 345UL, 8, 1573206),
			new MultiWorldClientSession.LocationData("Skull Woods - Compass Chest", 206UL, 16, 59794),
			new MultiWorldClientSession.LocationData("Skull Woods - Map Chest", 176UL, 32, 59803),
			new MultiWorldClientSession.LocationData("Skull Woods - Big Chest", 176UL, 16, 59800),
			new MultiWorldClientSession.LocationData("Skull Woods - Pot Prison", 174UL, 32, 59809),
			new MultiWorldClientSession.LocationData("Skull Woods - Pinball Room", 208UL, 16, 59848),
			new MultiWorldClientSession.LocationData("Skull Woods - Big Key Chest", 174UL, 16, 59806),
			new MultiWorldClientSession.LocationData("Skull Woods - Bridge Room", 178UL, 16, 59902),
			new MultiWorldClientSession.LocationData("Skull Woods - Boss", 83UL, 8, 1573205),
			new MultiWorldClientSession.LocationData("Ice Palace - Compass Chest", 92UL, 16, 59860),
			new MultiWorldClientSession.LocationData("Ice Palace - Freezor Chest", 252UL, 16, 59797),
			new MultiWorldClientSession.LocationData("Ice Palace - Big Chest", 316UL, 16, 59818),
			new MultiWorldClientSession.LocationData("Ice Palace - Iced T Room", 348UL, 16, 59875),
			new MultiWorldClientSession.LocationData("Ice Palace - Spike Room", 190UL, 16, 59872),
			new MultiWorldClientSession.LocationData("Ice Palace - Big Key Chest", 62UL, 16, 59812),
			new MultiWorldClientSession.LocationData("Ice Palace - Map Chest", 126UL, 16, 59869),
			new MultiWorldClientSession.LocationData("Ice Palace - Boss", 445UL, 8, 1573207),
			new MultiWorldClientSession.LocationData("Misery Mire - Big Chest", 390UL, 16, 60007),
			new MultiWorldClientSession.LocationData("Misery Mire - Map Chest", 390UL, 32, 60010),
			new MultiWorldClientSession.LocationData("Misery Mire - Main Lobby", 388UL, 16, 59998),
			new MultiWorldClientSession.LocationData("Misery Mire - Bridge Chest", 324UL, 16, 60001),
			new MultiWorldClientSession.LocationData("Misery Mire - Spike Chest", 358UL, 16, 59866),
			new MultiWorldClientSession.LocationData("Misery Mire - Compass Chest", 386UL, 16, 60004),
			new MultiWorldClientSession.LocationData("Misery Mire - Big Key Chest", 418UL, 16, 60013),
			new MultiWorldClientSession.LocationData("Misery Mire - Boss", 289UL, 8, 1573208),
			new MultiWorldClientSession.LocationData("Turtle Rock - Compass Chest", 428UL, 16, 59938),
			new MultiWorldClientSession.LocationData("Turtle Rock - Roller Room - Left", 366UL, 16, 59932),
			new MultiWorldClientSession.LocationData("Turtle Rock - Roller Room - Right", 366UL, 32, 59935),
			new MultiWorldClientSession.LocationData("Turtle Rock - Chain Chomps", 364UL, 16, 59926),
			new MultiWorldClientSession.LocationData("Turtle Rock - Big Key Chest", 40UL, 16, 59941),
			new MultiWorldClientSession.LocationData("Turtle Rock - Big Chest", 72UL, 16, 59929),
			new MultiWorldClientSession.LocationData("Turtle Rock - Crystaroller Room", 8UL, 16, 59956),
			new MultiWorldClientSession.LocationData("Turtle Rock - Eye Bridge - Bottom Left", 426UL, 128, 59953),
			new MultiWorldClientSession.LocationData("Turtle Rock - Eye Bridge - Bottom Right", 426UL, 64, 59950),
			new MultiWorldClientSession.LocationData("Turtle Rock - Eye Bridge - Top Left", 426UL, 32, 59947),
			new MultiWorldClientSession.LocationData("Turtle Rock - Eye Bridge - Top Right", 426UL, 16, 59944),
			new MultiWorldClientSession.LocationData("Turtle Rock - Boss", 329UL, 8, 1573209),
			new MultiWorldClientSession.LocationData("Palace of Darkness - Shooter Room", 18UL, 16, 59995),
			new MultiWorldClientSession.LocationData("Palace of Darkness - The Arena - Bridge", 84UL, 32, 59965),
			new MultiWorldClientSession.LocationData("Palace of Darkness - Stalfos Basement", 20UL, 16, 59977),
			new MultiWorldClientSession.LocationData("Palace of Darkness - Big Key Chest", 116UL, 16, 59959),
			new MultiWorldClientSession.LocationData("Palace of Darkness - The Arena - Ledge", 84UL, 16, 59962),
			new MultiWorldClientSession.LocationData("Palace of Darkness - Map Chest", 86UL, 16, 59986),
			new MultiWorldClientSession.LocationData("Palace of Darkness - Compass Chest", 52UL, 32, 59971),
			new MultiWorldClientSession.LocationData("Palace of Darkness - Dark Basement - Left", 212UL, 16, 59980),
			new MultiWorldClientSession.LocationData("Palace of Darkness - Dark Basement - Right", 212UL, 32, 59983),
			new MultiWorldClientSession.LocationData("Palace of Darkness - Dark Maze - Top", 50UL, 16, 59989),
			new MultiWorldClientSession.LocationData("Palace of Darkness - Dark Maze - Bottom", 50UL, 32, 59992),
			new MultiWorldClientSession.LocationData("Palace of Darkness - Big Chest", 52UL, 16, 59968),
			new MultiWorldClientSession.LocationData("Palace of Darkness - Harmless Hellway", 52UL, 64, 59974),
			new MultiWorldClientSession.LocationData("Palace of Darkness - Boss", 181UL, 8, 1573203),
			new MultiWorldClientSession.LocationData("Ganons Tower - Bob's Torch", 281UL, 4, 1573217),
			new MultiWorldClientSession.LocationData("Ganons Tower - Hope Room - Left", 280UL, 32, 60121),
			new MultiWorldClientSession.LocationData("Ganons Tower - Hope Room - Right", 280UL, 64, 60124),
			new MultiWorldClientSession.LocationData("Ganons Tower - Tile Room", 282UL, 16, 60130),
			new MultiWorldClientSession.LocationData("Ganons Tower - Compass Room - Top Left", 314UL, 16, 60133),
			new MultiWorldClientSession.LocationData("Ganons Tower - Compass Room - Top Right", 314UL, 32, 60136),
			new MultiWorldClientSession.LocationData("Ganons Tower - Compass Room - Bottom Left", 314UL, 64, 60139),
			new MultiWorldClientSession.LocationData("Ganons Tower - Compass Room - Bottom Right", 314UL, 128, 60142),
			new MultiWorldClientSession.LocationData("Ganons Tower - DMs Room - Top Left", 246UL, 16, 60088),
			new MultiWorldClientSession.LocationData("Ganons Tower - DMs Room - Top Right", 246UL, 32, 60091),
			new MultiWorldClientSession.LocationData("Ganons Tower - DMs Room - Bottom Left", 246UL, 64, 60094),
			new MultiWorldClientSession.LocationData("Ganons Tower - DMs Room - Bottom Right", 246UL, 128, 60097),
			new MultiWorldClientSession.LocationData("Ganons Tower - Map Chest", 278UL, 16, 60115),
			new MultiWorldClientSession.LocationData("Ganons Tower - Firesnake Room", 250UL, 16, 60112),
			new MultiWorldClientSession.LocationData("Ganons Tower - Randomizer Room - Top Left", 248UL, 16, 60100),
			new MultiWorldClientSession.LocationData("Ganons Tower - Randomizer Room - Top Right", 248UL, 32, 60103),
			new MultiWorldClientSession.LocationData("Ganons Tower - Randomizer Room - Bottom Left", 248UL, 64, 60106),
			new MultiWorldClientSession.LocationData("Ganons Tower - Randomizer Room - Bottom Right", 248UL, 128, 60109),
			new MultiWorldClientSession.LocationData("Ganons Tower - Bob's Chest", 280UL, 128, 60127),
			new MultiWorldClientSession.LocationData("Ganons Tower - Big Chest", 280UL, 16, 60118),
			new MultiWorldClientSession.LocationData("Ganons Tower - Big Key Room - Left", 56UL, 32, 60148),
			new MultiWorldClientSession.LocationData("Ganons Tower - Big Key Room - Right", 56UL, 64, 60151),
			new MultiWorldClientSession.LocationData("Ganons Tower - Big Key Chest", 56UL, 16, 60145),
			new MultiWorldClientSession.LocationData("Ganons Tower - Mini Helmasaur Room - Left", 122UL, 16, 60157),
			new MultiWorldClientSession.LocationData("Ganons Tower - Mini Helmasaur Room - Right", 122UL, 32, 60160),
			new MultiWorldClientSession.LocationData("Ganons Tower - Pre-Moldorm Chest", 122UL, 64, 60163),
			new MultiWorldClientSession.LocationData("Ganons Tower - Validation Chest", 154UL, 16, 60166)
		};

		// Token: 0x040000E9 RID: 233
		private static MultiWorldClientSession.ItemDescription[] Items = new MultiWorldClientSession.ItemDescription[]
		{
			new MultiWorldClientSession.ItemDescription("Bow", 11),
			new MultiWorldClientSession.ItemDescription("Progressive Bow", 100),
			new MultiWorldClientSession.ItemDescription("Progressive Bow", 101),
			new MultiWorldClientSession.ItemDescription("Book of Mudora", 29),
			new MultiWorldClientSession.ItemDescription("Hammer", 9),
			new MultiWorldClientSession.ItemDescription("Hookshot", 10),
			new MultiWorldClientSession.ItemDescription("Magic Mirror", 26),
			new MultiWorldClientSession.ItemDescription("Ocarina", 20),
			new MultiWorldClientSession.ItemDescription("Pegasus Boots", 75),
			new MultiWorldClientSession.ItemDescription("Power Glove", 27),
			new MultiWorldClientSession.ItemDescription("Cape", 25),
			new MultiWorldClientSession.ItemDescription("Mushroom", 41),
			new MultiWorldClientSession.ItemDescription("Shovel", 19),
			new MultiWorldClientSession.ItemDescription("Lamp", 18),
			new MultiWorldClientSession.ItemDescription("Magic Powder", 13),
			new MultiWorldClientSession.ItemDescription("Moon Pearl", 31),
			new MultiWorldClientSession.ItemDescription("Cane of Somaria", 21),
			new MultiWorldClientSession.ItemDescription("Fire Rod", 7),
			new MultiWorldClientSession.ItemDescription("Flippers", 30),
			new MultiWorldClientSession.ItemDescription("Ice Rod", 8),
			new MultiWorldClientSession.ItemDescription("Titans Mitts", 28),
			new MultiWorldClientSession.ItemDescription("Ether", 16),
			new MultiWorldClientSession.ItemDescription("Bombos", 15),
			new MultiWorldClientSession.ItemDescription("Quake", 17),
			new MultiWorldClientSession.ItemDescription("Bottle", 22),
			new MultiWorldClientSession.ItemDescription("Bottle (Red Potion)", 43),
			new MultiWorldClientSession.ItemDescription("Bottle (Green Potion)", 44),
			new MultiWorldClientSession.ItemDescription("Bottle (Blue Potion)", 45),
			new MultiWorldClientSession.ItemDescription("Bottle (Fairy)", 61),
			new MultiWorldClientSession.ItemDescription("Bottle (Bee)", 60),
			new MultiWorldClientSession.ItemDescription("Bottle (Good Bee)", 72),
			new MultiWorldClientSession.ItemDescription("Master Sword", 80),
			new MultiWorldClientSession.ItemDescription("Tempered Sword", 2),
			new MultiWorldClientSession.ItemDescription("Fighter Sword", 73),
			new MultiWorldClientSession.ItemDescription("Golden Sword", 3),
			new MultiWorldClientSession.ItemDescription("Progressive Sword", 94),
			new MultiWorldClientSession.ItemDescription("Progressive Glove", 97),
			new MultiWorldClientSession.ItemDescription("Silver Arrows", 88),
			new MultiWorldClientSession.ItemDescription("Triforce", 106),
			new MultiWorldClientSession.ItemDescription("Power Star", 107),
			new MultiWorldClientSession.ItemDescription("Triforce Piece", 108),
			new MultiWorldClientSession.ItemDescription("Single Arrow", 67),
			new MultiWorldClientSession.ItemDescription("Arrows (10)", 68),
			new MultiWorldClientSession.ItemDescription("Arrow Upgrade (+10)", 84),
			new MultiWorldClientSession.ItemDescription("Arrow Upgrade (+5)", 83),
			new MultiWorldClientSession.ItemDescription("Single Bomb", 39),
			new MultiWorldClientSession.ItemDescription("Bombs (3)", 40),
			new MultiWorldClientSession.ItemDescription("Bombs (10)", 49),
			new MultiWorldClientSession.ItemDescription("Bomb Upgrade (+10)", 82),
			new MultiWorldClientSession.ItemDescription("Bomb Upgrade (+5)", 81),
			new MultiWorldClientSession.ItemDescription("Blue Mail", 34),
			new MultiWorldClientSession.ItemDescription("Red Mail", 35),
			new MultiWorldClientSession.ItemDescription("Progressive Armor", 96),
			new MultiWorldClientSession.ItemDescription("Blue Boomerang", 12),
			new MultiWorldClientSession.ItemDescription("Red Boomerang", 42),
			new MultiWorldClientSession.ItemDescription("Blue Shield", 4),
			new MultiWorldClientSession.ItemDescription("Red Shield", 5),
			new MultiWorldClientSession.ItemDescription("Mirror Shield", 6),
			new MultiWorldClientSession.ItemDescription("Progressive Shield", 95),
			new MultiWorldClientSession.ItemDescription("Bug Catching Net", 33),
			new MultiWorldClientSession.ItemDescription("Cane of Byrna", 24),
			new MultiWorldClientSession.ItemDescription("Boss Heart Container", 62),
			new MultiWorldClientSession.ItemDescription("Sanctuary Heart Container", 63),
			new MultiWorldClientSession.ItemDescription("Piece of Heart", 23),
			new MultiWorldClientSession.ItemDescription("Rupee (1)", 52),
			new MultiWorldClientSession.ItemDescription("Rupees (5)", 53),
			new MultiWorldClientSession.ItemDescription("Rupees (20)", 54),
			new MultiWorldClientSession.ItemDescription("Rupees (50)", 65),
			new MultiWorldClientSession.ItemDescription("Rupees (100)", 64),
			new MultiWorldClientSession.ItemDescription("Rupees (300)", 70),
			new MultiWorldClientSession.ItemDescription("Rupoor", 89),
			new MultiWorldClientSession.ItemDescription("Red Clock", 91),
			new MultiWorldClientSession.ItemDescription("Blue Clock", 92),
			new MultiWorldClientSession.ItemDescription("Green Clock", 93),
			new MultiWorldClientSession.ItemDescription("Single RNG", 98),
			new MultiWorldClientSession.ItemDescription("Multi RNG", 99),
			new MultiWorldClientSession.ItemDescription("Magic Upgrade (1/2)", 78),
			new MultiWorldClientSession.ItemDescription("Magic Upgrade (1/4)", 79),
			new MultiWorldClientSession.ItemDescription("Small Key (Eastern Palace)", 162),
			new MultiWorldClientSession.ItemDescription("Big Key (Eastern Palace)", 157),
			new MultiWorldClientSession.ItemDescription("Compass (Eastern Palace)", 141),
			new MultiWorldClientSession.ItemDescription("Map (Eastern Palace)", 125),
			new MultiWorldClientSession.ItemDescription("Small Key (Desert Palace)", 163),
			new MultiWorldClientSession.ItemDescription("Big Key (Desert Palace)", 156),
			new MultiWorldClientSession.ItemDescription("Compass (Desert Palace)", 140),
			new MultiWorldClientSession.ItemDescription("Map (Desert Palace)", 124),
			new MultiWorldClientSession.ItemDescription("Small Key (Tower of Hera)", 170),
			new MultiWorldClientSession.ItemDescription("Big Key (Tower of Hera)", 149),
			new MultiWorldClientSession.ItemDescription("Compass (Tower of Hera)", 133),
			new MultiWorldClientSession.ItemDescription("Map (Tower of Hera)", 117),
			new MultiWorldClientSession.ItemDescription("Small Key (Escape)", 160),
			new MultiWorldClientSession.ItemDescription("Big Key (Escape)", 159),
			new MultiWorldClientSession.ItemDescription("Compass (Escape)", 143),
			new MultiWorldClientSession.ItemDescription("Map (Escape)", 127),
			new MultiWorldClientSession.ItemDescription("Small Key (Agahnims Tower)", 164),
			new MultiWorldClientSession.ItemDescription("Small Key (Palace of Darkness)", 166),
			new MultiWorldClientSession.ItemDescription("Big Key (Palace of Darkness)", 153),
			new MultiWorldClientSession.ItemDescription("Compass (Palace of Darkness)", 137),
			new MultiWorldClientSession.ItemDescription("Map (Palace of Darkness)", 121),
			new MultiWorldClientSession.ItemDescription("Small Key (Thieves Town)", 171),
			new MultiWorldClientSession.ItemDescription("Big Key (Thieves Town)", 148),
			new MultiWorldClientSession.ItemDescription("Compass (Thieves Town)", 132),
			new MultiWorldClientSession.ItemDescription("Map (Thieves Town)", 116),
			new MultiWorldClientSession.ItemDescription("Small Key (Skull Woods)", 168),
			new MultiWorldClientSession.ItemDescription("Big Key (Skull Woods)", 151),
			new MultiWorldClientSession.ItemDescription("Compass (Skull Woods)", 135),
			new MultiWorldClientSession.ItemDescription("Map (Skull Woods)", 119),
			new MultiWorldClientSession.ItemDescription("Small Key (Swamp Palace)", 165),
			new MultiWorldClientSession.ItemDescription("Big Key (Swamp Palace)", 154),
			new MultiWorldClientSession.ItemDescription("Compass (Swamp Palace)", 138),
			new MultiWorldClientSession.ItemDescription("Map (Swamp Palace)", 122),
			new MultiWorldClientSession.ItemDescription("Small Key (Ice Palace)", 169),
			new MultiWorldClientSession.ItemDescription("Big Key (Ice Palace)", 150),
			new MultiWorldClientSession.ItemDescription("Compass (Ice Palace)", 134),
			new MultiWorldClientSession.ItemDescription("Map (Ice Palace)", 118),
			new MultiWorldClientSession.ItemDescription("Small Key (Misery Mire)", 167),
			new MultiWorldClientSession.ItemDescription("Big Key (Misery Mire)", 152),
			new MultiWorldClientSession.ItemDescription("Compass (Misery Mire)", 136),
			new MultiWorldClientSession.ItemDescription("Map (Misery Mire)", 120),
			new MultiWorldClientSession.ItemDescription("Small Key (Turtle Rock)", 172),
			new MultiWorldClientSession.ItemDescription("Big Key (Turtle Rock)", 147),
			new MultiWorldClientSession.ItemDescription("Compass (Turtle Rock)", 131),
			new MultiWorldClientSession.ItemDescription("Map (Turtle Rock)", 115),
			new MultiWorldClientSession.ItemDescription("Small Key (Ganons Tower)", 173),
			new MultiWorldClientSession.ItemDescription("Big Key (Ganons Tower)", 146),
			new MultiWorldClientSession.ItemDescription("Compass (Ganons Tower)", 130),
			new MultiWorldClientSession.ItemDescription("Map (Ganons Tower)", 114),
			new MultiWorldClientSession.ItemDescription("Small Key (Universal)", 175),
			new MultiWorldClientSession.ItemDescription("Nothing", 90),
			new MultiWorldClientSession.ItemDescription("Red Potion", 46),
			new MultiWorldClientSession.ItemDescription("Green Potion", 47),
			new MultiWorldClientSession.ItemDescription("Blue Potion", 48),
			new MultiWorldClientSession.ItemDescription("Bee", 14),
			new MultiWorldClientSession.ItemDescription("Small Heart", 66)
		};

		// Token: 0x0200005F RID: 95
		public class SessionDescription : ObservableObject
		{
			// Token: 0x170000AC RID: 172
			// (get) Token: 0x06000342 RID: 834 RVA: 0x00010898 File Offset: 0x0000EA98
			// (set) Token: 0x06000343 RID: 835 RVA: 0x000108A0 File Offset: 0x0000EAA0
			public bool PasswordRequired
			{
				get
				{
					return this.mbPasswordRequired;
				}
				private set
				{
					base.SetProperty<bool>(ref this.mbPasswordRequired, value, "PasswordRequired");
				}
			}

			// Token: 0x170000AD RID: 173
			// (get) Token: 0x06000344 RID: 836 RVA: 0x000108B5 File Offset: 0x0000EAB5
			public IReadOnlyList<MultiWorldClientSession.SessionDescription.Slot> Slots
			{
				get
				{
					return this.mSlots;
				}
			}

			// Token: 0x170000AE RID: 174
			// (get) Token: 0x06000345 RID: 837 RVA: 0x000108BD File Offset: 0x0000EABD
			// (set) Token: 0x06000346 RID: 838 RVA: 0x000108CC File Offset: 0x0000EACC
			public int PlayerCount
			{
				get
				{
					return this.mSlots.Count;
				}
				private set
				{
					this.mSlots.Clear();
					for (int i = 0; i < value; i++)
					{
						this.mSlots.Add(new MultiWorldClientSession.SessionDescription.Slot
						{
							Index = i + 1
						});
					}
				}
			}

			// Token: 0x06000347 RID: 839 RVA: 0x0001090C File Offset: 0x0000EB0C
			public SessionDescription(JToken data, MultiWorldClientSession session)
			{
				JObject dataObject = data as JObject;
				if (dataObject == null)
				{
					throw new InvalidDataException("Unsupported data format for session description");
				}
				this.PlayerCount = dataObject.GetValue<int>("slots", 0);
				this.PasswordRequired = dataObject.GetValue("password", false);
				if (this.PlayerCount > 0 || this.PasswordRequired)
				{
					session.Log("--------------------------------", Array.Empty<object>());
					session.Log("Room Information:", Array.Empty<object>());
					session.Log("--------------------------------", Array.Empty<object>());
				}
				if (this.PlayerCount > 0)
				{
					session.Log("{0} player seed", new object[]
					{
						this.PlayerCount
					});
				}
				if (this.PasswordRequired)
				{
					session.Log("Password Required", Array.Empty<object>());
				}
			}

			// Token: 0x04000186 RID: 390
			private bool mbPasswordRequired;

			// Token: 0x04000187 RID: 391
			private ObservableCollection<MultiWorldClientSession.SessionDescription.Slot> mSlots = new ObservableCollection<MultiWorldClientSession.SessionDescription.Slot>();

			// Token: 0x02000070 RID: 112
			public class Slot : ObservableObject
			{
				// Token: 0x170000BB RID: 187
				// (get) Token: 0x06000385 RID: 901 RVA: 0x00010E1B File Offset: 0x0000F01B
				// (set) Token: 0x06000386 RID: 902 RVA: 0x00010E23 File Offset: 0x0000F023
				public string PlayerName
				{
					get
					{
						return this.mPlayerName;
					}
					set
					{
						base.SetProperty<string>(ref this.mPlayerName, value, "PlayerName");
					}
				}

				// Token: 0x170000BC RID: 188
				// (get) Token: 0x06000387 RID: 903 RVA: 0x00010E38 File Offset: 0x0000F038
				// (set) Token: 0x06000388 RID: 904 RVA: 0x00010E40 File Offset: 0x0000F040
				public string TeamName
				{
					get
					{
						return this.mTeamName;
					}
					set
					{
						base.SetProperty<string>(ref this.mTeamName, value, "TeamName");
					}
				}

				// Token: 0x170000BD RID: 189
				// (get) Token: 0x06000389 RID: 905 RVA: 0x00010E55 File Offset: 0x0000F055
				// (set) Token: 0x0600038A RID: 906 RVA: 0x00010E5D File Offset: 0x0000F05D
				[DependentProperty("DisplayIndex")]
				public int Index
				{
					get
					{
						return this.mIndex;
					}
					set
					{
						base.SetProperty<int>(ref this.mIndex, value, "Index");
					}
				}

				// Token: 0x170000BE RID: 190
				// (get) Token: 0x0600038B RID: 907 RVA: 0x00010E72 File Offset: 0x0000F072
				public int ServerIndex
				{
					get
					{
						return this.Index;
					}
				}

				// Token: 0x170000BF RID: 191
				// (get) Token: 0x0600038C RID: 908 RVA: 0x00010E72 File Offset: 0x0000F072
				public int DisplayIndex
				{
					get
					{
						return this.Index;
					}
				}

				// Token: 0x170000C0 RID: 192
				// (get) Token: 0x0600038D RID: 909 RVA: 0x00010E7A File Offset: 0x0000F07A
				// (set) Token: 0x0600038E RID: 910 RVA: 0x00010E82 File Offset: 0x0000F082
				public bool Available
				{
					get
					{
						return this.mbAvailable;
					}
					set
					{
						base.SetProperty<bool>(ref this.mbAvailable, value, "Available");
					}
				}

				// Token: 0x0600038F RID: 911 RVA: 0x00010E97 File Offset: 0x0000F097
				public Slot()
				{
				}

				// Token: 0x040001BA RID: 442
				private string mPlayerName;

				// Token: 0x040001BB RID: 443
				private string mTeamName;

				// Token: 0x040001BC RID: 444
				private int mIndex = -1;

				// Token: 0x040001BD RID: 445
				private bool mbAvailable = true;
			}
		}

		// Token: 0x02000060 RID: 96
		private class Message
		{
			// Token: 0x06000348 RID: 840 RVA: 0x00002071 File Offset: 0x00000271
			public Message()
			{
			}

			// Token: 0x04000188 RID: 392
			public string Command;

			// Token: 0x04000189 RID: 393
			public JToken Data;
		}

		// Token: 0x02000061 RID: 97
		private class LegacyConnectMsg
		{
			// Token: 0x170000AF RID: 175
			// (get) Token: 0x06000349 RID: 841 RVA: 0x000109E4 File Offset: 0x0000EBE4
			// (set) Token: 0x0600034A RID: 842 RVA: 0x000109EC File Offset: 0x0000EBEC
			public string password { get; set; }

			// Token: 0x170000B0 RID: 176
			// (get) Token: 0x0600034B RID: 843 RVA: 0x000109F5 File Offset: 0x0000EBF5
			// (set) Token: 0x0600034C RID: 844 RVA: 0x000109FD File Offset: 0x0000EBFD
			public string name { get; set; }

			// Token: 0x170000B1 RID: 177
			// (get) Token: 0x0600034D RID: 845 RVA: 0x00010A06 File Offset: 0x0000EC06
			// (set) Token: 0x0600034E RID: 846 RVA: 0x00010A0E File Offset: 0x0000EC0E
			public string team { get; set; }

			// Token: 0x170000B2 RID: 178
			// (get) Token: 0x0600034F RID: 847 RVA: 0x00010A17 File Offset: 0x0000EC17
			// (set) Token: 0x06000350 RID: 848 RVA: 0x00010A1F File Offset: 0x0000EC1F
			public int slot { get; set; }

			// Token: 0x06000351 RID: 849 RVA: 0x00002071 File Offset: 0x00000271
			public LegacyConnectMsg()
			{
			}
		}

		// Token: 0x02000062 RID: 98
		private class ConnectMsg
		{
			// Token: 0x170000B3 RID: 179
			// (get) Token: 0x06000352 RID: 850 RVA: 0x00010A28 File Offset: 0x0000EC28
			// (set) Token: 0x06000353 RID: 851 RVA: 0x00010A30 File Offset: 0x0000EC30
			public List<byte> rom { get; set; }

			// Token: 0x170000B4 RID: 180
			// (get) Token: 0x06000354 RID: 852 RVA: 0x00010A39 File Offset: 0x0000EC39
			// (set) Token: 0x06000355 RID: 853 RVA: 0x00010A41 File Offset: 0x0000EC41
			public string password { get; set; }

			// Token: 0x06000356 RID: 854 RVA: 0x00002071 File Offset: 0x00000271
			public ConnectMsg()
			{
			}
		}

		// Token: 0x02000063 RID: 99
		private class ReceivedItem
		{
			// Token: 0x170000B5 RID: 181
			// (get) Token: 0x06000357 RID: 855 RVA: 0x00010A4A File Offset: 0x0000EC4A
			// (set) Token: 0x06000358 RID: 856 RVA: 0x00010A52 File Offset: 0x0000EC52
			public int Item { get; set; }

			// Token: 0x170000B6 RID: 182
			// (get) Token: 0x06000359 RID: 857 RVA: 0x00010A5B File Offset: 0x0000EC5B
			// (set) Token: 0x0600035A RID: 858 RVA: 0x00010A63 File Offset: 0x0000EC63
			public string Location { get; set; }

			// Token: 0x170000B7 RID: 183
			// (get) Token: 0x0600035B RID: 859 RVA: 0x00010A6C File Offset: 0x0000EC6C
			// (set) Token: 0x0600035C RID: 860 RVA: 0x00010A74 File Offset: 0x0000EC74
			public string PlayerName { get; set; }

			// Token: 0x0600035D RID: 861 RVA: 0x00002071 File Offset: 0x00000271
			public ReceivedItem()
			{
			}
		}

		// Token: 0x02000064 RID: 100
		private struct LocationData
		{
			// Token: 0x0600035E RID: 862 RVA: 0x00010A7D File Offset: 0x0000EC7D
			public LocationData(string name, ulong offset, byte mask, int id)
			{
				this.Name = name;
				this.Offset = offset;
				this.Mask = mask;
				this.ID = id;
			}

			// Token: 0x04000193 RID: 403
			public string Name;

			// Token: 0x04000194 RID: 404
			public ulong Offset;

			// Token: 0x04000195 RID: 405
			public byte Mask;

			// Token: 0x04000196 RID: 406
			public int ID;
		}

		// Token: 0x02000065 RID: 101
		private struct ItemDescription
		{
			// Token: 0x0600035F RID: 863 RVA: 0x00010A9C File Offset: 0x0000EC9C
			public ItemDescription(string name, int id)
			{
				this.Name = name;
				this.ID = id;
			}

			// Token: 0x04000197 RID: 407
			public string Name;

			// Token: 0x04000198 RID: 408
			public int ID;
		}
	}
}
