﻿// Decompiled with JetBrains decompiler
// Type: ConnectorLib.sd2snes.Scheduler
// Assembly: ConnectorLib.sd2snes, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: 743ADA37-D030-4892-BC8D-59E965B0A960
// Assembly location: C:\Program Files (x86)\EmoTracker\ConnectorLib.sd2snes.dll

using ConnectorLib.Patches;
using ConnectorLib.usb2snes;
using log4net;
using Newtonsoft.Json;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Security.Cryptography;
using System.Threading.Tasks;
using WebSocketSharp;

namespace ConnectorLib.sd2snes
{
  public class Scheduler : IDisposable
  {
    private readonly Core _core = new Core();
    private readonly SortedDictionary<string, Scheduler> _ports;
    private volatile bool _stop;
    private readonly Task _task;
    private static readonly ILog _log = LogManager.GetLogger(typeof (Scheduler));

    public bool CoreConnected => this._core.Connected;

    public event EventHandler<bool> ConnectionStateChanged;

    public bool ConnectionState { get; private set; }

    public BlockingCollection<sd2snesServer.RequestQueueElementType> Queue { get; }

    public Dictionary<int, ClientSocket> Clients { get; }

    public void Dispose()
    {
      try
      {
        this._task?.Dispose();
      }
      catch
      {
      }
      try
      {
        this._core?.Dispose();
      }
      catch
      {
      }
      try
      {
        this.Queue.Dispose();
      }
      catch
      {
      }
    }

    ~Scheduler() => this.Dispose();

    public Scheduler(string name, SortedDictionary<string, Scheduler> ports)
    {
      this.Queue = new BlockingCollection<sd2snesServer.RequestQueueElementType>();
      this.Clients = new Dictionary<int, ClientSocket>();
      this._core.ConnectionStateChanged += new EventHandler<bool>(this._core_ConnectionStateChanged);
      this._core.Connect(name);
      this._core.ClearData();
      this._ports = ports;
      this._task = new Task(new Action(this.Run), TaskCreationOptions.LongRunning);
      this._task.Start();
    }

    private void _core_ConnectionStateChanged(object sender, bool e)
    {
      this.ConnectionState = e;
      EventHandler<bool> connectionStateChanged = this.ConnectionStateChanged;
      if (connectionStateChanged == null)
        return;
      connectionStateChanged(sender, e);
    }

    public void Run()
    {
      MD5 md5 = MD5.Create();
      while (!this._stop)
      {
        sd2snesServer.RequestQueueElementType queueElementType1 = (sd2snesServer.RequestQueueElementType) null;
        bool flag1 = false;
        try
        {
          queueElementType1 = this.Queue.Take();
        }
        catch
        {
          flag1 = true;
        }
        if (!flag1)
        {
          sd2snesServer.RequestQueueElementType queueElementType2 = queueElementType1;
          ConnectorLib.usb2snes.RequestType request = queueElementType2.Request;
          ClientSocket socket = queueElementType2.Socket;
          Scheduler._log.Info((object) ("USB Begin: " + JsonConvert.SerializeObject((object) request)));
          OpcodeType opcodeType;
          usbint_server_flags_e flags1;
          try
          {
            opcodeType = (OpcodeType) Enum.Parse(typeof (OpcodeType), request.Opcode);
            flags1 = usbint_server_flags_e.NONE;
            foreach (string str in (IEnumerable<string>) request.Flags ?? Enumerable.Empty<string>())
              flags1 |= (usbint_server_flags_e) Enum.Parse(typeof (usbint_server_flags_e), str);
          }
          catch (Exception ex)
          {
            Scheduler._log.Warn((object) ("Invalid Command Exception: " + ex.Message));
            socket.Context.WebSocket.Close(CloseStatusCode.ProtocolError, "Invalid Command Exception: " + ex.Message);
            continue;
          }
          try
          {
            switch (opcodeType)
            {
              case OpcodeType.Info:
                List<string> stringList = (List<string>) this._core.SendCommand(usbint_server_opcode_e.INFO, usbint_server_space_e.SNES, flags1);
                ResponseType responseType1 = new ResponseType()
                {
                  Results = stringList
                };
                socket.Queue.Add(new sd2snesServer.ResponseQueueElementType()
                {
                  Response = responseType1,
                  Done = true
                });
                break;
              case OpcodeType.Boot:
                this._core.SendCommand(usbint_server_opcode_e.BOOT, usbint_server_space_e.FILE, flags1, (object) request.Operands[0]);
                break;
              case OpcodeType.Menu:
              case OpcodeType.Reset:
                this._core.SendCommand(opcodeType == OpcodeType.Menu ? usbint_server_opcode_e.MENU_RESET : usbint_server_opcode_e.RESET, usbint_server_space_e.FILE, flags1);
                break;
              case OpcodeType.Stream:
                this._core.SendCommand(usbint_server_opcode_e.STREAM, usbint_server_space_e.MSU, flags1 | usbint_server_flags_e.DATA64B);
                int offset1 = 0;
                byte[] numArray1 = new byte[64];
                bool flag2 = false;
                while (!flag2)
                {
                  int count = 64;
                  offset1 += this._core.GetData(numArray1, offset1, 64 - offset1);
                  if (offset1 == count)
                  {
                    if ((flags1 & usbint_server_flags_e.STREAM_BURST) != usbint_server_flags_e.NONE)
                      flag2 = numArray1[count - 2] == byte.MaxValue && numArray1[count - 1] == byte.MaxValue;
                    socket.Queue.Add(new sd2snesServer.ResponseQueueElementType()
                    {
                      Data = new ArraySegment<byte>(numArray1, 0, count).ToArray<byte>(),
                      Done = flag2
                    });
                    offset1 = 0;
                  }
                }
                break;
              case OpcodeType.GetAddress:
                int count1 = 0;
                bool flag3 = false;
                Tuple<int, int>[] tupleArray1 = request.Operands.Count <= 16 ? new Tuple<int, int>[request.Operands.Count / 2] : (Tuple<int, int>[]) null;
                string s1 = "";
                int num1 = 0;
                foreach (string operand in request.Operands)
                {
                  if (flag3)
                  {
                    int num2 = int.Parse(operand, NumberStyles.HexNumber);
                    int num3 = int.Parse(s1, NumberStyles.HexNumber);
                    count1 += num2;
                    if (num2 > (int) byte.MaxValue)
                      tupleArray1 = (Tuple<int, int>[]) null;
                    else if (tupleArray1 != null)
                      tupleArray1[num1++] = Tuple.Create<int, int>(num3, num2);
                  }
                  else
                    s1 = operand;
                  flag3 = !flag3;
                }
                byte[] numArray2 = new byte[count1 + 512];
                int offset2 = 0;
                int num4 = 0;
                int index1 = 0;
label_47:
                if (index1 < request.Operands.Count)
                {
                  string operand1 = request.Operands[index1];
                  string operand2 = request.Operands[index1 + 1];
                  usbint_server_space_e space = (usbint_server_space_e) Enum.Parse(typeof (usbint_server_space_e), request.Space);
                  uint num5 = uint.Parse(operand1, NumberStyles.HexNumber);
                  int val2 = tupleArray1 != null ? count1 : int.Parse(operand2, NumberStyles.HexNumber);
                  if (tupleArray1 == null)
                  {
                    flags1 |= usbint_server_flags_e.NORESP;
                    this._core.SendCommand(usbint_server_opcode_e.GET, space, flags1, (object) num5, (object) (uint) val2);
                  }
                  else
                  {
                    flags1 |= usbint_server_flags_e.NORESP | usbint_server_flags_e.DATA64B;
                    index1 = request.Operands.Count - 2;
                    this._core.SendCommand(usbint_server_opcode_e.VGET, space, flags1, (object[]) tupleArray1);
                  }
                  int num6 = (flags1 & usbint_server_flags_e.DATA64B) == usbint_server_flags_e.NONE ? 512 : 64;
                  int num7 = val2 + num6 - 1 & ~(num6 - 1);
                  int val1 = 0;
                  int num8 = 0;
                  while (true)
                  {
                    int data;
                    do
                    {
                      while (val1 >= num7 || num4 + num8 - offset2 >= 1024)
                      {
                        int count2;
                        for (; num4 + num8 - offset2 >= Math.Min(count1 - offset2, 1024) && offset2 < count1; offset2 += count2)
                        {
                          count2 = Math.Min(count1 - offset2, 1024);
                          socket.Queue.Add(new sd2snesServer.ResponseQueueElementType()
                          {
                            Data = (byte[]) new ArraySegment<byte>(numArray2, offset2, count2).ToArray<byte>().Clone(),
                            Done = offset2 + count2 >= count1
                          });
                        }
                        if (val1 >= num7)
                        {
                          num4 += num8;
                          index1 += 2;
                          goto label_47;
                        }
                      }
                      int num9 = num8 % num6;
                      data = this._core.GetData(numArray2, num4 + num8, num6 - num9);
                    }
                    while (data == 0);
                    val1 += data;
                    num8 = Math.Min(val1, val2);
                  }
                }
                else
                {
                  Scheduler._log.Info((object) ("USB MD5: " + BitConverter.ToString(md5.ComputeHash(numArray2, 0, count1)).Replace("-", "")));
                  break;
                }
              case OpcodeType.PutAddress:
                int count3 = 0;
                bool flag4 = false;
                Tuple<int, int>[] tupleArray2 = request.Operands.Count <= 16 ? new Tuple<int, int>[request.Operands.Count / 2] : (Tuple<int, int>[]) null;
                string s2 = "";
                int num10 = 0;
                foreach (string operand in request.Operands)
                {
                  if (flag4)
                  {
                    int num11 = int.Parse(operand, NumberStyles.HexNumber);
                    int num12 = int.Parse(s2, NumberStyles.HexNumber);
                    count3 += num11;
                    if (num11 > (int) byte.MaxValue)
                      tupleArray2 = (Tuple<int, int>[]) null;
                    else if (tupleArray2 != null)
                      tupleArray2[num10++] = Tuple.Create<int, int>(num12, num11);
                  }
                  else
                    s2 = operand;
                  flag4 = !flag4;
                }
                byte[] numArray3 = new byte[count3 + 512];
                int num13 = 0;
                int destinationIndex1 = 0;
                int index2 = 0;
label_75:
                if (index2 < request.Operands.Count)
                {
                  string operand3 = request.Operands[index2];
                  string operand4 = request.Operands[index2 + 1];
                  usbint_server_space_e space = (usbint_server_space_e) Enum.Parse(typeof (usbint_server_space_e), request.Space);
                  uint num14 = uint.Parse(operand3, NumberStyles.HexNumber);
                  int val2 = tupleArray2 != null ? count3 : int.Parse(operand4, NumberStyles.HexNumber);
                  if (tupleArray2 == null)
                  {
                    flags1 |= usbint_server_flags_e.NORESP;
                    this._core.SendCommand(usbint_server_opcode_e.PUT, space, flags1, (object) num14, (object) (uint) val2);
                  }
                  else
                  {
                    flags1 |= usbint_server_flags_e.NORESP | usbint_server_flags_e.DATA64B;
                    index2 = request.Operands.Count - 2;
                    this._core.SendCommand(usbint_server_opcode_e.VPUT, space, flags1, (object[]) tupleArray2);
                  }
                  int num15 = (flags1 & usbint_server_flags_e.DATA64B) == usbint_server_flags_e.NONE ? 512 : 64;
                  int val1 = 0;
                  while (true)
                  {
                    while (destinationIndex1 - (num13 + val1) >= num15 || destinationIndex1 >= count3)
                    {
                      bool flag5 = num13 + num15 >= count3;
                      for (; val1 < val2 && destinationIndex1 - (num13 + val1) >= Math.Min(count3 - (num13 + val1), num15); val1 += num15)
                        this._core.SendData(new ArraySegment<byte>(numArray3, num13 + val1, num15).ToArray<byte>(), num15);
                      val1 = Math.Min(val1, val2);
                      if (val1 >= val2)
                      {
                        num13 += val1;
                        index2 += 2;
                        goto label_75;
                      }
                    }
                    byte[] sourceArray;
                    try
                    {
                      sourceArray = socket.DataQueue.Take();
                    }
                    catch
                    {
                      continue;
                    }
                    Array.Copy((Array) sourceArray, 0, (Array) numArray3, destinationIndex1, sourceArray.Length);
                    destinationIndex1 += sourceArray.Length;
                  }
                }
                else
                {
                  Scheduler._log.Info((object) ("USB MD5: " + BitConverter.ToString(md5.ComputeHash(numArray3, 0, count3)).Replace("-", "")));
                  break;
                }
              case OpcodeType.PutIPS:
                string operand5 = request.Operands[0];
                string operand6 = request.Operands[1];
                usbint_server_space_e space1 = (usbint_server_space_e) Enum.Parse(typeof (usbint_server_space_e), request.Space);
                int length1 = int.Parse(operand6, NumberStyles.HexNumber);
                byte[] numArray4 = new byte[length1];
                int num16 = (flags1 & usbint_server_flags_e.DATA64B) == usbint_server_flags_e.NONE ? 512 : 64;
                int destinationIndex2 = 0;
                while (destinationIndex2 < length1)
                {
                  byte[] sourceArray;
                  try
                  {
                    sourceArray = socket.DataQueue.Take();
                  }
                  catch
                  {
                    continue;
                  }
                  Array.Copy((Array) sourceArray, 0, (Array) numArray4, destinationIndex2, sourceArray.Length);
                  destinationIndex2 += sourceArray.Length;
                }
                IPS ips = new IPS();
                ips.Parse(numArray4);
                int num17 = 0;
                using (List<IPS.Patch>.Enumerator enumerator = ips.Items.GetEnumerator())
                {
                  while (enumerator.MoveNext())
                  {
                    IPS.Patch current = enumerator.Current;
                    byte[] numArray5 = new byte[current.Data.Count + num16];
                    Array.Copy((Array) current.Data.ToArray(), 0, (Array) numArray5, 0, current.Data.Count);
                    usbint_server_flags_e flags2 = flags1 | usbint_server_flags_e.NORESP;
                    if (operand5 == "hook" && num17 == 0)
                      flags2 |= usbint_server_flags_e.CLRX;
                    if (operand5 == "hook" && num17 == ips.Items.Count - 1)
                      flags2 |= usbint_server_flags_e.SETX;
                    this._core.SendCommand(usbint_server_opcode_e.PUT, space1, flags2, (object) (uint) current.Address, (object) (uint) current.Data.Count);
                    for (int offset3 = 0; offset3 < current.Data.Count; offset3 += num16)
                    {
                      Math.Min(current.Data.Count - offset3, 1024);
                      this._core.SendData(new ArraySegment<byte>(numArray5, offset3, num16).ToArray<byte>(), num16);
                    }
                    ++num17;
                  }
                  break;
                }
              case OpcodeType.GetFile:
                using (List<string>.Enumerator enumerator = request.Operands.GetEnumerator())
                {
                  while (enumerator.MoveNext())
                  {
                    string current = enumerator.Current;
                    int num18 = (int) this._core.SendCommand(usbint_server_opcode_e.GET, usbint_server_space_e.FILE, flags1, (object) current);
                    byte[] numArray6 = new byte[1024];
                    ResponseType responseType2 = new ResponseType()
                    {
                      Results = new List<string>()
                      {
                        num18.ToString("X")
                      }
                    };
                    socket.Queue.Add(new sd2snesServer.ResponseQueueElementType()
                    {
                      Response = responseType2,
                      Done = false
                    });
                    int num19 = (flags1 & usbint_server_flags_e.DATA64B) == usbint_server_flags_e.NONE ? 512 : 64;
                    int num20 = num18 + num19 - 1 & ~(num19 - 1);
                    int num21 = 0;
                    int num22 = num18;
                    int num23 = 0;
                    while (num21 < num20)
                    {
                      int offset4 = num21 % 1024;
                      int num24 = num21 % num19;
                      int data = this._core.GetData(numArray6, offset4, num19 - num24);
                      if (data != 0)
                      {
                        num21 += data;
                        if ((num21 == num20 || num21 - num23 >= 1024) && num23 < num22)
                        {
                          int count4 = Math.Min(num22 - num23, 1024);
                          num23 += count4;
                          socket.Queue.Add(new sd2snesServer.ResponseQueueElementType()
                          {
                            Data = new ArraySegment<byte>(numArray6, 0, count4).ToArray<byte>(),
                            Done = num23 >= num22
                          });
                        }
                      }
                    }
                  }
                  break;
                }
              case OpcodeType.PutFile:
                for (int index3 = 0; index3 < request.Operands.Count; index3 += 2)
                {
                  string operand7 = request.Operands[index3];
                  string operand8 = request.Operands[index3 + 1];
                  usbint_server_space_e space2 = usbint_server_space_e.FILE;
                  int num25 = int.Parse(operand8, NumberStyles.HexNumber);
                  this._core.SendCommand(usbint_server_opcode_e.PUT, space2, flags1, (object) operand7, (object) (uint) num25);
                  int length2 = (flags1 & usbint_server_flags_e.DATA64B) == usbint_server_flags_e.NONE ? 512 : 64;
                  byte[] numArray7 = new byte[length2];
                  byte[] numArray8 = new byte[num25 + 1024];
                  int destinationIndex3 = 0;
                  int sourceIndex = 0;
                  do
                  {
                    byte[] sourceArray = (byte[]) null;
                    bool flag6 = false;
                    try
                    {
                      sourceArray = socket.DataQueue.Take();
                    }
                    catch
                    {
                      flag6 = true;
                    }
                    if (!flag6)
                    {
                      Array.Copy((Array) sourceArray, 0, (Array) numArray8, destinationIndex3, sourceArray.Length);
                      destinationIndex3 += sourceArray.Length;
                      int num26 = destinationIndex3 >= num25 ? num25 + length2 - 1 & ~(length2 - 1) : destinationIndex3 & ~(length2 - 1);
                      while (sourceIndex < num26)
                      {
                        Array.Copy((Array) numArray8, sourceIndex, (Array) numArray7, 0, length2);
                        sourceIndex += length2;
                        this._core.SendData(numArray7, length2);
                      }
                    }
                  }
                  while (sourceIndex < num25);
                }
                break;
              case OpcodeType.List:
                using (List<string>.Enumerator enumerator = request.Operands.GetEnumerator())
                {
                  while (enumerator.MoveNext())
                  {
                    string current = enumerator.Current;
                    List<Tuple<int, string>> tupleList = (List<Tuple<int, string>>) this._core.SendCommand(usbint_server_opcode_e.LS, usbint_server_space_e.FILE, flags1, (object) current);
                    ResponseType responseType3 = new ResponseType()
                    {
                      Results = new List<string>()
                    };
                    foreach (Tuple<int, string> tuple in tupleList)
                    {
                      responseType3.Results.Add(tuple.Item1.ToString());
                      responseType3.Results.Add(tuple.Item2);
                    }
                    socket.Queue.Add(new sd2snesServer.ResponseQueueElementType()
                    {
                      Response = responseType3,
                      Done = true
                    });
                  }
                  break;
                }
              case OpcodeType.Remove:
              case OpcodeType.MakeDir:
                using (List<string>.Enumerator enumerator = request.Operands.GetEnumerator())
                {
                  while (enumerator.MoveNext())
                  {
                    string current = enumerator.Current;
                    this._core.SendCommand(opcodeType == OpcodeType.MakeDir ? usbint_server_opcode_e.MKDIR : usbint_server_opcode_e.RM, usbint_server_space_e.FILE, flags1, (object) current);
                  }
                  break;
                }
              case OpcodeType.Rename:
                for (int index4 = 0; index4 < request.Operands.Count; index4 += 2)
                {
                  string operand9 = request.Operands[index4];
                  string operand10 = request.Operands[index4 + 1];
                  this._core.SendCommand(usbint_server_opcode_e.MV, usbint_server_space_e.FILE, flags1, (object) operand9, (object) operand10);
                }
                break;
            }
          }
          catch (Exception ex)
          {
            Scheduler._log.Error((object) ("USB Exception: " + ex.Message));
            lock (this._ports)
            {
              Scheduler port = this._ports[this._core.PortName()];
              lock (port.Clients)
              {
                foreach (KeyValuePair<int, ClientSocket> keyValuePair in port.Clients.ToList<KeyValuePair<int, ClientSocket>>())
                {
                  try
                  {
                    keyValuePair.Value.Context.WebSocket.Close();
                  }
                  catch
                  {
                  }
                }
                port.Clients.Clear();
              }
              this._ports.Remove(this._core.PortName());
            }
            try
            {
              this.Stop();
            }
            catch
            {
            }
            try
            {
              this._core.Disconnect();
            }
            catch
            {
            }
            try
            {
              this._core.Dispose();
            }
            catch
            {
            }
            EventHandler<bool> connectionStateChanged = this.ConnectionStateChanged;
            if (connectionStateChanged == null)
              break;
            connectionStateChanged((object) this, this.CoreConnected);
            break;
          }
          Scheduler._log.Info((object) ("USB End: " + JsonConvert.SerializeObject((object) request)));
        }
      }
    }

    public async void Stop()
    {
      this._stop = true;
      this.Queue.CompleteAdding();
      await this._task;
    }
  }
}
