﻿// Decompiled with JetBrains decompiler
// Type: ConnectorLib.sd2snesConnectionManager
// Assembly: ConnectorLib.sd2snes, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: 743ADA37-D030-4892-BC8D-59E965B0A960
// Assembly location: C:\Program Files (x86)\EmoTracker\ConnectorLib.sd2snes.dll

using ConnectorLib.sd2snes;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Management;
using System.Runtime.CompilerServices;
using System.Threading;
using System.Threading.Tasks;

namespace ConnectorLib
{
  public class sd2snesConnectionManager : IDisposable, INotifyPropertyChanged
  {
    private sd2snesServer _sd2snes_server;
    private SortedDictionary<string, Scheduler> _sd2snes_ports;
    private readonly ManualResetEvent _sd2snes_disconnect = new ManualResetEvent(false);
    private readonly Task _usb_watcher;
    private volatile bool _quitting;

    public bool Issd2snesConnected { get; private set; }

    public void Dispose()
    {
      this._quitting = true;
      try
      {
        this._usb_watcher?.Dispose();
      }
      catch
      {
      }
    }

    public sd2snesConnectionManager()
    {
      if (this.DetectCartridge())
        this.CreateInstance();
      this._usb_watcher = Task.Factory.StartNew(new Action(this.USBWatcher), TaskCreationOptions.LongRunning);
    }

    private void USBWatcher()
    {
      while (!this._quitting)
      {
        if (this._sd2snes_server == null)
        {
          if (this.DetectCartridge())
            this.CreateInstance();
          else
            Thread.Sleep(500);
        }
        else
        {
          this._sd2snes_disconnect.WaitOne();
          this.DestroyInstance();
        }
      }
    }

    private void DestroyInstance()
    {
      try
      {
        this._sd2snes_server?.Stop();
      }
      catch
      {
      }
      this._sd2snes_server = (sd2snesServer) null;
      GC.Collect();
      GC.WaitForPendingFinalizers();
    }

    private void CreateInstance()
    {
      this.DestroyInstance();
      this._sd2snes_disconnect.Reset();
      this._sd2snes_ports = new SortedDictionary<string, Scheduler>();
      this._sd2snes_server = new sd2snesServer(this._sd2snes_ports);
      this._sd2snes_server.ConnectionStateChanged += new EventHandler<bool>(this._sd2snes_server_Onsd2snesConnected);
      this._sd2snes_server.Start();
    }

    private void _sd2snes_server_Onsd2snesConnected(object sender, bool e)
    {
      this.Issd2snesConnected = e;
      this.OnPropertyChanged("Issd2snesConnected");
      if (e)
        this._sd2snes_disconnect.Reset();
      else
        this._sd2snes_disconnect.Set();
    }

    private bool DetectCartridge()
    {
      foreach (sd2snesConnectionManager.PNPDeviceInfo pnpDevice in sd2snesConnectionManager.GetPNPDevices())
      {
        if (pnpDevice.Description.ToLowerInvariant().Contains("sd2snes") || pnpDevice.DeviceID.ToLowerInvariant().Contains("5a22"))
          return true;
      }
      return false;
    }

    private static List<sd2snesConnectionManager.PNPDeviceInfo> GetPNPDevices()
    {
      List<sd2snesConnectionManager.PNPDeviceInfo> pnpDevices = new List<sd2snesConnectionManager.PNPDeviceInfo>();
      ManagementObjectCollection objectCollection;
      using (ManagementObjectSearcher managementObjectSearcher = new ManagementObjectSearcher("Select * From Win32_PnPEntity"))
        objectCollection = managementObjectSearcher.Get();
      foreach (ManagementBaseObject managementBaseObject in objectCollection)
        pnpDevices.Add(new sd2snesConnectionManager.PNPDeviceInfo((string) managementBaseObject.GetPropertyValue("DeviceID"), (string) managementBaseObject.GetPropertyValue("PNPDeviceID"), (string) managementBaseObject.GetPropertyValue("Description")));
      objectCollection.Dispose();
      return pnpDevices;
    }

    public event PropertyChangedEventHandler PropertyChanged;

    protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
    {
      PropertyChangedEventHandler propertyChanged = this.PropertyChanged;
      if (propertyChanged == null)
        return;
      propertyChanged((object) this, new PropertyChangedEventArgs(propertyName));
    }

    public class PNPDeviceInfo
    {
      public string DeviceID { get; }

      public string PnpDeviceID { get; }

      public string Description { get; }

      public override string ToString() => this.Description;

      public PNPDeviceInfo(string deviceID, string pnpDeviceID, string description)
      {
        this.DeviceID = deviceID ?? "[NULL]";
        if (string.IsNullOrWhiteSpace(this.DeviceID))
          this.DeviceID = "[EMPTY]";
        this.PnpDeviceID = pnpDeviceID ?? "[NULL]";
        if (string.IsNullOrWhiteSpace(this.PnpDeviceID))
          this.PnpDeviceID = "[EMPTY]";
        this.Description = description ?? "[NULL]";
        if (!string.IsNullOrWhiteSpace(this.Description))
          return;
        this.Description = "[EMPTY]";
      }
    }
  }
}
