﻿// Decompiled with JetBrains decompiler
// Type: ConnectorLib.sd2snesConnector
// Assembly: ConnectorLib.sd2snes, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: 743ADA37-D030-4892-BC8D-59E965B0A960
// Assembly location: C:\Program Files (x86)\EmoTracker\ConnectorLib.sd2snes.dll

using ConnectorLib.Patches;
using ConnectorLib.usb2snes;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using System.Timers;
using WebSocketSharp;

namespace ConnectorLib
{
  public class sd2snesConnector : 
    ConnectorBase,
    ISNESConnector,
    IGameConnector,
    IDisposable,
    IAddressableConnector,
    IROMProcessor,
    I16BitConnector,
    I8BitConnector
  {
    private const uint WORK_BANK = 16384000;
    private const uint BASE_ADDRESS = 11264;
    private const int READ_TIMEOUT = 4096;
    private readonly object mTransmitLock = new object();
    private CancellationTokenSource _keepalive_cancel;
    private readonly object _cancel_lock = new object();
    private readonly Action<string> mMessageProcessor;
    private WebSocket mSocket;
    private readonly object _socket_lock = new object();
    private ulong mWrittenMessageIndex;
    private System.Timers.Timer mMessageClearTimer;
    private Action<MessageEventArgs> mPendingMessageHandler;
    private readonly ManualResetEvent mMemoryReadEvent = new ManualResetEvent(false);
    private ThreadLocal<sd2snesBatchWriteContext> mActiveBatchWriteContext = new ThreadLocal<sd2snesBatchWriteContext>();
    private readonly object writelock = new object();

    public Endianness Endianness => Endianness.LittleEndian;

    public static string Usb2SnesApplicationName { get; set; } = "EmoTracker";

    public event EventHandler<(ConnectionStatus, string)> ConnectionStatusChanged;

    protected override void Dispose(bool disposing) => this.Disconnect(true);

    public IFeatureAware Features { get; } = (IFeatureAware) new sd2snesFeatures();

    public sd2snesConnector()
      : this((Action<string>) null)
    {
    }

    public sd2snesConnector(Action<string> messageHandler)
    {
      this.mMessageProcessor = messageHandler;
      this.ConnectIfNecessary();
    }

    private void Output(string msg, params object[] tokens)
    {
      Action<string> messageProcessor = this.mMessageProcessor;
      if (messageProcessor == null)
        return;
      messageProcessor(string.Format(msg, tokens));
    }

    public bool Connected
    {
      get
      {
        lock (this._socket_lock)
        {
          WebSocket mSocket = this.mSocket;
          return mSocket != null && mSocket.IsAlive;
        }
      }
    }

    private void Disconnect(bool shutdown)
    {
      lock (this._socket_lock)
      {
        this.mSocket?.Close();
        this.mSocket = (WebSocket) null;
      }
      GC.Collect();
      GC.WaitForPendingFinalizers();
      if (shutdown)
      {
        lock (this._cancel_lock)
          this._keepalive_cancel?.Cancel();
      }
      EventHandler<(ConnectionStatus, string)> connectionStatusChanged = this.ConnectionStatusChanged;
      if (connectionStatusChanged == null)
        return;
      connectionStatusChanged((object) this, (ConnectionStatus.Closed, "Disconnected from sd2snes websocket."));
    }

    [DebuggerStepThrough]
    public void KeepAlive() => this.ReadByte(8257536U);

    private WebSocket NewSocket(ushort port)
    {
      WebSocket webSocket = new WebSocket(string.Format("ws://localhost:{0}", (object) port), Array.Empty<string>());
      webSocket.Log.Output = (Action<LogData, string>) ((data, message) =>
      {
        if (string.IsNullOrWhiteSpace(data.Message))
          return;
        this.Output(data.Message);
      });
      webSocket.OnMessage += new EventHandler<MessageEventArgs>(this.Socket_OnMessageReceived);
      return webSocket;
    }

    private bool? ConnectIfNecessary()
    {
      lock (this._socket_lock)
      {
        if (!this.Connected)
        {
          this.Disconnect(false);
          this.mSocket = this.NewSocket((ushort) 23074);
          this.mSocket.Connect();
          Thread.Sleep(100);
          if (!this.mSocket.IsAlive)
          {
            this.mSocket = this.NewSocket((ushort) 8080);
            this.mSocket.Connect();
            Thread.Sleep(100);
          }
          if (!this.mSocket.IsAlive)
          {
            this.mSocket = this.NewSocket((ushort) 36952);
            this.mSocket.Connect();
            Thread.Sleep(100);
          }
          if (!this.mSocket.IsAlive)
          {
            this.mSocket = this.NewSocket((ushort) 64213);
            this.mSocket.Connect();
            Thread.Sleep(100);
          }
          if (this.mSocket.IsAlive)
          {
            this.Output("Connected to WebSocket");
            this.InvokeOnReady();
            string port = (string) null;
            this.SetPendingMessageHandler((Action<MessageEventArgs>) (e =>
            {
              try
              {
                Dictionary<string, object> dictionary = JsonConvert.DeserializeObject<Dictionary<string, object>>(e.Data);
                object obj;
                if (dictionary == null || !dictionary.TryGetValue("Results", out obj))
                  return;
                if (!(obj is IEnumerable<JToken> jtokens2))
                  jtokens2 = Enumerable.Empty<JToken>();
                foreach (IEnumerable<JToken> jtokens3 in jtokens2)
                {
                  port = jtokens3.Value<string>();
                  if (!string.IsNullOrWhiteSpace(port))
                    break;
                }
              }
              catch
              {
              }
              finally
              {
                this.mMemoryReadEvent.Set();
              }
            }));
            this.mSocket.Send(JsonConvert.SerializeObject((object) new ConnectorLib.usb2snes.RequestType()
            {
              Opcode = OpcodeType.DeviceList.ToString(),
              Space = "SNES"
            }));
            this.mMemoryReadEvent.WaitOne();
            this.mMemoryReadEvent.Reset();
            if (!string.IsNullOrWhiteSpace(port))
            {
              this.Output("Connecting to SD2SNES via {0}", (object) port);
              WebSocket mSocket1 = this.mSocket;
              ConnectorLib.usb2snes.RequestType requestType1 = new ConnectorLib.usb2snes.RequestType();
              ConnectorLib.usb2snes.RequestType requestType2 = requestType1;
              OpcodeType opcodeType = OpcodeType.Attach;
              string str1 = opcodeType.ToString();
              requestType2.Opcode = str1;
              requestType1.Space = "SNES";
              requestType1.Operands = new List<string>((IEnumerable<string>) new string[1]
              {
                port
              });
              string data1 = JsonConvert.SerializeObject((object) requestType1);
              mSocket1.Send(data1);
              WebSocket mSocket2 = this.mSocket;
              ConnectorLib.usb2snes.RequestType requestType3 = new ConnectorLib.usb2snes.RequestType();
              ConnectorLib.usb2snes.RequestType requestType4 = requestType3;
              opcodeType = OpcodeType.Name;
              string str2 = opcodeType.ToString();
              requestType4.Opcode = str2;
              requestType3.Space = "SNES";
              requestType3.Operands = new List<string>((IEnumerable<string>) new string[1]
              {
                sd2snesConnector.Usb2SnesApplicationName
              });
              string data2 = JsonConvert.SerializeObject((object) requestType3);
              mSocket2.Send(data2);
              EventHandler<(ConnectionStatus, string)> connectionStatusChanged = this.ConnectionStatusChanged;
              if (connectionStatusChanged != null)
                connectionStatusChanged((object) this, (ConnectionStatus.Open, "Connection to sd2snes socket service established."));
              lock (this._cancel_lock)
              {
                CancellationTokenSource keepaliveCancel = this._keepalive_cancel;
                if ((keepaliveCancel != null ? (keepaliveCancel.IsCancellationRequested ? 1 : 0) : 1) != 0)
                {
                  this._keepalive_cancel = new CancellationTokenSource();
                  CancellationToken tok = this._keepalive_cancel.Token;
                  Task.Factory.StartNew((Action) (() =>
                  {
                    while (!tok.IsCancellationRequested)
                    {
                      try
                      {
                        this.KeepAlive();
                      }
                      catch
                      {
                      }
                      Thread.Sleep(1500);
                    }
                    Action<string> messageProcessor = this.mMessageProcessor;
                    if (messageProcessor == null)
                      return;
                    messageProcessor("sd2snesConnector keepalive is quitting.");
                  }), tok, TaskCreationOptions.LongRunning, TaskScheduler.Default);
                }
              }
              return new bool?(true);
            }
            this.Output("ERROR: No sd2snes was detected by the system.");
            EventHandler<(ConnectionStatus, string)> connectionStatusChanged1 = this.ConnectionStatusChanged;
            if (connectionStatusChanged1 != null)
              connectionStatusChanged1((object) this, (ConnectionStatus.Closed, "No sd2snes was detected by the system."));
            return new bool?(false);
          }
          this.Output("ERROR: Failed to connect to web socket; the usb2snes websocket application may not be running.");
          EventHandler<(ConnectionStatus, string)> connectionStatusChanged2 = this.ConnectionStatusChanged;
          if (connectionStatusChanged2 != null)
            connectionStatusChanged2((object) this, (ConnectionStatus.Closed, "Failed to connect to websocket; the usb2snes websocket application may not be running."));
          return new bool?(false);
        }
      }
      return new bool?();
    }

    private void SetPendingMessageHandler(Action<MessageEventArgs> handler) => this.mPendingMessageHandler = handler;

    private void Socket_OnMessageReceived(object sender, MessageEventArgs e)
    {
      Action<MessageEventArgs> pendingMessageHandler = this.mPendingMessageHandler;
      this.mPendingMessageHandler = (Action<MessageEventArgs>) null;
      if (pendingMessageHandler == null)
        return;
      pendingMessageHandler(e);
    }

    public static bool MapAddressInRange(
      uint address,
      uint srcRangeBegin,
      uint srcRangeEnd,
      uint dstRangeBegin,
      out uint mappedAddress)
    {
      if (address >= srcRangeBegin && address <= srcRangeEnd)
      {
        mappedAddress = address - srcRangeBegin + dstRangeBegin;
        return true;
      }
      mappedAddress = 0U;
      return false;
    }

    public uint TranslateAddress(uint address, sd2snesConnector.TranslationMode mode)
    {
      if (this.Mapper != null && mode == sd2snesConnector.TranslationMode.Read)
      {
        (ulong address, string domain) tuple = this.Mapper.Translate((ulong) address);
        ulong address1 = tuple.address;
        switch (tuple.domain)
        {
          case "WRAM":
            return (uint) (address1 + 16056320UL);
          case "CARTROM":
            return (uint) address1;
          case "CARTRAM":
            return (uint) (address1 + 14680064UL);
        }
      }
      uint mappedAddress;
      if (mode == sd2snesConnector.TranslationMode.Read && sd2snesConnector.MapAddressInRange(address, 8257536U, 8388607U, 16056320U, out mappedAddress))
        return mappedAddress;
      for (uint index = 0; index < 63U; ++index)
      {
        if (sd2snesConnector.MapAddressInRange(address, (uint) ((int) index * 65536 + 32768), (uint) ((int) index * 65536 + (int) ushort.MaxValue), index * 32768U, out mappedAddress) || sd2snesConnector.MapAddressInRange(address, (uint) ((int) index * 65536 + 8421376), (uint) ((int) index * 65536 + 8454143), index * 32768U, out mappedAddress))
          return mappedAddress;
      }
      for (uint index = 0; index < 8U; ++index)
      {
        if (sd2snesConnector.MapAddressInRange(address, (uint) (7340032 + (int) index * 65536), (uint) (7372799 + (int) index * 65536), (uint) (14680064 + (int) index * 32768), out mappedAddress))
          return mappedAddress;
      }
      return address;
    }

    public bool ReadBytes(ulong address, byte[] buffer) => this.Read(address, buffer);

    public bool Read(ulong address, byte[] buffer)
    {
      bool bSuccess = false;
      while (!bSuccess)
      {
        this.ConnectIfNecessary();
        if (!this.Connected)
          return false;
        try
        {
          using (ManualResetEvent readEvent = new ManualResetEvent(false))
          {
            lock (this.mTransmitLock)
            {
              ConnectorLib.usb2snes.RequestType requestType = new ConnectorLib.usb2snes.RequestType()
              {
                Opcode = OpcodeType.GetAddress.ToString(),
                Space = "SNES",
                Operands = new List<string>((IEnumerable<string>) new string[2]
                {
                  this.TranslateAddress((uint) address, sd2snesConnector.TranslationMode.Read).ToString("X"),
                  buffer.Length.ToString("X")
                })
              };
              this.SetPendingMessageHandler((Action<MessageEventArgs>) (e =>
              {
                try
                {
                  if (e.Type != Opcode.Binary || e.RawData == null)
                    return;
                  for (int val1 = 0; val1 < buffer.Length; ++val1)
                    buffer[val1] = e.RawData[Math.Min(val1, e.RawData.Length - 1)];
                  bSuccess = true;
                }
                catch
                {
                }
                finally
                {
                  readEvent.Set();
                }
              }));
              try
              {
                lock (this._socket_lock)
                  this.mSocket?.Send(JsonConvert.SerializeObject((object) requestType));
                if (!readEvent.WaitOne(4096))
                {
                  this.Disconnect(false);
                  return false;
                }
              }
              catch
              {
                this.Disconnect(false);
                return false;
              }
              finally
              {
                readEvent.Reset();
              }
            }
          }
        }
        catch
        {
          return false;
        }
      }
      return bSuccess;
    }

    internal void TransmitBatchWrites()
    {
      lock (this.mTransmitLock)
      {
        try
        {
          sd2snesBatchWriteContext batchWriteContext = this.mActiveBatchWriteContext.Value;
          if (batchWriteContext == null || batchWriteContext.BatchWriteEntries.Count == 0)
            return;
          this.TransmitWriteDescriptors((IEnumerable<sd2snesConnector.WriteDescriptor>) batchWriteContext.BatchWriteEntries);
        }
        finally
        {
          this.mActiveBatchWriteContext.Value = (sd2snesBatchWriteContext) null;
        }
      }
    }

    public IBatchWriteContext OpenBatchWriteContext()
    {
      lock (this.mTransmitLock)
      {
        if (this.mActiveBatchWriteContext.Value != null)
          return (IBatchWriteContext) this.mActiveBatchWriteContext.Value;
        this.mActiveBatchWriteContext.Value = new sd2snesBatchWriteContext(this);
        return (IBatchWriteContext) this.mActiveBatchWriteContext.Value;
      }
    }

    private void BeginCommand(List<byte> cmd)
    {
      cmd.Clear();
      cmd.Add((byte) 8);
      cmd.Add((byte) 139);
      cmd.Add((byte) 226);
      cmd.Add((byte) 32);
      cmd.Add((byte) 72);
      cmd.Add((byte) 235);
      cmd.Add((byte) 72);
      cmd.Add((byte) 169);
      cmd.Add((byte) 0);
      cmd.Add((byte) 72);
      cmd.Add((byte) 171);
    }

    private static void EndCommand(List<byte> cmd)
    {
      cmd.Add((byte) 169);
      cmd.Add((byte) 0);
      cmd.Add((byte) 141);
      cmd.Add((byte) 0);
      cmd.Add((byte) 44);
      cmd.Add((byte) 104);
      cmd.Add((byte) 235);
      cmd.Add((byte) 104);
      cmd.Add((byte) 171);
      cmd.Add((byte) 40);
      cmd.Add((byte) 108);
      cmd.Add((byte) 234);
      cmd.Add(byte.MaxValue);
    }

    private void AddSystemWriteToCommand(List<byte> cmd, uint address, byte[] data)
    {
      for (uint index = 0; (long) index < (long) data.Length; ++index)
      {
        uint num = address + index;
        cmd.Add((byte) 169);
        cmd.Add(data[(int) index]);
        cmd.Add((byte) 143);
        cmd.Add(Convert.ToByte(num & (uint) byte.MaxValue));
        cmd.Add(Convert.ToByte(num >> 8 & (uint) byte.MaxValue));
        cmd.Add(Convert.ToByte(num >> 16 & (uint) byte.MaxValue));
      }
    }

    private void CommitCommand(
      List<byte> cmd,
      List<sd2snesConnector.WriteDescriptor> shadowWrites = null)
    {
      sd2snesConnector.EndCommand(cmd);
      ConnectorLib.usb2snes.RequestType requestType1 = new ConnectorLib.usb2snes.RequestType()
      {
        Opcode = OpcodeType.PutAddress.ToString(),
        Space = "CMD",
        Operands = new List<string>((IEnumerable<string>) new string[4]
        {
          11264.ToString("X"),
          cmd.Count.ToString("X"),
          11264.ToString("X"),
          1.ToString("X")
        })
      };
      cmd.Add(cmd[0]);
      cmd[0] = (byte) 0;
      lock (this._socket_lock)
      {
        this.mSocket?.Send(JsonConvert.SerializeObject((object) requestType1));
        this.mSocket?.Send(cmd.ToArray());
      }
      foreach (sd2snesConnector.WriteDescriptor writeDescriptor in (IEnumerable<sd2snesConnector.WriteDescriptor>) shadowWrites ?? Enumerable.Empty<sd2snesConnector.WriteDescriptor>())
      {
        uint num = this.TranslateAddress(writeDescriptor.address, sd2snesConnector.TranslationMode.Read);
        if ((int) num != (int) writeDescriptor.address)
        {
          ConnectorLib.usb2snes.RequestType requestType2 = new ConnectorLib.usb2snes.RequestType()
          {
            Opcode = OpcodeType.PutAddress.ToString(),
            Space = "SNES",
            Operands = new List<string>((IEnumerable<string>) new string[2]
            {
              num.ToString("X"),
              writeDescriptor.data.Length.ToString("X")
            })
          };
          lock (this._socket_lock)
          {
            this.mSocket?.Send(JsonConvert.SerializeObject((object) requestType2));
            this.mSocket?.Send(writeDescriptor.data);
          }
        }
      }
      cmd.Clear();
      shadowWrites?.Clear();
    }

    private void TransmitWriteDescriptors(
      IEnumerable<sd2snesConnector.WriteDescriptor> descriptors)
    {
      try
      {
        this.ConnectIfNecessary();
        do
          ;
        while (!this.Connected);
        List<byte> cmd = new List<byte>();
        List<sd2snesConnector.WriteDescriptor> shadowWrites = new List<sd2snesConnector.WriteDescriptor>();
        int num = 0;
        foreach (sd2snesConnector.WriteDescriptor descriptor in descriptors)
        {
          bool flag1 = (int) descriptor.address != (int) descriptor.translatedAddress;
          bool flag2 = cmd.Count > 0;
          if (!flag1)
          {
            if (descriptor.data.Length < 16)
            {
              if (!flag2)
                this.BeginCommand(cmd);
              this.AddSystemWriteToCommand(cmd, descriptor.address, descriptor.data);
              shadowWrites.Add(descriptor);
              num += descriptor.data.Length;
              if (num > 16)
              {
                this.CommitCommand(cmd, shadowWrites);
                num = 0;
              }
            }
            else
            {
              if (flag2)
                this.CommitCommand(cmd, shadowWrites);
              this.DMAToWRAM(descriptor.address, descriptor.data);
            }
          }
          else
          {
            if (flag2)
              this.CommitCommand(cmd, shadowWrites);
            ConnectorLib.usb2snes.RequestType requestType = new ConnectorLib.usb2snes.RequestType()
            {
              Opcode = OpcodeType.PutAddress.ToString(),
              Space = "SNES",
              Operands = new List<string>((IEnumerable<string>) new string[2]
              {
                descriptor.translatedAddress.ToString("X"),
                descriptor.data.Length.ToString("X")
              })
            };
            lock (this._socket_lock)
            {
              this.mSocket?.Send(JsonConvert.SerializeObject((object) requestType));
              this.mSocket?.Send(descriptor.data);
            }
          }
        }
        if (cmd.Count <= 0)
          return;
        this.CommitCommand(cmd, shadowWrites);
      }
      catch
      {
      }
      finally
      {
        this.mActiveBatchWriteContext.Value = (sd2snesBatchWriteContext) null;
      }
    }

    private bool DMAToWRAM(uint address, byte[] data)
    {
      if (data == null)
        return false;
      if (data.Length > 65536)
        throw new ArgumentException("The maximum DMA request size is 65536 bytes.", nameof (data));
      try
      {
        this.ConnectIfNecessary();
        if (!this.Connected)
          return false;
        ConnectorLib.usb2snes.RequestType requestType = new ConnectorLib.usb2snes.RequestType()
        {
          Opcode = OpcodeType.PutAddress.ToString(),
          Space = "SNES",
          Operands = new List<string>((IEnumerable<string>) new string[2]
          {
            16384000U.ToString("X"),
            data.Length.ToString("X")
          })
        };
        lock (this._socket_lock)
        {
          if (this.mSocket == null)
            return false;
          this.mSocket.Send(JsonConvert.SerializeObject((object) requestType));
          this.mSocket.Send(data);
        }
        List<byte> cmd = new List<byte>();
        this.BeginCommand(cmd);
        for (int index = 0; index <= 6; ++index)
        {
          cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.LDA_as((ushort) (17152 + index)));
          cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.STA_as((ushort) (11760 + index)));
        }
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.STZ_as((ushort) 17152));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.LDA_c((byte) 128));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.STA_as((ushort) 17153));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.LDA_c((byte) (address & (uint) byte.MaxValue)));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.STA_as((ushort) 8577));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.LDA_c((byte) (address >> 8 & (uint) byte.MaxValue)));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.STA_as((ushort) 8578));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.LDA_c((byte) (address >> 16)));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.STA_as((ushort) 8579));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.STZ_as((ushort) 17154));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.STZ_as((ushort) 17155));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.LDA_c((byte) 250));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.STA_as((ushort) 17156));
        ushort num = data.Length >= 65536 ? (ushort) 0 : (ushort) data.Length;
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.LDA_c((byte) ((uint) num & (uint) byte.MaxValue)));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.STA_as((ushort) 17157));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.LDA_c((byte) ((uint) num >> 8)));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.STA_as((ushort) 17158));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.STZ_as((ushort) 17159));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.LDA_c((byte) 1));
        cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.STA_as((ushort) 16907));
        for (int index = 0; index <= 6; ++index)
        {
          cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.LDA_as((ushort) (11760 + index)));
          cmd.AddRange((IEnumerable<byte>) SNESAssembler.Instructions.STA_as((ushort) (17152 + index)));
        }
        this.CommitCommand(cmd, new List<sd2snesConnector.WriteDescriptor>((IEnumerable<sd2snesConnector.WriteDescriptor>) new sd2snesConnector.WriteDescriptor[1]
        {
          new sd2snesConnector.WriteDescriptor(address, this.TranslateAddress(address, sd2snesConnector.TranslationMode.Read), data)
        }));
        return true;
      }
      catch
      {
        return false;
      }
      finally
      {
        this.mActiveBatchWriteContext.Value = (sd2snesBatchWriteContext) null;
      }
    }

    public bool WriteBytes(ulong address, byte[] data) => this.Write(address, data);

    public bool Write(ulong address, byte[] data)
    {
      lock (this.mTransmitLock)
      {
        if (this.mActiveBatchWriteContext.Value != null && data != null && data.Length != 0)
        {
          this.mActiveBatchWriteContext.Value.BatchWriteEntries.Add(new sd2snesConnector.WriteDescriptor((uint) address, this.TranslateAddress((uint) address, sd2snesConnector.TranslationMode.Write), data));
          return true;
        }
        if (data.Length < 16)
          this.TransmitWriteDescriptors((IEnumerable<sd2snesConnector.WriteDescriptor>) new sd2snesConnector.WriteDescriptor[1]
          {
            new sd2snesConnector.WriteDescriptor((uint) address, this.TranslateAddress((uint) address, sd2snesConnector.TranslationMode.Write), data)
          });
        else
          this.DMAToWRAM((uint) address, data);
      }
      return true;
    }

    public bool Freeze(
      ulong address,
      ulong value,
      byte size,
      WriteType writeType,
      ulong cmpAddress,
      ulong cmpValue,
      byte cmpSize,
      CompareType condition)
    {
      throw new NotImplementedException();
    }

    public bool Freeze(
      ulong address,
      ulong value,
      ulong mask,
      byte size,
      WriteType writeType,
      ulong cmpAddress,
      ulong cmpValue,
      byte cmpSize,
      CompareType condition)
    {
      throw new NotImplementedException();
    }

    public string GetROMPath() => throw new NotImplementedException();

    public bool OpenROM(string path) => throw new NotImplementedException();

    public bool CloseROM() => throw new NotImplementedException();

    public bool PatchROM(IROMPatch patch) => this.PatchROM(patch.Data);

    public bool PatchROM(byte[] data)
    {
      lock (this._socket_lock)
      {
        if (this.mSocket == null)
          return false;
        this.mSocket.Send(JsonConvert.SerializeObject((object) new ConnectorLib.usb2snes.RequestType()
        {
          Opcode = OpcodeType.PutIPS.ToString(),
          Space = "SNES",
          Operands = new List<string>((IEnumerable<string>) new string[2]
          {
            "rom",
            data.Length.ToString("X")
          })
        }));
        this.mSocket.Send(data);
      }
      this.ResetConsole();
      return true;
    }

    public bool ResetConsole()
    {
      lock (this._socket_lock)
      {
        if (this.mSocket == null)
          return false;
        this.mSocket.Send(JsonConvert.SerializeObject((object) new ConnectorLib.usb2snes.RequestType()
        {
          Opcode = OpcodeType.Reset.ToString()
        }));
      }
      return true;
    }

    public bool Freeze8(ulong address, byte value) => this.WriteByte(address, value);

    public bool Freeze16(ulong address, ushort value) => this.WriteWord(address, value);

    public bool Unfreeze(ulong address) => true;

    public IBatchContext8 GetBatchContext8() => (IBatchContext8) null;

    public IBatchContext16 GetBatchContext16() => (IBatchContext16) null;

    public byte? ReadByte(ulong address) => this.ReadByte((uint) address);

    public byte? ReadByte(uint address)
    {
      this.Output(string.Format("sd2snesConnector is reading a byte: [${0:X6}]", (object) address));
      byte[] buffer = new byte[1];
      if (this.Read((ulong) address, buffer))
        return new byte?(buffer[0]);
      this.Output(string.Format("sd2snesConnector failed to read a byte: [${0:X6}]", (object) address));
      return new byte?();
    }

    bool I8BitConnector.Read8(ulong address, out byte value) => this.ReadByte(address, out value);

    public bool ReadByte(ulong address, out byte value)
    {
      this.Output(string.Format("sd2snesConnector is reading a byte: [${0:X6}]", (object) address));
      byte[] buffer = new byte[1];
      if (this.Read(address, buffer))
      {
        value = buffer[0];
        return true;
      }
      this.Output(string.Format("sd2snesConnector failed to read a byte: [${0:X6}]", (object) address));
      value = (byte) 0;
      return false;
    }

    public ushort? ReadWord(ulong address)
    {
      this.Output(string.Format("sd2snesConnector is reading a word: [${0:X6}]", (object) address));
      byte[] buffer = new byte[2];
      if (this.Read(address, buffer))
        return new ushort?((ushort) ((uint) buffer[1] << 8 | (uint) buffer[0]));
      this.Output(string.Format("sd2snesConnector failed to read a word: [${0:X6}]", (object) address));
      return new ushort?();
    }

    bool I16BitConnector.Read16(ulong address, out ushort value) => this.ReadWord(address, out value);

    public bool ReadWord(ulong address, out ushort value)
    {
      this.Output(string.Format("sd2snesConnector is reading a word: [${0:X6}]", (object) address));
      byte[] buffer = new byte[2];
      if (this.Read(address, buffer))
      {
        value = (ushort) ((uint) buffer[1] << 8 | (uint) buffer[0]);
        return true;
      }
      this.Output(string.Format("sd2snesConnector failed to read a word: [${0:X6}]", (object) address));
      value = (ushort) 0;
      return false;
    }

    public bool SendMessage(string message)
    {
      EventHandler<string> onMessage = this.OnMessage;
      if (onMessage != null)
        onMessage((object) this, message);
      string str = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Personal), "CrowdControl");
      string outputPath = Path.Combine(str, "sd2snes_message_log.txt");
      if (!string.IsNullOrWhiteSpace(message))
      {
        ulong messageIdx = ++this.mWrittenMessageIndex;
        try
        {
          Directory.CreateDirectory(str);
          using (TextWriter textWriter = (TextWriter) new StreamWriter(outputPath, messageIdx != 1UL))
          {
            if (messageIdx != 1UL)
              textWriter.Write('\n');
            textWriter.Write(message);
            return true;
          }
        }
        catch
        {
        }
        finally
        {
          if (this.mMessageClearTimer != null)
          {
            this.mMessageClearTimer.Dispose();
            this.mMessageClearTimer = (System.Timers.Timer) null;
          }
          this.mMessageClearTimer = new System.Timers.Timer()
          {
            Interval = 5000.0,
            AutoReset = false
          };
          this.mMessageClearTimer.Elapsed += (ElapsedEventHandler) ((sender, args) =>
          {
            if ((long) this.mWrittenMessageIndex != (long) messageIdx)
              return;
            try
            {
              using (TextWriter textWriter = (TextWriter) new StreamWriter(outputPath, true))
              {
                textWriter.WriteLine("");
                textWriter.WriteLine("");
              }
            }
            catch
            {
            }
          });
          this.mMessageClearTimer.Start();
        }
      }
      return false;
    }

    public event EventHandler<string> OnMessage;

    public bool SetBits(ulong address, byte value, out byte oldVal)
    {
      this.Output(string.Format("sd2snesConnector is setting a bit: [${0:X6} = #${1:X2}]", (object) address, (object) value));
      bool flag = false;
      byte[] numArray = new byte[1];
      if (this.Read(address, numArray))
      {
        numArray[0] = (byte) (((int) numArray[0] | (int) value) & (int) byte.MaxValue);
        flag = this.Write(address, numArray);
      }
      oldVal = (byte) 0;
      return flag;
    }

    public bool UnsetBits(ulong address, byte value, out byte oldVal)
    {
      this.Output(string.Format("sd2snesConnector is clearing a bit: [${0:X6} = #${1:X2}]", (object) address, (object) value));
      bool flag = false;
      byte[] numArray = new byte[1];
      if (this.Read(address, numArray))
      {
        numArray[0] = (byte) ((int) numArray[0] & (int) (byte) ((uint) ~value & (uint) byte.MaxValue) & (int) byte.MaxValue);
        flag = this.Write(address, numArray);
      }
      oldVal = (byte) 0;
      return flag;
    }

    public bool CompareSwap(ulong address, byte test, byte value, out byte oldValue)
    {
      oldValue = (byte) 0;
      return false;
    }

    public bool FreezeU8(ulong address, byte value, byte cmpValue, CompareType cmpType) => throw new NotImplementedException();

    public bool FreezeU8TU8(ulong address, byte value, byte cmpValue, CompareType cmpType) => throw new NotImplementedException();

    public bool FreezeU8TU8(
      ulong address,
      byte value,
      ulong cmpAddress,
      byte cmpValue,
      CompareType cmpType)
    {
      throw new NotImplementedException();
    }

    public bool FreezeU16(ulong address, ushort value, ushort cmpValue, CompareType cmpType) => throw new NotImplementedException();

    public bool FreezeU8TU16(ulong address, byte value, ushort cmpValue, CompareType cmpType) => throw new NotImplementedException();

    public bool FreezeU16TU8(ulong address, ushort value, byte cmpValue, CompareType cmpType) => throw new NotImplementedException();

    public bool FreezeU16TU16(ulong address, ushort value, ushort cmpValue, CompareType cmpType) => throw new NotImplementedException();

    public bool FreezeU8TU16(
      ulong address,
      byte value,
      ulong cmpAddress,
      ushort cmpValue,
      CompareType cmpType)
    {
      throw new NotImplementedException();
    }

    public bool FreezeU16TU8(
      ulong address,
      ushort value,
      ulong cmpAddress,
      byte cmpValue,
      CompareType cmpType)
    {
      throw new NotImplementedException();
    }

    public bool FreezeU16TU16(
      ulong address,
      ushort value,
      ulong cmpAddress,
      ushort cmpValue,
      CompareType cmpType)
    {
      throw new NotImplementedException();
    }

    public IBatchContext GetBatchContext() => (IBatchContext) null;

    bool I8BitConnector.Write8(ulong address, byte value) => this.WriteByte((ulong) (uint) address, value);

    public bool WriteByte(ulong address, byte value)
    {
      this.Output(string.Format("sd2snesConnector is writing a byte: [${0:X6} = #${1:X2}]", (object) address, (object) value));
      lock (this.writelock)
      {
        Thread.Sleep(25);
        byte[] data = new byte[1]{ value };
        if (this.Write(address, data))
          return true;
        this.Output(string.Format("sd2snesConnector failed to write a byte: [${0:X6} = #${1:X2}]", (object) address, (object) value));
        return false;
      }
    }

    bool I16BitConnector.Write16(ulong address, ushort value) => this.WriteWord((ulong) (uint) address, value);

    public bool WriteWord(ulong address, ushort value)
    {
      this.Output(string.Format("sd2snesConnector is writing a word: [${0:X6} = #${1:X4}]", (object) address, (object) value));
      lock (this.writelock)
      {
        byte[] data = new byte[2]
        {
          (byte) ((uint) value % 256U),
          (byte) ((uint) value / 256U)
        };
        if (this.Write(address, data))
          return true;
        this.Output(string.Format("sd2snesConnector failed to write a word: [${0:X6} = #${1:X4}]", (object) address, (object) value));
        return false;
      }
    }

    public struct WriteDescriptor
    {
      public uint address;
      public uint translatedAddress;
      public byte[] data;

      public WriteDescriptor(uint _address, uint _translatedAddress, byte[] _data)
      {
        this.address = _address;
        this.translatedAddress = _translatedAddress;
        this.data = new byte[_data.Length];
        _data.CopyTo((Array) this.data, 0);
      }
    }

    public enum TranslationMode
    {
      Read,
      Write,
    }
  }
}
