﻿// Decompiled with JetBrains decompiler
// Type: ConnectorLib.usb2snes.Core
// Assembly: ConnectorLib.sd2snes, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: 743ADA37-D030-4892-BC8D-59E965B0A960
// Assembly location: C:\Program Files (x86)\EmoTracker\ConnectorLib.sd2snes.dll

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO.Ports;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading;

namespace ConnectorLib.usb2snes
{
  public class Core : IDisposable, INotifyPropertyChanged
  {
    private readonly object _port_lock = new object();
    private SafeSerialPort _serial_port;

    public void Dispose()
    {
      try
      {
        this.DestroyPort();
      }
      catch
      {
      }
    }

    public bool Connected
    {
      get
      {
        SafeSerialPort serialPort = this._serial_port;
        return serialPort != null && serialPort.IsOpen;
      }
    }

    public event EventHandler<bool> ConnectionStateChanged;

    public string PortName() => this._serial_port?.PortName;

    public static List<Core.Port> GetDeviceList(bool disableFilter = false)
    {
      List<Core.Port> deviceList = new List<Core.Port>();
      foreach (usb2snes.utils.Win32DeviceMgmt.DeviceInfo allComPort in usb2snes.utils.Win32DeviceMgmt.GetAllCOMPorts())
      {
        if (!disableFilter)
        {
          string busDescription = allComPort.bus_description;
          if ((busDescription != null ? (busDescription.ToLowerInvariant().Contains("sd2snes") ? 1 : 0) : 0) == 0)
          {
            string instancePath = allComPort.instance_path;
            if ((instancePath != null ? (instancePath.ToLowerInvariant().Contains("PID_5a22") ? 1 : 0) : 0) == 0)
              continue;
          }
        }
        deviceList.Add(new Core.Port(allComPort.name?.Trim() ?? string.Empty, allComPort.name?.Trim() ?? string.Empty + " | " + allComPort.description?.Trim(), allComPort.instance_path?.Trim() ?? string.Empty));
      }
      deviceList.Sort();
      return deviceList;
    }

    public void Connect(string portName)
    {
      this.RebuildPort(portName);
      this._serial_port.Open();
      this.OnPropertyChanged("Connected");
      EventHandler<bool> connectionStateChanged = this.ConnectionStateChanged;
      if (connectionStateChanged == null)
        return;
      connectionStateChanged((object) this, this.Connected);
    }

    public void Reset()
    {
      this._serial_port.DtrEnable = false;
      Thread.Sleep(500);
    }

    private void DestroyPort()
    {
      lock (this._port_lock)
      {
        SafeSerialPort serialPort = this._serial_port;
        if ((serialPort != null ? (serialPort.IsOpen ? 1 : 0) : 0) != 0)
          this.Disconnect();
        try
        {
          this._serial_port?.Dispose();
        }
        catch
        {
        }
        this._serial_port = (SafeSerialPort) null;
        GC.Collect();
        GC.WaitForPendingFinalizers();
        this.OnPropertyChanged("Connected");
        EventHandler<bool> connectionStateChanged = this.ConnectionStateChanged;
        if (connectionStateChanged == null)
          return;
        connectionStateChanged((object) this, this.Connected);
      }
    }

    private void RebuildPort(string newPort = null)
    {
      lock (this._port_lock)
      {
        string str1 = newPort;
        if (str1 == null)
          str1 = this._serial_port?.PortName ?? throw new InvalidOperationException("No port specified.");
        string str2 = str1;
        this.DestroyPort();
        SafeSerialPort safeSerialPort = new SafeSerialPort();
        safeSerialPort.PortName = str2;
        safeSerialPort.BaudRate = 9600;
        safeSerialPort.Parity = Parity.None;
        safeSerialPort.DataBits = 8;
        safeSerialPort.StopBits = StopBits.One;
        safeSerialPort.Handshake = Handshake.None;
        safeSerialPort.ReadTimeout = 5000;
        safeSerialPort.WriteTimeout = 5000;
        safeSerialPort.DtrEnable = true;
        this._serial_port = safeSerialPort;
      }
    }

    public void Disconnect()
    {
      try
      {
        this._serial_port.DtrEnable = false;
      }
      catch
      {
      }
      try
      {
        this._serial_port.Close();
      }
      catch
      {
      }
      this.DestroyPort();
    }

    public object SendCommand(
      usbint_server_opcode_e opcode,
      usbint_server_space_e space,
      usbint_server_flags_e flags,
      params object[] args)
    {
      byte[] numArray1 = new byte[512];
      object obj = (object) null;
      Array.Clear((Array) numArray1, 0, numArray1.Length);
      numArray1[0] = Convert.ToByte('U');
      numArray1[1] = Convert.ToByte('S');
      numArray1[2] = Convert.ToByte('B');
      numArray1[3] = Convert.ToByte('A');
      numArray1[4] = Convert.ToByte((object) opcode);
      numArray1[5] = Convert.ToByte((object) space);
      numArray1[6] = Convert.ToByte((object) flags);
      if (space == usbint_server_space_e.FILE)
      {
        switch (opcode)
        {
          case usbint_server_opcode_e.GET:
          case usbint_server_opcode_e.PUT:
          case usbint_server_opcode_e.LS:
          case usbint_server_opcode_e.MKDIR:
          case usbint_server_opcode_e.RM:
          case usbint_server_opcode_e.MV:
          case usbint_server_opcode_e.BOOT:
            string source1 = args[0] != null && args[0] is string ? (string) args[0] : throw new Exception("Command: " + opcode.ToString() + " missing arg[0] string");
            Buffer.BlockCopy((Array) Encoding.ASCII.GetBytes(source1.ToArray<char>()), 0, (Array) numArray1, 256, Math.Min((int) byte.MaxValue, source1.Length));
            switch (opcode)
            {
              case usbint_server_opcode_e.PUT:
                uint num1 = args[1] != null && args[1] is uint ? (uint) args[1] : throw new Exception("Command: " + opcode.ToString() + " missing arg[1] uint");
                numArray1[252] = Convert.ToByte(num1 >> 24 & (uint) byte.MaxValue);
                numArray1[253] = Convert.ToByte(num1 >> 16 & (uint) byte.MaxValue);
                numArray1[254] = Convert.ToByte(num1 >> 8 & (uint) byte.MaxValue);
                numArray1[(int) byte.MaxValue] = Convert.ToByte(num1 & (uint) byte.MaxValue);
                break;
              case usbint_server_opcode_e.MV:
                string source2 = args[1] != null && args[1] is string ? (string) args[1] : throw new Exception("Command: " + opcode.ToString() + " missing arg[1] string");
                Buffer.BlockCopy((Array) Encoding.ASCII.GetBytes(source2.ToArray<char>()), 0, (Array) numArray1, 8, source2.Length);
                break;
            }
            break;
          case usbint_server_opcode_e.RESET:
          case usbint_server_opcode_e.POWER_CYCLE:
          case usbint_server_opcode_e.INFO:
          case usbint_server_opcode_e.MENU_RESET:
          case usbint_server_opcode_e.STREAM:
            break;
          default:
            throw new Exception("Unhandled Command: " + opcode.ToString() + " space: " + space.ToString() + " flags: " + flags.ToString());
        }
      }
      else
      {
        switch (opcode)
        {
          case usbint_server_opcode_e.GET:
          case usbint_server_opcode_e.PUT:
            uint num2 = args[0] != null && args[0] is uint ? (uint) args[0] : throw new Exception("Command: " + opcode.ToString() + " missing arg[0] uint");
            numArray1[256] = Convert.ToByte(num2 >> 24 & (uint) byte.MaxValue);
            numArray1[257] = Convert.ToByte(num2 >> 16 & (uint) byte.MaxValue);
            numArray1[258] = Convert.ToByte(num2 >> 8 & (uint) byte.MaxValue);
            numArray1[259] = Convert.ToByte(num2 & (uint) byte.MaxValue);
            uint num3 = args[1] != null && args[1] is uint ? (uint) args[1] : throw new Exception("Command: " + opcode.ToString() + " missing arg[1] uint");
            numArray1[252] = Convert.ToByte(num3 >> 24 & (uint) byte.MaxValue);
            numArray1[253] = Convert.ToByte(num3 >> 16 & (uint) byte.MaxValue);
            numArray1[254] = Convert.ToByte(num3 >> 8 & (uint) byte.MaxValue);
            numArray1[(int) byte.MaxValue] = Convert.ToByte(num3 & (uint) byte.MaxValue);
            break;
          case usbint_server_opcode_e.VGET:
          case usbint_server_opcode_e.VPUT:
            if (args.Length == 0 || args.Length > 8)
              throw new Exception("Command: " + opcode.ToString() + " need 2 <= args <= 16 and a multiple of 2.  Format: (size0, offset0), ...");
            uint num4 = 0;
            foreach (Tuple<int, int> tuple in args)
            {
              byte num5 = Convert.ToByte(tuple.Item2);
              uint uint32 = Convert.ToUInt32(tuple.Item1);
              numArray1[32 + (int) num4 * 4] = num5;
              numArray1[33 + (int) num4 * 4] = Convert.ToByte(uint32 >> 16 & (uint) byte.MaxValue);
              numArray1[34 + (int) num4 * 4] = Convert.ToByte(uint32 >> 8 & (uint) byte.MaxValue);
              numArray1[35 + (int) num4 * 4] = Convert.ToByte(uint32 & (uint) byte.MaxValue);
              ++num4;
            }
            break;
          case usbint_server_opcode_e.RESET:
          case usbint_server_opcode_e.POWER_CYCLE:
          case usbint_server_opcode_e.INFO:
          case usbint_server_opcode_e.MENU_RESET:
          case usbint_server_opcode_e.STREAM:
            break;
          default:
            throw new Exception("Unhandled Request: " + opcode.ToString() + " space: " + space.ToString() + " flags: " + flags.ToString());
        }
      }
      int count = opcode == usbint_server_opcode_e.VGET || opcode == usbint_server_opcode_e.VPUT ? 64 : numArray1.Length;
      this._serial_port.Write(numArray1, 0, count);
      int num6 = 0;
      if ((flags & usbint_server_flags_e.NORESP) == usbint_server_flags_e.NONE)
      {
        Array.Clear((Array) numArray1, 0, numArray1.Length);
        while (num6 < 512)
          num6 += this._serial_port.Read(numArray1, num6 % 512, 512 - num6 % 512);
        if (numArray1[0] != (byte) 85 || numArray1[1] != (byte) 83 || numArray1[2] != (byte) 66 || numArray1[3] != (byte) 65 || (int) numArray1[4] != (int) Convert.ToByte((object) usbint_server_opcode_e.RESPONSE) || numArray1[5] == (byte) 1)
          throw new Exception("Response Error Request: " + opcode.ToString() + " space: " + space.ToString() + " flags: " + flags.ToString() + " Response: " + numArray1[4].ToString());
      }
      switch (opcode)
      {
        case usbint_server_opcode_e.GET:
        case usbint_server_opcode_e.VGET:
          obj = (object) ((((0 | (int) numArray1[252]) << 8 | (int) numArray1[253]) << 8 | (int) numArray1[254]) << 8 | (int) numArray1[(int) byte.MaxValue]);
          break;
        case usbint_server_opcode_e.LS:
          int num7 = 0;
          List<Tuple<int, string>> tupleList = new List<Tuple<int, string>>();
          obj = (object) tupleList;
          do
          {
            int num8 = this._serial_port.Read(numArray1, num6 % 512, 512 - num6 % 512);
            if (num8 != 0)
            {
              num6 += num8;
              if (num6 % 512 == 0)
              {
                int num9 = 0;
                while (num9 < 512)
                {
                  byte[] numArray2 = numArray1;
                  int index1 = num9;
                  int index2 = index1 + 1;
                  num7 = (int) numArray2[index1];
                  switch (num7)
                  {
                    case 0:
                    case 1:
                      string str = "";
                      while (numArray1[index2] != (byte) 0)
                        str += ((char) numArray1[index2++]).ToString();
                      num9 = index2 + 1;
                      tupleList.Add(Tuple.Create<int, string>(num7, str));
                      continue;
                    case 2:
                    case (int) byte.MaxValue:
                      goto label_59;
                    default:
                      throw new IndexOutOfRangeException();
                  }
                }
              }
            }
label_59:;
          }
          while (num7 != (int) byte.MaxValue);
          break;
        case usbint_server_opcode_e.INFO:
          List<string> stringList = new List<string>();
          stringList.Add(Encoding.UTF8.GetString(numArray1, 260, Array.IndexOf<byte>(numArray1, (byte) 0, 260) - 260));
          stringList.Add(((int) numArray1[256] << 24 | (int) numArray1[257] << 16 | (int) numArray1[258] << 8 | (int) numArray1[259]).ToString("X"));
          stringList.Add(Encoding.UTF8.GetString(numArray1, 16, Array.IndexOf<byte>(numArray1, (byte) 0, 16) - 16));
          byte num10 = numArray1[6];
          List<string> values = new List<string>();
          if (((int) num10 & 1) != 0)
            values.Add("FEAT_DSPX");
          if (((int) num10 & 2) != 0)
            values.Add("FEAT_ST0010");
          if (((int) num10 & 4) != 0)
            values.Add("FEAT_SRTC");
          if (((int) num10 & 8) != 0)
            values.Add("FEAT_MSU1");
          if (((int) num10 & 16) != 0)
            values.Add("FEAT_213F");
          if (((int) num10 & 32) != 0)
            values.Add("FEAT_CMD_UNLOCK");
          if (((int) num10 & 64) != 0)
            values.Add("FEAT_USB1");
          if (((int) num10 & 128) != 0)
            values.Add("FEAT_DMA1");
          stringList.Add(string.Join("|", (IEnumerable<string>) values));
          obj = (object) stringList;
          break;
      }
      return obj;
    }

    public void SendData(byte[] data, int size)
    {
      if (size > data.Length)
        throw new Exception("Bytes to send larger than array size");
      this._serial_port.Write(data, 0, size);
    }

    public int GetData(byte[] data, int offset, int length) => this._serial_port.Read(data, offset, length);

    public void ClearData()
    {
      this.Reset();
      byte[] data = new byte[64];
      if (this._serial_port.BytesToRead != 0)
      {
        int readTimeout = this._serial_port.ReadTimeout;
        try
        {
          this._serial_port.ReadTimeout = 50;
          this._serial_port.WriteTimeout = 50;
          while (true)
            this.GetData(data, 0, 64);
        }
        catch
        {
        }
        finally
        {
          this._serial_port.ReadTimeout = readTimeout;
          this._serial_port.WriteTimeout = readTimeout;
        }
      }
      this._serial_port?.DiscardInBuffer();
      this._serial_port?.DiscardOutBuffer();
    }

    public event PropertyChangedEventHandler PropertyChanged;

    protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
    {
      PropertyChangedEventHandler propertyChanged = this.PropertyChanged;
      if (propertyChanged == null)
        return;
      propertyChanged((object) this, new PropertyChangedEventArgs(propertyName));
    }

    public class Port : IComparable<Core.Port>
    {
      public string Name { get; }

      public string Desc { get; }

      public string PNPID { get; }

      public Port(string name, string desc, string pnpID)
      {
        this.Name = name;
        this.Desc = desc;
        this.PNPID = pnpID;
      }

      public override string ToString() => this.Desc;

      public int CompareTo(Core.Port p) => p != null ? string.Compare(this.Name, p.Name, StringComparison.InvariantCultureIgnoreCase) : 1;
    }
  }
}
