﻿// Decompiled with JetBrains decompiler
// Type: ConnectorLib.Extensions
// Assembly: ConnectorLib, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: CC4FDCDA-98EE-4CD3-AF60-DEE32CDD48A0
// Assembly location: C:\Program Files (x86)\EmoTracker\ConnectorLib.dll

using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Linq;
using System.Net.Sockets;
using System.Threading.Tasks;

namespace ConnectorLib
{
    internal static class Extensions
    {
        public static void Put<K, V>(this Dictionary<K, V> dictionary, K key, V value)
        {
            lock (dictionary)
            {
                if (dictionary.ContainsKey(key))
                {
                    try
                    {
                        if (dictionary[key] is IDisposable disposable)
                            disposable.Dispose();
                    }
                    catch
                    {
                    }
                    dictionary[key] = value;
                }
                else
                    dictionary.Add(key, value);
            }
        }

        public static bool ReadBlocking(this SerialPort port, byte[] responseBytes, int bytesExpected)
        {
            int offset = 0;
            int num;
            while (bytesExpected > 0 && (num = port.Read(responseBytes, offset, bytesExpected)) > 0)
            {
                checked { offset += num; }
                checked { bytesExpected -= num; }
            }
            return bytesExpected == 0;
        }

        public static bool IsConnected(this TcpClient client)
        {
            try
            {
                if (client?.Client == null || !client.Client.Connected)
                    return false;
                if (!client.Client.Poll(0, SelectMode.SelectRead))
                    return true;
                byte[] buffer = new byte[1];
                return client.Client.Receive(buffer, SocketFlags.Peek) != 0;
            }
            catch
            {
                return false;
            }
        }

        internal static async void Forget(this Task task)
        {
            try
            {
                await task.ConfigureAwait(false);
            }
            catch (Exception ex)
            {
                Log.Error((object)ex);
            }
        }

        internal static IEnumerable<byte> AppendChecksum(this IEnumerable<byte> array)
        {
            return array.Append(array.Aggregate<byte, byte>(0, (byte current, byte b) => checked((byte)((int)(current + b) % 256))));
        }

        internal static byte[] AppendChecksum(this byte[] array)
        {
            byte[] numArray = new byte[checked(array.Length + 1)];
            byte num1 = 0;
            int index = 0;
            while (index < array.Length)
            {
                byte num2 = array[index];
                numArray[index] = num2;
                num1 = checked((byte)unchecked(checked((int)num1 + (int)num2) % 256));
                checked { ++index; }
            }
            numArray[checked(numArray.Length - 1)] = num1;
            return numArray;
        }

        public static ushort SwapBytes(this ushort x) => checked((ushort)(((int)x >> 8 | (int)x << 8) & (int)ushort.MaxValue));

        public static uint SwapBytes(this uint x)
        {
            x = x >> 16 | x << 16;
            return (x & 4278255360U) >> 8 | (uint)(((int)x & 16711935) << 8);
        }

        public static ulong SwapBytes(this ulong x)
        {
            x = x >> 32 | x << 32;
            x = (x & 18446462603027742720UL) >> 16 | (ulong)(((long)x & 281470681808895L) << 16);
            return (x & 18374966859414961920UL) >> 8 | (ulong)(((long)x & 71777214294589695L) << 8);
        }

        public static byte[] GetBytes(this uint val) => new byte[4]
        {
      checked ((byte) (unchecked ((int) val) & (int) byte.MaxValue)),
      checked ((byte) (unchecked ((int) (val >> 8)) & (int) byte.MaxValue)),
      checked ((byte) (unchecked ((int) (val >> 16)) & (int) byte.MaxValue)),
      checked ((byte) (unchecked ((int) (val >> 24)) & (int) byte.MaxValue))
        };

        public static string ToHexString(this byte[] val) => BitConverter.ToString(val).Replace('-', ' ');
    }
}
