﻿// Decompiled with JetBrains decompiler
// Type: ConnectorLib.FTDIGeckoProvider
// Assembly: ConnectorLib, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: CC4FDCDA-98EE-4CD3-AF60-DEE32CDD48A0
// Assembly location: C:\Program Files (x86)\EmoTracker\ConnectorLib.dll

using FTD2XX_NET;
using System;
using System.Threading;
using System.Threading.Tasks;

namespace ConnectorLib
{
  public sealed class FTDIGeckoProvider : IGeckoProvider, IDisposable
  {
    private readonly FTDI _ftdi = new FTDI();
    private readonly Task _connect_loop;
    private readonly Task _read_loop;
    private readonly EventWaitHandle _wait_modem = new EventWaitHandle(false, EventResetMode.AutoReset);
    private readonly EventWaitHandle _wait_rx = new EventWaitHandle(false, EventResetMode.AutoReset);
    private volatile bool _quitting;

    ~FTDIGeckoProvider() => this.Dispose(false);

    public void Dispose()
    {
      this.Dispose(true);
      GC.SuppressFinalize((object) this);
    }

    private void Dispose(bool disposing)
    {
      this.Disconnect();
      this._quitting = true;
      this._wait_modem.Set();
      this._wait_rx.Set();
      if (!disposing)
        return;
      this._wait_modem.Dispose();
      this._wait_rx.Dispose();
    }

    public FTDIGeckoProvider()
    {
      int num1 = (int) this._ftdi.SetBaudRate(9600U);
      int num2 = (int) this._ftdi.SetEventNotification(1U, this._wait_rx);
      int num3 = (int) this._ftdi.SetEventNotification(2U, this._wait_modem);
      this._connect_loop = (Task) Task.Factory.StartNew<Task>(new Func<Task>(this.ConnectLoop), TaskCreationOptions.LongRunning);
    }

    public bool GeckoInit()
    {
      Log.Message((object) "FTDIGeckoProvider is initializing device GECKUSB0/*.");
      if (this._ftdi.ResetDevice() != FTDI.FT_STATUS.FT_OK)
      {
        this.Disconnect();
        return false;
      }
      Log.Message((object) "FTDIGeckoProvider is purging the RX buffer for device GECKUSB0/*.");
      if (this._ftdi.Purge(1U) != FTDI.FT_STATUS.FT_OK)
      {
        this.Disconnect();
        return false;
      }
      Log.Message((object) "FTDIGeckoProvider is purging the TX buffer for device GECKUSB0/*.");
      if (this._ftdi.Purge(2U) == FTDI.FT_STATUS.FT_OK)
        return true;
      this.Disconnect();
      return false;
    }

    private void Disconnect()
    {
      try
      {
        int num = (int) this._ftdi.Close();
      }
      catch
      {
      }
      EventHandler<(ConnectionStatus, string)> connectionStatusChanged = this.ConnectionStatusChanged;
      if (connectionStatusChanged != null)
        connectionStatusChanged((object) this, (ConnectionStatus.Closed, string.Empty));
      Log.Message((object) "FTDIGeckoProvider disconnected from device GECKUSB0/*.");
    }

    public bool Connected => this._ftdi.IsOpen;

    public event EventHandler<(ConnectionStatus, string)> ConnectionStatusChanged;

    private async Task ConnectLoop()
    {
      FTDIGeckoProvider sender = this;
      while (!sender._quitting)
      {
        try
        {
          Log.Debug((object) "FTDIGeckoProvider is attempting to open device GECKUSB0/*.");
          if (sender._ftdi.OpenBySerialNumber("GECKUSB0") != FTDI.FT_STATUS.FT_OK)
          {
            await Task.Delay(TimeSpan.FromSeconds(1.0));
            Log.Message((object) "FTDIGeckoProvider failed to open device GECKUSB0/*.");
          }
          else
          {
            EventHandler<(ConnectionStatus, string)> connectionStatusChanged1 = sender.ConnectionStatusChanged;
            if (connectionStatusChanged1 != null)
              connectionStatusChanged1((object) sender, (ConnectionStatus.Open, string.Empty));
            Log.Message((object) "FTDIGeckoProvider connected to device GECKUSB0/*.");
            sender.GeckoInit();
            while (!sender._quitting && sender._wait_modem.WaitOne())
            {
              if (!sender._ftdi.IsOpen)
              {
                EventHandler<(ConnectionStatus, string)> connectionStatusChanged2 = sender.ConnectionStatusChanged;
                if (connectionStatusChanged2 != null)
                  connectionStatusChanged2((object) sender, (ConnectionStatus.Closed, string.Empty));
                Log.Message((object) "FTDIGeckoProvider disconnected from device GECKUSB0/*.");
                break;
              }
              Log.Message((object) "FTDIGeckoProvider got a state change from device GECKUSB0/*.");
            }
            sender.Disconnect();
          }
        }
        catch (Exception ex)
        {
          Log.Debug((object) ex);
        }
      }
    }

    private uint WiiU32(byte[] bytes) => (uint) ((int) bytes[3] | (int) bytes[2] << 8 | (int) bytes[1] << 16 | (int) bytes[0] << 24);

    private void ReadLoop()
    {
      while (!this._quitting)
      {
        if (!this._wait_rx.WaitOne())
          break;
        try
        {
          Log.Message((object) "FTDIGeckoProvider got an RX signal from device GECKUSB0/*.");
        }
        catch (Exception ex)
        {
          Log.Debug((object) ex);
        }
      }
    }

    public bool TryWrite(byte[] data) => this.TryWrite(data, out uint _);

    public bool TryWrite(byte[] data, out uint bytesWritten) => this._ftdi.Write(data, data.Length, out bytesWritten) == FTDI.FT_STATUS.FT_OK;

    public bool TryRead(byte[] data, bool allowPartial = false) => this.TryRead(data, allowPartial, out uint _);

    public bool TryRead(byte[] data, out uint bytesRead) => this.TryRead(data, false, out bytesRead);

    public bool TryRead(byte[] data, bool allowPartial, out uint bytesRead) => this._ftdi.Read(data, checked ((uint) data.Length), out bytesRead) == FTDI.FT_STATUS.FT_OK && (long) bytesRead == (long) data.Length;

    public event EventHandler<GeckoData> OnDataReceived;
  }
}
