﻿// Decompiled with JetBrains decompiler
// Type: ConnectorLib.LuaConnector
// Assembly: ConnectorLib, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: CC4FDCDA-98EE-4CD3-AF60-DEE32CDD48A0
// Assembly location: C:\Program Files (x86)\EmoTracker\ConnectorLib.dll

using ConnectorLib.Features;
using ConnectorLib.Patches;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ConnectorLib
{
  public class LuaConnector : 
    ConnectorBase,
    IGameConnector,
    IDisposable,
    INESConnector,
    IAddressableConnector,
    IROMProcessor,
    I8BitConnector,
    ISNESConnector,
    I16BitConnector,
    IN64Connector,
    I64BitConnector,
    I32BitConnector,
    IGenesisConnector,
    IGBConnector,
    IGBAConnector,
    IPS1Connector,
    INESProConnector,
    IFreezableConnector
  {
    public const uint VERSION_MAJOR = 2;
    public const uint VERSION_MINOR = 2;
    public const uint VERSION_PATCH = 0;
    public const uint VERSION = 131584;
    private static int _next_id;
    private TcpListener _listener;
    private volatile TcpClient _client;
    private volatile NetworkStream _stream;
    private volatile bool _quitting;
    private readonly Action<string> _message_handler;
    private readonly Dictionary<int, LuaConnector.Response> _response_events = new Dictionary<int, LuaConnector.Response>();
    private bool _connected;
    private bool _ready;
    private ManualResetEventSlim connectSync = new ManualResetEventSlim();
    private bool _mapper_set;

    public static int CommandResponseTimeoutMS { get; set; } = 1000;

    private static int NextID => Interlocked.Increment(ref LuaConnector._next_id);

    public Endianness Endianness { get; private set; }

    public override IMemoryMapper Mapper
    {
      set
      {
        this._mapper_set = true;
        base.Mapper = value;
      }
    }

    public bool Connected => this._client.IsConnected() && this._connected && this._ready;

    public IFeatureAware Features { get; private set; }

    protected override void Dispose(bool disposing)
    {
      this._quitting = true;
      if (disposing)
      {
        try
        {
          this._listener?.Stop();
        }
        catch
        {
        }
        try
        {
          this._client?.Dispose();
        }
        catch
        {
        }
      }
      base.Dispose(disposing);
    }

    ~LuaConnector() => this.Dispose();

    public LuaConnector()
      : this((IPAddress) null)
    {
    }

    public LuaConnector(IPAddress endpoint)
      : this((Action<string>) null, endpoint)
    {
    }

    public LuaConnector(Action<string> messageHandler, IPAddress endpoint = null)
    {
      if (endpoint == null)
        endpoint = IPAddress.Loopback;
      this._message_handler = messageHandler ?? (Action<string>) (_ => { });
      this._listener = new TcpListener(endpoint, 43884);
      this._listener.Start();
      Task.Factory.StartNew((Action) (() =>
      {
        try
        {
          while (!this._quitting)
          {
            try
            {
              this._client = this._listener.AcceptTcpClient();
            }
            catch (SocketException ex)
            {
              this._ready = this._connected = false;
              if (this._quitting)
              {
                this._message_handler("LuaConnector listener is quitting.");
                break;
              }
              Log.Error((Exception) ex, "Lua connector failed to accept connection");
              continue;
            }
            if (this._stream != null)
            {
              this._stream.Close();
              this.connectSync.Wait();
              this.connectSync.Reset();
            }
            this._stream = this._client.GetStream();
            this._message_handler("LuaConnector listener socket established successfully.");
            this._connected = true;
            EventHandler<(ConnectionStatus, string)> connectionStatusChanged = this.ConnectionStatusChanged;
            if (connectionStatusChanged != null)
              connectionStatusChanged((object) this, (ConnectionStatus.Open, "Connection to client script established."));
            Task.Factory.StartNew(new Action(this.Receive), TaskCreationOptions.LongRunning);
          }
        }
        catch (Exception ex)
        {
          if (this._quitting)
          {
            this._message_handler("LuaConnector listener is quitting.");
            this._ready = this._connected = false;
            EventHandler<(ConnectionStatus, string)> connectionStatusChanged = this.ConnectionStatusChanged;
            if (connectionStatusChanged == null)
              return;
            connectionStatusChanged((object) this, (ConnectionStatus.Closed, "LuaConnector has stopped."));
          }
          else
            Log.Error(ex, "Lua connector failure");
        }
      }), TaskCreationOptions.LongRunning);
      Task.Factory.StartNew<Task>((Func<Task>) (async () =>
      {
        while (!this._quitting)
        {
          if (this._connected)
          {
            try
            {
              this.KeepAlive();
            }
            catch
            {
            }
          }
          await Task.Delay(TimeSpan.FromSeconds(2.5));
        }
      }));
    }

    private void Receive()
    {
      byte[] numArray = new byte[4096];
      while (!this._quitting)
      {
        try
        {
          if (this._client.IsConnected())
          {
            if (this._stream != null)
            {
              if (this._stream.Read(numArray, 0, 4) != 4)
              {
                if (!this._quitting)
                  Log.Message((object) "Failed to read all bytes");
                else
                  break;
              }
              else
              {
                int count = (int) numArray[0] << 24 | (int) numArray[1] << 16 | (int) numArray[2] << 8 | (int) numArray[3];
                if (count > numArray.Length)
                  Log.Error((object) "Lua connector message too big");
                else if (this._stream.Read(numArray, 0, count) == count)
                {
                  string json = Encoding.UTF8.GetString(numArray, 0, count);
                  try
                  {
                    this.ProcessBlock(json);
                  }
                  catch (Exception ex)
                  {
                    Log.Error(ex, "Lua connector failed to process block");
                    this._message_handler("LuaConnector block processor caught an exception. [" + ex.GetType().Name + "]");
                  }
                }
                else if (!this._quitting)
                {
                  Log.Message((object) "Failed to read all bytes");
                  this._message_handler("Failed to read message.");
                }
                else
                  break;
              }
            }
            else
              break;
          }
          else
            break;
        }
        catch (Exception ex)
        {
          if (!this._quitting)
          {
            this._ready = this._connected = false;
            EventHandler<(ConnectionStatus, string)> connectionStatusChanged = this.ConnectionStatusChanged;
            if (connectionStatusChanged != null)
              connectionStatusChanged((object) this, (ConnectionStatus.Closed, "LuaConnector has disconnected"));
            Log.Error(ex, "Lua connector failed to read");
            this.connectSync.Set();
            return;
          }
          break;
        }
      }
      this._ready = this._connected = false;
      this._message_handler("LuaConnector block processor is quitting.");
      EventHandler<(ConnectionStatus, string)> connectionStatusChanged1 = this.ConnectionStatusChanged;
      if (connectionStatusChanged1 != null)
        connectionStatusChanged1((object) this, (ConnectionStatus.Closed, (string) null));
      this.connectSync.Set();
    }

    public IBatchWriteContext OpenBatchWriteContext() => (IBatchWriteContext) new NullContext();

    [DebuggerStepThrough]
    public void KeepAlive() => this.WriteBlock(new JObject()
    {
      {
        "id",
        (JToken) LuaConnector.NextID
      },
      {
        "type",
        (JToken) byte.MaxValue
      }
    });

    public event EventHandler<(ConnectionStatus, string)> ConnectionStatusChanged;

    public bool SetBits(ulong address, byte value, out byte oldValue)
    {
      this._message_handler(string.Format("LuaConnector is setting a bit: [${0:X16} = #${1:X2}]", (object) address, (object) value));
      ulong oldValue1;
      bool flag = this.ReadCMD(address, (ulong) value, out oldValue1, CommandType.SetBits);
      oldValue = checked ((byte) oldValue1);
      return flag;
    }

    public bool UnsetBits(ulong address, byte value, out byte oldValue)
    {
      this._message_handler(string.Format("LuaConnector is unsetting a bit: [${0:X16} = #${1:X2}]", (object) address, (object) value));
      ulong oldValue1;
      bool flag = this.ReadCMD(address, (ulong) value, out oldValue1, CommandType.UnsetBits);
      oldValue = checked ((byte) oldValue1);
      return flag;
    }

    public bool CompareSwap(ulong address, byte test, byte newVal, out byte orig)
    {
      this._message_handler("LuaConnector compare swap not yet supported.");
      orig = (byte) 0;
      return false;
    }

    private bool SetCoreModes(string name)
    {
      switch (name)
      {
        case "GB":
        case "GBC":
          if (!this._mapper_set)
            this.Mapper = (IMemoryMapper) null;
          this.Features = (IFeatureAware) null;
          this.Endianness = Endianness.LittleEndian;
          return true;
        case "GBA":
          if (!this._mapper_set)
            this.Mapper = (IMemoryMapper) null;
          this.Features = (IFeatureAware) null;
          this.Endianness = Endianness.LittleEndian;
          return true;
        case "GEN":
          if (!this._mapper_set)
            this.Mapper = (IMemoryMapper) new BusMapper("M68K BUS");
          this.Features = (IFeatureAware) null;
          this.Endianness = Endianness.BigEndian;
          return true;
        case "N64":
          if (!this._mapper_set)
            this.Mapper = (IMemoryMapper) new N64Mapper();
          this.Features = (IFeatureAware) new BizhawkMupen64PlusFeatures();
          this.Endianness = Endianness.BigEndian;
          return true;
        case "NES":
          if (!this._mapper_set)
            this.Mapper = (IMemoryMapper) null;
          this.Features = (IFeatureAware) null;
          this.Endianness = Endianness.LittleEndian;
          return true;
        case "PSX":
          if (!this._mapper_set)
            this.Mapper = (IMemoryMapper) new PS1Mapper();
          this.Features = (IFeatureAware) null;
          this.Endianness = Endianness.LittleEndian;
          return true;
        case "RAWSNES":
          this.Features = (IFeatureAware) new BizhawkBSNESFeatures();
          this.Endianness = Endianness.LittleEndian;
          return true;
        case "SNES":
          if (!this._mapper_set)
            this.Mapper = (IMemoryMapper) null;
          this.Features = (IFeatureAware) new BizhawkBSNESFeatures();
          this.Endianness = Endianness.LittleEndian;
          return true;
        default:
          return false;
      }
    }

    IBatchContext8 I8BitConnector.GetBatchContext8() => (IBatchContext8) null;

    IBatchContext16 I16BitConnector.GetBatchContext16() => (IBatchContext16) null;

    IBatchContext32 I32BitConnector.GetBatchContext32() => (IBatchContext32) null;

    IBatchContext64 I64BitConnector.GetBatchContext64() => (IBatchContext64) null;

    private bool SendBlock(CommandType type, ulong address, ulong value) => this.WriteBlock(new JObject()
    {
      {
        "id",
        (JToken) LuaConnector.NextID
      },
      {
        nameof (type),
        (JToken) (byte) type
      },
      {
        nameof (address),
        (JToken) address
      },
      {
        nameof (value),
        (JToken) value
      }
    });

    public bool Freeze(
      ulong address,
      ulong value,
      byte size,
      WriteType writeType,
      ulong cmpAddress,
      ulong cmpValue,
      byte cmpSize,
      CompareType condition)
    {
      return this.WriteBlock(new JObject()
      {
        {
          "id",
          (JToken) LuaConnector.NextID
        },
        {
          "type",
          (JToken) (byte) 48
        },
        {
          nameof (address),
          (JToken) address
        },
        {
          nameof (value),
          (JToken) value
        },
        {
          nameof (size),
          (JToken) size
        },
        {
          nameof (writeType),
          (JToken) (byte) writeType
        },
        {
          nameof (cmpAddress),
          (JToken) cmpAddress
        },
        {
          nameof (cmpValue),
          (JToken) cmpValue
        },
        {
          nameof (cmpSize),
          (JToken) cmpSize
        },
        {
          nameof (condition),
          (JToken) (byte) condition
        }
      });
    }

    public bool Freeze(
      ulong address,
      ulong value,
      ulong mask,
      byte size,
      WriteType writeType,
      ulong cmpAddress,
      ulong cmpValue,
      byte cmpSize,
      CompareType condition)
    {
      return this.WriteBlock(new JObject()
      {
        {
          "id",
          (JToken) LuaConnector.NextID
        },
        {
          "type",
          (JToken) (byte) 48
        },
        {
          nameof (address),
          (JToken) address
        },
        {
          nameof (value),
          (JToken) value
        },
        {
          nameof (mask),
          (JToken) mask
        },
        {
          nameof (size),
          (JToken) size
        },
        {
          nameof (writeType),
          (JToken) (byte) writeType
        },
        {
          nameof (cmpAddress),
          (JToken) cmpAddress
        },
        {
          nameof (cmpValue),
          (JToken) cmpValue
        },
        {
          nameof (cmpSize),
          (JToken) cmpSize
        },
        {
          nameof (condition),
          (JToken) (byte) condition
        }
      });
    }

    private bool Freeze(ulong address, ulong value, byte size) => this.WriteBlock(new JObject()
    {
      {
        "id",
        (JToken) LuaConnector.NextID
      },
      {
        "type",
        (JToken) (byte) 48
      },
      {
        nameof (address),
        (JToken) address
      },
      {
        nameof (value),
        (JToken) value
      },
      {
        nameof (size),
        (JToken) size
      },
      {
        "writeType",
        (JToken) (byte) 1
      },
      {
        "condition",
        (JToken) (byte) 3
      }
    });

    public bool FreezeU8(ulong address, byte value, byte cmpValue, CompareType cmpType) => this.FreezeU8TU8(address, value, cmpValue, cmpType);

    public bool FreezeU16(ulong address, ushort value, ushort cmpValue, CompareType cmpType) => this.FreezeU16TU16(address, value, cmpValue, cmpType);

    public bool FreezeU32(ulong address, uint value, uint cmpValue, CompareType cmpType) => this.FreezeU32TU32(address, value, cmpValue, cmpType);

    public bool FreezeU64(ulong address, ulong value, ulong cmpValue, CompareType cmpType) => this.FreezeU64TU64(address, value, cmpValue, cmpType);

    public bool FreezeU8TU8(ulong address, byte value, byte cmpValue, CompareType cmpType) => this.Freeze(address, (ulong) value, (byte) 1, WriteType.WriteValue, address, (ulong) cmpValue, (byte) 1, cmpType);

    public bool FreezeU8TU16(ulong address, byte value, ushort cmpValue, CompareType cmpType) => this.Freeze(address, (ulong) value, (byte) 1, WriteType.WriteValue, address, (ulong) cmpValue, (byte) 2, cmpType);

    public bool FreezeU8TU32(ulong address, byte value, uint cmpValue, CompareType cmpType) => this.Freeze(address, (ulong) value, (byte) 1, WriteType.WriteValue, address, (ulong) cmpValue, (byte) 4, cmpType);

    public bool FreezeU8TU64(ulong address, byte value, ulong cmpValue, CompareType cmpType) => this.Freeze(address, (ulong) value, (byte) 1, WriteType.WriteValue, address, cmpValue, (byte) 8, cmpType);

    public bool FreezeU16TU8(ulong address, ushort value, byte cmpValue, CompareType cmpType) => this.Freeze(address, (ulong) value, (byte) 2, WriteType.WriteValue, address, (ulong) cmpValue, (byte) 1, cmpType);

    public bool FreezeU16TU16(ulong address, ushort value, ushort cmpValue, CompareType cmpType) => this.Freeze(address, (ulong) value, (byte) 2, WriteType.WriteValue, address, (ulong) cmpValue, (byte) 2, cmpType);

    public bool FreezeU16TU32(ulong address, ushort value, uint cmpValue, CompareType cmpType) => this.Freeze(address, (ulong) value, (byte) 2, WriteType.WriteValue, address, (ulong) cmpValue, (byte) 4, cmpType);

    public bool FreezeU16TU64(ulong address, ushort value, ulong cmpValue, CompareType cmpType) => this.Freeze(address, (ulong) value, (byte) 2, WriteType.WriteValue, address, cmpValue, (byte) 8, cmpType);

    public bool FreezeU32TU8(ulong address, uint value, byte cmpValue, CompareType cmpType) => this.Freeze(address, (ulong) value, (byte) 4, WriteType.WriteValue, address, (ulong) cmpValue, (byte) 1, cmpType);

    public bool FreezeU32TU16(ulong address, uint value, ushort cmpValue, CompareType cmpType) => this.Freeze(address, (ulong) value, (byte) 4, WriteType.WriteValue, address, (ulong) cmpValue, (byte) 2, cmpType);

    public bool FreezeU32TU32(ulong address, uint value, uint cmpValue, CompareType cmpType) => this.Freeze(address, (ulong) value, (byte) 4, WriteType.WriteValue, address, (ulong) cmpValue, (byte) 4, cmpType);

    public bool FreezeU32TU64(ulong address, uint value, ulong cmpValue, CompareType cmpType) => this.Freeze(address, (ulong) value, (byte) 4, WriteType.WriteValue, address, cmpValue, (byte) 8, cmpType);

    public bool FreezeU64TU8(ulong address, ulong value, byte cmpValue, CompareType cmpType) => this.Freeze(address, value, (byte) 8, WriteType.WriteValue, address, (ulong) cmpValue, (byte) 1, cmpType);

    public bool FreezeU64TU16(ulong address, ulong value, ushort cmpValue, CompareType cmpType) => this.Freeze(address, value, (byte) 8, WriteType.WriteValue, address, (ulong) cmpValue, (byte) 2, cmpType);

    public bool FreezeU64TU32(ulong address, ulong value, uint cmpValue, CompareType cmpType) => this.Freeze(address, value, (byte) 8, WriteType.WriteValue, address, (ulong) cmpValue, (byte) 4, cmpType);

    public bool FreezeU64TU64(ulong address, ulong value, ulong cmpValue, CompareType cmpType) => this.Freeze(address, value, (byte) 8, WriteType.WriteValue, address, cmpValue, (byte) 8, cmpType);

    public bool FreezeU8TU8(
      ulong address,
      byte value,
      ulong cmpAddress,
      byte cmpValue,
      CompareType cmpType)
    {
      return this.Freeze(address, (ulong) value, (byte) 1, WriteType.WriteValue, cmpAddress, (ulong) cmpValue, (byte) 1, cmpType);
    }

    public bool FreezeU8TU16(
      ulong address,
      byte value,
      ulong cmpAddress,
      ushort cmpValue,
      CompareType cmpType)
    {
      return this.Freeze(address, (ulong) value, (byte) 1, WriteType.WriteValue, cmpAddress, (ulong) cmpValue, (byte) 2, cmpType);
    }

    public bool FreezeU8TU32(
      ulong address,
      byte value,
      ulong cmpAddress,
      uint cmpValue,
      CompareType cmpType)
    {
      return this.Freeze(address, (ulong) value, (byte) 1, WriteType.WriteValue, cmpAddress, (ulong) cmpValue, (byte) 4, cmpType);
    }

    public bool FreezeU8TU64(
      ulong address,
      byte value,
      ulong cmpAddress,
      ulong cmpValue,
      CompareType cmpType)
    {
      return this.Freeze(address, (ulong) value, (byte) 1, WriteType.WriteValue, cmpAddress, cmpValue, (byte) 8, cmpType);
    }

    public bool FreezeU16TU8(
      ulong address,
      ushort value,
      ulong cmpAddress,
      byte cmpValue,
      CompareType cmpType)
    {
      return this.Freeze(address, (ulong) value, (byte) 2, WriteType.WriteValue, cmpAddress, (ulong) cmpValue, (byte) 1, cmpType);
    }

    public bool FreezeU16TU16(
      ulong address,
      ushort value,
      ulong cmpAddress,
      ushort cmpValue,
      CompareType cmpType)
    {
      return this.Freeze(address, (ulong) value, (byte) 2, WriteType.WriteValue, cmpAddress, (ulong) cmpValue, (byte) 2, cmpType);
    }

    public bool FreezeU16TU32(
      ulong address,
      ushort value,
      ulong cmpAddress,
      uint cmpValue,
      CompareType cmpType)
    {
      return this.Freeze(address, (ulong) value, (byte) 2, WriteType.WriteValue, cmpAddress, (ulong) cmpValue, (byte) 4, cmpType);
    }

    public bool FreezeU16TU64(
      ulong address,
      ushort value,
      ulong cmpAddress,
      ulong cmpValue,
      CompareType cmpType)
    {
      return this.Freeze(address, (ulong) value, (byte) 2, WriteType.WriteValue, cmpAddress, cmpValue, (byte) 8, cmpType);
    }

    public bool FreezeU32TU8(
      ulong address,
      uint value,
      ulong cmpAddress,
      byte cmpValue,
      CompareType cmpType)
    {
      return this.Freeze(address, (ulong) value, (byte) 4, WriteType.WriteValue, cmpAddress, (ulong) cmpValue, (byte) 1, cmpType);
    }

    public bool FreezeU32TU16(
      ulong address,
      uint value,
      ulong cmpAddress,
      ushort cmpValue,
      CompareType cmpType)
    {
      return this.Freeze(address, (ulong) value, (byte) 4, WriteType.WriteValue, cmpAddress, (ulong) cmpValue, (byte) 2, cmpType);
    }

    public bool FreezeU32TU32(
      ulong address,
      uint value,
      ulong cmpAddress,
      uint cmpValue,
      CompareType cmpType)
    {
      return this.Freeze(address, (ulong) value, (byte) 4, WriteType.WriteValue, cmpAddress, (ulong) cmpValue, (byte) 4, cmpType);
    }

    public bool FreezeU32TU64(
      ulong address,
      uint value,
      ulong cmpAddress,
      ulong cmpValue,
      CompareType cmpType)
    {
      return this.Freeze(address, (ulong) value, (byte) 4, WriteType.WriteValue, cmpAddress, cmpValue, (byte) 8, cmpType);
    }

    public bool FreezeU64TU8(
      ulong address,
      ulong value,
      ulong cmpAddress,
      byte cmpValue,
      CompareType cmpType)
    {
      return this.Freeze(address, value, (byte) 8, WriteType.WriteValue, cmpAddress, (ulong) cmpValue, (byte) 1, cmpType);
    }

    public bool FreezeU64TU16(
      ulong address,
      ulong value,
      ulong cmpAddress,
      ushort cmpValue,
      CompareType cmpType)
    {
      return this.Freeze(address, value, (byte) 8, WriteType.WriteValue, cmpAddress, (ulong) cmpValue, (byte) 2, cmpType);
    }

    public bool FreezeU64TU32(
      ulong address,
      ulong value,
      ulong cmpAddress,
      uint cmpValue,
      CompareType cmpType)
    {
      return this.Freeze(address, value, (byte) 8, WriteType.WriteValue, cmpAddress, (ulong) cmpValue, (byte) 4, cmpType);
    }

    public bool FreezeU64TU64(
      ulong address,
      ulong value,
      ulong cmpAddress,
      ulong cmpValue,
      CompareType cmpType)
    {
      return this.Freeze(address, value, (byte) 8, WriteType.WriteValue, cmpAddress, cmpValue, (byte) 8, cmpType);
    }

    bool I8BitConnector.Freeze8(ulong address, byte value) => this.FreezeU8(address, value);

    public bool FreezeU8(ulong address, byte value)
    {
      this._message_handler(string.Format("LuaConnector is freezing: [${0:X8} = #${1:X2}]", (object) address, (object) value));
      return this.Freeze(address, (ulong) value, (byte) 1);
    }

    bool I16BitConnector.Freeze16(ulong address, ushort value) => this.FreezeU16(address, value);

    public bool FreezeU16(ulong address, ushort value)
    {
      this._message_handler(string.Format("LuaConnector is freezing: [${0:X8} = #${1:X4}]", (object) address, (object) value));
      return this.Freeze(address, (ulong) value, (byte) 2);
    }

    bool I32BitConnector.Freeze32(ulong address, uint value) => this.FreezeU32(address, value);

    public bool FreezeU32(ulong address, uint value)
    {
      this._message_handler(string.Format("LuaConnector is freezing: [${0:X8} = #${1:X8}]", (object) address, (object) value));
      return this.Freeze(address, (ulong) value, (byte) 4);
    }

    bool I64BitConnector.Freeze64(ulong address, ulong value) => this.FreezeU64(address, value);

    public bool FreezeU64(ulong address, ulong value)
    {
      this._message_handler(string.Format("LuaConnector is freezing: [${0:X8} = #${1:X16}]", (object) address, (object) value));
      return this.Freeze(address, value, (byte) 8);
    }

    public bool Unfreeze(ulong address)
    {
      this._message_handler(string.Format("LuaConnector is unfreezing: [${0:X8}]", (object) address));
      return this.WriteBlock(new JObject()
      {
        {
          "id",
          (JToken) LuaConnector.NextID
        },
        {
          "type",
          (JToken) (byte) 63
        },
        {
          nameof (address),
          (JToken) address
        }
      });
    }

    public bool WriteByte(ulong address, byte value) => this.Write8(address, value);

    public bool Write8(ulong address, byte value)
    {
      this._message_handler(string.Format("LuaConnector is writing: [${0:X8} = #${1:X2}]", (object) address, (object) value));
      return this.SendBlock(CommandType.WriteU8, address, (ulong) value);
    }

    public bool WriteWord(ulong address, ushort value) => this.Write16(address, value);

    public bool Write16(ulong address, ushort value)
    {
      this._message_handler(string.Format("LuaConnector is writing: [${0:X8} = #${1:X4}]", (object) address, (object) value));
      return this.SendBlock(CommandType.WriteU16, address, (ulong) value);
    }

    public bool Write32(ulong address, uint value)
    {
      this._message_handler(string.Format("LuaConnector is writing: [${0:X8} = #${1:X8}]", (object) address, (object) value));
      return this.SendBlock(CommandType.WriteU32, address, (ulong) value);
    }

    public bool Write64(ulong address, ulong value)
    {
      this._message_handler(string.Format("LuaConnector is writing: [${0:X8} = #${1:X16}]", (object) address, (object) value));
      return this.SendBlock(CommandType.WriteU64, address, value);
    }

    public byte? ReadByte(ulong address)
    {
      byte num;
      return !this.Read8(address, out num) ? new byte?() : new byte?(num);
    }

    public bool Read8(ulong address, out byte value)
    {
      this._message_handler(string.Format("LuaConnector is reading: [${0:X16}]", (object) address));
      return this.ReadCMD<byte>(address, out value, CommandType.ReadU8);
    }

    public bool Read16(ulong address, out ushort value)
    {
      this._message_handler(string.Format("LuaConnector is reading: [${0:X16}]", (object) address));
      return this.ReadCMD<ushort>(address, out value, CommandType.ReadU16);
    }

    public bool Read32(ulong address, out uint value)
    {
      this._message_handler(string.Format("LuaConnector is reading: [${0:X16}]", (object) address));
      return this.ReadCMD<uint>(address, out value, CommandType.ReadU32);
    }

    public bool Read64(ulong address, out ulong value)
    {
      this._message_handler(string.Format("LuaConnector is reading: [${0:X16}]", (object) address));
      return this.ReadCMD<ulong>(address, out value, CommandType.ReadU64);
    }

    private bool ReadCMD<T>(ulong address, out T value, CommandType commandType)
    {
      int nextId = LuaConnector.NextID;
      return this.ReadImpl<T>(new JObject()
      {
        {
          "id",
          (JToken) nextId
        },
        {
          "type",
          (JToken) (byte) commandType
        },
        {
          nameof (address),
          (JToken) address
        }
      }, nextId, out value);
    }

    private bool ReadCMD(ulong address, ulong value, out ulong oldValue, CommandType commandType)
    {
      int nextId = LuaConnector.NextID;
      return this.ReadImpl<ulong>(new JObject()
      {
        {
          "id",
          (JToken) nextId
        },
        {
          "type",
          (JToken) (byte) commandType
        },
        {
          nameof (address),
          (JToken) address
        },
        {
          nameof (value),
          (JToken) value
        }
      }, nextId, out oldValue);
    }

    private bool ReadImpl<T>(JObject block, int id, out T value)
    {
      if (!this.WriteBlock(block))
      {
        value = default (T);
        return false;
      }
      JObject jobject = this.ReadImpl(id);
      if (jobject == null)
      {
        value = default (T);
        return false;
      }
      value = jobject[nameof (value)].Value<T>();
      return true;
    }

    private JObject ReadImpl(int id)
    {
      LuaConnector.Response response = new LuaConnector.Response()
      {
        handle = new ManualResetEventSlim()
      };
      this._response_events.Put<int, LuaConnector.Response>(id, response);
      bool flag = response.handle.Wait(LuaConnector.CommandResponseTimeoutMS);
      this._response_events.Remove(id);
      return flag ? response.block : (JObject) null;
    }

    public bool WriteBytes(ulong address, byte[] buffer) => this.Write(address, buffer);

    public bool Write(ulong address, byte[] data)
    {
      if (data == null)
        return false;
      if (data.Length == 0)
        return true;
      this._message_handler(string.Format("LuaConnector is writing {0} bytes: [${1:X8}]", (object) data.Length, (object) address));
      return this.WriteBlock(new JObject()
      {
        {
          "id",
          (JToken) LuaConnector.NextID
        },
        {
          "type",
          (JToken) (byte) 31
        },
        {
          nameof (address),
          (JToken) address
        },
        {
          "block",
          (JToken) data
        },
        {
          "value",
          (JToken) data.Length
        }
      });
    }

    public bool ReadBytes(ulong address, byte[] buffer) => this.Read(address, buffer);

    public bool Read(ulong address, byte[] buffer)
    {
      if (buffer == null)
        return false;
      if (buffer.Length == 0)
        return true;
      this._message_handler(string.Format("LuaConnector is reading: [${0:X8}]", (object) address));
      int nextId = LuaConnector.NextID;
      if (!this.WriteBlock(new JObject()
      {
        {
          "id",
          (JToken) nextId
        },
        {
          "type",
          (JToken) (byte) 15
        },
        {
          nameof (address),
          (JToken) address
        },
        {
          "value",
          (JToken) buffer.Length
        }
      }))
        return false;
      JObject jobject = this.ReadImpl(nextId);
      if (jobject?["block"] == null)
        return false;
      Convert.FromBase64String(jobject["block"].Value<string>()).CopyTo((Array) buffer, 0);
      return true;
    }

    public bool SendMessage(string message)
    {
      EventHandler<string> onMessage = this.OnMessage;
      if (onMessage != null)
        onMessage((object) this, message);
      return this.WriteBlock(new JObject()
      {
        {
          "id",
          (JToken) LuaConnector.NextID
        },
        {
          "type",
          (JToken) (byte) 240
        },
        {
          nameof (message),
          (JToken) (message ?? string.Empty)
        }
      });
    }

    public event EventHandler<string> OnMessage;

    public string GetROMPath()
    {
      int nextId = LuaConnector.NextID;
      if (!this.WriteBlock(new JObject()
      {
        {
          "id",
          (JToken) LuaConnector.NextID
        },
        {
          "type",
          (JToken) (byte) 226
        }
      }))
        return (string) null;
      JObject jobject = this.ReadImpl(nextId);
      // ISSUE: explicit non-virtual call
      return jobject == null ? (string) null : (jobject["message"]).Value<string>();
    }

    public bool OpenROM(string path) => this.WriteBlock(new JObject()
    {
      {
        "id",
        (JToken) LuaConnector.NextID
      },
      {
        "type",
        (JToken) (byte) 224
      },
      {
        "message",
        (JToken) path
      }
    });

    public bool CloseROM() => this.WriteBlock(new JObject()
    {
      {
        "id",
        (JToken) LuaConnector.NextID
      },
      {
        "type",
        (JToken) (byte) 225
      }
    });

    public bool PatchROM(IROMPatch patch)
    {
      string romPath = this.GetROMPath();
      if (string.IsNullOrWhiteSpace(romPath) || !this.CloseROM())
        return false;
      string path = Path.ChangeExtension(romPath, "cc." + Path.GetExtension(romPath));
      using (FileStream fileStream = System.IO.File.Open(romPath, FileMode.Open, FileAccess.Read))
      {
        using (FileStream destination = System.IO.File.Open(path, FileMode.Truncate, FileAccess.Write))
        {
          fileStream.CopyTo((Stream) destination);
          foreach (IPatch patch1 in (IEnumerable<IPatch>) patch.Items)
          {
            destination.Position = (long) patch1.Address;
            byte[] data = patch1.Data;
            destination.Write(data, 0, data.Length);
          }
          return this.OpenROM(path);
        }
      }
    }

    private void PrepOutgoingBlock(JObject block)
    {
      CommandType commandType = (CommandType) block["type"].Value<byte>();
      if ((uint) commandType <= 48U)
      {
        switch (commandType)
        {
          case CommandType.WriteU16:
            if (this.Endianness == Endianness.BigEndian)
            {
              block["value"] = (JToken) block["value"].Value<ushort>().SwapBytes();
              break;
            }
            break;
          case CommandType.WriteU32:
            if (this.Endianness == Endianness.BigEndian)
            {
              block["value"] = (JToken) block["value"].Value<uint>().SwapBytes();
              break;
            }
            break;
          case CommandType.WriteU64:
            if (this.Endianness == Endianness.BigEndian)
            {
              block["value"] = (JToken) block["value"].Value<ulong>().SwapBytes();
              break;
            }
            break;
          case CommandType.FreezeU:
            if (block["size"].Value<byte>() != (byte) 2)
            {
              if (block["size"].Value<byte>() != (byte) 4)
              {
                if (block["size"].Value<byte>() != (byte) 8)
                  break;
                goto case CommandType.WriteU64;
              }
              else
                goto case CommandType.WriteU32;
            }
            else
              goto case CommandType.WriteU16;
        }
      }
      else
      {
        switch (commandType)
        {
          case CommandType.OpenROM:
          case CommandType.CloseROM:
          case CommandType.GetROMPath:
          case CommandType.GetEmulationCore:
            return;
          case CommandType.SendMessage:
            return;
          case CommandType.KeepAlive:
            return;
        }
      }
      JToken jtoken1 = block["domain"];
      string a = jtoken1 != null ? jtoken1.Value<string>() : (string) null;
      if (this.Mapper == null || !string.IsNullOrWhiteSpace(a) && !string.Equals(a, "System Bus", StringComparison.OrdinalIgnoreCase))
        return;
      JObject jobject1 = block;
      JObject jobject2 = block;
      (ulong address, string domain) tuple = this.Mapper.Translate(block["address"].Value<ulong>());
      JToken address1 = (JToken) tuple.address;
      JToken domain = (JToken) tuple.domain;
      JToken jtoken2;
      JToken jtoken3 = jtoken2 = address1;
      jobject1["address"] = jtoken2;
      jobject2["domain"] = jtoken3 = domain;
      JToken jtoken4 = block["cmpAddress"];
      int num1;
      if (jtoken4 == null)
      {
        num1 = 0;
      }
      else
      {
        long num2 = (long) jtoken4.Value<ulong>();
        num1 = 1;
      }
      if (num1 == 0)
        return;
      JObject jobject3 = block;
      JToken address2 = (JToken) this.Mapper.Translate(block["cmpAddress"].Value<ulong>()).address;
      JToken jtoken5;
      JToken jtoken6 = jtoken5 = address2;
      jobject3["cmpAddress"] = jtoken5;
    }

    private void PrepIncomingBlock(JObject block)
    {
      CommandType commandType = (CommandType) block["type"].Value<byte>();
      if ((uint) commandType <= 48U)
      {
        switch (commandType)
        {
          case CommandType.ReadU16:
          case CommandType.WriteU16:
            if (this.Endianness == Endianness.BigEndian)
            {
              block["value"] = (JToken) block["value"].Value<ushort>().SwapBytes();
              break;
            }
            break;
          case CommandType.ReadU32:
          case CommandType.WriteU32:
            if (this.Endianness == Endianness.BigEndian)
            {
              block["value"] = (JToken) block["value"].Value<uint>().SwapBytes();
              break;
            }
            break;
          case CommandType.ReadU64:
          case CommandType.WriteU64:
            if (this.Endianness == Endianness.BigEndian)
            {
              block["value"] = (JToken) block["value"].Value<ulong>().SwapBytes();
              break;
            }
            break;
          case CommandType.FreezeU:
            if (block["size"].Value<byte>() != (byte) 2)
            {
              if (block["size"].Value<byte>() != (byte) 4)
              {
                if (block["size"].Value<byte>() != (byte) 8)
                  break;
                goto case CommandType.ReadU64;
              }
              else
                goto case CommandType.ReadU32;
            }
            else
              goto case CommandType.ReadU16;
        }
      }
      else
      {
        switch (commandType)
        {
          case CommandType.OpenROM:
          case CommandType.CloseROM:
          case CommandType.GetROMPath:
          case CommandType.GetEmulationCore:
            return;
          case CommandType.SendMessage:
            return;
          case CommandType.KeepAlive:
            return;
        }
      }
      if (this.Mapper == null)
        return;
      block["address"] = (JToken) this.Mapper.Translate(block["address"].Value<ulong>(), block["domain"].Value<string>());
      block["domain"] = (JToken) "System Bus";
    }

    private bool WriteBlock(JObject block, bool ignoreReady = false)
    {
      if (!this._connected || this._stream == null)
        return false;
      if (!(this._ready | ignoreReady))
      {
        this.WriteBlock(new JObject()
        {
          {
            "id",
            (JToken) LuaConnector.NextID
          },
          {
            "type",
            (JToken) (byte) 227
          },
          {
            "value",
            (JToken) 131584U
          }
        }, true);
        return false;
      }
      this.PrepOutgoingBlock(block);
      byte[] bytes = Encoding.UTF8.GetBytes(JsonConvert.SerializeObject((object) block));
      uint length = checked ((uint) bytes.Length);
      try
      {
        NetworkStream stream = this._stream;
        lock (stream)
        {
          byte num1 = checked ((byte) (unchecked ((int) (length >> 24)) & (int) byte.MaxValue));
          byte num2 = checked ((byte) (unchecked ((int) (length >> 16)) & (int) byte.MaxValue));
          byte num3 = checked ((byte) (unchecked ((int) (length >> 8)) & (int) byte.MaxValue));
          byte num4 = checked ((byte) (unchecked ((int) length) & (int) byte.MaxValue));
          stream.WriteByte(num1);
          stream.WriteByte(num2);
          stream.WriteByte(num3);
          stream.WriteByte(num4);
          stream.Write(bytes, 0, bytes.Length);
          stream.Flush();
        }
      }
      catch (Exception ex)
      {
        this._ready = this._connected = false;
        EventHandler<(ConnectionStatus, string)> connectionStatusChanged = this.ConnectionStatusChanged;
        if (connectionStatusChanged != null)
          connectionStatusChanged((object) this, (ConnectionStatus.Closed, "LuaConnector has disconnected"));
        Log.Error(ex, "Lua connector failed to write");
        return false;
      }
      return true;
    }

    private void ProcessBlock(string json)
    {
      JObject block = JObject.Parse(json);
      switch (block["type"].Value<byte>())
      {
        case 227:
          uint num = block["value"].Value<uint>();
          this._message_handler(string.Format("LuaConnector v{0}.{1}.{2} initialized (client v{3}.{4}.{5})", (object) 2U, (object) 2U, (object) 0U, (object) (num >> 16 & (uint) byte.MaxValue), (object) (num >> 8 & (uint) byte.MaxValue), (object) (num & (uint) byte.MaxValue)));
          if (num != 131584U)
            throw new Exception("LuaConnector version mismatch.");
          this._ready = this.SetCoreModes(block["message"].Value<string>());
          if (!this._ready)
            break;
          this.InvokeOnReady();
          break;
        case byte.MaxValue:
          this.PrepIncomingBlock(block);
          LuaConnector.Response response;
          if (!this._response_events.TryGetValue(block["id"].Value<int>(), out response))
            break;
          response.block = block;
          response.handle.Set();
          break;
        default:
          this._message_handler(string.Format("LuaConnector got a block. [{0}]", (object) block["id"].Value<int>()));
          goto case byte.MaxValue;
      }
    }

    private class Response
    {
      public ManualResetEventSlim handle;
      public JObject block;
    }
  }
}
