﻿// Decompiled with JetBrains decompiler
// Type: ConnectorLib.NamedPipeClientConnector
// Assembly: ConnectorLib, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: CC4FDCDA-98EE-4CD3-AF60-DEE32CDD48A0
// Assembly location: C:\Program Files (x86)\EmoTracker\ConnectorLib.dll

using Newtonsoft.Json;
using System;
using System.IO;
using System.IO.Pipes;

namespace ConnectorLib
{
  public class NamedPipeClientConnector : ConnectorBase, IGameConnector, IDisposable
  {
    private NamedPipeClientStream _stream;
    private string _server_name;
    private string _pipe_name;
    private Func<string, string> _message_builder;
    private bool _connected;
    private NamedPipeClientConnector.MessageMode _message_mode;

    public bool Connected
    {
      get
      {
        switch (this._message_mode)
        {
          case NamedPipeClientConnector.MessageMode.SingleUse:
            return this._connected;
          case NamedPipeClientConnector.MessageMode.MessageFramed:
            NamedPipeClientStream stream = this._stream;
            return stream != null && stream.IsConnected;
          default:
            throw new ArgumentOutOfRangeException();
        }
      }
    }

    protected override void Dispose(bool disposing)
    {
      if (disposing)
        this._stream?.Dispose();
      base.Dispose(disposing);
    }

    public void Init(
      string pipeName,
      NamedPipeClientConnector.MessageMode alwaysReconnect = NamedPipeClientConnector.MessageMode.SingleUse)
    {
      this.Init(".", pipeName, alwaysReconnect);
    }

    public void Init(
      string serverName,
      string pipeName,
      NamedPipeClientConnector.MessageMode alwaysReconnect = NamedPipeClientConnector.MessageMode.SingleUse,
      Func<string, string> messageBuilder = null)
    {
      this._server_name = serverName;
      this._pipe_name = pipeName;
      this._message_mode = alwaysReconnect;
      this._message_builder = messageBuilder;
      switch (this._message_mode)
      {
        case NamedPipeClientConnector.MessageMode.SingleUse:
          this._connected = true;
          this.InvokeOnReady();
          EventHandler<(ConnectionStatus, string)> connectionStatusChanged = this.ConnectionStatusChanged;
          if (connectionStatusChanged == null)
            break;
          connectionStatusChanged((object) this, (ConnectionStatus.Open, (string) null));
          break;
        case NamedPipeClientConnector.MessageMode.MessageFramed:
          this._stream = new NamedPipeClientStream(serverName, pipeName, PipeDirection.InOut);
          this.ConnectIfNecessary();
          break;
        default:
          throw new ArgumentOutOfRangeException();
      }
    }

    public event EventHandler<string> PipeMessageReceived;

    public bool Send(string message) => this.SendRaw(message);

    public bool SendJSON(object message) => this.SendRaw(JsonConvert.SerializeObject(message));

    private bool SendRaw(string message)
    {
      try
      {
        switch (this._message_mode)
        {
          case NamedPipeClientConnector.MessageMode.SingleUse:
            using (NamedPipeClientStream pipeClientStream = new NamedPipeClientStream(this._server_name, this._pipe_name))
            {
              if (!pipeClientStream.IsConnected)
                pipeClientStream.Connect(1000);
              using (StreamWriter streamWriter = new StreamWriter((Stream) pipeClientStream)
              {
                AutoFlush = true
              })
                streamWriter.WriteLine(message);
              return true;
            }
          case NamedPipeClientConnector.MessageMode.MessageFramed:
            this.ConnectIfNecessary();
            using (StreamWriter streamWriter = new StreamWriter((Stream) this._stream)
            {
              AutoFlush = true
            })
              streamWriter.WriteLine(message);
            return true;
          default:
            throw new ArgumentOutOfRangeException();
        }
      }
      catch (Exception ex)
      {
        switch (this._message_mode)
        {
          case NamedPipeClientConnector.MessageMode.SingleUse:
            int messageMode = (int) this._message_mode;
            Log.Error((object) ex);
            return false;
          case NamedPipeClientConnector.MessageMode.MessageFramed:
            this.Disconnect();
            goto case NamedPipeClientConnector.MessageMode.SingleUse;
          default:
            throw new ArgumentOutOfRangeException();
        }
      }
    }

    public bool SendMessage(string message)
    {
      EventHandler<string> onMessage = this.OnMessage;
      if (onMessage != null)
        onMessage((object) this, message);
      return this._message_builder == null || this.SendRaw(this._message_builder(message));
    }

    private void ConnectIfNecessary()
    {
      if (this.Connected)
        return;
      this._stream?.Connect();
      Log.Message((object) ("NamedPipeConnector has connected to the pipe " + this._pipe_name + "."));
      this.InvokeOnReady();
      EventHandler<(ConnectionStatus, string)> connectionStatusChanged = this.ConnectionStatusChanged;
      if (connectionStatusChanged == null)
        return;
      connectionStatusChanged((object) this, (ConnectionStatus.Open, (string) null));
    }

    private void Disconnect()
    {
      if (!this.Connected)
        return;
      try
      {
        this._stream?.Dispose();
      }
      catch
      {
      }
      this._stream = (NamedPipeClientStream) null;
      Log.Message((object) "NamedPipeConnector has disconnected.");
      EventHandler<(ConnectionStatus, string)> connectionStatusChanged = this.ConnectionStatusChanged;
      if (connectionStatusChanged == null)
        return;
      connectionStatusChanged((object) this, (ConnectionStatus.Closed, (string) null));
    }

    public void KeepAlive()
    {
    }

    public event EventHandler<(ConnectionStatus, string)> ConnectionStatusChanged;

    public event EventHandler<string> OnMessage;

    public IBatchWriteContext OpenBatchWriteContext() => (IBatchWriteContext) NullContext.Context;

    public enum MessageMode : byte
    {
      SingleUse,
      MessageFramed,
    }
  }
}
