﻿// Decompiled with JetBrains decompiler
// Type: ConnectorLib.NamedPipeDuplexConnector
// Assembly: ConnectorLib, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: CC4FDCDA-98EE-4CD3-AF60-DEE32CDD48A0
// Assembly location: C:\Program Files (x86)\EmoTracker\ConnectorLib.dll

using Newtonsoft.Json;
using System;
using System.IO;
using System.IO.Pipes;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ConnectorLib
{
  public class NamedPipeDuplexConnector : ConnectorBase, IGameConnector, IDisposable
  {
    private string _server_name;
    private string _in_pipe_name;
    private string _out_pipe_name;
    private Func<string, string> _message_builder;
    private bool _connected;
    private NamedPipeDuplexConnector.MessageMode _message_mode;
    private volatile bool _quitting;
    private readonly ManualResetEventSlim _listener_event = new ManualResetEventSlim(true);

    public bool Connected
    {
      get
      {
        switch (this._message_mode)
        {
          case NamedPipeDuplexConnector.MessageMode.SingleUse:
            return this._connected;
          case NamedPipeDuplexConnector.MessageMode.MessageFramed:
            throw new NotImplementedException();
          default:
            throw new ArgumentOutOfRangeException();
        }
      }
    }

    protected override void Dispose(bool disposing)
    {
      this._listener_event.Set();
      if (disposing)
        this._listener_event.Dispose();
      this._quitting = true;
      this._connected = false;
      EventHandler<(ConnectionStatus, string)> connectionStatusChanged = this.ConnectionStatusChanged;
      if (connectionStatusChanged != null)
        connectionStatusChanged((object) this, (ConnectionStatus.Closed, string.Empty));
      base.Dispose(disposing);
    }

    public void Init(
      string inPipeName,
      string outPipeName,
      NamedPipeDuplexConnector.MessageMode alwaysReconnect = NamedPipeDuplexConnector.MessageMode.SingleUse)
    {
      this.Init(".", inPipeName, outPipeName, alwaysReconnect);
    }

    public void Init(
      string serverName,
      string inPipeName,
      string outPipeName,
      NamedPipeDuplexConnector.MessageMode messageMode = NamedPipeDuplexConnector.MessageMode.SingleUse,
      Func<string, string> messageBuilder = null)
    {
      this._server_name = serverName;
      this._in_pipe_name = inPipeName;
      this._out_pipe_name = outPipeName;
      this._message_mode = messageMode;
      this._message_builder = messageBuilder;
      Task.Factory.StartNew((Action) (() =>
      {
        while (!this._quitting)
        {
          Task.Run((Action) (() => this.ServerThread(messageMode))).Forget();
          this._listener_event.Reset();
          this._listener_event.Wait();
        }
      }), TaskCreationOptions.LongRunning).Forget();
    }

    private void ServerThread(NamedPipeDuplexConnector.MessageMode mode)
    {
      using (NamedPipeServerStream pipeServerStream = new NamedPipeServerStream(this._in_pipe_name, PipeDirection.InOut, -1))
      {
        pipeServerStream.WaitForConnection();
        this._listener_event.Set();
        using (StreamReader streamReader = new StreamReader((Stream) pipeServerStream, Encoding.UTF8))
        {
          EventHandler<string> onMessage = this.OnMessage;
          if (onMessage != null)
            onMessage((object) this, streamReader.ReadLine());
        }
        pipeServerStream.Close();
      }
    }

    public bool Send(string message) => this.SendRaw(message);

    public bool SendJSON(object message) => this.SendRaw(JsonConvert.SerializeObject(message));

    private bool SendRaw(string message)
    {
      try
      {
        switch (this._message_mode)
        {
          case NamedPipeDuplexConnector.MessageMode.SingleUse:
            using (NamedPipeClientStream pipeClientStream = new NamedPipeClientStream(this._server_name, this._out_pipe_name))
            {
              if (!pipeClientStream.IsConnected)
                pipeClientStream.Connect(1000);
              using (StreamWriter streamWriter = new StreamWriter((Stream) pipeClientStream)
              {
                AutoFlush = true
              })
                streamWriter.WriteLine(message);
              return true;
            }
          case NamedPipeDuplexConnector.MessageMode.MessageFramed:
            throw new NotImplementedException();
          default:
            throw new ArgumentOutOfRangeException();
        }
      }
      catch (Exception ex)
      {
        Log.Error((object) ex);
        return false;
      }
    }

    public bool SendMessage(string message)
    {
      EventHandler<string> onMessage = this.OnMessage;
      if (onMessage != null)
        onMessage((object) this, message);
      return this._message_builder == null || this.SendRaw(this._message_builder(message));
    }

    public void KeepAlive()
    {
    }

    public event EventHandler<(ConnectionStatus, string)> ConnectionStatusChanged;

    public event EventHandler<string> OnMessage;

    public IBatchWriteContext OpenBatchWriteContext() => (IBatchWriteContext) NullContext.Context;

    public enum MessageMode : byte
    {
      SingleUse,
      MessageFramed,
    }
  }
}
