﻿// Decompiled with JetBrains decompiler
// Type: ConnectorLib.NamedPipeServerConnector
// Assembly: ConnectorLib, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: CC4FDCDA-98EE-4CD3-AF60-DEE32CDD48A0
// Assembly location: C:\Program Files (x86)\EmoTracker\ConnectorLib.dll

using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO.Pipes;
using System.Text;
using System.Threading.Tasks;

namespace ConnectorLib
{
  public class NamedPipeServerConnector : ConnectorBase, IGameConnector, IDisposable
  {
    private NamedPipeServerStream _stream;
    private string _pipe_name;
    private Func<string, string> _message_builder;
    private bool _connected;
    private NamedPipeServerConnector.MessageMode _message_mode;

    public bool Connected
    {
      get
      {
        switch (this._message_mode)
        {
          case NamedPipeServerConnector.MessageMode.SingleUse:
            return this._connected;
          case NamedPipeServerConnector.MessageMode.MessageFramed:
            NamedPipeServerStream stream = this._stream;
            return stream != null && stream.IsConnected;
          default:
            throw new ArgumentOutOfRangeException();
        }
      }
    }

    protected override void Dispose(bool disposing)
    {
      if (disposing)
        this._stream?.Dispose();
      base.Dispose(disposing);
    }

    public void Init(string pipeName) => this.Init(pipeName, (Func<string, string>) null);

    public void Init(string pipeName, Func<string, string> messageBuilder = null)
    {
      this._pipe_name = pipeName;
      this._message_builder = messageBuilder;
      this._stream = new NamedPipeServerStream(pipeName, PipeDirection.InOut);
      Task.Factory.StartNew((Action) (() => this.ServerThread(NamedPipeServerConnector.MessageMode.MessageFramed)), TaskCreationOptions.LongRunning).Forget();
    }

    private void ServerThread(NamedPipeServerConnector.MessageMode mode)
    {
      NamedPipeServerStream pipeServerStream = new NamedPipeServerStream(this._pipe_name, PipeDirection.InOut);
      pipeServerStream.WaitForConnection();
      List<byte> byteList = new List<byte>();
      while (mode == NamedPipeServerConnector.MessageMode.MessageFramed && pipeServerStream.IsConnected)
      {
        int num;
        while ((num = pipeServerStream.ReadByte()) > 0)
          byteList.Add(checked ((byte) num));
        EventHandler<string> pipeMessageReceived = this.PipeMessageReceived;
        if (pipeMessageReceived != null)
          pipeMessageReceived((object) this, Encoding.UTF8.GetString(byteList.ToArray()));
      }
      pipeServerStream.Close();
    }

    public event EventHandler<string> PipeMessageReceived;

    public bool Send(string message) => this.SendRaw(message);

    public bool SendJSON(object message) => this.SendRaw(JsonConvert.SerializeObject(message));

    private bool SendRaw(string message)
    {
      try
      {
        if (!this._stream.IsConnected)
          return false;
        byte[] bytes = Encoding.UTF8.GetBytes(message);
        this._stream.Write(bytes, 0, bytes.Length);
        return true;
      }
      catch (Exception ex)
      {
        Log.Error((object) ex);
        return false;
      }
    }

    public bool SendMessage(string message)
    {
      EventHandler<string> onMessage = this.OnMessage;
      if (onMessage != null)
        onMessage((object) this, message);
      return this._message_builder == null || this.SendRaw(this._message_builder(message));
    }

    private void Disconnect()
    {
      if (!this.Connected)
        return;
      try
      {
        this._stream?.Dispose();
      }
      catch
      {
      }
      this._stream = (NamedPipeServerStream) null;
      Log.Message((object) "NamedPipeConnector has disconnected.");
      EventHandler<(ConnectionStatus, string)> connectionStatusChanged = this.ConnectionStatusChanged;
      if (connectionStatusChanged == null)
        return;
      connectionStatusChanged((object) this, (ConnectionStatus.Closed, (string) null));
    }

    public void KeepAlive()
    {
    }

    public event EventHandler<(ConnectionStatus, string)> ConnectionStatusChanged;

    public event EventHandler<string> OnMessage;

    public IBatchWriteContext OpenBatchWriteContext() => (IBatchWriteContext) NullContext.Context;

    public enum MessageMode : byte
    {
      SingleUse,
      MessageFramed,
    }
  }
}
