﻿// Decompiled with JetBrains decompiler
// Type: ConnectorLib.Patches.BPSUnpackStream
// Assembly: ConnectorLib, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: CC4FDCDA-98EE-4CD3-AF60-DEE32CDD48A0
// Assembly location: C:\Program Files (x86)\EmoTracker\ConnectorLib.dll

using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ConnectorLib.Patches
{
    public sealed class BPSUnpackStream : Stream
    {
        private readonly JObject _meta;
        private readonly byte[] _target_bytes;
        private readonly MemoryStream _target;
        private const string UNEXPECTED_EOF = "The BPS data ended unexpectedly.";

        protected override void Dispose(bool disposing)
        {
            if (disposing)
                this._target.Dispose();
            base.Dispose(disposing);
        }

        public BPSUnpackStream(Stream bps, Stream source, bool validate = true)
        {
            CRC32 chkBPS = new CRC32();
            long tRead = 0L;
            bps.Position = 0L;
            source.Position = 0L;
            if (bps.ReadByte(chkBPS) != 66 || bps.ReadByte(chkBPS) != 80 || bps.ReadByte(chkBPS) != 83 || bps.ReadByte(chkBPS) != 49)
            {
                throw new InvalidDataException("The BPS header was not found.");
            }
            ulong sourceSize = BPSUnpackStream.DecodeBPS(bps, chkBPS);
            if (sourceSize > 9223372036854775807UL)
            {
                throw new InvalidDataException("The BPS source size is too large.");
            }
            checked
            {
                if (validate && source.Length != (long)sourceSize)
                {
                    throw new InvalidDataException("The source data is the wrong size.");
                }
                ulong targetSize = BPSUnpackStream.DecodeBPS(bps, chkBPS);
                if (targetSize > 9223372036854775807UL)
                {
                    throw new InvalidDataException("The BPS target size is too large.");
                }
                this._target_bytes = new byte[targetSize];
                this._target = new MemoryStream(this._target_bytes);
                ulong metaLength = BPSUnpackStream.DecodeBPS(bps, chkBPS);
                if (metaLength > 9223372036854775807UL)
                {
                    throw new InvalidDataException("The BPS meta size is too large.");
                }
                if (metaLength > 0UL)
                {
                    byte[] metaArray = new byte[metaLength];
                    bps.Read(metaArray, 0, metaArray.Length, chkBPS);
                    this._meta = JObject.Parse(Encoding.UTF8.GetString(metaArray));
                }
                while (bps.Position < bps.Length - 12L)
                {
                    ulong cVal = BPSUnpackStream.DecodeBPS(bps, chkBPS);
                    ValueTuple<BPSUnpackStream.Command, ulong> action = new ValueTuple<BPSUnpackStream.Command, ulong>((BPSUnpackStream.Command)(cVal & 3UL), (cVal >> 2) + 1UL);
                    switch (action.Item1)
                    {
                        case BPSUnpackStream.Command.SourceRead:
                            {
                                long sp = source.Position;
                                source.Position = this._target.Position;
                                byte[] buf = new byte[action.Item2];
                                source.Read(buf, 0, buf.Length);
                                this._target.Write(buf, 0, buf.Length);
                                source.Position = sp;
                                break;
                            }
                        case BPSUnpackStream.Command.TargetRead:
                            {
                                byte[] buf2 = new byte[action.Item2];
                                bps.Read(buf2, 0, buf2.Length, chkBPS);
                                this._target.Write(buf2, 0, buf2.Length);
                                break;
                            }
                        case BPSUnpackStream.Command.SourceCopy:
                            {
                                source.Seek(BPSUnpackStream.DecodeREL(bps, chkBPS), SeekOrigin.Current);
                                byte[] buf3 = new byte[action.Item2];
                                source.Read(buf3, 0, buf3.Length);
                                this._target.Write(buf3, 0, buf3.Length);
                                break;
                            }
                        case BPSUnpackStream.Command.TargetCopy:
                            tRead += BPSUnpackStream.DecodeREL(bps, chkBPS);
                            for (ulong i = 0UL; i < action.Item2; i += 1UL)
                            {
                                Stream target = this._target;
                                byte[] target_bytes = this._target_bytes;
                                long num = tRead;
                                tRead = num + 1L;
                                target.WriteByte(target_bytes[(int)((IntPtr)num)]);
                            }
                            break;
                    }
                }
                uint sourceChecksum = BPSUnpackStream.ReadUIntThrow(bps, chkBPS);
                if (validate && sourceChecksum != CRC32.Calculate(source))
                {
                    throw new InvalidDataException("The source data checksum did not match.");
                }
                uint targetChecksum = BPSUnpackStream.ReadUIntThrow(bps, chkBPS);
                if (validate)
                {
                    if (targetChecksum != CRC32.Calculate(this._target_bytes))
                    {
                        throw new InvalidDataException("The target data checksum did not match.");
                    }
                    if ((long)targetSize != this._target.Length)
                    {
                        throw new InvalidDataException("The target data size did not match.");
                    }
                }
                this._target.Position = 0L;
                uint patchChecksum = BPSUnpackStream.ReadUIntThrow(bps, null);
                if (patchChecksum != chkBPS.Checksum)
                {
                    throw new InvalidDataException("The BPS checksum did not match.");
                }
            }
        }

        private static long DecodeREL(Stream s, CRC32 checksum)
        {
            ulong bpsNum = BPSUnpackStream.DecodeBPS(s, checksum);
            if (bpsNum > (ulong)long.MaxValue)
            {
                throw new InvalidDataException("A relative offset that is too large was encountered.");
            }
            return checked((((bpsNum & 1UL) != 0UL) ? -1L : 1L) * ((long)bpsNum >> 1));
        }

        private static ulong DecodeBPS(Stream s, CRC32 checksum)
        {
            ulong num1 = 0;
            ulong num2 = 1;
            ulong num3;
            while (true)
            {
                byte num4 = BPSUnpackStream.ReadByteThrow(s, "The BPS data ended unexpectedly.", checksum);
                num3 = checked(num1 + (ulong)((uint)num4 & (uint)sbyte.MaxValue) * num2);
                if (((int)num4 & 128) == 0)
                {
                    num2 <<= 7;
                    num1 = checked(num3 + num2);
                }
                else
                    break;
            }
            return num3;
        }

        private static byte ReadByteThrow(Stream s, string error, CRC32 checksum)
        {
            int r = s.ReadByte();
            if (r < 0)
                throw new InvalidDataException(error);
            checksum?.Append(checked((byte)r));
            return checked((byte)r);
        }

        private static uint ReadUIntThrow(Stream s, CRC32 checksum)
        {
            uint num1 = 0;
            int num2 = s.ReadByte(checksum);
            if (num2 < 0)
                throw new InvalidDataException("The BPS data ended unexpectedly.");
            uint num3 = num1 | checked((uint)num2);
            int num4 = s.ReadByte(checksum);
            if (num4 < 0)
                throw new InvalidDataException("The BPS data ended unexpectedly.");
            uint num5 = num3 | checked((uint)num4) << 8;
            int num6 = s.ReadByte(checksum);
            if (num6 < 0)
                throw new InvalidDataException("The BPS data ended unexpectedly.");
            uint num7 = num5 | checked((uint)num6) << 16;
            int num8 = s.ReadByte(checksum);
            if (num8 < 0)
                throw new InvalidDataException("The BPS data ended unexpectedly.");
            return num7 | checked((uint)num8) << 24;
        }

        public override long Seek(long offset, SeekOrigin origin) => this._target.Seek(offset, origin);

        public override void SetLength(long value) => throw new NotSupportedException();

        public override int Read(byte[] buffer, int offset, int count) => this._target.Read(buffer, offset, count);

        public override bool CanRead => this._target.CanRead;

        public override bool CanSeek => this._target.CanSeek;

        public override bool CanWrite => false;

        public override long Length => this._target.Length;

        public override long Position
        {
            get => this._target.Position;
            set => this._target.Position = value;
        }

        public override void Write(byte[] buffer, int offset, int count) => throw new NotSupportedException();

        public override void Flush() => this._target.Flush();

        public override Task WriteAsync(
          byte[] buffer,
          int offset,
          int count,
          CancellationToken cancellationToken)
        {
            throw new NotSupportedException();
        }

        public override void WriteByte(byte value) => throw new NotSupportedException();

        public void WriteTo(Stream stream) => this._target.WriteTo(stream);

        private enum Command : byte
        {
            SourceRead,
            TargetRead,
            SourceCopy,
            TargetCopy,
        }
    }
}
