﻿// Decompiled with JetBrains decompiler
// Type: ConnectorLib.RESTConnector
// Assembly: ConnectorLib, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: CC4FDCDA-98EE-4CD3-AF60-DEE32CDD48A0
// Assembly location: C:\Program Files (x86)\EmoTracker\ConnectorLib.dll

using Microsoft.CSharp.RuntimeBinder;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Net;
using System.Runtime.CompilerServices;
using System.Threading.Tasks;

namespace ConnectorLib
{
  public class RESTConnector : ConnectorBase, IRESTConnector, IGameConnector, IDisposable
  {
    private WebClient client;
    private WebClient keepAliveClient;
    private bool running = true;

    public bool Connected { get; private set; }

    protected override void Dispose(bool disposing)
    {
      this.running = false;
      if (!disposing)
        return;
      this.client.Dispose();
      this.keepAliveClient.Dispose();
    }

    void IRESTConnector.Init(string baseAddress)
    {
      this.client = new WebClient()
      {
        BaseAddress = baseAddress
      };
      this.keepAliveClient = new WebClient()
      {
        BaseAddress = baseAddress
      };
      Task.Factory.StartNew<Task>((Func<Task>) (async () =>
      {
        while (this.running)
        {
          this.KeepAlive();
          await Task.Delay(TimeSpan.FromSeconds(2.5));
        }
      }));
    }

    private void SetHeaders()
    {
      this.client.Headers["Accept"] = "application/json";
      this.client.Headers["Content-Type"] = "application/json";
    }

    public bool Send(string path) => this.SendJson(path, (string) null);

    public T Send<T>(string path) => this.SendJson<T>(path, (string) null);

    public bool Send<V>(string path, V value) => this.SendJson(path, JsonConvert.SerializeObject((object) value));

    public T Send<T, V>(string path, V value) => this.SendJson<T>(path, JsonConvert.SerializeObject((object) value));

    public bool SendValue<V>(string path, V value) => this.SendValue<RESTConnector.GenericResponse, V>(path, value).result;

    public T SendValue<T, V>(string path, V value)
    {
        // decomp was garbage; best guess - viz
        string json = JsonConvert.SerializeObject(value);
        return this.SendRaw<T>(path, json);
    }

    public bool SendJson(string path, string json) => this.SendRaw<RESTConnector.GenericResponse>(path, json).result;

    public T SendJson<T>(string path, string json) => this.SendRaw<T>(path, json);

    private T SendRaw<T>(string path, string json, WebClient http = null)
    {
      try
      {
        this.SetHeaders();
        WebClient webClient = http ?? this.client;
        string str = json == null ? webClient.DownloadString(path) : webClient.UploadString(path, json);
        try
        {
          return JsonConvert.DeserializeObject<T>(str);
        }
        catch (Exception ex)
        {
          Log.Error(ex, "REST Connector failed to deserialize response.");
          return default (T);
        }
      }
      catch
      {
        this.SetDisconnected();
        return default (T);
      }
    }

    bool IGameConnector.SendMessage(string message)
    {
      EventHandler<string> onMessage = this.OnMessage;
      if (onMessage != null)
        onMessage((object) this, message);
      return message != null && this.Send<string>(nameof (message), message);
    }

    [DebuggerStepThrough]
    public void KeepAlive() => this.SendRaw<RESTConnector.KeepAliveResponse>("keepalive", "{}", this.keepAliveClient);

    public void SetConnected()
    {
      if (this.Connected)
        return;
      this.Connected = true;
      this.InvokeOnReady();
      EventHandler<(ConnectionStatus, string)> connectionStatusChanged = this.ConnectionStatusChanged;
      if (connectionStatusChanged == null)
        return;
      connectionStatusChanged((object) this, (ConnectionStatus.Open, (string) null));
    }

    public void SetDisconnected()
    {
      if (!this.Connected)
        return;
      this.Connected = false;
      EventHandler<(ConnectionStatus, string)> connectionStatusChanged = this.ConnectionStatusChanged;
      if (connectionStatusChanged == null)
        return;
      connectionStatusChanged((object) this, (ConnectionStatus.Closed, (string) null));
    }

    public event EventHandler<(ConnectionStatus, string)> ConnectionStatusChanged;

    public event EventHandler<string> OnMessage;

    public IBatchWriteContext OpenBatchWriteContext() => (IBatchWriteContext) NullContext.Context;

    protected class GenericResponse
    {
      public bool result;
    }

    private class KeepAliveResponse
    {
    }
  }
}
