﻿// Decompiled with JetBrains decompiler
// Type: ConnectorLib.SNESMapper
// Assembly: ConnectorLib, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: CC4FDCDA-98EE-4CD3-AF60-DEE32CDD48A0
// Assembly location: C:\Program Files (x86)\EmoTracker\ConnectorLib.dll

using System;
using System.Collections.Generic;

namespace ConnectorLib
{
  [Serializable]
  public class SNESMapper : IMemoryMapper
  {
    public bool MAD1 { get; set; }

    public bool SFX { get; set; }

    public SNESMapper.MapperType Type { get; }

    public SNESMapper(SNESMapper.MapperType type) => this.Type = type;

    public SNESMapper(byte type)
    {
      switch ((uint) type & 15U)
      {
        case 0:
        case 2:
        case 3:
          this.Type = SNESMapper.MapperType.LoROM;
          break;
        case 1:
          this.Type = SNESMapper.MapperType.HiROM;
          break;
        case 5:
        case 10:
          this.Type = SNESMapper.MapperType.ExHiROM;
          break;
      }
    }

    public (ulong address, string domain) Translate(ulong address)
    {
      switch (this.Type)
      {
        case SNESMapper.MapperType.LoROM:
          return this.TranslateLoROM(address);
        case SNESMapper.MapperType.HiROM:
          return this.TranslateHiROM(address);
        case SNESMapper.MapperType.ExLoROM:
          return this.TranslateLoROM(address, true);
        case SNESMapper.MapperType.ExHiROM:
          return this.TranslateHiROM(address, true);
        default:
          throw new ArgumentOutOfRangeException();
      }
    }

    public (ulong address, string domain) TranslateHiROM(ulong address, bool ex = false)
    {
      byte num1 = (byte) (address >> 16);
      ushort num2 = (ushort) address;
      if ((num1 < (byte) 63 || num1 >= (byte) 128 && num1 <= (byte) 191) && num2 <= (ushort) 8191)
        return (address & 131071UL, "WRAM");
      if (num1 >= (byte) 126 && num1 <= (byte) 127)
        return (address & 8519679UL, "WRAM");
      if (num1 >= (byte) 254)
        return (checked (address + (ulong) (65536U * ((uint) num1 - 254U)) + 4063232UL), "CARTROM");
      if ((num1 >= (byte) 32 && num1 <= (byte) 63 || num1 >= (byte) 160 && num1 <= (byte) 191) && num2 >= (ushort) 24576 && num2 <= (ushort) short.MaxValue)
        return (checked (address & 8191UL + (ulong) (8192U * ((uint) num1 & 63U))), "CARTRAM");
      if (ex)
      {
        if (num1 <= (byte) 63 && num2 >= (ushort) 32768)
          return (checked ((address & (ulong) short.MaxValue) + (ulong) (32768U * (uint) num1) + 4194304UL), "CARTROM");
        if (num1 >= (byte) 128 && num1 <= (byte) 191 && num2 >= (ushort) 32768)
          return (checked (address & (ulong) short.MaxValue + (ulong) (32768U * ((uint) num1 - 128U))), "CARTROM");
        if (num1 >= (byte) 64 && num1 <= (byte) 125)
          return (checked (address + (ulong) (65536U * ((uint) num1 - 64U) + 4194304U)), "CARTROM");
        if (num1 >= (byte) 192 && num1 <= (byte) 253)
          return (checked (address + (ulong) (65536U * ((uint) num1 - 192U))), "CARTROM");
      }
      else
      {
        if (num1 <= (byte) 63 && num2 >= (ushort) 32768)
          return (checked (address & (ulong) short.MaxValue + (ulong) (32768U * (uint) num1)), "CARTROM");
        if (num1 >= (byte) 64 && num1 <= (byte) 125)
          return (checked (address + (ulong) (65536U * ((uint) num1 - 64U))), "CARTROM");
        if (num1 >= (byte) 254)
          return (checked (address + (ulong) (65536U * ((uint) num1 - 64U)) + 4063232UL), "CARTROM");
      }
      throw new ArgumentOutOfRangeException(nameof (address));
    }

    public (ulong address, string domain) TranslateLoROM(ulong address, bool ex = false)
    {
      if (ex)
        throw new NotImplementedException("ExLoROM is not supported.");
      byte num1 = (byte) (address >> 16);
      ushort num2 = (ushort) address;
      if ((num1 < (byte) 63 || num1 >= (byte) 128 && num1 <= (byte) 191) && num2 <= (ushort) 8191)
        return (address & 131071UL, "WRAM");
      if (num1 >= (byte) 126 && num1 <= (byte) 127)
        return (address & 8519679UL, "WRAM");
      if ((num1 < (byte) 63 || num1 >= (byte) 128 && num1 <= (byte) 191) && num2 >= (ushort) 24576 && num2 <= (ushort) short.MaxValue)
        return (address & 8191UL, "CARTRAM");
      if (num2 >= (ushort) 32768 && num1 != (byte) 126 && num1 != (byte) 127)
        return ((ulong) checked ((uint) num2 & (uint) short.MaxValue + 32768U * (uint) num1), "CARTROM");
      if (!this.MAD1 && num1 >= (byte) 64 && num1 <= (byte) 111)
        return ((ulong) checked ((uint) num2 & (uint) short.MaxValue + 32768U * (uint) num1), "CARTROM");
      if (num1 >= (byte) 112 && num1 <= (byte) 125)
        return ((ulong) checked ((uint) num2 & (uint) short.MaxValue + 32768U * ((uint) num1 - 112U)), "CARTRAM");
      return num1 == (byte) 254 || num1 == byte.MaxValue ? ((ulong) checked ((uint) num2 & (uint) short.MaxValue + 32768U * ((uint) num1 - 254U)), "CARTRAM") : throw new ArgumentOutOfRangeException(nameof (address));
    }

    public ulong Translate(ulong address, string domain)
    {
      switch (this.Type)
      {
        case SNESMapper.MapperType.LoROM:
          return this.TranslateLoROM(address, domain);
        case SNESMapper.MapperType.HiROM:
          throw new NotImplementedException();
        case SNESMapper.MapperType.ExLoROM:
          throw new NotImplementedException();
        case SNESMapper.MapperType.ExHiROM:
          throw new NotImplementedException();
        default:
          throw new ArgumentOutOfRangeException();
      }
    }

    public ulong TranslateLoROM(ulong address, string domain)
    {
      switch (domain)
      {
        case "System Bus":
          return address;
        case "WRAM":
          return address | 7340032UL;
        case "CARTRAM":
          return address | (this.SFX ? 8257536UL : 24576UL);
        case "CARTROM":
          return address | (this.SFX ? 8257536UL : 24576UL);
        default:
          throw new ArgumentOutOfRangeException(nameof (domain));
      }
    }

    public IEnumerable<string> Domains => (IEnumerable<string>) new string[7]
    {
      "WRAM",
      "CARTROM",
      "CARTRAM",
      "VRAM",
      "OAM",
      "CGRAM",
      "APURAM"
    };

    public enum MapperType : byte
    {
      LoROM = 1,
      HiROM = 2,
      ExLoROM = 17, // 0x11
      ExHiROM = 18, // 0x12
    }
  }
}
