﻿// Decompiled with JetBrains decompiler
// Type: ConnectorLib.SimpleTCPConnector
// Assembly: ConnectorLib, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: CC4FDCDA-98EE-4CD3-AF60-DEE32CDD48A0
// Assembly location: C:\Program Files (x86)\EmoTracker\ConnectorLib.dll

using Newtonsoft.Json;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ConnectorLib
{
  public class SimpleTCPConnector : ConnectorBase, IGameConnector, IDisposable
  {
    private TcpListener _server;
    private readonly ConcurrentDictionary<TcpClient, NetworkStream> _clients = new ConcurrentDictionary<TcpClient, NetworkStream>();
    private volatile bool _quitting;
    private readonly Task _listener;
    private readonly ManualResetEventSlim _start_wait = new ManualResetEventSlim(false);
    private SimpleTCPConnector.MessageType _message_type;
    private SimpleTCPConnector.FramingType _framing_type;

    private bool HasClients => !this._clients.IsEmpty;

    protected override void Dispose(bool disposing)
    {
      this._quitting = true;
      try
      {
        this._server.Stop();
      }
      catch
      {
      }
      this.CloseAllSockets();
      base.Dispose(disposing);
    }

    private void CloseAllSockets()
    {
      foreach (TcpClient key in (IEnumerable<TcpClient>) this._clients.Keys)
      {
        try
        {
          key.Close();
        }
        catch (Exception ex)
        {
          Log.Error((object) ex);
        }
        this._clients.TryRemove(key, out NetworkStream _);
      }
    }

    public SimpleTCPConnector() => this._listener = Task.Factory.StartNew(new Action(this.Listen), TaskCreationOptions.LongRunning);

    public void Init(
      string host,
      ushort port,
      SimpleTCPConnector.MessageType messageType,
      SimpleTCPConnector.FramingType framingType)
    {
      this._message_type = messageType;
      this._framing_type = framingType;
      this._server = new TcpListener(IPAddress.Parse(host), (int) port);
      this._server.Server.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);
      this._server.Start();
      this._start_wait.Set();
      this.InvokeOnReady();
    }

    private void Listen()
    {
      this._start_wait.Wait();
      while (!this._quitting)
      {
        Log.Debug((object) "Service is listening for incoming connections...");
        TcpClient client = this._server.AcceptTcpClient();
        Log.Debug((object) string.Format("Starting message handler for game {0}.", (object) client.GetHashCode()));
        Task.Run((Action) (() =>
        {
          Log.Debug((object) string.Format("Game {0} connected.", (object) client.GetHashCode()));
          try
          {
            EventHandler<(ConnectionStatus, string)> connectionStatusChanged = this.ConnectionStatusChanged;
            if (connectionStatusChanged != null)
              connectionStatusChanged((object) this, (ConnectionStatus.Open, string.Empty));
            NetworkStream stream = client.GetStream();
            this._clients[client] = stream;
            StringBuilder stringBuilder = new StringBuilder();
            using (StreamReader streamReader = new StreamReader((Stream) stream))
            {
              while (client.Connected)
              {
                int num;
                while ((num = streamReader.Read()) > 0)
                  stringBuilder.Append(checked ((char) num));
                if (num == 0 && stringBuilder.Length > 0)
                {
                  Log.Debug((object) string.Format("Got a message from game {0}.", (object) client.GetHashCode()));
                  string e = stringBuilder.ToString();
                  EventHandler<string> onMessage = this.OnMessage;
                  if (onMessage != null)
                    onMessage((object) this, e);
                  EventHandler<SimpleTCPConnector.Response> onMessageParsed = this.OnMessageParsed;
                  if (onMessageParsed != null)
                    onMessageParsed((object) this, JsonConvert.DeserializeObject<SimpleTCPConnector.Response>(e));
                  stringBuilder.Clear();
                }
              }
            }
          }
          catch
          {
            Log.Debug((object) string.Format("Game {0}'s socket threw an exception.", (object) client.GetHashCode()));
          }
          finally
          {
            Log.Debug((object) string.Format("Game {0} was disconnected.", (object) client.GetHashCode()));
            client.Dispose();
            this._clients.TryRemove(client, out NetworkStream _);
            if (this._clients.IsEmpty)
            {
              EventHandler<(ConnectionStatus, string)> connectionStatusChanged = this.ConnectionStatusChanged;
              if (connectionStatusChanged != null)
                connectionStatusChanged((object) this, (ConnectionStatus.Closed, string.Empty));
            }
          }
        })).Forget();
      }
    }

    public bool Send(SimpleTCPConnector.Request req)
    {
      if (this._message_type == SimpleTCPConnector.MessageType.JSON)
        return this.SendJSON(req);
      throw new ArgumentOutOfRangeException();
    }

    private bool SendJSON(SimpleTCPConnector.Request req)
    {
      if (!this.HasClients)
        return false;
      bool flag = false;
      byte[] bytes = Encoding.UTF8.GetBytes(JsonConvert.SerializeObject((object) req));
      foreach (TcpClient key in (IEnumerable<TcpClient>) this._clients.Keys)
      {
        try
        {
          NetworkStream stream = key.GetStream();
          if (stream.CanWrite)
          {
            lock (stream)
            {
              stream.Write(bytes, 0, bytes.Length);
              stream.WriteByte((byte) 0);
            }
            flag = true;
            Log.Debug((object) string.Format("Sent message {0} successfully to game {1}.", (object) req.id, (object) key.GetHashCode()));
          }
        }
        catch
        {
          Log.Debug((object) string.Format("Failed to send message {0} to game {1}.", (object) req.id, (object) key.GetHashCode()));
        }
      }
      return flag;
    }

    public IBatchWriteContext OpenBatchWriteContext() => (IBatchWriteContext) new NullContext();

    public bool SendMessage(string message) => false;

    public event EventHandler<string> OnMessage;

    public event EventHandler<SimpleTCPConnector.Response> OnMessageParsed;

    public bool Connected => this._clients.Keys.Any<TcpClient>((Func<TcpClient, bool>) (c => c.Connected));

    public void KeepAlive()
    {
    }

    public event EventHandler<(ConnectionStatus, string)> ConnectionStatusChanged;

    public enum MessageType
    {
      JSON,
    }

    public enum FramingType
    {
      Null,
    }

    public enum RequestType
    {
      Test,
      Start,
      Stop,
    }

    [Serializable]
    public class Request
    {
      private static int _next_id;
      public int id = Interlocked.Increment(ref SimpleTCPConnector.Request._next_id);
      public string code;
      public string viewer;
      public SimpleTCPConnector.RequestType type;
    }

    [Serializable]
    public class Response
    {
      public int id;
      public SimpleTCPConnector.EffectResult status;
      public string message;
    }

    public enum EffectResult
    {
      Success,
      Failure,
      Unavailable,
      Retry,
      Queue,
      Running,
    }
  }
}
