﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Core.ObservableCollectionAggregatorSynchronizer`2
// Assembly: EmoTracker.Core, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: 4F6DD65D-7348-46EE-83CB-CEA4A6F85CDC
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Core.dll

using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Linq;

namespace EmoTracker.Core
{
  public class ObservableCollectionAggregatorSynchronizer<SrcType, DstType> : IDisposable
  {
    private List<ObservableCollectionAggregatorSynchronizer<SrcType, DstType>.ListSegment> Segments = new List<ObservableCollectionAggregatorSynchronizer<SrcType, DstType>.ListSegment>();
    private ObservableCollection<DstType> mDest;
    private Func<SrcType, DstType> mItemConverter;

    public bool EnableDispose { get; set; }

    public ObservableCollectionAggregatorSynchronizer(
      ObservableCollection<DstType> dest,
      Func<SrcType, DstType> itemConverter,
      params IReadOnlyCollection<SrcType>[] sources)
    {
      this.mDest = dest;
      this.mItemConverter = itemConverter;
      foreach (IReadOnlyCollection<SrcType> source in sources)
        this.AddSource(source);
    }

    public void AddSource(IReadOnlyCollection<SrcType> source)
    {
      if (source == null)
        return;
      ObservableCollectionAggregatorSynchronizer<SrcType, DstType>.ListSegment listSegment = new ObservableCollectionAggregatorSynchronizer<SrcType, DstType>.ListSegment(source, this.mDest.Count);
      this.Segments.Add(listSegment);
      foreach (SrcType srcItem in (IEnumerable<SrcType>) source)
        this.AddItem(source, srcItem);
      listSegment.SourceChangeTracking.CollectionChanged += new NotifyCollectionChangedEventHandler(this.SourceCollectionChanged);
    }

    public void RemoveSource(IReadOnlyCollection<SrcType> source)
    {
      foreach (SrcType srcType in (IEnumerable<SrcType>) source)
        this.RemoveItem(source, 0, srcType);
      foreach (ObservableCollectionAggregatorSynchronizer<SrcType, DstType>.ListSegment segment in this.Segments)
      {
        if (segment.Source == source)
        {
          segment.SourceChangeTracking.CollectionChanged -= new NotifyCollectionChangedEventHandler(this.SourceCollectionChanged);
          this.Segments.Remove(segment);
          break;
        }
      }
    }

    private void OffsetSegmentBaseIndices(int startingSegmentIdx, int offset)
    {
      for (int index = startingSegmentIdx; index < this.Segments.Count; ++index)
        this.Segments[index].BaseIndex += offset;
    }

    private void AddItem(IReadOnlyCollection<SrcType> source, SrcType srcItem)
    {
      for (int index = 0; index < this.Segments.Count; ++index)
      {
        if (this.Segments[index].Source == source)
        {
          DstType dstType = this.mItemConverter(srcItem);
          if (index < this.Segments.Count - 1)
          {
            this.mDest.Insert(this.Segments[index + 1].BaseIndex, dstType);
            this.OffsetSegmentBaseIndices(index + 1, 1);
          }
          else
            this.mDest.Add(dstType);
        }
      }
    }

    private void InsertItem(IReadOnlyCollection<SrcType> source, int index, SrcType srcItem)
    {
      for (int index1 = 0; index1 < this.Segments.Count; ++index1)
      {
        ObservableCollectionAggregatorSynchronizer<SrcType, DstType>.ListSegment segment = this.Segments[index1];
        if (segment.Source == source)
        {
          DstType dstType = this.mItemConverter(srcItem);
          int index2 = segment.BaseIndex + index;
          if (index2 < this.mDest.Count)
            this.mDest.Insert(index2, dstType);
          else
            this.mDest.Add(dstType);
          this.OffsetSegmentBaseIndices(index1 + 1, 1);
        }
      }
    }

    protected void DisposeCreatedItem(DstType item)
    {
      if (!(item is IDisposable disposable) || !this.EnableDispose)
        return;
      disposable.Dispose();
    }

    private void RemoveItem(IReadOnlyCollection<SrcType> source, int idx, SrcType item)
    {
      for (int index = 0; index < this.Segments.Count; ++index)
      {
        ObservableCollectionAggregatorSynchronizer<SrcType, DstType>.ListSegment segment = this.Segments[index];
        if (segment.Source == source)
        {
          DstType dstType = this.mDest.ElementAt<DstType>(segment.BaseIndex + idx);
          this.mDest.RemoveAt(segment.BaseIndex + idx);
          this.OffsetSegmentBaseIndices(index + 1, -1);
          this.DisposeCreatedItem(dstType);
        }
      }
    }

    private void MoveItem(IReadOnlyCollection<SrcType> source, int fromIdx, int toIdx)
    {
      for (int index = 0; index < this.Segments.Count; ++index)
      {
        ObservableCollectionAggregatorSynchronizer<SrcType, DstType>.ListSegment segment = this.Segments[index];
        if (segment.Source == source)
          this.mDest.Move(fromIdx + segment.BaseIndex, toIdx + segment.BaseIndex);
      }
    }

    private void Clear(IReadOnlyCollection<SrcType> source)
    {
      for (int index = 0; index < this.Segments.Count; ++index)
      {
        ObservableCollectionAggregatorSynchronizer<SrcType, DstType>.ListSegment segment1 = this.Segments[index];
        if (segment1.Source == source)
        {
          if (index < this.Segments.Count - 1)
          {
            ObservableCollectionAggregatorSynchronizer<SrcType, DstType>.ListSegment segment2 = this.Segments[index + 1];
            while (segment2.BaseIndex > segment1.BaseIndex)
            {
              this.mDest.RemoveAt(segment1.BaseIndex);
              this.OffsetSegmentBaseIndices(index + 1, -1);
            }
          }
          else
          {
            while (this.mDest.Count != segment1.BaseIndex)
              this.mDest.RemoveAt(this.mDest.Count - 1);
          }
        }
      }
    }

    private void SourceCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
    {
      IReadOnlyCollection<SrcType> source = sender as IReadOnlyCollection<SrcType>;
      if (e.Action == NotifyCollectionChangedAction.Add)
      {
        int newStartingIndex = e.NewStartingIndex;
        foreach (SrcType newItem in (IEnumerable) e.NewItems)
        {
          this.InsertItem(source, newStartingIndex, newItem);
          ++newStartingIndex;
        }
      }
      else if (e.Action == NotifyCollectionChangedAction.Remove)
      {
        int oldStartingIndex = e.OldStartingIndex;
        foreach (SrcType oldItem in (IEnumerable) e.OldItems)
        {
          this.RemoveItem(source, oldStartingIndex, oldItem);
          ++oldStartingIndex;
        }
      }
      else if (e.Action == NotifyCollectionChangedAction.Move)
      {
        this.MoveItem(source, e.OldStartingIndex, e.NewStartingIndex);
      }
      else
      {
        if (e.Action != NotifyCollectionChangedAction.Reset)
          return;
        this.Clear(source);
      }
    }

    public void Dispose()
    {
      foreach (ObservableCollectionAggregatorSynchronizer<SrcType, DstType>.ListSegment listSegment in this.Segments.ToArray())
        this.RemoveSource(listSegment.Source);
    }

    private class ListSegment
    {
      public IReadOnlyCollection<SrcType> Source;
      public INotifyCollectionChanged SourceChangeTracking;
      public int BaseIndex;

      public ListSegment(IReadOnlyCollection<SrcType> source, int baseIdx)
      {
        this.Source = source;
        this.SourceChangeTracking = (INotifyCollectionChanged) this.Source;
        this.BaseIndex = baseIdx;
      }
    }
  }
}
