﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Core.ObservableCollectionSynchronizer`2
// Assembly: EmoTracker.Core, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: 4F6DD65D-7348-46EE-83CB-CEA4A6F85CDC
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Core.dll

using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;

namespace EmoTracker.Core
{
  public class ObservableCollectionSynchronizer<SrcType, DstType> : IDisposable where DstType : class
  {
    private IReadOnlyCollection<SrcType> mSource;
    private INotifyCollectionChanged mSourceChangeTracking;
    private ObservableCollection<DstType> mDest;
    private Func<SrcType, DstType> mItemConverter;
    private Action<DstType> mItemDisposer;

    public static B GenericConstructorConverter<A, B>(A src) where B : class
    {
      try
      {
        return Activator.CreateInstance(typeof (B), (object) src) as B;
      }
      catch
      {
      }
      return default (B);
    }

    public bool EnableDispose { get; set; }

    public ObservableCollectionSynchronizer(
      IReadOnlyCollection<SrcType> source,
      ObservableCollection<DstType> dest,
      Func<SrcType, DstType> itemConverter = null,
      Action<DstType> itemDisposer = null)
    {
      this.mSource = source;
      this.mSourceChangeTracking = this.mSource as INotifyCollectionChanged;
      this.mDest = dest;
      this.mItemConverter = itemConverter ?? new Func<SrcType, DstType>(ObservableCollectionSynchronizer<SrcType, DstType>.GenericConstructorConverter<SrcType, DstType>);
      this.mItemDisposer = itemDisposer;
      this.EnableDispose = true;
      if (this.mSource == null)
        return;
      this.InitializeCollection();
    }

    protected void AddItem(SrcType srcItem) => this.mDest.Add(this.mItemConverter(srcItem));

    protected void InsertItem(int index, SrcType srcItem)
    {
      DstType dstType = this.mItemConverter(srcItem);
      this.mDest.Insert(index, dstType);
    }

    protected void RemoveItems(int index, int count)
    {
      for (int index1 = 0; index1 < count; ++index1)
      {
        DstType dstType = this.mDest[index];
        this.mDest.RemoveAt(index);
        this.DisposeCreatedItem(dstType);
      }
    }

    protected void RemoveAllItems()
    {
      foreach (DstType dstType in (Collection<DstType>) this.mDest)
        this.DisposeCreatedItem(dstType);
      this.mDest.Clear();
    }

    protected void MoveItem(int fromIndex, int toIndex) => this.mDest.Move(fromIndex, toIndex);

    protected void ReplaceItem(int index, SrcType newSrcItem)
    {
      this.DisposeCreatedItem(this.mDest[index]);
      this.mDest[index] = this.mItemConverter(newSrcItem);
    }

    protected void DisposeCreatedItem(DstType item)
    {
      if (this.mItemDisposer != null)
        this.mItemDisposer(item);
      if (!(item is IDisposable disposable) || !this.EnableDispose)
        return;
      disposable.Dispose();
    }

    private void InitializeCollection()
    {
      foreach (SrcType srcItem in (IEnumerable<SrcType>) this.mSource)
        this.AddItem(srcItem);
      if (this.mSourceChangeTracking == null)
        return;
      this.mSourceChangeTracking.CollectionChanged += new NotifyCollectionChangedEventHandler(this.SourceCollectionChanged);
    }

    protected virtual void SourceCollectionChanged(
      object sender,
      NotifyCollectionChangedEventArgs e)
    {
      if (this.mSource == null || this.mDest == null || this.mItemConverter == null)
        return;
      if (e.Action == NotifyCollectionChangedAction.Add)
      {
        int newStartingIndex = e.NewStartingIndex;
        foreach (SrcType newItem in (IEnumerable) e.NewItems)
        {
          this.InsertItem(newStartingIndex, newItem);
          ++newStartingIndex;
        }
      }
      else if (e.Action == NotifyCollectionChangedAction.Remove)
        this.RemoveItems(e.OldStartingIndex, e.OldItems.Count);
      else if (e.Action == NotifyCollectionChangedAction.Move)
        this.MoveItem(e.OldStartingIndex, e.NewStartingIndex);
      else if (e.Action == NotifyCollectionChangedAction.Replace)
      {
        int newStartingIndex = e.NewStartingIndex;
        foreach (SrcType newItem in (IEnumerable) e.NewItems)
        {
          this.ReplaceItem(newStartingIndex, newItem);
          ++newStartingIndex;
        }
      }
      else
      {
        if (e.Action != NotifyCollectionChangedAction.Reset)
          throw new NotImplementedException();
        this.RemoveAllItems();
      }
    }

    public virtual void Dispose()
    {
      if (this.mSourceChangeTracking != null)
        this.mSourceChangeTracking.CollectionChanged -= new NotifyCollectionChangedEventHandler(this.SourceCollectionChanged);
      foreach (DstType dstType in (Collection<DstType>) this.mDest)
        this.DisposeCreatedItem(dstType);
      this.mDest.Clear();
      this.mSourceChangeTracking = (INotifyCollectionChanged) null;
      this.mDest = (ObservableCollection<DstType>) null;
      this.mSource = (IReadOnlyCollection<SrcType>) null;
      this.mItemConverter = (Func<SrcType, DstType>) null;
      this.mItemDisposer = (Action<DstType>) null;
    }
  }
}
