﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Core.ObservableObject
// Assembly: EmoTracker.Core, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: 4F6DD65D-7348-46EE-83CB-CEA4A6F85CDC
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Core.dll

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Reflection;
using System.Runtime.CompilerServices;

namespace EmoTracker.Core
{
  public abstract class ObservableObject : 
    INotifyPropertyChanged,
    INotifyPropertyChanging,
    IDisposable
  {
    public event PropertyChangedEventHandler PropertyChanged;

    public event PropertyChangingEventHandler PropertyChanging;

    private void NotifyDependentProperties(
      PropertyChangedEventHandler handler,
      Type hostType,
      string propertyName)
    {
      PropertyInfo property = hostType.GetProperty(propertyName, BindingFlags.Instance | BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic);
      if (!(property != (PropertyInfo) null))
        return;
      foreach (Attribute customAttribute in Attribute.GetCustomAttributes((MemberInfo) property, typeof (DependentPropertyAttribute)))
      {
        handler((object) this, new PropertyChangedEventArgs((customAttribute as DependentPropertyAttribute).Property));
        this.NotifyDependentProperties(handler, hostType, (customAttribute as DependentPropertyAttribute).Property);
      }
    }

    private void NotifyDependentProperties(
      PropertyChangingEventHandler handler,
      Type hostType,
      string propertyName)
    {
      PropertyInfo property = hostType.GetProperty(propertyName, BindingFlags.Instance | BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic);
      if (!(property != (PropertyInfo) null))
        return;
      foreach (Attribute customAttribute in Attribute.GetCustomAttributes((MemberInfo) property, typeof (DependentPropertyAttribute)))
      {
        handler((object) this, new PropertyChangingEventArgs((customAttribute as DependentPropertyAttribute).Property));
        this.NotifyDependentProperties(handler, hostType, (customAttribute as DependentPropertyAttribute).Property);
      }
    }

    protected virtual void NotifyPropertyChanged([CallerMemberName] string propertyName = null)
    {
      PropertyChangedEventHandler propertyChanged = this.PropertyChanged;
      if (propertyChanged == null)
        return;
      propertyChanged((object) this, new PropertyChangedEventArgs(propertyName));
      this.NotifyDependentProperties(propertyChanged, this.GetType(), propertyName);
    }

    protected virtual void NotifyPropertyChanging([CallerMemberName] string propertyName = null)
    {
      PropertyChangingEventHandler propertyChanging = this.PropertyChanging;
      if (propertyChanging == null)
        return;
      propertyChanging((object) this, new PropertyChangingEventArgs(propertyName));
      this.NotifyDependentProperties(propertyChanging, this.GetType(), propertyName);
    }

    protected bool SetProperty<T>(ref T field, T value, [CallerMemberName] string propertyName = null)
    {
      if (EqualityComparer<T>.Default.Equals(field, value))
        return false;
      this.NotifyPropertyChanging(propertyName);
      field = value;
      this.NotifyPropertyChanged(propertyName);
      return true;
    }

    protected void ForceSetProperty<T>(ref T field, T value, [CallerMemberName] string propertyName = null)
    {
      this.NotifyPropertyChanging(propertyName);
      field = value;
      this.NotifyPropertyChanged(propertyName);
    }

    public void ForceRefreshProperty(string propertyName = null) => this.NotifyPropertyChanged(propertyName);

    public virtual void Dispose()
    {
    }

    protected void DisposeObject(object obj)
    {
      if (!(obj is IDisposable disposable))
        return;
      disposable.Dispose();
    }

    protected void DisposeObjectAndDefault<T>(ref T obj)
    {
      try
      {
        if (!(obj is IDisposable disposable))
          return;
        disposable.Dispose();
      }
      catch
      {
      }
      finally
      {
        obj = default (T);
      }
    }

    protected void DisposeCollection<T>(IEnumerable<T> collection)
    {
      foreach (T obj in collection)
      {
        if (obj is IDisposable disposable)
          disposable.Dispose();
      }
    }
  }
}
