﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Core.ObservableValueCollectionSynchronizer`1
// Assembly: EmoTracker.Core, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: 4F6DD65D-7348-46EE-83CB-CEA4A6F85CDC
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Core.dll

using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;

namespace EmoTracker.Core
{
  public class ObservableValueCollectionSynchronizer<T> : IDisposable
  {
    private IReadOnlyCollection<Observable<T>> mSource;
    private INotifyCollectionChanged mSourceChangeTracking;
    private ObservableCollection<T> mDest;
    private Action<T> mItemDisposer;

    public bool EnableDispose { get; set; }

    public ObservableValueCollectionSynchronizer(
      IReadOnlyCollection<Observable<T>> source,
      ObservableCollection<T> dest,
      Action<T> itemDisposer = null)
    {
      this.mSource = source;
      this.mSourceChangeTracking = (INotifyCollectionChanged) this.mSource;
      this.mDest = dest;
      this.mItemDisposer = itemDisposer;
      this.EnableDispose = true;
      if (this.mSource == null)
        return;
      this.InitializeCollection();
    }

    protected void AddItem(Observable<T> srcItem)
    {
      srcItem.PropertyChanged += new PropertyChangedEventHandler(this.srcItem_PropertyChanged);
      this.mDest.Add(srcItem.Value);
    }

    private void srcItem_PropertyChanged(object sender, PropertyChangedEventArgs e)
    {
      int index = 0;
      foreach (Observable<T> observable in (IEnumerable<Observable<T>>) this.mSource)
      {
        if (observable == (Observable<T>) sender)
        {
          this.mDest[index] = observable.Value;
          break;
        }
        ++index;
      }
    }

    protected void InsertItem(int index, Observable<T> srcItem)
    {
      srcItem.PropertyChanged += new PropertyChangedEventHandler(this.srcItem_PropertyChanged);
      this.mDest.Insert(index, srcItem.Value);
    }

    protected void RemoveItems(int index, int count)
    {
      for (int index1 = 0; index1 < count; ++index1)
      {
        T obj = this.mDest[index];
        this.mDest.RemoveAt(index);
        this.DisposeCreatedItem(obj);
      }
    }

    protected void RemoveAllItems()
    {
      foreach (T obj in (Collection<T>) this.mDest)
        this.DisposeCreatedItem(obj);
      this.mDest.Clear();
    }

    protected void MoveItem(int fromIndex, int toIndex) => this.mDest.Move(fromIndex, toIndex);

    protected void DisposeCreatedItem(T item)
    {
      if (this.mItemDisposer != null)
        this.mItemDisposer(item);
      if (!(item is IDisposable disposable) || !this.EnableDispose)
        return;
      disposable.Dispose();
    }

    private void InitializeCollection()
    {
      foreach (Observable<T> srcItem in (IEnumerable<Observable<T>>) this.mSource)
        this.AddItem(srcItem);
      this.mSourceChangeTracking.CollectionChanged += new NotifyCollectionChangedEventHandler(this.SourceCollectionChanged);
    }

    protected virtual void SourceCollectionChanged(
      object sender,
      NotifyCollectionChangedEventArgs e)
    {
      if (e.Action == NotifyCollectionChangedAction.Add)
      {
        int newStartingIndex = e.NewStartingIndex;
        foreach (Observable<T> newItem in (IEnumerable) e.NewItems)
        {
          this.InsertItem(newStartingIndex, newItem);
          ++newStartingIndex;
        }
      }
      else if (e.Action == NotifyCollectionChangedAction.Remove)
        this.RemoveItems(e.OldStartingIndex, e.OldItems.Count);
      else if (e.Action == NotifyCollectionChangedAction.Move)
      {
        this.MoveItem(e.OldStartingIndex, e.NewStartingIndex);
      }
      else
      {
        if (e.Action != NotifyCollectionChangedAction.Reset)
          throw new NotImplementedException();
        this.RemoveAllItems();
      }
    }

    public void Dispose()
    {
      if (this.mSourceChangeTracking != null)
        this.mSourceChangeTracking.CollectionChanged -= new NotifyCollectionChangedEventHandler(this.SourceCollectionChanged);
      foreach (T obj in (Collection<T>) this.mDest)
        this.DisposeCreatedItem(obj);
      this.mDest.Clear();
      this.mSourceChangeTracking = (INotifyCollectionChanged) null;
      this.mDest = (ObservableCollection<T>) null;
      this.mSource = (IReadOnlyCollection<Observable<T>>) null;
      this.mItemDisposer = (Action<T>) null;
    }
  }
}
