﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.Core.Transactions.Processors.LocalTransactionProcessorWithUndo
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using System;
using System.Collections.Generic;
using System.Linq;

namespace EmoTracker.Data.Core.Transactions.Processors
{
  public class LocalTransactionProcessorWithUndo : 
    IUndoableTransactionProcessor,
    ITransactionProcessor
  {
    private List<LocalTransactionProcessorWithUndo.Transaction> mUndoStack = new List<LocalTransactionProcessorWithUndo.Transaction>();
    private LocalTransactionProcessorWithUndo.Transaction mOpenTransaction;
    private static readonly int MaxUndoSteps = 250;

    public ITransactionScope OpenTransaction()
    {
      if (this.mOpenTransaction != null)
        return (ITransactionScope) null;
      this.mOpenTransaction = new LocalTransactionProcessorWithUndo.Transaction(this);
      return (ITransactionScope) this.mOpenTransaction;
    }

    public ITransactionScope CurrentScope => (ITransactionScope) this.mOpenTransaction;

    public void WriteProperty<T>(
      TransactableObject obj,
      string fieldName,
      T value,
      Action<ITransaction> transactionStateCallback = null)
    {
      using (this.OpenTransaction())
        this.mOpenTransaction.WriteProperty<T>(obj, fieldName, value, transactionStateCallback);
    }

    private void ProcessTransaction(
      LocalTransactionProcessorWithUndo.Transaction transaction)
    {
      try
      {
        LocalTransactionProcessorWithUndo.Transaction mOpenTransaction = this.mOpenTransaction;
        this.mOpenTransaction = (LocalTransactionProcessorWithUndo.Transaction) null;
        if (mOpenTransaction.Empty)
          return;
        this.mUndoStack.Add(mOpenTransaction);
        mOpenTransaction.Commit();
        while (this.mUndoStack.Count > LocalTransactionProcessorWithUndo.MaxUndoSteps)
          this.mUndoStack.RemoveAt(0);
      }
      finally
      {
        this.mOpenTransaction = (LocalTransactionProcessorWithUndo.Transaction) null;
      }
    }

    public void ClearUndoHistory()
    {
      if (this.mOpenTransaction != null)
        throw new InvalidOperationException("Cannot clear undo stack while a transaction is open for write");
      this.mUndoStack.Clear();
    }

    public void Undo()
    {
      LocalTransactionProcessorWithUndo.Transaction transaction = this.mUndoStack.LastOrDefault<LocalTransactionProcessorWithUndo.Transaction>();
      if (transaction == null)
        return;
      this.mUndoStack.Remove(transaction);
      transaction.Undo();
    }

    private class Transaction : ITransaction, ITransactionScope, IDisposable
    {
      private TransactionStatus mTransactionStatus = TransactionStatus.Pending;
      private LocalTransactionProcessorWithUndo.Transaction.ReadSource mReadSource = LocalTransactionProcessorWithUndo.Transaction.ReadSource.Target;
      private LocalTransactionProcessorWithUndo mProcessor;
      private Dictionary<KeyValuePair<TransactableObject, string>, LocalTransactionProcessorWithUndo.Transaction.ValueStateBase> mValueStates = new Dictionary<KeyValuePair<TransactableObject, string>, LocalTransactionProcessorWithUndo.Transaction.ValueStateBase>();

      ITransaction ITransactionScope.Transaction => (ITransaction) this;

      public TransactionStatus Status => this.mTransactionStatus;

      public bool HasPropertyValue(TransactableObject src, string propertyName) => this.mValueStates.ContainsKey(new KeyValuePair<TransactableObject, string>(src, propertyName));

      public T GetPropertyValue<T>(TransactableObject src, string propertyName)
      {
        try
        {
          LocalTransactionProcessorWithUndo.Transaction.ValueStateBase valueStateBase;
          if (this.mValueStates.TryGetValue(new KeyValuePair<TransactableObject, string>(src, propertyName), out valueStateBase))
          {
            LocalTransactionProcessorWithUndo.Transaction.ValueState<T> valueState = (LocalTransactionProcessorWithUndo.Transaction.ValueState<T>) valueStateBase;
            switch (this.mReadSource)
            {
              case LocalTransactionProcessorWithUndo.Transaction.ReadSource.Original:
                return valueState.OriginalValue;
              case LocalTransactionProcessorWithUndo.Transaction.ReadSource.Target:
                return valueState.TargetValue;
            }
          }
        }
        catch
        {
        }
        return default (T);
      }

      public Transaction(LocalTransactionProcessorWithUndo processor) => this.mProcessor = processor;

      public bool Empty => this.mValueStates.Count == 0;

      public void WriteProperty<T>(
        TransactableObject source,
        string fieldName,
        T value,
        Action<ITransaction> action = null)
      {
        this.mValueStates[new KeyValuePair<TransactableObject, string>(source, fieldName)] = (LocalTransactionProcessorWithUndo.Transaction.ValueStateBase) new LocalTransactionProcessorWithUndo.Transaction.ValueState<T>(source, source.GetTransactableProperty<T>(fieldName), value, action);
      }

      public void Dispose() => this.mProcessor.ProcessTransaction(this);

      public void Commit()
      {
        this.mTransactionStatus = TransactionStatus.Completed;
        foreach (KeyValuePair<KeyValuePair<TransactableObject, string>, LocalTransactionProcessorWithUndo.Transaction.ValueStateBase> mValueState in this.mValueStates)
          mValueState.Value.ProcessedAction((ITransaction) this);
      }

      public void Undo()
      {
        using (new LocationDatabase.SuspendRefreshScope())
        {
          this.mReadSource = LocalTransactionProcessorWithUndo.Transaction.ReadSource.Original;
          foreach (KeyValuePair<KeyValuePair<TransactableObject, string>, LocalTransactionProcessorWithUndo.Transaction.ValueStateBase> mValueState in this.mValueStates)
            mValueState.Value.ProcessedAction((ITransaction) this);
        }
      }

      private enum ReadSource
      {
        Original,
        Target,
      }

      private abstract class ValueStateBase
      {
        public TransactableObject Source;
        public Action<ITransaction> ProcessedAction;
      }

      private class ValueState<T> : LocalTransactionProcessorWithUndo.Transaction.ValueStateBase
      {
        public T OriginalValue;
        public T TargetValue;

        public ValueState(
          TransactableObject source,
          T originalValue,
          T targetValue,
          Action<ITransaction> action)
        {
          this.Source = source;
          this.ProcessedAction = action;
          this.OriginalValue = originalValue;
          this.TargetValue = targetValue;
        }
      }
    }
  }
}
