﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.Core.Transactions.TransactableObject
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using EmoTracker.Core;
using System;
using System.Collections.Generic;
using System.Reflection;
using System.Runtime.CompilerServices;

namespace EmoTracker.Data.Core.Transactions
{
  public class TransactableObject : ObservableObject
  {
    private static Dictionary<Type, Dictionary<string, bool>> mGlobalReadFromOpenTransactionCache = new Dictionary<Type, Dictionary<string, bool>>();
    private Dictionary<string, bool> mLocalReadFromOpenTransactionCache;
    private Dictionary<string, object> mPropertyStore = new Dictionary<string, object>();

    private bool ShouldReadFromOpenTransaction(Type hostType, string propertyName)
    {
      bool flag = false;
      if (this.mLocalReadFromOpenTransactionCache == null && !TransactableObject.mGlobalReadFromOpenTransactionCache.TryGetValue(hostType, out this.mLocalReadFromOpenTransactionCache))
        TransactableObject.mGlobalReadFromOpenTransactionCache[hostType] = this.mLocalReadFromOpenTransactionCache = new Dictionary<string, bool>();
      if (this.mLocalReadFromOpenTransactionCache.TryGetValue(propertyName, out flag))
        return flag;
      PropertyInfo property = hostType.GetProperty(propertyName, BindingFlags.Instance | BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic);
      if (property != (PropertyInfo) null && Attribute.GetCustomAttribute((MemberInfo) property, typeof (TransactablePropertyReadBehaviorAttribute)) is TransactablePropertyReadBehaviorAttribute customAttribute)
        flag = customAttribute.ReadBehavior == TransactablePropertyReadBehavior.AllowOpenTransactionRead;
      this.mLocalReadFromOpenTransactionCache[propertyName] = flag;
      return flag;
    }

    private T GetCurrentTransactablePropertyValue<T>(string propertyName)
    {
      object transactablePropertyValue;
      if (!this.mPropertyStore.TryGetValue(propertyName, out transactablePropertyValue))
        return default (T);
      try
      {
        return (T) transactablePropertyValue;
      }
      catch
      {
        throw new InvalidCastException(string.Format("Failed to read property '{0}' as type `{1}`", (object) propertyName, (object) typeof (T)));
      }
    }

    public T GetTransactableProperty<T>([CallerMemberName] string propertyName = null, bool bForceReadFromOpenTransaction = false)
    {
      if (bForceReadFromOpenTransaction || this.ShouldReadFromOpenTransaction(this.GetType(), propertyName))
      {
        ITransactionScope currentScope = TransactionProcessor.Current.CurrentScope;
        if (currentScope != null && currentScope.Transaction != null && currentScope.Transaction.HasPropertyValue(this, propertyName))
          return currentScope.Transaction.GetPropertyValue<T>(this, propertyName);
      }
      return this.GetCurrentTransactablePropertyValue<T>(propertyName);
    }

    protected bool ForceSetTransactableProperty<T>(
      T value,
      Action<T> onTransactionProcessed = null,
      [CallerMemberName] string propertyName = null)
    {
      if (!this.SetTransactableProperty<T>(value, onTransactionProcessed, propertyName) && onTransactionProcessed != null)
        onTransactionProcessed(value);
      return true;
    }

    protected bool SetTransactableProperty<T>(
      T value,
      Action<T> onTransactionProcessed = null,
      [CallerMemberName] string propertyName = null)
    {
      if (EqualityComparer<T>.Default.Equals(this.GetTransactableProperty<T>(propertyName), value))
        return false;
      TransactionProcessor.Current.WriteProperty<T>(this, propertyName, value, (Action<ITransaction>) (transactionState =>
      {
        if (transactionState.Status != TransactionStatus.Completed)
          return;
        try
        {
          T propertyValue = transactionState.GetPropertyValue<T>(this, propertyName);
          this.NotifyPropertyChanging(propertyName);
          this.mPropertyStore[propertyName] = (object) propertyValue;
          Action<T> action = onTransactionProcessed;
          if (action != null)
            action(propertyValue);
          this.NotifyPropertyChanged(propertyName);
        }
        catch
        {
        }
      }));
      return true;
    }
  }
}
