﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.ItemDatabase
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using EmoTracker.Core;
using EmoTracker.Data.Items;
using EmoTracker.Data.JSON;
using EmoTracker.Data.Locations;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;

namespace EmoTracker.Data
{
  public class ItemDatabase : Singleton<ItemDatabase>, ICodeProvider
  {
    private ObservableCollection<ITrackableItem> mItems = new ObservableCollection<ITrackableItem>();

    public IEnumerable<ITrackableItem> Items => (IEnumerable<ITrackableItem>) this.mItems;

    public void Reset()
    {
      foreach (ITrackableItem mItem in (Collection<ITrackableItem>) this.mItems)
        mItem?.Dispose();
      this.mItems.Clear();
    }

    public void RegisterItem(ITrackableItem item)
    {
      if (this.mItems.Contains(item))
        return;
      this.mItems.Add(item);
    }

    public bool LegacyLoad(IGamePackage package) => this.mItems.Count > 0 || this.IncrementalLoad("items.json", package, true);

    public bool IncrementalLoad(string path, IGamePackage package, bool bLegacy = false)
    {
      bool flag = false;
      if (bLegacy)
        ObservableSingleton<ScriptManager>.Instance.OutputWarning("Loading legacy items");
      else
        ObservableSingleton<ScriptManager>.Instance.Output("Loading Items: {0}", (object) path);
      using (new LoggingBlock())
      {
        try
        {
          ObservableSingleton<LocationDatabase>.Instance.SuspendRefresh = true;
          using (StreamReader reader = new StreamReader(package.Open(path)))
          {
            foreach (JObject data in (JArray) JToken.ReadFrom((JsonReader) new JsonTextReader((TextReader) reader)))
            {
              ITrackableItem trackableItem = ItemBase.CreateItem(data, package);
              if (trackableItem != null)
                this.mItems.Add(trackableItem);
            }
          }
          flag = true;
        }
        catch (Exception ex)
        {
          ObservableSingleton<ScriptManager>.Instance.OutputException(ex);
        }
        finally
        {
          ObservableSingleton<LocationDatabase>.Instance.SuspendRefresh = false;
        }
      }
      return flag;
    }

    internal bool CodeIsProvided(string code)
    {
      foreach (ITrackableItem trackableItem in this.Items)
      {
        if (trackableItem.ProvidesCode(code) > 0U)
          return true;
      }
      return false;
    }

    public object FindObjectForCode(string code) => (object) this.FindProvidingItemForCode(code);

    public uint ProviderCountForCode(string code, out AccessibilityLevel maxAccessibilityLevel)
    {
      code = code.ToLower();
      maxAccessibilityLevel = AccessibilityLevel.Normal;
      uint num = 0;
      foreach (ITrackableItem trackableItem in this.Items)
        num += trackableItem.ProvidesCode(code);
      return num;
    }

    internal ITrackableItem FindProvidingItemForCode(string code)
    {
      if (!string.IsNullOrWhiteSpace(code))
      {
        foreach (ITrackableItem providingItemForCode in this.Items)
        {
          if (providingItemForCode.CanProvideCode(code))
            return providingItemForCode;
        }
      }
      return (ITrackableItem) null;
    }

    public ITrackableItem[] FindProvidingItemsForCode(string code)
    {
      List<ITrackableItem> trackableItemList = new List<ITrackableItem>();
      if (!string.IsNullOrWhiteSpace(code))
      {
        foreach (ITrackableItem trackableItem in this.Items)
        {
          if (trackableItem.CanProvideCode(code))
            trackableItemList.Add(trackableItem);
        }
      }
      return trackableItemList.ToArray();
    }

    internal void Save(JObject root)
    {
      JArray jarray = new JArray();
      foreach (ITrackableItem mItem in (Collection<ITrackableItem>) this.mItems)
      {
        JObject itemData = new JObject();
        itemData["item_reference"] = (JToken) this.GetPersistableItemReference(mItem);
        try
        {
          if (mItem.Save(itemData))
            jarray.Add((JToken) itemData);
        }
        catch (Exception ex)
        {
          ObservableSingleton<ScriptManager>.Instance.OutputException(ex);
        }
      }
      if (jarray.Count <= 0)
        return;
      root["item_database"] = (JToken) jarray;
    }

    internal bool Load(JObject root)
    {
      JArray jarray = root.GetValue<JArray>("item_database");
      if (jarray != null)
      {
        foreach (JObject jobject in jarray)
        {
          ITrackableItem trackableItem = this.ResolvePersistableItemReference(jobject.GetValue<string>("item_reference", (string) null));
          if (trackableItem == null)
          {
            trackableItem = this.ResolvePersistableItemReference(jobject.GetValue<string>("item_reference", (string) null), true);
            if (trackableItem != null)
              ObservableSingleton<ScriptManager>.Instance.OutputWarning("Item index consistency issue found (and allowed) for reference \"{0}\" while loading a save. Ensure that items are loaded/created in a consistent order.", (object) jobject.GetValue<string>("item_reference", (string) null));
            if (trackableItem == null)
              return false;
          }
          if (!trackableItem.Load(jobject))
            return false;
        }
      }
      return true;
    }

    public string GetPersistableItemReference(ITrackableItem item, bool allowAnyType = false)
    {
      int num = this.mItems.IndexOf(item);
      if (num < 0)
        throw new InvalidOperationException("Cannot generate persistable reference for item that is not in the ItemDatabase");
      string defaultTagForType = JsonTypeTagsAttribute.GetDefaultTagForType(item.GetType());
      if (string.IsNullOrWhiteSpace(defaultTagForType))
        throw new InvalidOperationException("Cannot generate persistable reference for item that does not provide at least one json type tag");
      return !string.IsNullOrWhiteSpace(item.Name) ? string.Format("{0}:{1}:{2}", (object) num, allowAnyType ? (object) "*" : (object) Uri.EscapeDataString(defaultTagForType), (object) Uri.EscapeDataString(item.Name)) : string.Format("{0}:{1}", (object) num, allowAnyType ? (object) "*" : (object) Uri.EscapeDataString(defaultTagForType));
    }

    private ITrackableItem FindItemByName(string name)
    {
      foreach (ITrackableItem itemByName in this.Items)
      {
        if (string.Equals(itemByName.Name, name, StringComparison.OrdinalIgnoreCase))
          return itemByName;
      }
      return (ITrackableItem) null;
    }

    public ITrackableItem ResolvePersistableItemReference(
      string reference,
      bool allowIndexMismatch = false)
    {
      if (string.IsNullOrWhiteSpace(reference))
        return (ITrackableItem) null;
      string[] strArray = reference.Split(new char[1]{ ':' }, StringSplitOptions.RemoveEmptyEntries);
      if (strArray.Length < 2)
        return (ITrackableItem) null;
      int result = -1;
      if (!int.TryParse(strArray[0], out result) || result < 0 || result > this.mItems.Count - 1)
        return (ITrackableItem) null;
      ITrackableItem mItem = this.mItems[result];
      if (!string.Equals(strArray[1], "*") && !JsonTypeTagsAttribute.GetTypeSupportsTag(mItem.GetType(), Uri.UnescapeDataString(strArray[1])))
        return (ITrackableItem) null;
      if (mItem == null || strArray.Length < 3 || string.Equals(Uri.UnescapeDataString(strArray[2]), mItem.Name, StringComparison.OrdinalIgnoreCase))
        return mItem;
      return allowIndexMismatch ? this.FindItemByName(Uri.UnescapeDataString(strArray[2])) : (ITrackableItem) null;
    }
  }
}
