﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.Items.ConsumableItem
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using EmoTracker.Core;
using EmoTracker.Data.JSON;
using EmoTracker.Data.Media;
using Newtonsoft.Json.Linq;
using System;

namespace EmoTracker.Data.Items
{
  [JsonTypeTags(new string[] {"consumable"})]
  public class ConsumableItem : ItemBase
  {
    private int mMaxCount = int.MaxValue;
    private int mMinCount;
    private int mIncrement = 1;
    private bool mbSwapActions;
    private bool mbDisplayAsFractionOfMax;
    private ImageReference mEmptyIcon;
    private ImageReference mFullIcon;
    private CodeProvider mCodeProvider = new CodeProvider();

    public bool SwapActions
    {
      get => this.mbSwapActions;
      set => this.SetProperty<bool>(ref this.mbSwapActions, value, nameof (SwapActions));
    }

    public bool DisplayAsFractionOfMax
    {
      get => this.mbDisplayAsFractionOfMax;
      set
      {
        if (!this.SetProperty<bool>(ref this.mbDisplayAsFractionOfMax, value, nameof (DisplayAsFractionOfMax)))
          return;
        this.UpdateBadgeAndIcon();
      }
    }

    [DependentProperty("AvailableCount")]
    public int AcquiredCount
    {
      get => this.GetTransactableProperty<int>(nameof (AcquiredCount));
      set => this.SetTransactableProperty<int>(Math.Min(Math.Max(Math.Max(value, this.ConsumedCount), this.mMinCount), this.mMaxCount), (Action<int>) (processedValue =>
      {
        this.UpdateBadgeAndIcon();
        ObservableSingleton<LocationDatabase>.Instance.RefeshAccessibility();
      }), nameof (AcquiredCount));
    }

    [DependentProperty("AvailableCount")]
    public int ConsumedCount
    {
      get => this.GetTransactableProperty<int>(nameof (ConsumedCount));
      set => this.SetTransactableProperty<int>(Math.Max(Math.Min(value, this.AvailableCount), 0), (Action<int>) (processedValue =>
      {
        this.UpdateBadgeAndIcon();
        ObservableSingleton<LocationDatabase>.Instance.RefeshAccessibility();
      }), nameof (ConsumedCount));
    }

    public int AvailableCount => this.AcquiredCount - this.ConsumedCount;

    public int CountIncrement
    {
      get => this.mIncrement;
      set => this.SetProperty<int>(ref this.mIncrement, value, nameof (CountIncrement));
    }

    public int MinCount
    {
      get => this.mMinCount;
      set
      {
        if (!this.SetProperty<int>(ref this.mMinCount, value, nameof (MinCount)))
          return;
        this.AcquiredCount = this.AcquiredCount;
        this.ConsumedCount = this.ConsumedCount;
        this.UpdateBadgeAndIcon();
      }
    }

    public int MaxCount
    {
      get => this.mMaxCount;
      set
      {
        if (!this.SetProperty<int>(ref this.mMaxCount, value, nameof (MaxCount)))
          return;
        this.AcquiredCount = this.AcquiredCount;
        this.ConsumedCount = this.ConsumedCount;
        this.UpdateBadgeAndIcon();
      }
    }

    public override bool CanProvideCode(string code) => this.mCodeProvider.ProvidesCode(code);

    public override uint ProvidesCode(string code) => this.AvailableCount > 0 && this.mCodeProvider.ProvidesCode(code) ? (uint) this.AvailableCount : 0U;

    public override void AdvanceToCode(string code = null) => this.OnLeftClick();

    public override void OnLeftClick()
    {
      if (!this.mbSwapActions)
        this.Increment();
      else
        this.Decrement();
    }

    public int Increment(int count = 1)
    {
      int num = Math.Min(this.MaxCount, Math.Max(this.MinCount, this.AcquiredCount + this.mIncrement * count));
      this.AcquiredCount = num;
      return num;
    }

    public int Decrement(int count = 1)
    {
      int num = Math.Min(this.MaxCount, Math.Max(this.MinCount, this.AcquiredCount - this.mIncrement * count));
      this.AcquiredCount = num;
      return num;
    }

    public override void OnRightClick()
    {
      if (!this.mbSwapActions)
        this.Decrement();
      else
        this.Increment();
    }

    public bool Consume(int quantity = 1)
    {
      if (this.AvailableCount < quantity)
        return false;
      this.ConsumedCount += quantity;
      return true;
    }

    public bool Release(int quantity = 1)
    {
      if (this.ConsumedCount < quantity)
        return false;
      this.ConsumedCount -= quantity;
      return true;
    }

    protected void UpdateBadgeAndIcon()
    {
      if (this.AvailableCount == 0)
      {
        this.Icon = this.AcquiredCount > 0 ? this.mFullIcon : this.mEmptyIcon;
        this.BadgeText = (string) null;
      }
      else
      {
        this.Icon = this.mFullIcon;
        if (!this.DisplayAsFractionOfMax)
          this.BadgeText = this.AvailableCount.ToString();
        else
          this.BadgeText = string.Format("{0}/{1}", (object) this.AvailableCount.ToString(), (object) this.MaxCount.ToString());
      }
      if (this.AcquiredCount >= this.mMaxCount)
        this.BadgeTextColor = "#00ff00";
      else
        this.BadgeTextColor = "WhiteSmoke";
    }

    protected override void ParseDataInternal(JObject data, IGamePackage package)
    {
      this.mCodeProvider.Clear();
      this.mCodeProvider.AddCodes(data.GetValue<string>("codes", (string) null));
      this.mFullIcon = ImageReference.FromPackRelativePath(package, data.GetValue<string>("img", (string) null), data.GetValue<string>("img_mods", (string) null));
      this.mEmptyIcon = ImageReference.FromPackRelativePath(package, data.GetValue<string>("disabled_img", (string) null), data.GetValue<string>("disabled_img_mods", (string) null) ?? this.DisabledImageFilterSpec);
      if (this.mEmptyIcon == null)
        this.mEmptyIcon = ImageReference.FromImageReference(this.mFullIcon, data.GetValue<string>("disabled_img_mods", (string) null) ?? this.DisabledImageFilterSpec);
      this.MaxCount = data.GetValue<int>("max_quantity", int.MaxValue);
      this.MinCount = data.GetValue<int>("min_quantity");
      this.AcquiredCount = data.GetValue<int>("initial_quantity");
      this.CountIncrement = data.GetValue<int>("increment", 1);
      this.mbSwapActions = data.GetValue<bool>("swap_actions");
      this.mbDisplayAsFractionOfMax = data.GetValue<bool>("display_as_fraction_of_max");
      this.UpdateBadgeAndIcon();
    }

    protected override bool Save(JObject data)
    {
      data["acquired_count"] = (JToken) this.AcquiredCount;
      data["consumed_count"] = (JToken) this.ConsumedCount;
      data["max_count"] = (JToken) this.MaxCount;
      data["min_count"] = (JToken) this.MinCount;
      return true;
    }

    protected override bool Load(JObject data)
    {
      int num1 = data.GetValue<int>("acquired_count", -1);
      int num2 = data.GetValue<int>("consumed_count", -1);
      if (num1 < 0 || num2 < 0)
        return false;
      this.AcquiredCount = num1;
      this.ConsumedCount = num2;
      this.MaxCount = data.GetValue<int>("max_count", this.MaxCount);
      this.MinCount = data.GetValue<int>("min_count", this.MinCount);
      return true;
    }
  }
}
