﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.Items.ProgressiveItem
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using EmoTracker.Core;
using EmoTracker.Data.JSON;
using EmoTracker.Data.Media;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;

namespace EmoTracker.Data.Items
{
  [JsonTypeTags(new string[] {"progressive"})]
  public class ProgressiveItem : ItemBase
  {
    protected List<ProgressiveItem.Stage> StagesInternal = new List<ProgressiveItem.Stage>();
    private string mDisabledImageMods;
    private bool mbLoop;

    public IEnumerable<ProgressiveItem.Stage> Stages => (IEnumerable<ProgressiveItem.Stage>) this.StagesInternal;

    public bool Loop
    {
      get => this.mbLoop;
      set => this.SetProperty<bool>(ref this.mbLoop, value, nameof (Loop));
    }

    private ProgressiveItem.Stage CurrentStageInstance => this.StagesInternal.Count > 0 ? this.StagesInternal[this.CurrentStage] : (ProgressiveItem.Stage) null;

    [DependentProperty("CurrentStageInstance")]
    public int CurrentStage
    {
      get => this.GetTransactableProperty<int>(nameof (CurrentStage));
      set
      {
        if (value < 0 || value >= this.StagesInternal.Count)
          return;
        this.SetTransactableProperty<int>(value, (Action<int>) (processedValue => this.UpdateAppearance()), nameof (CurrentStage));
      }
    }

    public void AddStage(
      ProgressiveItem.Stage stage,
      bool enableDisabledStage,
      IGamePackage package)
    {
      if (this.StagesInternal.Count == 0 & enableDisabledStage)
        this.StagesInternal.Add(new ProgressiveItem.Stage()
        {
          Icon = ImageReference.FromImageReference(stage.Icon, this.mDisabledImageMods ?? this.DisabledImageFilterSpec)
        });
      this.StagesInternal.Add(stage);
      this.UpdateAppearance();
    }

    protected void UpdateAppearance()
    {
      if (this.CurrentStageInstance != null)
        this.Icon = this.CurrentStageInstance.Icon;
      else
        this.Icon = (ImageReference) null;
      this.UpdatePotentialIcon();
    }

    protected void UpdatePotentialIcon()
    {
      int index;
      if ((index = this.CurrentStage + 1) < this.StagesInternal.Count)
        this.PotentialIcon = this.StagesInternal[index].Icon;
      else
        this.PotentialIcon = this.Icon;
    }

    public override bool CanProvideCode(string code)
    {
      foreach (CodeProvider codeProvider in this.StagesInternal)
      {
        if (codeProvider.ProvidesCode(code))
          return true;
      }
      return false;
    }

    public override uint ProvidesCode(string code) => this.CurrentStageInstance != null && this.CurrentStageInstance.ProvidesCode(code) ? 1U : 0U;

    public override void AdvanceToCode(string code = null)
    {
      if (code != null)
      {
        for (int index = this.CurrentStage + 1; index < this.StagesInternal.Count; ++index)
        {
          if (this.StagesInternal[index].ProvidesCode(code))
          {
            this.CurrentStage = index;
            return;
          }
        }
        if (this.Loop)
        {
          for (int index = 0; index < this.StagesInternal.Count; ++index)
          {
            if (this.StagesInternal[index].ProvidesCode(code))
            {
              this.CurrentStage = index;
              return;
            }
          }
        }
      }
      this.Advance();
    }

    public void Advance()
    {
      int num = this.CurrentStage + 1;
      if (this.Loop)
        num %= this.StagesInternal.Count;
      if (num >= this.StagesInternal.Count)
        return;
      this.CurrentStage = num;
    }

    public void Downgrade()
    {
      int num = this.CurrentStage - 1;
      if (this.Loop)
        num = (num + this.StagesInternal.Count) % this.StagesInternal.Count;
      if (num < 0)
        return;
      this.CurrentStage = num;
    }

    public override void OnLeftClick() => this.Advance();

    public override void OnRightClick() => this.Downgrade();

    protected override void ParseDataInternal(JObject data, IGamePackage package)
    {
      this.Loop = data.GetValue<bool>("loop");
      bool enableDisabledStage = data.GetValue<bool>("allow_disabled", true);
      this.mDisabledImageMods = data.GetValue<string>("disabled_img_mods", (string) null);
      string spec = "";
      foreach (JObject token in (JArray) data.GetValue("stages"))
      {
        ImageReference imageReference = ImageReference.FromPackRelativePath(package, token.GetValue<string>("img", (string) null), token.GetValue<string>("img_mods", (string) null));
        if (!token.GetValue<bool>("inherit_codes", true))
          spec = "";
        if (imageReference != null)
        {
          ProgressiveItem.Stage stage = new ProgressiveItem.Stage();
          stage.Icon = imageReference;
          string str = token.GetValue<string>("codes", (string) null);
          if (!string.IsNullOrWhiteSpace(str))
            spec = string.Format("{0},{1}", (object) spec, (object) str);
          if (!string.IsNullOrWhiteSpace(spec))
            stage.AddCodes(spec);
          this.AddStage(stage, enableDisabledStage, package);
        }
      }
      int num = data.GetValue<int>("initial_stage_idx", -1);
      if (num < 0 || this.StagesInternal.Count <= 0)
        return;
      this.CurrentStage = (num + (enableDisabledStage ? 1 : 0)) % this.StagesInternal.Count;
    }

    protected override bool Save(JObject data)
    {
      data["stage_index"] = (JToken) this.CurrentStage;
      return true;
    }

    protected override bool Load(JObject data)
    {
      int num = data.GetValue<int>("stage_index", -1);
      int count = this.StagesInternal.Count;
      if (count == 0)
        return true;
      if (num < 0 || num > count - 1)
        return false;
      this.CurrentStage = num;
      return true;
    }

    public class Stage : CodeProvider
    {
      public ImageReference Icon { get; set; }

      public string Name { get; set; }
    }
  }
}
