﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.Items.ProgressiveToggleItem
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using EmoTracker.Core;
using EmoTracker.Data.JSON;
using EmoTracker.Data.Media;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;

namespace EmoTracker.Data.Items
{
  [JsonTypeTags(new string[] {"progressive_toggle"})]
  public class ProgressiveToggleItem : ItemBase
  {
    public bool SwapActions;
    public uint StageCount;
    private Dictionary<KeyValuePair<bool, uint>, ProgressiveToggleItem.Stage> mStages = new Dictionary<KeyValuePair<bool, uint>, ProgressiveToggleItem.Stage>();

    public bool Active
    {
      get => this.GetTransactableProperty<bool>(nameof (Active));
      set => this.SetTransactableProperty<bool>(value, (Action<bool>) (processedValue => this.UpdateImage()), nameof (Active));
    }

    public uint CurrentStage
    {
      get => this.GetTransactableProperty<uint>(nameof (CurrentStage));
      set => this.SetTransactableProperty<uint>(value, (Action<uint>) (processedValue => this.UpdateImage()), nameof (CurrentStage));
    }

    public ProgressiveToggleItem.Stage GetActiveStageForIndex(uint index)
    {
      ProgressiveToggleItem.Stage stage;
      return this.mStages.TryGetValue(new KeyValuePair<bool, uint>(true, index), out stage) ? stage : (ProgressiveToggleItem.Stage) null;
    }

    public void AddStage(
      IGamePackage package,
      uint stage,
      ProgressiveToggleItem.Stage stageDef,
      ImageReference disabledImage = null)
    {
      this.StageCount = Math.Max(this.StageCount, stage + 1U);
      disabledImage = disabledImage ?? ImageReference.FromImageReference(stageDef.Icon, this.DisabledImageFilterSpec);
      this.mStages[new KeyValuePair<bool, uint>(false, stage)] = new ProgressiveToggleItem.Stage()
      {
        Icon = disabledImage
      };
      this.mStages[new KeyValuePair<bool, uint>(true, stage)] = stageDef;
      this.UpdateImage();
    }

    public override bool CanProvideCode(string code)
    {
      foreach (ProgressiveToggleItem.Stage stage in this.mStages.Values)
      {
        if (stage != null && stage.ProvidesCode(code))
          return true;
      }
      return false;
    }

    public override uint ProvidesCode(string code)
    {
      ProgressiveToggleItem.Stage stage;
      return this.mStages.TryGetValue(new KeyValuePair<bool, uint>(this.Active, this.CurrentStage), out stage) && stage.ProvidesCode(code) ? 1U : 0U;
    }

    public override void AdvanceToCode(string code = null)
    {
      this.Active = true;
      this.UpdateImage();
    }

    public void Advance()
    {
      this.CurrentStage = (this.CurrentStage + 1U) % this.StageCount;
      this.UpdateImage();
    }

    public void AdvanceToPrivateCode(string code = null)
    {
      if (string.IsNullOrWhiteSpace(code) || this.StageCount <= 0U)
        return;
      int currentStage = (int) this.CurrentStage;
      for (int index = currentStage + 1; (long) index < (long) this.StageCount; ++index)
      {
        ProgressiveToggleItem.Stage mStage = this.mStages[new KeyValuePair<bool, uint>(true, (uint) index)];
        if (mStage.PrivateCodes.ProvidesCode(code) || mStage.ProvidesCode(code))
        {
          this.CurrentStage = (uint) index;
          this.UpdateImage();
          return;
        }
      }
      for (int index = currentStage - 1; index >= 0; --index)
      {
        ProgressiveToggleItem.Stage mStage = this.mStages[new KeyValuePair<bool, uint>(true, (uint) index)];
        if (mStage.PrivateCodes.ProvidesCode(code) || mStage.ProvidesCode(code))
        {
          this.CurrentStage = (uint) index;
          this.UpdateImage();
          break;
        }
      }
    }

    public override void OnLeftClick()
    {
      if (!this.SwapActions)
        this.Active = !this.Active;
      else
        this.Advance();
    }

    public override void OnRightClick()
    {
      if (this.SwapActions)
        this.Active = !this.Active;
      else
        this.Advance();
    }

    protected void UpdateImage()
    {
      ProgressiveToggleItem.Stage stage1;
      if (this.mStages.TryGetValue(new KeyValuePair<bool, uint>(this.Active, this.CurrentStage), out stage1))
        this.Icon = stage1.Icon;
      ProgressiveToggleItem.Stage stage2;
      if (!this.mStages.TryGetValue(new KeyValuePair<bool, uint>(true, this.CurrentStage), out stage2))
        return;
      this.PotentialIcon = stage2.Icon;
    }

    protected override void ParseDataInternal(JObject data, IGamePackage package)
    {
      this.SwapActions = data.GetValue<bool>("swap_actions");
      uint num = 0;
      foreach (JObject token in (JArray) data.GetValue("stages"))
      {
        ImageReference imageReference = ImageReference.FromPackRelativePath(package, token.GetValue<string>("img", (string) null), token.GetValue<string>("img_mods", (string) null));
        ImageReference disabledImage = ImageReference.FromPackRelativePath(package, token.GetValue<string>("disabled_img", (string) null), token.GetValue<string>("disabled_img_mods", (string) null) ?? this.DisabledImageFilterSpec);
        if (imageReference != null)
        {
          ProgressiveToggleItem.Stage stageDef = new ProgressiveToggleItem.Stage();
          stageDef.Icon = imageReference;
          string spec1 = token.GetValue<string>("codes", (string) null);
          if (!string.IsNullOrWhiteSpace(spec1))
            stageDef.AddCodes(spec1);
          string spec2 = token.GetValue<string>("secondary_codes", (string) null);
          if (!string.IsNullOrWhiteSpace(spec2))
            stageDef.PrivateCodes.AddCodes(spec2);
          this.AddStage(package, num++, stageDef, disabledImage);
        }
      }
      if (this.StageCount <= 0U)
        return;
      this.CurrentStage = data.GetValue<uint>("initial_stage_idx", this.CurrentStage) % this.StageCount;
      this.Active = data.GetValue<bool>("initial_active_state", this.Active);
    }

    protected override bool Save(JObject data)
    {
      data["stage_index"] = (JToken) this.CurrentStage;
      data["active"] = (JToken) this.Active;
      return true;
    }

    protected override bool Load(JObject data)
    {
      int num = data.GetValue<int>("stage_index", -1);
      int count = this.mStages.Count;
      if (num < 0 || (long) num > (long) (this.StageCount - 1U))
        return false;
      this.CurrentStage = (uint) num;
      this.Active = data.GetValue<bool>("active", this.Active);
      this.UpdateImage();
      return true;
    }

    public class Stage : CodeProvider
    {
      private CodeProvider mPrivateCodes = new CodeProvider();

      public ImageReference Icon { get; set; }

      public CodeProvider PrivateCodes => this.mPrivateCodes;
    }
  }
}
