﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.LocationDatabase
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using EmoTracker.Core;
using EmoTracker.Data.JSON;
using EmoTracker.Data.Locations;
using EmoTracker.Data.Media;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;

namespace EmoTracker.Data
{
  public class LocationDatabase : ObservableSingleton<LocationDatabase>, ICodeProvider
  {
    private Location mRoot;
    private Location mLastClearedLocation;
    private ObservableCollection<Location> mAllLocations = new ObservableCollection<Location>();
    private ObservableCollection<Location> mPinnedLocations = new ObservableCollection<Location>();
    private ObservableCollection<Location> mVisibleLocations = new ObservableCollection<Location>();
    private bool mbSuspendRefresh;
    private bool mbInRefresh;
    private uint mPendingRefreshCount;

    public bool SuspendRefresh
    {
      get => this.mbSuspendRefresh;
      set
      {
        if (!this.SetProperty<bool>(ref this.mbSuspendRefresh, value, nameof (SuspendRefresh)) || this.mbSuspendRefresh)
          return;
        this.RefeshAccessibility(true);
      }
    }

    public Location Root => this.mRoot;

    public Location LastClearedLocation
    {
      get => this.mLastClearedLocation;
      set => this.SetProperty<Location>(ref this.mLastClearedLocation, value, nameof (LastClearedLocation));
    }

    public IEnumerable<Location> AllLocations => (IEnumerable<Location>) this.mAllLocations;

    public IEnumerable<Location> VisibleLocations => (IEnumerable<Location>) this.mVisibleLocations;

    public IEnumerable<Location> PinnedLocations => (IEnumerable<Location>) this.mPinnedLocations;

    public void Reset()
    {
      this.LastClearedLocation = (Location) null;
      this.mAllLocations.Clear();
      this.mPinnedLocations.Clear();
      this.mVisibleLocations.Clear();
      Location location = new Location();
      location.Color = "#212121";
      location.OpenChestImage = ImageReference.FromExternalURI(new Uri("pack://application:,,,/EmoTracker;component/Resources/chest_open.png"));
      location.ClosedChestImage = ImageReference.FromExternalURI(new Uri("pack://application:,,,/EmoTracker;component/Resources/chest_closed.png"));
      this.mRoot = location;
    }

    public void ParseLocationVisualProperties(
      JObject data,
      LocationVisualProperties visual,
      IGamePackage package)
    {
      ImageReference imageReference1 = ImageReference.FromPackRelativePath(package, data.GetValue<string>("chest_opened_img", (string) null));
      if (imageReference1 != null)
        visual.OpenChestImage = imageReference1;
      ImageReference imageReference2 = ImageReference.FromPackRelativePath(package, data.GetValue<string>("chest_unopened_img", (string) null));
      if (imageReference2 != null)
        visual.ClosedChestImage = imageReference2;
      if (!string.IsNullOrWhiteSpace(data.GetValue<string>("always_allow_chest_manipulation", (string) null)))
        visual.AlwaysAllowChestManipulation = data.GetValue<bool>("always_allow_chest_manipulation");
      if (visual == this.Root || string.IsNullOrWhiteSpace(data.GetValue<string>("auto_unpin_on_clear", (string) null)))
        return;
      visual.AutoUnpinOnClear = data.GetValue<bool>("auto_unpin_on_clear");
    }

    public bool LegacyLoad(IGamePackage package) => this.mAllLocations.Count > 0 || this.IncrementalLoad("locations.json", package, true);

    internal bool IncrementalLoad(string path, IGamePackage package, bool bLegacy = false)
    {
      if (bLegacy)
        ObservableSingleton<ScriptManager>.Instance.OutputWarning("Loading legacy locations");
      else
        ObservableSingleton<ScriptManager>.Instance.Output("Loading Locations: {0}", (object) path);
      using (new LoggingBlock())
      {
        try
        {
          this.mbSuspendRefresh = true;
          using (Stream stream = package.Open(path))
          {
            if (stream != null)
            {
              using (StreamReader reader = new StreamReader(stream))
              {
                foreach (JObject data in (JArray) JToken.ReadFrom((JsonReader) new JsonTextReader((TextReader) reader)))
                {
                  Location child = this.LoadLocation(package, this.mRoot, data);
                  if (child != null)
                    this.mRoot.AddChild(child);
                }
              }
            }
            else
              ObservableSingleton<ScriptManager>.Instance.Output("File not found");
          }
        }
        catch (Exception ex)
        {
          ObservableSingleton<ScriptManager>.Instance.OutputException(ex);
        }
        finally
        {
          this.mbSuspendRefresh = false;
        }
        this.RefeshAccessibility();
      }
      return true;
    }

    internal AccessibilityLevel GetAccessibilityForCode(string code)
    {
      switch (this.FindObjectForCode(code))
      {
        case Location location:
          return location.BaseAccessibilityLevel;
        case Section section:
          return section.AccessibilityLevel;
        default:
          return AccessibilityLevel.Normal;
      }
    }

    public uint ProviderCountForCode(string code, out AccessibilityLevel maxAccessibilityLevel)
    {
      code = code.ToLower();
      maxAccessibilityLevel = this.GetAccessibilityForCode(code);
      string[] strArray = code.Split('/');
      if (strArray.Length != 0)
      {
        Location location = this.FindLocation(strArray[0]);
        if (location != null)
        {
          if (strArray.Length >= 2)
          {
            Section section = location.FindSection(strArray[1]);
            if (section != null && section.AccessibilityLevel >= AccessibilityLevel.Unlockable)
              return 1;
          }
          else if (location.BaseAccessibilityLevel >= AccessibilityLevel.Unlockable)
            return 1;
        }
      }
      return 0;
    }

    public object FindObjectForCode(string code)
    {
      code = code.ToLower();
      string[] strArray = code.Split('/');
      if (strArray.Length != 0)
      {
        Location location = this.FindLocation(strArray[0]);
        if (location != null)
          return strArray.Length >= 2 ? (object) location.FindSection(strArray[1]) : (object) location;
      }
      return (object) null;
    }

    public Location FindLocation(string name)
    {
      if (!string.IsNullOrWhiteSpace(name))
      {
        foreach (Location allLocation in this.AllLocations)
        {
          if (name.Equals(allLocation.Name, StringComparison.OrdinalIgnoreCase))
            return allLocation;
        }
      }
      return (Location) null;
    }

    public void PinLocation(Location location)
    {
      if (!this.mPinnedLocations.Contains(location))
        this.mPinnedLocations.Insert(0, location);
      else
        this.mPinnedLocations.Move(this.mPinnedLocations.IndexOf(location), 0);
    }

    public void UnpinLocation(Location location) => this.mPinnedLocations.Remove(location);

    internal void RefeshAccessibility(bool bPendingOnly = false)
    {
      if (!this.mbSuspendRefresh)
      {
        if (!bPendingOnly)
          ++this.mPendingRefreshCount;
        if (this.mbInRefresh)
          return;
        bool flag = false;
        try
        {
          this.mbInRefresh = true;
          while (this.mPendingRefreshCount > 0U)
          {
            this.mPendingRefreshCount = 0U;
            flag = true;
            AccessibilityRule.ClearCaches();
            ObservableSingleton<ScriptManager>.Instance.ClearExpressionCache();
            ObservableSingleton<ScriptManager>.Instance.InvokeStandardCallback(ScriptManager.StandardCallback.AccessibilityUpdating);
            if (this.mRoot != null)
              this.mRoot.RefreshAccessibility();
            ObservableSingleton<MapDatabase>.Instance.MarkVisibilityDirty();
          }
        }
        finally
        {
          this.mbInRefresh = false;
          AccessibilityRule.ClearCaches();
          ObservableSingleton<MapDatabase>.Instance.UpdateVisibilityIfNecessary();
          if (flag)
            ObservableSingleton<ScriptManager>.Instance.InvokeStandardCallback(ScriptManager.StandardCallback.AccessibilityUpdated);
        }
      }
      else
      {
        AccessibilityRule.ClearCaches();
        ++this.mPendingRefreshCount;
      }
    }

    private void LoadLocationList(IGamePackage package, Location parent, JArray locationNodes)
    {
      if (locationNodes == null)
        return;
      foreach (JObject locationNode in locationNodes)
      {
        Location child = this.LoadLocation(package, parent, locationNode);
        if (child != null && child.Parent != null)
          child.Parent.AddChild(child);
      }
    }

    private Location LoadLocation(IGamePackage package, Location parent, JObject data)
    {
      Location location1 = new Location()
      {
        Name = data.GetValue<string>("name", (string) null),
        ShortName = data.GetValue<string>("short_name", (string) null),
        Parent = parent
      };
      string code = data.GetValue<string>(nameof (parent), (string) null);
      if (!string.IsNullOrWhiteSpace(code) && this.FindObjectForCode(code) is Location objectForCode)
        location1.Parent = objectForCode;
      this.ParseLocationVisualProperties(data, (LocationVisualProperties) location1, package);
      string str = data.GetValue<string>("color", (string) null);
      if (!string.IsNullOrWhiteSpace(str))
        location1.Color = str;
      JArray jarray1 = data.GetValue<JArray>("access_rules");
      if (jarray1 != null)
      {
        foreach (JToken jtoken in jarray1)
          location1.AccessibilityRules.AddRule(jtoken.Value<string>());
      }
      JArray jarray2 = data.GetValue<JArray>("sections");
      if (jarray2 != null)
      {
        foreach (JObject jobject in jarray2)
        {
          Section section = new Section(location1);
          this.ParseLocationVisualProperties(jobject, (LocationVisualProperties) section, package);
          section.Name = jobject.GetValue<string>("name", (string) null);
          section.ItemCaptureLayout = jobject.GetValue<string>("capture_item_layout", (string) null);
          section.ShortName = jobject.GetValue<string>("short_name", (string) null);
          section.ChestCount = section.AvailableChestCount = jobject.GetValue<uint>("item_count");
          section.ClearAsGroup = jobject.GetValue<bool>("clear_as_group", true);
          section.CaptureItem = jobject.GetValue<bool>("capture_item");
          section.ShowGateItem = jobject.GetValue<bool>("show_gate_item", true);
          JArray jarray3 = jobject.GetValue<JArray>("access_rules");
          if (jarray3 != null)
          {
            foreach (JToken jtoken in jarray3)
              section.AccessibilityRules.AddRule(jtoken.Value<string>());
          }
          JArray jarray4 = jobject.GetValue<JArray>("visibility_rules");
          if (jarray4 != null)
          {
            foreach (JToken jtoken in jarray4)
              section.VisibilityRules.AddRule(jtoken.Value<string>());
          }
          JArray jarray5 = jobject.GetValue<JArray>("gate_access_rules");
          if (jarray5 != null)
          {
            foreach (JToken jtoken in jarray5)
              section.GateAccessibilityRules.AddRule(jtoken.Value<string>());
          }
          JArray jarray6 = jobject.GetValue<JArray>("gate_bypass_rules");
          if (jarray6 != null)
          {
            foreach (JToken jtoken in jarray6)
              section.GateBypassRules.AddRule(jtoken.Value<string>());
          }
          string path = jobject.GetValue<string>("thumbnail", (string) null);
          if (!string.IsNullOrWhiteSpace(path))
            section.Thumbnail = ImageReference.FromPackRelativePath(package, path);
          section.HostedItemCode = jobject.GetValue<string>("hosted_item", (string) null);
          section.GateItemCode = jobject.GetValue<string>("gate_item", (string) null);
          if (section.ChestCount > 0U || section.HostedItem != null)
            location1.AddSection(section);
        }
      }
      if (location1.HasLocalItems)
      {
        this.mVisibleLocations.Add(location1);
        if (location1.Group != null)
          location1.Group.AddLocation(location1);
        JArray jarray7 = data.GetValue<JArray>("map_locations");
        if (jarray7 != null)
        {
          foreach (JObject token in jarray7)
          {
            Map map = ObservableSingleton<MapDatabase>.Instance.FindMap(token.GetValue<string>("map", (string) null));
            if (map != null)
            {
              double num1 = token.GetValue<double>("x");
              double num2 = token.GetValue<double>("y");
              double num3 = token.GetValue<double>("size", -1.0);
              double num4 = token.GetValue<double>("border_thickness", -1.0);
              double num5 = token.GetValue<double>("badge_size", -1.0);
              double num6 = num3 < 0.0 ? map.LocationSize : num3;
              double num7 = num4 < 0.0 ? map.LocationBorderThickness : num4;
              MapLocation location2 = new MapLocation()
              {
                X = num1,
                Y = num2,
                Size = num6,
                BorderThickness = num7,
                Location = location1
              };
              location2.AlwaysVisible = token.GetValue<bool>("always_visible");
              location2.EnableBadgeHitTest = token.GetValue<bool>("enable_badge_hit_test");
              if (num5 > 0.0)
              {
                location2.OverrideBadgeSize = true;
                location2.BadgeSize = num5;
              }
              JArray jarray8 = token.GetValue<JArray>("restrict_visibility_rules", token.GetValue<JArray>("visibility_rules"));
              if (jarray8 != null)
              {
                foreach (JToken jtoken in jarray8)
                  location2.RestrictVisibilityRules.AddRule(jtoken.Value<string>());
              }
              JArray jarray9 = token.GetValue<JArray>("force_visibility_rules");
              if (jarray9 != null)
              {
                foreach (JToken jtoken in jarray9)
                  location2.ForceVisibilityRules.AddRule(jtoken.Value<string>());
              }
              JArray jarray10 = token.GetValue<JArray>("force_invisibility_rules");
              if (jarray10 != null)
              {
                foreach (JToken jtoken in jarray10)
                  location2.ForceInvisibilityRules.AddRule(jtoken.Value<string>());
              }
              map.AddLocation(location2);
            }
          }
        }
      }
      this.mAllLocations.Add(location1);
      JArray locationNodes = data.GetValue<JArray>("children");
      this.LoadLocationList(package, location1, locationNodes);
      return location1;
    }

    internal void Save(JObject root)
    {
      JObject jobject1 = new JObject();
      JArray jarray1 = new JArray();
      foreach (Location visibleLocation in this.VisibleLocations)
      {
        JObject jobject2 = new JObject();
        jobject2["location_reference"] = (JToken) this.GetPersistableLocationReference(visibleLocation);
        jobject2["modified_by_user"] = (JToken) visibleLocation.ModifiedByUser;
        JArray jarray2 = new JArray();
        foreach (Section section in visibleLocation.Sections)
        {
          JObject jobject3 = new JObject();
          jobject3["section_reference"] = (JToken) visibleLocation.GetPersistableSectionReference(section);
          jobject3["available_chest_count"] = (JToken) section.AvailableChestCount;
          if (section.CapturedItem != null)
            jobject3["captured_item"] = (JToken) Singleton<ItemDatabase>.Instance.GetPersistableItemReference(section.CapturedItem, true);
          jarray2.Add((JToken) jobject3);
        }
        if (jarray2.Count > 0)
          jobject2["sections"] = (JToken) jarray2;
        JArray jarray3 = visibleLocation.NoteTakingSite.AsJsonArray();
        if (jarray3 != null)
          jobject2["notes"] = (JToken) jarray3;
        jarray1.Add((JToken) jobject2);
      }
      if (jarray1.Count > 0)
        jobject1["locations"] = (JToken) jarray1;
      JArray jarray4 = new JArray();
      foreach (Location location in this.mPinnedLocations.Reverse<Location>())
        jarray4.Add((JToken) this.GetPersistableLocationReference(location));
      if (jarray4.Count > 0)
        jobject1["pinned_locations"] = (JToken) jarray4;
      if (!jobject1.Properties().Any<JProperty>())
        return;
      root["location_database"] = (JToken) jobject1;
    }

    internal bool Load(JObject root)
    {
      try
      {
        this.SuspendRefresh = true;
        JObject token1 = root.GetValue<JObject>("location_database");
        if (token1 == null)
          return true;
        JArray jarray1 = token1.GetValue<JArray>("locations");
        if (jarray1 == null)
          return false;
        foreach (JObject token2 in jarray1)
        {
          Location location = this.ResolvePersistableLocationReference(token2.GetValue<string>("location_reference", (string) null));
          if (location == null)
            return false;
          location.ModifiedByUser = token2.GetValue<bool>("modified_by_user");
          JArray jarray2 = token2.GetValue<JArray>("sections");
          if (jarray2 == null && location.Sections.Count<Section>() != 0 || jarray2 != null && location.Sections.Count<Section>() != jarray2.Count)
            return false;
          foreach (JObject token3 in jarray2)
          {
            Section section = location.ResolvePersistableSectionReference(token3.GetValue<string>("section_reference", (string) null));
            if (section == null)
              return false;
            section.AvailableChestCount = token3.GetValue<uint>("available_chest_count");
            string reference = token3.GetValue<string>("captured_item", (string) null);
            ITrackableItem trackableItem = Singleton<ItemDatabase>.Instance.ResolvePersistableItemReference(reference);
            if (!string.IsNullOrWhiteSpace(reference) && trackableItem == null || trackableItem != null && !section.CaptureItem)
              return false;
            section.CapturedItem = trackableItem;
          }
          location.NoteTakingSite.PopulateWithJsonArray(token2.GetValue<JArray>("notes"));
        }
        JArray jarray3 = token1.GetValue<JArray>("pinned_locations");
        if (jarray3 != null)
        {
          foreach (JToken reference in jarray3)
          {
            Location location = this.ResolvePersistableLocationReference((string) reference);
            if (location == null)
              return false;
            location.Pinned = true;
          }
        }
        return true;
      }
      finally
      {
        this.SuspendRefresh = false;
      }
    }

    public string GetPersistableLocationReference(Location location)
    {
      int num = this.mAllLocations.IndexOf(location);
      if (num < 0)
        throw new InvalidOperationException("Cannot generate persistable reference for location that is not in the LocationDatabase");
      return !string.IsNullOrWhiteSpace(location.Name) ? string.Format("{0}:{1}", (object) num, (object) Uri.EscapeDataString(location.Name)) : string.Format("{0}", (object) num);
    }

    public Location ResolvePersistableLocationReference(string reference)
    {
      if (string.IsNullOrWhiteSpace(reference))
        return (Location) null;
      string[] strArray = reference.Split(new char[1]{ ':' }, StringSplitOptions.RemoveEmptyEntries);
      if (strArray.Length < 1)
        return (Location) null;
      int result = -1;
      if (!int.TryParse(strArray[0], out result) || result < 0 || result > this.mAllLocations.Count - 1)
        return (Location) null;
      Location mAllLocation = this.mAllLocations[result];
      return strArray.Length >= 2 && !string.Equals(Uri.UnescapeDataString(strArray[1]), mAllLocation.Name, StringComparison.OrdinalIgnoreCase) ? (Location) null : mAllLocation;
    }

    public class SuspendRefreshScope : IDisposable
    {
      private bool mbSuspend;

      public SuspendRefreshScope()
      {
        this.mbSuspend = ObservableSingleton<LocationDatabase>.Instance.SuspendRefresh;
        ObservableSingleton<LocationDatabase>.Instance.SuspendRefresh = true;
      }

      public virtual void Dispose() => ObservableSingleton<LocationDatabase>.Instance.SuspendRefresh = this.mbSuspend;
    }
  }
}
