﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.Locations.Location
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using EmoTracker.Core;
using EmoTracker.Data.Core.Transactions;
using EmoTracker.Data.Media;
using EmoTracker.Data.Notes;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Linq;

namespace EmoTracker.Data.Locations
{
  public class Location : LocationVisualProperties
  {
    private ObservableCollection<Location> mChildren = new ObservableCollection<Location>();
    private ObservableCollection<Section> mSections = new ObservableCollection<Section>();
    private ObservableCollection<ImageReference> mBadges = new ObservableCollection<ImageReference>();
    private AccessibilityRuleSet mAccessibility = new AccessibilityRuleSet();
    private AccessibilityLevel mCachedAccessibility;
    private AccessibilityLevel mCachedBaseAccessibility;
    private Location mParent;
    private Group mGroup;
    private ImageReference mThumbnail;
    private string mName;
    private string mShortName;
    private string mColor;
    private bool mModifiedByUser;
    private bool mbHasAvailableItems;
    private bool mbHasVisibleSections;
    private NoteTakingSite mNoteTakingSite = new NoteTakingSite();

    private static AccessibilityLevel Min(
      AccessibilityLevel a,
      AccessibilityLevel b)
    {
      return a >= b ? b : a;
    }

    private static AccessibilityLevel Max(
      AccessibilityLevel a,
      AccessibilityLevel b)
    {
      return a <= b ? b : a;
    }

    public Section FindSection(string name)
    {
      foreach (Section section in this.Sections)
      {
        if (name.Equals(section.Name, StringComparison.OrdinalIgnoreCase))
          return section;
      }
      return (Section) null;
    }

    public void FullClearAllPossible()
    {
      using (TransactionProcessor.Current.OpenTransaction())
      {
        int num = 0;
        foreach (Section section in this.Sections)
        {
          if ((this.AlwaysAllowChestManipulation || section.AccessibilityLevel >= AccessibilityLevel.Unlockable || ObservableSingleton<ApplicationSettings>.Instance.AlwaysAllowClearing) && section.Visible)
          {
            if (section.HostedItem != null)
              section.HostedItem.AdvanceToCode(section.HostedItemCode);
            section.AvailableChestCount = 0U;
            ++num;
          }
        }
        if (num < this.Sections.Count<Section>() || !this.AutoUnpinOnClear)
          return;
        this.Pinned = false;
      }
    }

    public string Name
    {
      get => this.mName;
      set
      {
        this.SetProperty<string>(ref this.mName, value, nameof (Name));
        this.NotifyPropertyChanged("ShortName");
      }
    }

    public string ShortName
    {
      get => this.mShortName != null ? this.mShortName : this.Name;
      set => this.SetProperty<string>(ref this.mShortName, value, nameof (ShortName));
    }

    public bool ModifiedByUser
    {
      get => this.mModifiedByUser;
      set => this.SetProperty<bool>(ref this.mModifiedByUser, value, nameof (ModifiedByUser));
    }

    public AccessibilityRuleSet AccessibilityRules => this.mAccessibility;

    public AccessibilityLevel BaseAccessibilityLevel => this.mCachedBaseAccessibility;

    public AccessibilityLevel AccessibilityLevel => this.mCachedAccessibility;

    public bool HasLocalItems => this.mSections.Count > 0;

    public bool HasAvailableItems
    {
      get => this.mbHasAvailableItems;
      set
      {
        this.mbHasAvailableItems = value;
        this.NotifyPropertyChanged(nameof (HasAvailableItems));
      }
    }

    public bool HasVisibleSections
    {
      get => this.mbHasVisibleSections;
      set => this.SetProperty<bool>(ref this.mbHasVisibleSections, value, nameof (HasVisibleSections));
    }

    public Location Parent
    {
      get => this.mParent;
      set
      {
        if (!this.SetProperty<Location>(ref this.mParent, value, nameof (Parent)))
          return;
        this.VisualParent = (LocationVisualProperties) this.mParent;
      }
    }

    public Group Group
    {
      get
      {
        if (this.mGroup != null)
          return this.mGroup;
        return this.Parent != null ? this.Parent.Group : (Group) null;
      }
      set
      {
        this.mGroup = value;
        this.NotifyPropertyChanged(nameof (Group));
      }
    }

    public string Color
    {
      get
      {
        if (this.mColor != null)
          return this.mColor;
        return this.mParent != null ? this.mParent.Color : (string) null;
      }
      set => this.SetProperty<string>(ref this.mColor, value, nameof (Color));
    }

    public IEnumerable<Location> Children => (IEnumerable<Location>) this.mChildren;

    public IEnumerable<Section> Sections => (IEnumerable<Section>) this.mSections;

    public ImageReference Thumbnail
    {
      get => this.mThumbnail;
      set
      {
        this.mThumbnail = value;
        this.NotifyPropertyChanged(nameof (Thumbnail));
      }
    }

    [TransactablePropertyReadBehavior(TransactablePropertyReadBehavior.AllowOpenTransactionRead)]
    public bool Pinned
    {
      get => this.GetTransactableProperty<bool>(nameof (Pinned));
      set => this.ForceSetTransactableProperty<bool>(value, (Action<bool>) (processedValue =>
      {
        if (processedValue)
          ObservableSingleton<LocationDatabase>.Instance.PinLocation(this);
        else
          ObservableSingleton<LocationDatabase>.Instance.UnpinLocation(this);
      }), nameof (Pinned));
    }

    public bool AddChild(Location child)
    {
      if (child == null || this.mChildren.Contains(child))
        return false;
      this.mChildren.Add(child);
      return true;
    }

    public bool AddSection(Section section)
    {
      if (section == null || this.mSections.Contains(section))
        return false;
      this.mSections.Add(section);
      return true;
    }

    public uint AvailableItemCount
    {
      get
      {
        uint availableItemCount = 0;
        foreach (Section section in this.Sections)
        {
          availableItemCount += section.AvailableChestCount;
          if (section.HostedItem != null && section.HostedItem.ProvidesCode(section.HostedItemCode) == 0U)
            ++availableItemCount;
        }
        return availableItemCount;
      }
    }

    public void AutoUnpinIfAppropriate()
    {
      if (!this.AutoUnpinOnClear || !this.Pinned)
        return;
      foreach (Section section in this.Sections)
      {
        if (section.Visible && section.HasUnclaimedItems)
          return;
      }
      this.Pinned = false;
    }

    public void RefreshAccessibility()
    {
      AccessibilityLevel baseAccessibility = this.mCachedBaseAccessibility;
      int cachedAccessibility = (int) this.mCachedAccessibility;
      this.mCachedBaseAccessibility = this.AccessibilityRules.Accessibility;
      if (this.Parent != null)
        this.mCachedBaseAccessibility = Location.Min(this.mCachedBaseAccessibility, this.Parent.BaseAccessibilityLevel);
      AccessibilityLevel a1 = AccessibilityLevel.None;
      AccessibilityLevel a2 = AccessibilityLevel.Normal;
      this.HasAvailableItems = false;
      this.HasVisibleSections = false;
      if (this.mSections.Count > 0)
      {
        Dictionary<string, uint> dictionary = new Dictionary<string, uint>();
        HashSet<IConsumingItem> consumingItemSet = new HashSet<IConsumingItem>();
        int num1 = 0;
        int num2 = 0;
        int num3 = 0;
        int num4 = 0;
        uint num5 = 0;
        uint num6 = 0;
        foreach (Section section in this.Sections)
        {
          section.RefreshAccessibility();
          if (section.Visible)
          {
            this.HasVisibleSections = true;
            if (section.HasUnclaimedItems)
            {
              ++num4;
              if (section.AccessibilityLevel >= AccessibilityLevel.SequenceBreak)
                a2 = Location.Min(a2, section.AccessibilityLevel);
              if (section.AccessibilityLevel >= AccessibilityLevel.Unlockable)
              {
                a1 = Location.Max(a1, section.AccessibilityLevel);
                ++num1;
                if (section.HasUnclaimedItems)
                {
                  ++num2;
                  if (section.AccessibilityLevel == AccessibilityLevel.Inspect)
                  {
                    ++num3;
                    num6 += section.AvailableChestCount;
                  }
                }
                num5 += section.AvailableChestCount;
                if (section.HostedItem != null && section.HostedItem.ProvidesCode(section.HostedItemCode) == 0U)
                  ++num5;
                if (section.AccessibilityLevel == AccessibilityLevel.Unlockable && section.GateItem is IConsumingItem gateItem && !consumingItemSet.Contains(gateItem))
                {
                  string code;
                  uint count;
                  if (gateItem.GetPotentialConsumedItem(out code, out count) && count > 0U && !string.IsNullOrWhiteSpace(code))
                    dictionary[code] = !dictionary.ContainsKey(code) ? count : count + dictionary[code];
                  consumingItemSet.Add(gateItem);
                }
              }
            }
          }
        }
        this.mCachedAccessibility = this.mCachedBaseAccessibility;
        if (num5 > 0U)
        {
          this.HasAvailableItems = true;
          if (this.Group != null)
            this.Group.HasAvailableItems = true;
          this.mCachedAccessibility = num2 < num4 || num6 > 0U && num6 < num5 ? AccessibilityLevel.Partial : (a1 < AccessibilityLevel.SequenceBreak ? a1 : a2);
          foreach (KeyValuePair<string, uint> keyValuePair in dictionary)
          {
            if (Singleton<ItemDatabase>.Instance.ProviderCountForCode(keyValuePair.Key, out AccessibilityLevel _) < keyValuePair.Value)
            {
              this.mCachedAccessibility = AccessibilityLevel.Partial;
              break;
            }
          }
        }
        else if (num4 > 0)
          this.mCachedAccessibility = AccessibilityLevel.None;
        else if (num4 == 0)
        {
          this.mCachedAccessibility = AccessibilityLevel.Cleared;
          if (this.AutoUnpinOnClear)
            this.Pinned = false;
          if (this.mCachedBaseAccessibility != baseAccessibility)
            ObservableSingleton<LocationDatabase>.Instance.LastClearedLocation = this;
        }
      }
      else
        this.mCachedAccessibility = this.mCachedBaseAccessibility;
      this.NotifyPropertyChanged("BaseAccessibilityLevel");
      this.NotifyPropertyChanged("AccessibilityLevel");
      foreach (Location child in this.Children)
        child.RefreshAccessibility();
    }

    public string GetPersistableSectionReference(Section section)
    {
      int num = this.mSections.IndexOf(section);
      if (num < 0)
        throw new InvalidOperationException("Cannot generate persistable reference for section that is not in the parent location");
      return !string.IsNullOrWhiteSpace(section.Name) ? string.Format("{0}:{1}", (object) num, (object) Uri.EscapeDataString(section.Name)) : string.Format("{0}", (object) num);
    }

    public Section ResolvePersistableSectionReference(string reference)
    {
      if (string.IsNullOrWhiteSpace(reference))
        return (Section) null;
      string[] strArray = reference.Split(new char[1]{ ':' }, StringSplitOptions.RemoveEmptyEntries);
      if (strArray.Length < 1)
        return (Section) null;
      int result = -1;
      if (!int.TryParse(strArray[0], out result) || result < 0 || result > this.mSections.Count - 1)
        return (Section) null;
      Section mSection = this.mSections[result];
      return strArray.Length >= 2 && !string.Equals(Uri.UnescapeDataString(strArray[1]), mSection.Name, StringComparison.OrdinalIgnoreCase) ? (Section) null : mSection;
    }

    public NoteTakingSite NoteTakingSite => this.mNoteTakingSite;

    public IEnumerable<ImageReference> Badges => (IEnumerable<ImageReference>) this.mBadges;

    public bool HasBadges => this.mBadges.Count > 0;

    public ImageReference AddBadge(string imageRef, string filterSpec = null)
    {
      try
      {
        ImageReference imageReference = ImageReference.FromPackRelativePath(ObservableSingleton<Tracker>.Instance.ActiveGamePackage, imageRef, filterSpec);
        this.mBadges.Add(imageReference);
        return imageReference;
      }
      catch
      {
        return (ImageReference) null;
      }
    }

    public ImageReference AddBadge(ImageReference imageRef, string filter = null)
    {
      try
      {
        ImageReference imageReference = ImageReference.FromImageReference(imageRef, filter);
        this.mBadges.Add(imageReference);
        return imageReference;
      }
      catch
      {
        return (ImageReference) null;
      }
    }

    public void RemoveBadge(ImageReference badge)
    {
      try
      {
        if (badge == null || !this.mBadges.Contains(badge))
          return;
        this.mBadges.Remove(badge);
      }
      catch
      {
      }
    }

    private void Badges_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e) => this.NotifyPropertyChanged("HasBadges");

    public Location() => this.mBadges.CollectionChanged += new NotifyCollectionChangedEventHandler(this.Badges_CollectionChanged);
  }
}
