﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.Locations.MapLocation
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using EmoTracker.Core;
using EmoTracker.Data.Media;
using System;
using System.ComponentModel;

namespace EmoTracker.Data.Locations
{
  public class MapLocation : ObservableObject
  {
    private AccessibilityRuleSet mRestrictVisibilityRules = new AccessibilityRuleSet();
    private AccessibilityRuleSet mForceVisibilityRules = new AccessibilityRuleSet();
    private AccessibilityRuleSet mForceInvisibilityRules = new AccessibilityRuleSet();
    private Location mLocation;
    private double mX;
    private double mY;
    private double mSize;
    private double mBorderThickness = 8.0;
    private Thickness mItemMargin = new Thickness(-35.0, -35.0, 0.0, 0.0);
    private double mBadgeSize = 35.0;
    private Thickness mBadgeMargin = new Thickness(-17.5, -17.5, 0.0, 0.0);
    private double mNoteIndicatorSize = 35.0;
    private Thickness mNoteIndicatorMargin = new Thickness(-17.5, -17.5, 0.0, 0.0);
    private bool mbAlwaysVisible;
    private bool mbBadgeSizeOverride;
    private bool mbEnableBadgeHitTest;
    private bool mbVisibilityDirty = true;

    public bool AlwaysVisible
    {
      get => this.mbAlwaysVisible;
      set => this.SetProperty<bool>(ref this.mbAlwaysVisible, value, nameof (AlwaysVisible));
    }

    public bool OverrideBadgeSize
    {
      get => this.mbBadgeSizeOverride;
      set => this.SetProperty<bool>(ref this.mbBadgeSizeOverride, value, nameof (OverrideBadgeSize));
    }

    public bool EnableBadgeHitTest
    {
      get => this.mbEnableBadgeHitTest;
      set => this.SetProperty<bool>(ref this.mbEnableBadgeHitTest, value, nameof (EnableBadgeHitTest));
    }

    public AccessibilityRuleSet RestrictVisibilityRules => this.mRestrictVisibilityRules;

    public AccessibilityRuleSet ForceVisibilityRules => this.mForceVisibilityRules;

    public AccessibilityRuleSet ForceInvisibilityRules => this.mForceInvisibilityRules;

    public bool ForceVisible => this.AlwaysVisible || !this.ForceVisibilityRules.Empty && this.ForceVisibilityRules.AccessibilityForVisibility == AccessibilityLevel.Normal;

    public bool ForceInvisible => !this.AlwaysVisible && (!this.ForceInvisibilityRules.Empty && this.ForceInvisibilityRules.AccessibilityForVisibility == AccessibilityLevel.Normal || !this.RestrictVisibilityRules.Empty && this.RestrictVisibilityRules.AccessibilityForVisibility != AccessibilityLevel.Normal);

    public double X
    {
      get => this.mX;
      set => this.SetProperty<double>(ref this.mX, value, nameof (X));
    }

    public double Y
    {
      get => this.mY;
      set => this.SetProperty<double>(ref this.mY, value, nameof (Y));
    }

    public double Size
    {
      get => this.mSize;
      set
      {
        this.SetProperty<double>(ref this.mSize, value, nameof (Size));
        this.mItemMargin.Top = this.mItemMargin.Left = -1.0 * (this.mSize / 2.0);
        this.NotifyPropertyChanged("LocationMargin");
        this.mNoteIndicatorSize = Math.Min(this.mSize * 0.75, 50.0);
        this.mNoteIndicatorMargin = new Thickness(0.0, this.mNoteIndicatorSize * -0.5, this.mNoteIndicatorSize * -0.5, 0.0);
        this.NotifyPropertyChanged("NoteIndicatorSize");
        this.NotifyPropertyChanged("NoteIndicatorMargin");
        if (!this.OverrideBadgeSize)
          this.BadgeSize = this.mSize * 0.75;
        this.UpdateBadgeMargin();
      }
    }

    private void UpdateBadgeMargin()
    {
      this.mBadgeMargin = new Thickness(this.mSize * 0.5 + this.mBadgeSize * -0.5, this.mSize * 0.5 + this.mBadgeSize * -0.5, 0.0, 0.0);
      this.NotifyPropertyChanged("BadgeMargin");
    }

    public double BorderThickness
    {
      get => this.mBorderThickness;
      set => this.SetProperty<double>(ref this.mBorderThickness, value, nameof (BorderThickness));
    }

    public Thickness Margin => this.mItemMargin;

    public double BadgeSize
    {
      get => this.mBadgeSize;
      set
      {
        if (!this.SetProperty<double>(ref this.mBadgeSize, value, nameof (BadgeSize)))
          return;
        this.UpdateBadgeMargin();
      }
    }

    public Thickness BadgeMargin => this.mBadgeMargin;

    public double NoteIndicatorSize => this.mNoteIndicatorSize;

    public Thickness NoteIndicatorMargin => this.mNoteIndicatorMargin;

    public Location Location
    {
      get => this.mLocation;
      set
      {
        if (!this.SetProperty<Location>(ref this.mLocation, value, nameof (Location)) || this.mLocation == null)
          return;
        this.mLocation.PropertyChanged += new PropertyChangedEventHandler(this.MLocation_PropertyChanged);
      }
    }

    public override void Dispose()
    {
      if (this.Location != null)
        this.Location.PropertyChanged -= new PropertyChangedEventHandler(this.MLocation_PropertyChanged);
      this.Location = (Location) null;
      base.Dispose();
    }

    private void MLocation_PropertyChanged(object sender, PropertyChangedEventArgs e) => this.MarkVisibilityDirty();

    internal void MarkVisibilityDirty() => this.mbVisibilityDirty = true;

    internal void UpdateVisibilityIfNecessary()
    {
      if (!this.mbVisibilityDirty)
        return;
      try
      {
        this.NotifyPropertyChanged("ForceVisible");
        this.NotifyPropertyChanged("ForceInvisible");
      }
      finally
      {
        this.mbVisibilityDirty = false;
      }
    }
  }
}
