﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.Locations.Section
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using EmoTracker.Core;
using EmoTracker.Data.Core.Transactions;
using EmoTracker.Data.Media;
using System;
using System.Collections.Generic;

namespace EmoTracker.Data.Locations
{
  public class Section : LocationVisualProperties
  {
    private Location mOwner;
    private string mName;
    private string mShortName;
    private string mGateCode;
    private string mItemCaptureLayout;
    private uint mnNumChests;
    private ImageReference mThumbnail;
    private AccessibilityRuleSet mGateAccessibilityRules = new AccessibilityRuleSet();
    private AccessibilityRuleSet mGateBypassRules = new AccessibilityRuleSet();
    private AccessibilityRuleSet mAccessibilityRules = new AccessibilityRuleSet();
    private AccessibilityRuleSet mVisibilityRules = new AccessibilityRuleSet();
    private ITrackableItem mGateItem;
    private AccessibilityLevel mCachedAccessibility;
    private AccessibilityLevel mCachedGateAccessibility;
    private bool mbCachedVisibilty = true;
    private bool mbClearAsGroup;
    private bool mbCaptureItem;
    private bool mbShowGateItem = true;
    private string mHostedItemCode;

    private static AccessibilityLevel Min(
      AccessibilityLevel a,
      AccessibilityLevel b)
    {
      return a >= b ? b : a;
    }

    private static AccessibilityLevel Max(
      AccessibilityLevel a,
      AccessibilityLevel b)
    {
      return a <= b ? b : a;
    }

    public Section(Location owner)
    {
      this.VisualParent = (LocationVisualProperties) owner;
      this.mOwner = owner;
    }

    public Location Owner => this.mOwner;

    public string Name
    {
      get => this.mName;
      set
      {
        this.SetProperty<string>(ref this.mName, value, nameof (Name));
        this.NotifyPropertyChanged("ShortName");
      }
    }

    public string ShortName
    {
      get => this.mShortName != null ? this.mShortName : this.Name;
      set => this.SetProperty<string>(ref this.mShortName, value, nameof (ShortName));
    }

    public string ItemCaptureLayout
    {
      get => this.mItemCaptureLayout;
      set => this.SetProperty<string>(ref this.mItemCaptureLayout, value, nameof (ItemCaptureLayout));
    }

    public ImageReference Thumbnail
    {
      get => this.mThumbnail;
      set
      {
        this.mThumbnail = value;
        this.NotifyPropertyChanged(nameof (Thumbnail));
      }
    }

    public ITrackableItem HostedItem
    {
      get => this.GetTransactableProperty<ITrackableItem>(nameof (HostedItem));
      private set => this.SetTransactableProperty<ITrackableItem>(value, propertyName: nameof (HostedItem));
    }

    [TransactablePropertyReadBehavior(TransactablePropertyReadBehavior.AllowOpenTransactionRead)]
    public ITrackableItem CapturedItem
    {
      get => this.GetTransactableProperty<ITrackableItem>(nameof (CapturedItem));
      set
      {
        using (TransactionProcessor.Current.OpenTransaction())
        {
          if (!this.SetTransactableProperty<ITrackableItem>(value, (Action<ITrackableItem>) (processedValue => ObservableSingleton<LocationDatabase>.Instance.RefeshAccessibility()), nameof (CapturedItem)))
            return;
          this.Owner.AutoUnpinIfAppropriate();
        }
      }
    }

    public string GateItemCode
    {
      get => this.mGateCode;
      set
      {
        if (!this.SetProperty<string>(ref this.mGateCode, value, nameof (GateItemCode)))
          return;
        this.GateItem = Singleton<ItemDatabase>.Instance.FindProvidingItemForCode(this.mGateCode);
      }
    }

    public ITrackableItem GateItem
    {
      get => this.mGateItem;
      private set
      {
        if (!this.SetProperty<ITrackableItem>(ref this.mGateItem, value, nameof (GateItem)))
          return;
        ObservableSingleton<LocationDatabase>.Instance.RefeshAccessibility();
      }
    }

    public string HostedItemCode
    {
      get => this.mHostedItemCode;
      set
      {
        this.mHostedItemCode = value;
        this.NotifyPropertyChanged(nameof (HostedItemCode));
        this.HostedItem = Singleton<ItemDatabase>.Instance.FindProvidingItemForCode(this.mHostedItemCode);
      }
    }

    public uint ChestCount
    {
      get => this.mnNumChests;
      set
      {
        this.mnNumChests = value;
        this.NotifyPropertyChanged(nameof (ChestCount));
      }
    }

    [TransactablePropertyReadBehavior(TransactablePropertyReadBehavior.AllowOpenTransactionRead)]
    public uint AvailableChestCount
    {
      get => this.GetTransactableProperty<uint>(nameof (AvailableChestCount));
      set
      {
        using (TransactionProcessor.Current.OpenTransaction())
        {
          if (value == 0U && this.CapturedItem != null)
          {
            this.CapturedItem.AdvanceToCode();
            this.CapturedItem = (ITrackableItem) null;
          }
          if (!this.SetTransactableProperty<uint>(value, (Action<uint>) (processedValue => ObservableSingleton<LocationDatabase>.Instance.RefeshAccessibility()), nameof (AvailableChestCount)))
            return;
          this.Owner.AutoUnpinIfAppropriate();
        }
      }
    }

    public bool HasUnclaimedItems => this.AvailableChestCount > 0U || this.HostedItem != null && this.HostedItem.ProvidesCode(this.HostedItemCode) == 0U;

    public bool ClearAsGroup
    {
      get => this.mbClearAsGroup;
      set
      {
        this.mbClearAsGroup = value;
        this.NotifyPropertyChanged(nameof (ClearAsGroup));
      }
    }

    public bool CaptureItem
    {
      get => this.mbCaptureItem;
      set
      {
        this.mbCaptureItem = value;
        this.NotifyPropertyChanged(nameof (CaptureItem));
      }
    }

    public bool ShowGateItem
    {
      get => this.mbShowGateItem;
      set => this.SetProperty<bool>(ref this.mbShowGateItem, value, nameof (ShowGateItem));
    }

    public AccessibilityRuleSet GateAccessibilityRules => this.mGateAccessibilityRules;

    public AccessibilityRuleSet GateBypassRules => this.mGateBypassRules;

    public AccessibilityRuleSet AccessibilityRules => this.mAccessibilityRules;

    public AccessibilityRuleSet VisibilityRules => this.mVisibilityRules;

    public AccessibilityLevel AccessibilityLevel => this.mCachedAccessibility;

    public AccessibilityLevel GateAccessibilityLevel => this.mCachedGateAccessibility;

    public bool Visible => this.mbCachedVisibilty;

    public void ComputeGateDependencies(
      Dictionary<string, uint> aggregateGateRequirements,
      bool bIsRoot = true)
    {
      string code1;
      uint count;
      if ((bIsRoot || this.AccessibilityLevel == AccessibilityLevel.Unlockable) && this.GateItem is IConsumingItem gateItem && gateItem.GetPotentialConsumedItem(out code1, out count))
        aggregateGateRequirements[code1] = !aggregateGateRequirements.ContainsKey(code1) ? count : count + aggregateGateRequirements[code1];
      foreach (AccessibilityRule rule in this.GateAccessibilityRules.Rules)
      {
        foreach (AccessibilityRule.CodeRule code2 in rule.Codes)
        {
          if (ObservableSingleton<Tracker>.Instance.FindObjectForCode(code2.mCode) is Section objectForCode)
            objectForCode.ComputeGateDependencies(aggregateGateRequirements, false);
        }
      }
      foreach (AccessibilityRule rule in this.AccessibilityRules.Rules)
      {
        foreach (AccessibilityRule.CodeRule code3 in rule.Codes)
        {
          if (ObservableSingleton<Tracker>.Instance.FindObjectForCode(code3.mCode) is Section objectForCode)
            objectForCode.ComputeGateDependencies(aggregateGateRequirements, false);
        }
      }
    }

    public void RefreshAccessibility()
    {
      if (this.GateItem != null)
      {
        this.mCachedGateAccessibility = Section.Min(this.mOwner.BaseAccessibilityLevel, this.GateAccessibilityRules.AccessibilityWithoutModifiers);
        if (this.GateItem.ProvidesCode(this.GateItemCode) > 0U)
          this.mCachedGateAccessibility = AccessibilityLevel.Normal;
      }
      this.mCachedAccessibility = this.CapturedItem == null ? Section.Min(this.mOwner.BaseAccessibilityLevel, this.AccessibilityRules.Accessibility) : Section.Min(this.mOwner.BaseAccessibilityLevel, this.AccessibilityRules.AccessibilityWithoutModifiers);
      if (this.mCachedAccessibility >= AccessibilityLevel.Inspect && this.GateItem != null && this.GateItem.ProvidesCode(this.GateItemCode) == 0U)
      {
        Dictionary<string, uint> aggregateGateRequirements = new Dictionary<string, uint>();
        this.ComputeGateDependencies(aggregateGateRequirements);
        string code;
        uint count;
        if (this.GateItem is IConsumingItem gateItem && gateItem.GetPotentialConsumedItem(out code, out count))
        {
          uint num1 = count;
          if (aggregateGateRequirements.ContainsKey(code))
            num1 = aggregateGateRequirements[code];
          uint num2 = Singleton<ItemDatabase>.Instance.ProviderCountForCode(code, out AccessibilityLevel _);
          AccessibilityLevel a = this.GateBypassRules.Empty || num2 < num1 - count ? AccessibilityLevel.None : this.GateBypassRules.AccessibilityWithoutModifiers;
          AccessibilityLevel b = num2 < num1 || this.GateAccessibilityLevel < AccessibilityLevel.Unlockable ? AccessibilityLevel.None : this.GateAccessibilityLevel;
          this.mCachedAccessibility = a > AccessibilityLevel.SequenceBreak || b < AccessibilityLevel.SequenceBreak ? Section.Max(Section.Min(a, this.mCachedAccessibility), b) : AccessibilityLevel.Unlockable;
        }
      }
      this.mbCachedVisibilty = this.VisibilityRules.AccessibilityForVisibility >= AccessibilityLevel.Normal;
      this.NotifyPropertyChanged("AccessibilityLevel");
      this.NotifyPropertyChanged("GateAccessibilityLevel");
      this.NotifyPropertyChanged("Visible");
    }
  }
}
