﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.MapDatabase
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using EmoTracker.Core;
using EmoTracker.Data.JSON;
using EmoTracker.Data.Locations;
using EmoTracker.Data.Media;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;

namespace EmoTracker.Data
{
  public class MapDatabase : ObservableSingleton<MapDatabase>
  {
    private ObservableCollection<Map> mMaps = new ObservableCollection<Map>();

    public IEnumerable<Map> Maps => (IEnumerable<Map>) this.mMaps;

    public void Reset()
    {
      this.DisposeCollection<Map>((IEnumerable<Map>) this.mMaps);
      this.mMaps.Clear();
    }

    public bool LegacyLoad(IGamePackage package)
    {
      if (this.mMaps.Count > 0)
        return true;
      ObservableSingleton<ScriptManager>.Instance.OutputWarning("Loading Legacy Maps");
      using (new LoggingBlock())
        return this.IncrementalLoad("maps.json", package);
    }

    internal bool IncrementalLoad(string path, IGamePackage package)
    {
      ObservableSingleton<ScriptManager>.Instance.Output("Loading Maps: {0}", (object) path);
      using (new LoggingBlock())
      {
        try
        {
          using (Stream stream = package.Open(path))
          {
            if (stream != null)
            {
              using (StreamReader reader = new StreamReader(stream))
              {
                foreach (JObject token in (JArray) JToken.ReadFrom((JsonReader) new JsonTextReader((TextReader) reader)))
                  this.mMaps.Add(new Map()
                  {
                    Name = token.GetValue<string>("name", (string) null),
                    LocationSize = token.GetValue<double>("location_size", 70.0),
                    LocationBorderThickness = token.GetValue<double>("location_border_thickness", 8.0),
                    Image = ImageReference.FromPackRelativePath(package, token.GetValue<string>("img", (string) null))
                  });
              }
            }
            else
              ObservableSingleton<ScriptManager>.Instance.Output("File not found");
          }
          return true;
        }
        catch (Exception ex)
        {
          ObservableSingleton<ScriptManager>.Instance.OutputException(ex);
          return false;
        }
      }
    }

    public Map FindMap(string name)
    {
      if (!string.IsNullOrWhiteSpace(name))
      {
        foreach (Map mMap in (Collection<Map>) this.mMaps)
        {
          if (name.Equals(mMap.Name, StringComparison.OrdinalIgnoreCase))
            return mMap;
        }
      }
      return (Map) null;
    }

    internal void MarkVisibilityDirty()
    {
      foreach (Map mMap in (Collection<Map>) this.mMaps)
      {
        foreach (MapLocation location in mMap.Locations)
          location.MarkVisibilityDirty();
      }
    }

    internal void UpdateVisibilityIfNecessary()
    {
      foreach (Map mMap in (Collection<Map>) this.mMaps)
      {
        foreach (MapLocation location in mMap.Locations)
          location.UpdateVisibilityIfNecessary();
      }
    }
  }
}
