﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.Packages.GamePackage
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using EmoTracker.Core;
using EmoTracker.Data.JSON;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;

namespace EmoTracker.Data.Packages
{
  public class GamePackage : ObservableObject, IGamePackage, IMenuItemsProvider
  {
    private IGamePackageSource mSource;
    private string mName;
    private string mGameName;
    private string mGameVariant;
    private string mAuthor;
    private string mPackageUID;
    private Version mVersion;
    private Version mLayoutEngineVersion;
    private GamePlatform mPlatform;
    private bool mUnsafe = true;
    private ObservableCollection<IGamePackageVariant> mAvailableVariants = new ObservableCollection<IGamePackageVariant>();
    private GamePackage.Variant mActiveVariant;
    private string mOverridePath;

    public string OverridePath
    {
      get
      {
        if (!string.IsNullOrEmpty(this.mOverridePath) || string.IsNullOrWhiteSpace(this.mPackageUID))
          return this.mOverridePath;
        this.mOverridePath = Path.Combine(UserDirectory.Path, "user_overrides", this.mPackageUID);
        return this.mOverridePath;
      }
    }

    public string VariantOverridePath
    {
      get
      {
        if (!string.IsNullOrEmpty(this.mOverridePath) || string.IsNullOrWhiteSpace(this.mPackageUID))
          return this.mOverridePath;
        this.mOverridePath = Path.Combine(UserDirectory.Path, "user_overrides", this.mPackageUID);
        return this.mOverridePath;
      }
    }

    public GamePackage(IGamePackageSource source)
    {
      this.mSource = source;
      this.LoadManifest();
    }

    public IGamePackageSource Source => this.mSource;

    public bool IsActive => ObservableSingleton<Tracker>.Instance.ActiveGamePackage == this;

    public string Name => this.mName;

    public string DisplayName => this.mName ?? this.mPackageUID;

    public GamePlatform Platform => this.mPlatform;

    public string Game => this.mGameName;

    public string GameVariant => this.mGameVariant;

    public string Author => this.mAuthor;

    public string UniqueID => this.mPackageUID;

    public Version Version => this.mVersion;

    public Version LayoutEngineVersion => this.mLayoutEngineVersion;

    public bool FlaggedAsUnsafe => this.mUnsafe;

    string IGamePackage.OverridePath => this.OverridePath;

    [DependentProperty("VariantOverridePath")]
    private GamePackage.Variant ActiveVariant
    {
      get => this.mActiveVariant;
      set => this.SetProperty<GamePackage.Variant>(ref this.mActiveVariant, value, nameof (ActiveVariant));
    }

    public IEnumerable<IGamePackageVariant> AvailableVariants => (IEnumerable<IGamePackageVariant>) this.mAvailableVariants;

    public IGamePackageVariant FindVariant(string uid)
    {
      foreach (IGamePackageVariant availableVariant in this.AvailableVariants)
      {
        if (string.Equals(uid, availableVariant.UniqueID, StringComparison.OrdinalIgnoreCase))
          return availableVariant;
      }
      return (IGamePackageVariant) null;
    }

    IGamePackageVariant IGamePackage.ActiveVariant
    {
      get => (IGamePackageVariant) this.mActiveVariant;
      set => this.ActiveVariant = value as GamePackage.Variant;
    }

    public bool IsValid => !string.IsNullOrWhiteSpace(this.mPackageUID) && this.Version != (Version) null;

    IEnumerable<object> IMenuItemsProvider.Items => (IEnumerable<object>) this.mAvailableVariants;

    public void ResetUserOverrides()
    {
      if (!Directory.Exists(this.OverridePath))
        return;
      try
      {
        Directory.Delete(this.OverridePath, true);
      }
      catch
      {
      }
    }

    public void ExportUserOverride(string filename) => this.DeploySourceFileToOverride(filename, true);

    private string NormalizePath(string path)
    {
      if (string.IsNullOrWhiteSpace(path))
        return (string) null;
      while (path.StartsWith("/"))
        path = path.Substring(1);
      path = path.Replace(Path.AltDirectorySeparatorChar, '/');
      path = path.Replace(Path.DirectorySeparatorChar, '/');
      return path;
    }

    private Stream OpenOverrideUsingRoot(string path, string root)
    {
      if (!string.IsNullOrWhiteSpace(path) && !string.IsNullOrWhiteSpace(this.OverridePath) && Directory.Exists(this.OverridePath))
      {
        string path1 = Path.Combine(root, path);
        if (File.Exists(path1))
          return (Stream) File.OpenRead(path1);
      }
      return (Stream) null;
    }

    public Stream Open(string path, bool ignoreVariants = false, bool ignoreOverrides = false)
    {
      if (string.IsNullOrWhiteSpace(path))
        return (Stream) null;
      path = this.NormalizePath(path);
      Stream stream = (Stream) null;
      if (!ignoreOverrides && !ignoreVariants && stream == null && this.ActiveVariant != null)
        stream = this.OpenOverrideUsingRoot(path, this.ActiveVariant.OverridePath);
      if (!ignoreVariants && stream == null && this.ActiveVariant != null)
        stream = this.mSource.Open(this.NormalizePath(Path.Combine(this.ActiveVariant.BasePath, path)));
      if (!ignoreOverrides && stream == null)
        stream = this.OpenOverrideUsingRoot(path, this.OverridePath);
      if (stream == null)
        stream = this.mSource.Open(path);
      return stream;
    }

    public bool Exists(string path, bool ignoreVariants = false, bool ignoreOverrides = false)
    {
      using (Stream stream = this.Open(path, ignoreVariants, ignoreOverrides))
        return stream != null;
    }

    private void LoadManifest()
    {
      this.mAvailableVariants.Clear();
      if (this.mSource == null)
        return;
      try
      {
        using (Stream stream = this.mSource.Open("manifest.json"))
        {
          if (stream == null)
            return;
          using (StreamReader reader = new StreamReader(stream))
          {
            JObject token1 = (JObject) JToken.ReadFrom((JsonReader) new JsonTextReader((TextReader) reader));
            if (token1 == null)
              return;
            this.mName = token1.GetValue<string>("name", (string) null);
            this.mPlatform = token1.GetEnumValue<GamePlatform>("platform");
            this.mGameName = token1.GetValue<string>("game_name", (string) null);
            this.mGameVariant = token1.GetValue<string>("game_variant", (string) null);
            this.mAuthor = token1.GetValue<string>("author", (string) null);
            this.mUnsafe = token1.GetValue<bool>("enable_unsafe_scripting");
            this.mPackageUID = token1.GetValue<string>("uid", (string) null);
            if (string.IsNullOrWhiteSpace(this.mPackageUID))
              this.mPackageUID = token1.GetValue<string>("package_uid", (string) null);
            if (!Version.TryParse(token1.GetValue<string>("version", (string) null), out this.mVersion))
              Version.TryParse(token1.GetValue<string>("package_version", (string) null), out this.mVersion);
            Version.TryParse(token1.GetValue<string>("layout_engine_version", (string) null), out this.mLayoutEngineVersion);
            JObject token2 = token1.GetValue<JObject>("variants");
            if (token2 == null)
              return;
            foreach (JProperty property in token2.Properties())
            {
              string name = property.Name;
              JObject token3 = token2.GetValue<JObject>(property.Name);
              if (token3 != null)
              {
                string str = token3.GetValue<string>("display_name", (string) null);
                if (!string.IsNullOrWhiteSpace(str))
                  this.mAvailableVariants.Add((IGamePackageVariant) new GamePackage.Variant((IGamePackage) this)
                  {
                    UniqueID = name,
                    DisplayName = str,
                    OverridePath = Path.Combine(this.OverridePath, name)
                  });
              }
            }
          }
        }
      }
      catch
      {
      }
    }

    private void DeploySourceFileToOverride(string path, bool bReplaceExisting = false)
    {
      if (!Directory.Exists(this.OverridePath))
        Directory.CreateDirectory(this.OverridePath);
      path = this.NormalizePath(path);
      string path1 = Path.Combine(this.OverridePath, path);
      string directoryName = Path.GetDirectoryName(path1);
      if (!Directory.Exists(directoryName))
        Directory.CreateDirectory(directoryName);
      if (!bReplaceExisting && File.Exists(path1) || this.mSource == null)
        return;
      using (Stream stream = this.mSource.Open(path))
      {
        if (stream == null)
          return;
        if (File.Exists(path1))
          File.Delete(path1);
        using (Stream destination = (Stream) File.OpenWrite(path1))
          stream.CopyTo(destination);
      }
    }

    private class Variant : ObservableObject, IGamePackageVariant
    {
      private IGamePackage mOwner;
      private string mUniqueID;
      private string mOverridePath;
      private string mDisplayName;

      public Variant(IGamePackage owner) => this.mOwner = owner;

      public string BasePath => this.mUniqueID;

      public string OverridePath
      {
        get => this.mOverridePath;
        set => this.SetProperty<string>(ref this.mOverridePath, value, nameof (OverridePath));
      }

      public string DisplayName
      {
        get => this.mDisplayName;
        set => this.SetProperty<string>(ref this.mDisplayName, value, nameof (DisplayName));
      }

      public bool IsActive => ObservableSingleton<Tracker>.Instance.ActiveGamePackageVariant == this;

      public string UniqueID
      {
        get => this.mUniqueID;
        set => this.SetProperty<string>(ref this.mUniqueID, value, nameof (UniqueID));
      }

      public IGamePackage Package => this.mOwner;
    }
  }
}
