﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.Packages.PackageManager
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using EmoTracker.Core;
using EmoTracker.Core.Services;
using EmoTracker.Data.JSON;
using EmoTracker.Data.Media;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Net;
using System.Net.Cache;

namespace EmoTracker.Data.Packages
{
  public class PackageManager : ObservableSingleton<PackageManager>
  {
    private static string mServiceRootURI = "https://emotracker.net/service/";
    private WebClient mGameListWebClient;
    private WebClient mRepositoryListWebClient;
    private ObservableCollection<PackageRepositoryEntry> mAvailablePackages = new ObservableCollection<PackageRepositoryEntry>();
    private TrivialObservableCollectionAggregatorSynchronizer<PackageRepositoryEntry> mAvailablePackagesSync;
    private ObservableCollection<PackageManager.Game> mAvailableGames = new ObservableCollection<PackageManager.Game>();
    private PackageManager.Game mDefaultGame = new PackageManager.Game()
    {
      Name = "Other",
      Key = "Other",
      ImageURL = "https://emotracker.net/service/games/default.png"
    };
    private bool mbCommunityRepositoryListError;
    private bool mbDownloadNotInProgress = true;
    private ObservableCollection<PackageRepository> mRepositories = new ObservableCollection<PackageRepository>();
    public static string PackInstallPath = Path.Combine(UserDirectory.Path, "packs");
    private ObservableCollection<GamePackage> mInstalledPackages = new ObservableCollection<GamePackage>();

    public static string BaseServiceUri => PackageManager.mServiceRootURI;

    public static Uri BuildServiceUri(string path) => new Uri(string.Format("{0}{1}", (object) PackageManager.mServiceRootURI, (object) path.TrimStart('/')));

    public event EventHandler OnGameListDownloaded;

    public IEnumerable<PackageRepositoryEntry> AvailablePackages => (IEnumerable<PackageRepositoryEntry>) this.mAvailablePackages;

    public IEnumerable<PackageManager.Game> AvailableGames => (IEnumerable<PackageManager.Game>) this.mAvailableGames;

    public PackageManager.Game DefaultGame => this.mDefaultGame;

    public PackageManager.Game FindGame(string name)
    {
      if (!string.IsNullOrWhiteSpace(name))
      {
        foreach (PackageManager.Game availableGame in this.AvailableGames)
        {
          if (availableGame.Name.Equals(name, StringComparison.OrdinalIgnoreCase) || availableGame.Key.Equals(name, StringComparison.OrdinalIgnoreCase))
            return availableGame;
        }
      }
      return this.DefaultGame;
    }

    public bool CommunityRepositoryListError
    {
      get => this.mbCommunityRepositoryListError;
      private set => this.SetProperty<bool>(ref this.mbCommunityRepositoryListError, value, nameof (CommunityRepositoryListError));
    }

    public bool DownloadNotInProgress
    {
      get => this.mbDownloadNotInProgress;
      set => this.SetProperty<bool>(ref this.mbDownloadNotInProgress, value, nameof (DownloadNotInProgress));
    }

    public IEnumerable<PackageRepository> Repositories => (IEnumerable<PackageRepository>) this.mRepositories;

    public bool UpdatesAvailable
    {
      get
      {
        foreach (PackageRepository repository in this.Repositories)
        {
          foreach (PackageRepositoryEntry package in (IEnumerable<PackageRepositoryEntry>) repository.Packages)
          {
            if (package.Status == PackageRepositoryEntry.PackageStatus.UpdateAvailable)
              return true;
          }
        }
        return false;
      }
    }

    public bool CurrentPackageHasUpdateAvailable
    {
      get
      {
        foreach (PackageRepository repository in this.Repositories)
        {
          foreach (PackageRepositoryEntry package in (IEnumerable<PackageRepositoryEntry>) repository.Packages)
          {
            if (package.Status == PackageRepositoryEntry.PackageStatus.UpdateAvailable && package.ExistingPackage == ObservableSingleton<Tracker>.Instance.ActiveGamePackage)
              return true;
          }
        }
        return false;
      }
    }

    public bool AnyPackagesInstalled => this.mInstalledPackages.Count > 0;

    public PackageRepository AddRepository(PackageRepository repository)
    {
      foreach (PackageRepository repository1 in this.Repositories)
      {
        if (repository1.URL.Equals(repository.URL, StringComparison.OrdinalIgnoreCase))
        {
          repository1.Reload();
          return repository1;
        }
      }
      this.mRepositories.Add(repository);
      this.mAvailablePackagesSync.AddSource(repository.Packages);
      return repository;
    }

    public void RemoveRepository(PackageRepository repository)
    {
      this.mRepositories.Remove(repository);
      if (repository == null || repository.Packages == null)
        return;
      this.mAvailablePackagesSync.RemoveSource(repository.Packages);
    }

    public event EventHandler<PackageRepository> OnRepositoryUpdated;

    public IEnumerable<IGamePackage> InstalledPackages => (IEnumerable<IGamePackage>) this.mInstalledPackages;

    public PackageManager() => this.mAvailablePackagesSync = new TrivialObservableCollectionAggregatorSynchronizer<PackageRepositoryEntry>(this.mAvailablePackages, Array.Empty<IReadOnlyCollection<PackageRepositoryEntry>>());

    public void Initialize()
    {
      Directory.CreateDirectory(PackageManager.PackInstallPath);
      this.ScanForInstalledPackages();
      foreach (string additionalRepository in ObservableSingleton<ApplicationSettings>.Instance.AdditionalRepositories)
        this.AddRepository(new PackageRepository(additionalRepository));
      this.DownloadGameList();
      this.DownloadRepositoryList();
    }

    private void DownloadGameList()
    {
      try
      {
        this.mGameListWebClient = new WebClient();
        this.mGameListWebClient.Headers.Add("User-Agent", string.Format("EmoTracker/{0} (Windows)", (object) ApplicationVersion.Current));
        this.mGameListWebClient.CachePolicy = new RequestCachePolicy(RequestCacheLevel.NoCacheNoStore);
        this.mGameListWebClient.DownloadDataCompleted += new DownloadDataCompletedEventHandler(this.OnGameListDownloadCompleted);
        this.mGameListWebClient.DownloadDataAsync(PackageManager.BuildServiceUri("supported_games.json"));
      }
      catch
      {
      }
    }

    private void DownloadRepositoryList()
    {
      try
      {
        this.mRepositoryListWebClient = new WebClient();
        this.mRepositoryListWebClient.Headers.Add("User-Agent", string.Format("EmoTracker/{0} (Windows)", (object) ApplicationVersion.Current));
        this.mRepositoryListWebClient.CachePolicy = new RequestCachePolicy(RequestCacheLevel.NoCacheNoStore);
        this.mRepositoryListWebClient.DownloadDataCompleted += new DownloadDataCompletedEventHandler(this.OnRepositoryListDownloadCompleted);
        this.mRepositoryListWebClient.DownloadDataAsync(PackageManager.BuildServiceUri("package_repositories.json"));
      }
      catch
      {
      }
    }

    private void ParseMemoryRangeList(IList<PackageManager.Game.MemoryRange> ranges, JArray src)
    {
      if (src == null)
        return;
      foreach (JToken jtoken in src)
      {
        PackageManager.Game.MemoryRange result;
        if (!PackageManager.Game.MemoryRange.TryParse((string) jtoken, out result) || result == null)
          throw new InvalidDataException("Invalid data was passed to MemoryRange.TryParse(...)");
        ranges.Add(result);
      }
    }

    private void OnGameListDownloadCompleted(object sender, DownloadDataCompletedEventArgs e)
    {
      if (e.Error != null || e.Result == null)
        return;
      if (e.Result.Length == 0)
        return;
      try
      {
        using (Stream stream = (Stream) new MemoryStream(e.Result))
        {
          using (StreamReader reader = new StreamReader(stream))
          {
            JObject token1 = (JObject) JToken.ReadFrom((JsonReader) new JsonTextReader((TextReader) reader));
            if (token1 == null)
              return;
            foreach (JProperty property in token1.Properties())
            {
              try
              {
                JObject token2 = token1.GetValue<JObject>(property.Name);
                if (token2 != null)
                {
                  PackageManager.Game game = new PackageManager.Game()
                  {
                    Key = property.Name,
                    Name = token2.GetValue<string>("name", property.Name),
                    Series = token2.GetValue<string>("series", (string) null),
                    ImageURL = token2.GetValue<string>("image", (string) null),
                    Priority = token2.GetValue<int>("priority", 10000),
                    SeriesPriority = token2.GetValue<int>("series_priority", 10000)
                  };
                  bool flag = false;
                  try
                  {
                    JObject token3 = token2.GetValue<JObject>("memory_watch_config");
                    if (token3 != null)
                    {
                      List<PackageManager.Game.MemoryRange> ranges1 = new List<PackageManager.Game.MemoryRange>();
                      this.ParseMemoryRangeList((IList<PackageManager.Game.MemoryRange>) ranges1, token3.GetValue<JArray>("memory_range_blacklist"));
                      List<PackageManager.Game.MemoryRange> ranges2 = new List<PackageManager.Game.MemoryRange>();
                      this.ParseMemoryRangeList((IList<PackageManager.Game.MemoryRange>) ranges2, token3.GetValue<JArray>("memory_range_whitelist"));
                      foreach (PackageManager.Game.MemoryRange memoryRange in ranges2)
                        game.MemoryRangeWhitelist.Add(memoryRange);
                      foreach (PackageManager.Game.MemoryRange memoryRange in ranges1)
                        game.MemoryRangeBlacklist.Add(memoryRange);
                    }
                  }
                  catch
                  {
                    flag = true;
                  }
                  if (flag)
                    game.MemoryRangeBlacklist.Add(new PackageManager.Game.MemoryRange()
                    {
                      Begin = 0UL,
                      End = ulong.MaxValue
                    });
                  this.mAvailableGames.Add(game);
                }
              }
              catch
              {
              }
            }
          }
        }
      }
      catch
      {
      }
      finally
      {
        if (this.OnGameListDownloaded != null)
          this.OnGameListDownloaded((object) this, EventArgs.Empty);
      }
    }

    internal void NotifyRepositoryUpdated(PackageRepository packageRepository) => Dispatch.BeginInvoke((Action) (() =>
    {
      this.NotifyPropertyChanged("CurrentPackageHasUpdateAvailable");
      this.NotifyPropertyChanged("UpdatesAvailable");
      EventHandler<PackageRepository> repositoryUpdated = this.OnRepositoryUpdated;
      if (repositoryUpdated == null)
        return;
      repositoryUpdated((object) this, packageRepository);
    }));

    private void OnRepositoryListDownloadCompleted(object sender, DownloadDataCompletedEventArgs e)
    {
      if (e.Error == null && e.Result != null)
      {
        if (e.Result.Length != 0)
        {
          try
          {
            HashSet<PackageRepository> packageRepositorySet = new HashSet<PackageRepository>();
            foreach (PackageRepository repository in this.Repositories)
              packageRepositorySet.Add(repository);
            using (Stream stream = (Stream) new MemoryStream(e.Result))
            {
              using (StreamReader reader = new StreamReader(stream))
              {
                JObject token = (JObject) JToken.ReadFrom((JsonReader) new JsonTextReader((TextReader) reader));
                if (token != null)
                {
                  foreach (JProperty property in token.Properties())
                  {
                    try
                    {
                      string url = token.GetValue<string>(property.Name, (string) null);
                      packageRepositorySet.Remove(this.AddRepository(new PackageRepository(url, property.Name)));
                    }
                    catch
                    {
                    }
                  }
                }
              }
            }
            foreach (PackageRepository repository in packageRepositorySet)
              this.RemoveRepository(repository);
            foreach (string additionalRepository in ObservableSingleton<ApplicationSettings>.Instance.AdditionalRepositories)
              this.AddRepository(new PackageRepository(additionalRepository));
            this.CommunityRepositoryListError = false;
            return;
          }
          catch
          {
            return;
          }
        }
      }
      this.CommunityRepositoryListError = true;
    }

    private void ScanForInstalledPackages()
    {
      this.mInstalledPackages.Clear();
      this.NotifyPropertyChanged("AnyPackagesInstalled");
      foreach (string directory in Directory.GetDirectories(PackageManager.PackInstallPath))
      {
        GamePackage package = new GamePackage((IGamePackageSource) new DirectoryPackageSource(directory));
        if (package.IsValid)
          this.AddInstalledPackage(package);
      }
      foreach (string file in Directory.GetFiles(PackageManager.PackInstallPath, "*.zip"))
      {
        GamePackage package = new GamePackage((IGamePackageSource) new ZipPackageSource(file));
        if (package.IsValid)
          this.AddInstalledPackage(package);
      }
    }

    private void AddInstalledPackage(GamePackage package, bool bOverride = false)
    {
      foreach (GamePackage installedPackage in (Collection<GamePackage>) this.mInstalledPackages)
      {
        if (string.Equals(package.UniqueID, installedPackage.UniqueID, StringComparison.OrdinalIgnoreCase) && !bOverride)
          return;
      }
      this.mInstalledPackages.Add(package);
      this.NotifyPropertyChanged("AnyPackagesInstalled");
    }

    public void RefreshActiveState()
    {
      foreach (GamePackage installedPackage in this.InstalledPackages)
      {
        installedPackage.ForceRefreshProperty("IsActive");
        foreach (IGamePackageVariant availableVariant in installedPackage.AvailableVariants)
        {
          if (availableVariant is ObservableObject observableObject)
            observableObject.ForceRefreshProperty("IsActive");
        }
      }
      this.ForceRefreshProperty("CurrentPackageHasUpdateAvailable");
    }

    public GamePackage FindInstalledPackage(string uid)
    {
      if (!string.IsNullOrWhiteSpace(uid))
      {
        foreach (GamePackage installedPackage in this.InstalledPackages)
        {
          if (string.Equals(installedPackage.UniqueID, uid, StringComparison.OrdinalIgnoreCase))
            return installedPackage;
        }
      }
      return (GamePackage) null;
    }

    public PackageRepositoryEntry FindRepositoryEntry(string uid)
    {
      if (!string.IsNullOrWhiteSpace(uid))
      {
        foreach (PackageRepositoryEntry availablePackage in this.AvailablePackages)
        {
          if (string.Equals(availablePackage.UID, uid, StringComparison.OrdinalIgnoreCase))
            return availablePackage;
        }
      }
      return (PackageRepositoryEntry) null;
    }

    public void Rescan() => this.ScanForInstalledPackages();

    public void RefreshRemoteRepositories() => this.DownloadRepositoryList();

    public class Game : ObservableObject
    {
      private ObservableCollection<IGamePackage> mInstalledPackages = new ObservableCollection<IGamePackage>();
      private ObservableCollection<PackageRepositoryEntry> mAvailablePackages = new ObservableCollection<PackageRepositoryEntry>();
      private ObservableCollection<PackageManager.Game.MemoryRange> mMemoryRangeWhitelist = new ObservableCollection<PackageManager.Game.MemoryRange>();
      private ObservableCollection<PackageManager.Game.MemoryRange> mMemoryRangeBlacklist = new ObservableCollection<PackageManager.Game.MemoryRange>();
      private string mKey;
      private string mName;
      private string mSeries;
      private string mImageURL;
      private int mPriority = 10000;
      private int mSeriesPriority = 10000;
      private ImageReference mImage = ImageReference.FromExternalURI(new Uri("pack://application:,,,/EmoTracker;component/Resources/PackageManager/default_game_banner.png"));

      public string Key
      {
        get => this.mKey;
        set => this.SetProperty<string>(ref this.mKey, value, nameof (Key));
      }

      public string Name
      {
        get => this.mName;
        set => this.SetProperty<string>(ref this.mName, value, nameof (Name));
      }

      public string Series
      {
        get => this.mSeries;
        set => this.SetProperty<string>(ref this.mSeries, value, nameof (Series));
      }

      public string ImageURL
      {
        get => this.mImageURL;
        set
        {
          if (!this.SetProperty<string>(ref this.mImageURL, value, nameof (ImageURL)))
            return;
          this.Image = ImageReference.FromExternalURI(new Uri(value));
        }
      }

      public ImageReference Image
      {
        get => this.mImage;
        private set => this.SetProperty<ImageReference>(ref this.mImage, value, nameof (Image));
      }

      public int Priority
      {
        get => this.mPriority;
        set => this.SetProperty<int>(ref this.mPriority, value, nameof (Priority));
      }

      public int SeriesPriority
      {
        get => this.mSeriesPriority;
        set => this.SetProperty<int>(ref this.mSeriesPriority, value, nameof (SeriesPriority));
      }

      public ObservableCollection<IGamePackage> InstalledPackages
      {
        get => this.mInstalledPackages;
        private set => this.SetProperty<ObservableCollection<IGamePackage>>(ref this.mInstalledPackages, value, nameof (InstalledPackages));
      }

      public ObservableCollection<PackageRepositoryEntry> AvailablePackages
      {
        get => this.mAvailablePackages;
        private set => this.SetProperty<ObservableCollection<PackageRepositoryEntry>>(ref this.mAvailablePackages, value, nameof (AvailablePackages));
      }

      public IList<PackageManager.Game.MemoryRange> MemoryRangeWhitelist => (IList<PackageManager.Game.MemoryRange>) this.mMemoryRangeWhitelist;

      public IList<PackageManager.Game.MemoryRange> MemoryRangeBlacklist => (IList<PackageManager.Game.MemoryRange>) this.mMemoryRangeBlacklist;

      private bool MemoryRangeContains(ulong rangeStart, ulong rangeEnd, ulong address) => address >= rangeStart && address <= rangeEnd;

      public bool IsMemoryRangeAccessAllowed(ulong start, ulong end)
      {
        foreach (PackageManager.Game.MemoryRange memoryRange in (IEnumerable<PackageManager.Game.MemoryRange>) this.MemoryRangeBlacklist)
        {
          if (this.MemoryRangeContains(memoryRange.Begin, memoryRange.End, start) || this.MemoryRangeContains(memoryRange.Begin, memoryRange.End, end) || this.MemoryRangeContains(start, end, memoryRange.Begin) || this.MemoryRangeContains(start, end, memoryRange.End))
            return false;
        }
        bool flag = true;
        if (this.MemoryRangeWhitelist.Count > 0)
        {
          flag = false;
          foreach (PackageManager.Game.MemoryRange memoryRange in (IEnumerable<PackageManager.Game.MemoryRange>) this.MemoryRangeWhitelist)
          {
            if (this.MemoryRangeContains(memoryRange.Begin, memoryRange.End, start) && this.MemoryRangeContains(memoryRange.Begin, memoryRange.End, end))
              flag = true;
          }
        }
        return flag;
      }

      public override string ToString() => this.Name;

      public class MemoryRange
      {
        public ulong Begin;
        public ulong End;

        public static bool TryParse(string value, out PackageManager.Game.MemoryRange result)
        {
          result = (PackageManager.Game.MemoryRange) null;
          if (!string.IsNullOrWhiteSpace(value))
          {
            string[] strArray = value.Split(new char[1]
            {
              ':'
            }, StringSplitOptions.RemoveEmptyEntries);
            if (strArray != null)
            {
              if (strArray.Length == 2)
              {
                try
                {
                  ulong uint64_1 = Convert.ToUInt64(strArray[0], 16);
                  ulong uint64_2 = Convert.ToUInt64(strArray[1], 16);
                  if (uint64_2 >= uint64_1)
                  {
                    result = new PackageManager.Game.MemoryRange()
                    {
                      Begin = uint64_1,
                      End = uint64_2
                    };
                    return true;
                  }
                }
                catch
                {
                }
              }
            }
          }
          return false;
        }

        public override string ToString() => string.Format("0x{0:x}:0x{1:x}", (object) this.Begin, (object) this.End);
      }
    }
  }
}
