﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.Packages.PackageRepository
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using EmoTracker.Core;
using EmoTracker.Data.JSON;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Net;
using System.Net.Cache;

namespace EmoTracker.Data.Packages
{
  public class PackageRepository : ObservableObject
  {
    private DownloadStatus mStatus;
    private WebClient mWebClient;
    private string mURL;
    private string mName;
    private ObservableCollection<PackageRepositoryEntry> mPackages = new ObservableCollection<PackageRepositoryEntry>();

    public IReadOnlyCollection<PackageRepositoryEntry> Packages => (IReadOnlyCollection<PackageRepositoryEntry>) this.mPackages;

    public DownloadStatus DownloadStatus
    {
      get => this.mStatus;
      protected set => this.SetProperty<DownloadStatus>(ref this.mStatus, value, nameof (DownloadStatus));
    }

    public string Name
    {
      get => this.mName;
      private set => this.SetProperty<string>(ref this.mName, value, nameof (Name));
    }

    public string URL
    {
      get => this.mURL;
      private set => this.SetProperty<string>(ref this.mURL, value, nameof (URL));
    }

    public PackageRepository(string url, string defaultName = null)
    {
      this.Name = defaultName;
      this.URL = url;
      this.mWebClient = new WebClient();
      this.mWebClient.Headers.Add(HttpRequestHeader.UserAgent, string.Format("EmoTracker/{0} (Windows)", (object) ApplicationVersion.Current));
      this.mWebClient.Headers.Add(HttpRequestHeader.CacheControl, "no-cache");
      this.mWebClient.CachePolicy = new RequestCachePolicy(RequestCacheLevel.NoCacheNoStore);
      this.mWebClient.DownloadDataCompleted += new DownloadDataCompletedEventHandler(this.MWebClient_DownloadDataCompleted);
      this.mWebClient.DownloadProgressChanged += new DownloadProgressChangedEventHandler(this.MWebClient_DownloadProgressChanged);
      this.Reload();
    }

    public void Reload()
    {
      if (this.DownloadStatus == DownloadStatus.InProgress)
        return;
      this.DownloadStatus = DownloadStatus.InProgress;
      try
      {
        this.mWebClient.DownloadDataAsync(new Uri(this.URL));
      }
      catch
      {
        this.DownloadStatus = DownloadStatus.Error;
      }
    }

    private void MWebClient_DownloadProgressChanged(
      object sender,
      DownloadProgressChangedEventArgs e)
    {
    }

    private PackageFlags ParsePackageFlagsArray(
      JArray attributes,
      bool bAllowPrivilegedFlags)
    {
      if (attributes == null)
        return PackageFlags.None;
      PackageFlags packageFlagsArray = PackageFlags.None;
      foreach (JToken attribute in attributes)
      {
        PackageFlags result;
        if (System.Enum.TryParse<PackageFlags>((string) attribute, true, out result))
        {
          if (!bAllowPrivilegedFlags && packageFlagsArray.HasFlag((System.Enum) (PackageFlags.Official | PackageFlags.Featured)))
            packageFlagsArray = PackageFlags.None;
          packageFlagsArray |= result;
        }
      }
      return packageFlagsArray;
    }

    private void MWebClient_DownloadDataCompleted(object sender, DownloadDataCompletedEventArgs e)
    {
      if (e.Error == null && e.Result != null)
      {
        if (e.Result.Length != 0)
        {
          try
          {
            this.mPackages.Clear();
            using (Stream stream = (Stream) new MemoryStream(e.Result))
            {
              using (StreamReader reader = new StreamReader(stream))
              {
                JObject token1 = (JObject) JToken.ReadFrom((JsonReader) new JsonTextReader((TextReader) reader));
                if (token1 != null)
                {
                  this.Name = token1.GetValue<string>("name", this.Name);
                  JArray jarray1 = token1.GetValue<JArray>("packages");
                  if (jarray1 != null)
                  {
                    foreach (JObject token2 in jarray1)
                    {
                      PackageRepositoryEntry packageRepositoryEntry = new PackageRepositoryEntry(this);
                      string input1 = token2.GetValue<string>("version", (string) null);
                      Version result1;
                      if (!string.IsNullOrWhiteSpace(input1) && Version.TryParse(input1, out result1))
                      {
                        packageRepositoryEntry.Version = result1;
                        string input2 = token2.GetValue<string>("required_app_version", (string) null);
                        Version result2;
                        if (!string.IsNullOrWhiteSpace(input2) && Version.TryParse(input2, out result2))
                          packageRepositoryEntry.RequiredAppVersion = result2;
                        packageRepositoryEntry.Name = token2.GetValue<string>("name", (string) null);
                        packageRepositoryEntry.Game = token2.GetValue<string>("game_name", (string) null);
                        packageRepositoryEntry.Author = token2.GetValue<string>("author", "Unknown");
                        packageRepositoryEntry.UID = token2.GetValue<string>("uid", (string) null);
                        packageRepositoryEntry.URL = token2.GetValue<string>("link", (string) null);
                        packageRepositoryEntry.DocumentationURL = token2.GetValue<string>("documentation_url", (string) null);
                        packageRepositoryEntry.ExistingPackage = ObservableSingleton<PackageManager>.Instance.FindInstalledPackage(packageRepositoryEntry.UID);
                        packageRepositoryEntry.Flags = this.ParsePackageFlagsArray(token2.GetValue<JArray>("flags"), packageRepositoryEntry.URL.ToLower().Contains("https://emotracker.net"));
                        JArray jarray2 = token2.GetValue<JArray>("variants");
                        if (jarray2 != null)
                        {
                          foreach (JObject token3 in jarray2)
                            packageRepositoryEntry.Variants.Add(new VariantEntry()
                            {
                              Name = token3.GetValue<string>("name", (string) null),
                              Flags = this.ParsePackageFlagsArray(token3.GetValue<JArray>("flags"), packageRepositoryEntry.URL.ToLower().Contains("https://emotracker.net"))
                            });
                        }
                        if (!string.IsNullOrWhiteSpace(packageRepositoryEntry.Name) && !string.IsNullOrWhiteSpace(packageRepositoryEntry.UID) && !string.IsNullOrWhiteSpace(packageRepositoryEntry.URL))
                        {
                          this.mPackages.Add(packageRepositoryEntry);
                          ObservableSingleton<PackageManager>.Instance.ForceRefreshProperty("UpdatesAvailable");
                          ObservableSingleton<PackageManager>.Instance.ForceRefreshProperty("CurrentPackageHasUpdateAvailable");
                        }
                      }
                    }
                  }
                }
              }
            }
            this.DownloadStatus = DownloadStatus.Complete;
          }
          catch
          {
            this.DownloadStatus = DownloadStatus.Error;
          }
          ObservableSingleton<PackageManager>.Instance.NotifyRepositoryUpdated(this);
          return;
        }
      }
      this.DownloadStatus = DownloadStatus.Error;
    }
  }
}
