﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.Packages.PackageRepositoryEntry
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using EmoTracker.Core;
using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Cache;
using System.Threading;

namespace EmoTracker.Data.Packages
{
  public class PackageRepositoryEntry : ObservableObject
  {
    private PackageRepository mOwner;
    private string mName;
    private string mGame;
    private string mAuthor;
    private string mUID;
    private string mURL;
    private string mDocumentationURL;
    private Version mVersion;
    private Version mRequiredAppVersion;
    private GamePackage mExistingPackage;
    private PackageRepositoryEntry.PackageStatus mStatus = PackageRepositoryEntry.PackageStatus.Available;
    private PackageFlags mFlags;
    private WebClient mWebClient;
    private DownloadStatus mDownloadStatus;
    private int mDownloadProgress;
    private ObservableCollection<VariantEntry> mVariants = new ObservableCollection<VariantEntry>();

    public PackageRepositoryEntry(PackageRepository owner)
    {
      this.mOwner = owner;
      this.mWebClient = new WebClient();
      this.mWebClient.Headers.Add("User-Agent", string.Format("EmoTracker/{0} (Windows)", (object) ApplicationVersion.Current));
      this.mWebClient.CachePolicy = new RequestCachePolicy(RequestCacheLevel.BypassCache);
      this.mWebClient.DownloadFileCompleted += new AsyncCompletedEventHandler(this.MWebClient_DownloadFileCompleted);
      this.mWebClient.DownloadProgressChanged += new DownloadProgressChangedEventHandler(this.MWebClient_DownloadProgressChanged);
    }

    public PackageRepositoryEntry.PackageStatus Status
    {
      get => this.mStatus;
      set
      {
        if (!this.SetProperty<PackageRepositoryEntry.PackageStatus>(ref this.mStatus, value, nameof (Status)))
          return;
        ObservableSingleton<PackageManager>.Instance.ForceRefreshProperty("UpdatesAvailable");
        ObservableSingleton<PackageManager>.Instance.ForceRefreshProperty("CurrentPackageHasUpdateAvailable");
      }
    }

    public PackageFlags Flags
    {
      get => this.mFlags;
      set => this.SetProperty<PackageFlags>(ref this.mFlags, value, nameof (Flags));
    }

    public DownloadStatus DownloadStatus
    {
      get => this.mDownloadStatus;
      set => this.SetProperty<DownloadStatus>(ref this.mDownloadStatus, value, nameof (DownloadStatus));
    }

    public string Name
    {
      get => this.mName;
      set => this.SetProperty<string>(ref this.mName, value, nameof (Name));
    }

    public string Game
    {
      get => this.mGame;
      set => this.SetProperty<string>(ref this.mGame, value, nameof (Game));
    }

    public string Author
    {
      get => this.mAuthor;
      set => this.SetProperty<string>(ref this.mAuthor, value, nameof (Author));
    }

    public string UID
    {
      get => this.mUID;
      set => this.SetProperty<string>(ref this.mUID, value, nameof (UID));
    }

    public string URL
    {
      get => this.mURL;
      set => this.SetProperty<string>(ref this.mURL, value, nameof (URL));
    }

    public string DocumentationURL
    {
      get => this.mDocumentationURL;
      set => this.SetProperty<string>(ref this.mDocumentationURL, value, nameof (DocumentationURL));
    }

    public Version Version
    {
      get => this.mVersion;
      set => this.SetProperty<Version>(ref this.mVersion, value, nameof (Version));
    }

    [DependentProperty("RequiresNewerAppVersion")]
    public Version RequiredAppVersion
    {
      get => this.mRequiredAppVersion;
      set
      {
        if (!this.SetProperty<Version>(ref this.mRequiredAppVersion, value, nameof (RequiredAppVersion)) || !this.RequiresNewerAppVersion)
          return;
        this.Status = PackageRepositoryEntry.PackageStatus.AppUpdateRequired;
      }
    }

    public bool RequiresNewerAppVersion => this.RequiredAppVersion > ApplicationVersion.Current;

    public int DownloadProgress
    {
      get => this.mDownloadProgress;
      set => this.SetProperty<int>(ref this.mDownloadProgress, value, nameof (DownloadProgress));
    }

    public GamePackage ExistingPackage
    {
      get => this.mExistingPackage;
      set
      {
        if (!this.SetProperty<GamePackage>(ref this.mExistingPackage, value, nameof (ExistingPackage)))
          return;
        if (this.RequiresNewerAppVersion)
          this.Status = PackageRepositoryEntry.PackageStatus.AppUpdateRequired;
        else if (this.mExistingPackage == null)
          this.Status = PackageRepositoryEntry.PackageStatus.Available;
        else if (this.mExistingPackage != null && this.Version > this.mExistingPackage.Version)
          this.Status = PackageRepositoryEntry.PackageStatus.UpdateAvailable;
        else if (this.mExistingPackage != null && this.mExistingPackage.Source is DirectoryPackageSource)
          this.Status = PackageRepositoryEntry.PackageStatus.Development;
        else
          this.Status = PackageRepositoryEntry.PackageStatus.Installed;
      }
    }

    public ObservableCollection<VariantEntry> Variants => this.mVariants;

    public void Install()
    {
      ObservableSingleton<PackageManager>.Instance.DownloadNotInProgress = false;
      string tempFileName = Path.GetTempFileName();
      this.mWebClient.DownloadFileAsync(new Uri(this.mURL), tempFileName, (object) tempFileName);
    }

    public UninstallResult Uninstall()
    {
      string packPath = this.ExistingPackage.Source.PackPath;
      FileSystemInfo fileSystemInfo = !System.IO.File.GetAttributes(packPath).HasFlag((System.Enum) FileAttributes.Directory) ? (FileSystemInfo) new FileInfo(packPath) : (FileSystemInfo) new DirectoryInfo(packPath);
      DirectoryInfo directoryInfo = new DirectoryInfo(this.ExistingPackage.OverridePath);
      try
      {
        fileSystemInfo.Delete();
      }
      catch
      {
        return UninstallResult.FailedUninstall;
      }
      try
      {
        if (directoryInfo.Exists)
          directoryInfo.Delete(true);
      }
      catch
      {
        return UninstallResult.FailedOverrides;
      }
      finally
      {
        if (ObservableSingleton<Tracker>.Instance.IsActivePackage(this.mUID))
          ObservableSingleton<Tracker>.Instance.ActiveGamePackage = (IGamePackage) null;
        ObservableSingleton<PackageManager>.Instance.Rescan();
        this.mOwner.Reload();
      }
      return UninstallResult.Success;
    }

    public BackupOverrideResult BackupOverride()
    {
      string destDirName = this.ExistingPackage.OverridePath + "_backup";
      DirectoryInfo directoryInfo = new DirectoryInfo(this.ExistingPackage.OverridePath);
      try
      {
        directoryInfo.MoveTo(destDirName);
      }
      catch
      {
        return BackupOverrideResult.Failed;
      }
      return BackupOverrideResult.Success;
    }

    private void MWebClient_DownloadProgressChanged(
      object sender,
      DownloadProgressChangedEventArgs e)
    {
      if (Environment.OSVersion.Version.Major < 10)
        return;
      this.DownloadProgress = e.ProgressPercentage;
    }

    private void MWebClient_DownloadFileCompleted(object sender, AsyncCompletedEventArgs e)
    {
      try
      {
        if (e.Error == null)
        {
          string userState = e.UserState as string;
          if (!string.IsNullOrWhiteSpace(userState) && System.IO.File.Exists(userState) && new FileInfo(userState).Length > 0L)
          {
            bool flag1 = ObservableSingleton<PackageManager>.Instance.InstalledPackages.Count<IGamePackage>() == 0;
            bool flag2 = this.ExistingPackage == ObservableSingleton<Tracker>.Instance.ActiveGamePackage;
            if (this.ExistingPackage != null)
              this.ExistingPackage.Source.ReleaseStorage();
            for (int index = 0; index < 10; ++index)
            {
              try
              {
                System.IO.File.Copy(userState, Path.Combine(PackageManager.PackInstallPath, string.Format("{0}.zip", (object) this.UID)), true);
                break;
              }
              catch (Exception ex)
              {
                ObservableSingleton<ScriptManager>.Instance.OutputException(ex);
                Thread.Sleep(500);
              }
            }
            if (this.ExistingPackage != null)
              this.ExistingPackage.Source.AcquireStorage();
            ObservableSingleton<PackageManager>.Instance.Rescan();
            this.mOwner.Reload();
            this.ExistingPackage = ObservableSingleton<PackageManager>.Instance.FindInstalledPackage(this.UID);
            if (flag1)
            {
              if (this.ExistingPackage.AvailableVariants.Count<IGamePackageVariant>() > 0)
              {
                ObservableSingleton<Tracker>.Instance.ActiveGamePackageVariant = this.ExistingPackage.AvailableVariants.First<IGamePackageVariant>();
                return;
              }
              ObservableSingleton<Tracker>.Instance.ActiveGamePackage = (IGamePackage) this.ExistingPackage;
              return;
            }
            if (!flag2 || this.ExistingPackage == null)
              return;
            IGamePackageVariant gamePackageVariant = this.ExistingPackage.FindVariant(ObservableSingleton<ApplicationSettings>.Instance.LastActivePackageVariant) ?? this.ExistingPackage.AvailableVariants.FirstOrDefault<IGamePackageVariant>();
            if (gamePackageVariant != null)
              ObservableSingleton<Tracker>.Instance.ActiveGamePackageVariant = gamePackageVariant;
            else
              ObservableSingleton<Tracker>.Instance.ActiveGamePackage = (IGamePackage) this.ExistingPackage;
            ObservableSingleton<Tracker>.Instance.Reload();
            return;
          }
        }
        this.Status = PackageRepositoryEntry.PackageStatus.DownloadError;
      }
      catch (Exception ex)
      {
        ObservableSingleton<ScriptManager>.Instance.OutputException(ex);
      }
      finally
      {
        ObservableSingleton<PackageManager>.Instance.DownloadNotInProgress = true;
      }
    }

    public enum PackageStatus
    {
      Development,
      AppUpdateRequired,
      Available,
      Installed,
      UpdateAvailable,
      DownloadError,
    }
  }
}
