﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.ScriptManager
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using EmoTracker.Core;
using EmoTracker.Data.Locations;
using EmoTracker.Data.Scripting;
using Newtonsoft.Json;
using NLua;
using NLua.Event;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Reflection;

namespace EmoTracker.Data
{
  public class ScriptManager : ObservableSingleton<ScriptManager>, ICodeProvider
  {
    private ObservableCollection<ScriptManager.LogLine> mLogOutput = new ObservableCollection<ScriptManager.LogLine>();
    private DelegateCommand mClearLogCommand;
    private static readonly string SystemLua = "\r\n\r\nimport = function () end\r\n\r\nfunction print(...)\r\n    local printResult = \"\"\r\n    for i, v in ipairs(table.pack(...)) do\r\n        printResult = printResult..tostring(v).. \"\\t\"\r\n    end\r\n    if string.len(printResult) > 0 then\r\n        _output(printResult)\r\n    end\r\n end\r\n ";
    private IGamePackage mPackage;
    private Lua mLua;
    private static readonly string kFirstIndentString = "   ";
    private static readonly string kIndentString = "   ";
    private static readonly string kLastIndentString = "   ";
    private string mIndentText = "";
    private int mLogIndent;
    private Dictionary<string, object> mGlobals = new Dictionary<string, object>();
    private IMemoryWatchService mMemoryService;
    private INotificationService mNotificationService;
    private Dictionary<string, ScriptManager.CacheEntry> mExpressionCache = new Dictionary<string, ScriptManager.CacheEntry>();
    private bool mbInPostLogicUpdate;

    [LuaHide]
    public IEnumerable<ScriptManager.LogLine> LogOutput => (IEnumerable<ScriptManager.LogLine>) this.mLogOutput;

    public DelegateCommand ClearLogCommand => this.mClearLogCommand;

    public int LogIndent
    {
      get => this.mLogIndent;
      set
      {
        this.mLogIndent = Math.Max(value, 0);
        this.mIndentText = "";
        for (int index = 0; index < this.mLogIndent; ++index)
          this.mIndentText = index != 0 ? (index != this.mLogIndent - 1 ? this.mIndentText + ScriptManager.kIndentString : this.mIndentText + ScriptManager.kLastIndentString) : this.mIndentText + ScriptManager.kFirstIndentString;
      }
    }

    [LuaHide]
    public void SetGlobalObject(string key, object value)
    {
      try
      {
        this.mGlobals[key] = value;
        if (this.mLua == null)
          return;
        this.mLua[key] = value;
      }
      catch
      {
      }
    }

    [LuaHide]
    private object[] LoadScript(IGamePackage package, string path)
    {
      this.Output(string.Format("Loading Script: {0}", (object) path));
      using (new LoggingBlock())
      {
        try
        {
          object[] objArray = (object[]) null;
          using (Stream stream = package.Open(path))
          {
            if (stream != null && stream.Length > 0L)
            {
              byte[] numArray = new byte[stream.Length];
              if (stream.Read(numArray, 0, numArray.Length) == numArray.Length)
                objArray = this.mLua.DoString(numArray);
            }
            else
              this.Output("Script not found");
          }
          return objArray;
        }
        catch (Exception ex)
        {
          this.Output(string.Format("A C# exception occurred while loading script: {0}", (object) path));
          using (new LoggingBlock())
            this.Output(ex.Message);
          return (object[]) null;
        }
      }
    }

    public object[] LoadScript(string path) => this.LoadScript(this.mPackage, path);

    public ScriptManager() => this.mClearLogCommand = new DelegateCommand(new Action<object>(this.ClearLogOutput));

    private void ClearLogOutput(object obj = null) => this.mLogOutput.Clear();

    public void SetMemoryWatchService(IMemoryWatchService service) => this.mMemoryService = service;

    public void SetNotificationService(INotificationService service) => this.mNotificationService = service;

    [LuaHide]
    public void Load(IGamePackage package)
    {
      this.mPackage = package;
      if (this.mPackage == null)
        return;
      this.DisposeObjectAndDefault<Lua>(ref this.mLua);
      this.mLua = new Lua();
      this.mLua.DebugHook += new EventHandler<DebugHookEventArgs>(this.MLua_DebugHook);
      this.mLua.HookException += new EventHandler<HookExceptionEventArgs>(this.MLua_HookException);
      this.mLua.RegisterFunction("_output", (object) this, (MethodBase) this.GetType().GetMethod("OutputRaw"));
      try
      {
        using (LuaTable luaTable = (LuaTable) this.mLua["os"])
        {
          luaTable["execute"] = (object) null;
          luaTable["exit"] = (object) null;
          luaTable["setlocale"] = (object) null;
        }
      }
      catch
      {
      }
      if (!this.mPackage.FlaggedAsUnsafe)
      {
        try
        {
          using (LuaTable luaTable = (LuaTable) this.mLua["os"])
          {
            luaTable["tmpname"] = (object) null;
            luaTable["rename"] = (object) null;
            luaTable["getenv"] = (object) null;
            luaTable["remove"] = (object) null;
          }
        }
        catch
        {
          this.mLua["os"] = (object) null;
        }
        this.mLua["io"] = (object) null;
      }
      this.mLua["Tracker"] = (object) Singleton<TrackerScriptInterface>.Instance;
      this.mLua["Layout"] = (object) Singleton<LayoutScriptInterface>.Instance;
      this.mLua["AccessibilityLevel"] = (object) AccessibilityLevel.None;
      this.mLua["NotificationType"] = (object) NotificationType.Message;
      this.mLua["ScriptHost"] = (object) this;
      this.mLua["ImageReference"] = (object) new ImageReferenceProvider();
      foreach (KeyValuePair<string, object> mGlobal in this.mGlobals)
        this.mLua[mGlobal.Key] = mGlobal.Value;
      this.mLua.DoString(ScriptManager.SystemLua);
      this.LoadScript(this.mPackage, "scripts/init.lua");
    }

    [LuaHide]
    public void OutputRaw(string text, string color = "Goldenrod")
    {
      if (text == null)
        return;
      using (StringReader stringReader = new StringReader(text))
      {
        for (string str = stringReader.ReadLine(); str != null; str = stringReader.ReadLine())
        {
          while (this.mLogOutput.Count > 500)
            this.mLogOutput.RemoveAt(0);
          this.mLogOutput.Add(new ScriptManager.LogLine()
          {
            Text = string.Format("{0}{1}", (object) this.mIndentText, (object) str),
            Color = color
          });
        }
      }
    }

    [LuaHide]
    public void Output(string text)
    {
      if (text == null)
        return;
      this.OutputRaw(text, "DarkGray");
    }

    [LuaHide]
    public void Output(string format, params object[] args)
    {
      if (format == null)
        return;
      this.OutputRaw(string.Format(format, args), "DarkGray");
    }

    [LuaHide]
    public void OutputException(Exception e)
    {
      if (e is JsonReaderException jsonReaderException)
      {
        ObservableSingleton<ScriptManager>.Instance.OutputError("JSON Parse Error");
        using (new LoggingBlock())
        {
          ObservableSingleton<ScriptManager>.Instance.OutputError(jsonReaderException.Message);
          if (string.IsNullOrWhiteSpace(jsonReaderException.HelpLink))
            return;
          this.OutputError("  For more information, see: {0}", (object) jsonReaderException.HelpLink);
        }
      }
      else
        this.OutputError("Exception: {0}\n{1}", (object) e.Message, (object) e.StackTrace);
    }

    [LuaHide]
    public void OutputError(string text)
    {
      if (text == null)
        return;
      this.OutputRaw(text, "Red");
    }

    [LuaHide]
    public void OutputError(string format, params object[] args)
    {
      if (format == null)
        return;
      this.OutputError(string.Format(format, args));
    }

    [LuaHide]
    public void OutputWarning(string text)
    {
      if (text == null)
        return;
      this.OutputRaw(text, "Yellow");
    }

    [LuaHide]
    public void OutputWarning(string format, params object[] args)
    {
      if (format == null)
        return;
      this.OutputWarning(string.Format(format, args));
    }

    private void MLua_HookException(object sender, HookExceptionEventArgs e)
    {
      if (e == null)
        return;
      this.OutputError(e.ToString());
    }

    private void MLua_DebugHook(object sender, DebugHookEventArgs e)
    {
      if (e == null)
        return;
      this.OutputError(e.ToString());
    }

    [LuaHide]
    public void Reset()
    {
      if (this.mLua != null)
      {
        this.mLua.Close();
        this.mLua = (Lua) null;
      }
      this.ClearLogOutput();
    }

    [LuaHide]
    public object FindObjectForCode(string code) => (object) null;

    [LuaHide]
    public uint ProviderCountForCode(string code, out AccessibilityLevel maxAccessibility)
    {
      maxAccessibility = AccessibilityLevel.Normal;
      uint num = 0;
      ScriptManager.CacheEntry cacheEntry;
      if (this.mExpressionCache.TryGetValue(code, out cacheEntry))
      {
        maxAccessibility = cacheEntry.maxAccessibility;
        return cacheEntry.count;
      }
      try
      {
        string[] source1 = code.Split('|');
        for (int index = 0; index < source1.Length; ++index)
          source1[index] = source1[index].Trim();
        using (LuaFunction luaFunction = this.mLua[source1[0]] as LuaFunction)
        {
          if (luaFunction != null)
          {
            IEnumerable<string> source2 = ((IEnumerable<string>) source1).Skip<string>(1);
            object[] source3 = source2 == null || !source2.Any<string>() ? luaFunction.Call() : luaFunction.Call(((IEnumerable<object>) source2).ToArray<object>());
            if (source3 == null)
            {
              this.OutputError("Lua function `{0}` did not return a count. All Lua functions used as logical expressions must return a count.", (object) code);
            }
            else
            {
              if (source3.Length >= 2)
              {
                AccessibilityLevel? nullable = source3[1] as AccessibilityLevel?;
                if (nullable.HasValue && nullable.HasValue)
                  maxAccessibility = nullable.Value;
              }
              num = Convert.ToUInt32(((IEnumerable<object>) source3).First<object>());
            }
          }
          else
            this.OutputError("Couldn't execute lua function `{0}` because it does not exist", (object) code);
        }
      }
      catch (Exception ex)
      {
        this.Output(ex.ToString());
      }
      this.mExpressionCache[code] = new ScriptManager.CacheEntry()
      {
        count = num,
        maxAccessibility = maxAccessibility
      };
      return num;
    }

    [LuaHide]
    internal void ClearExpressionCache() => this.mExpressionCache.Clear();

    [LuaHide]
    public void InvokeStandardCallback(ScriptManager.StandardCallback callback)
    {
      string fullPath = (string) null;
      switch (callback)
      {
        case ScriptManager.StandardCallback.AccessibilityUpdating:
          fullPath = "tracker_on_accessibility_updating";
          break;
        case ScriptManager.StandardCallback.AccessibilityUpdated:
          fullPath = "tracker_on_accessibility_updated";
          break;
        case ScriptManager.StandardCallback.StartLoadingSaveFile:
          fullPath = "tracker_on_begin_loading_save_file";
          break;
        case ScriptManager.StandardCallback.FinishLoadingSaveFile:
          fullPath = "tracker_on_finish_loading_save_file";
          break;
        case ScriptManager.StandardCallback.PackReady:
          fullPath = "tracker_on_pack_ready";
          break;
        case ScriptManager.StandardCallback.AutoTrackerStarted:
          fullPath = "autotracker_started";
          break;
        case ScriptManager.StandardCallback.AutoTrackerStopped:
          fullPath = "autotracker_stopped";
          break;
      }
      if (this.mbInPostLogicUpdate)
        return;
      try
      {
        this.mbInPostLogicUpdate = true;
        if (string.IsNullOrWhiteSpace(fullPath))
          return;
        using (LuaFunction luaFunction = this.mLua[fullPath] as LuaFunction)
          luaFunction?.Call();
      }
      catch (Exception ex)
      {
        this.OutputException(ex);
      }
      finally
      {
        this.mbInPostLogicUpdate = false;
      }
    }

    public IMemorySegment AddMemoryWatch(
      string name,
      ulong startAddress,
      ulong length,
      LuaFunction callback,
      int period = 1000)
    {
      return this.mMemoryService != null ? this.mMemoryService.AddMemoryWatch(name, startAddress, length, (Func<IMemorySegment, bool>) (segment =>
      {
        try
        {
          ObservableSingleton<LocationDatabase>.Instance.SuspendRefresh = true;
          if (callback != null)
          {
            object[] source = callback.Call((object) segment);
            if (source != null)
            {
              if (source.Length != 0)
                return Convert.ToBoolean(((IEnumerable<object>) source).First<object>());
            }
          }
        }
        catch (Exception ex)
        {
          this.Output(ex.ToString());
        }
        finally
        {
          ObservableSingleton<LocationDatabase>.Instance.SuspendRefresh = false;
        }
        return true;
      }), (Action<IMemorySegment>) (segment => callback?.Dispose()), period) : (IMemorySegment) null;
    }

    public void RemoveMemoryWatch(IMemorySegment segment)
    {
      if (this.mMemoryService == null)
        return;
      this.mMemoryService.RemoveMemoryWatch(segment);
    }

    public INotificationService NotificationService => this.mNotificationService;

    public void PushMarkdownNotification(NotificationType type, string markdown, int timeout = -1)
    {
      if (this.mNotificationService == null)
        return;
      this.mNotificationService.PushMarkdownNotification(type, markdown, timeout);
    }

    public LuaItem CreateLuaItem()
    {
      LuaItem luaItem = new LuaItem();
      Singleton<ItemDatabase>.Instance.RegisterItem((ITrackableItem) luaItem);
      return luaItem;
    }

    public class LogLine
    {
      public string Text { get; set; }

      public string Color { get; set; }
    }

    private struct CacheEntry
    {
      public uint count;
      public AccessibilityLevel maxAccessibility;
    }

    public enum StandardCallback
    {
      AccessibilityUpdating,
      AccessibilityUpdated,
      StartLoadingSaveFile,
      FinishLoadingSaveFile,
      PackReady,
      AutoTrackerStarted,
      AutoTrackerStopped,
    }
  }
}
