﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Data.Tracker
// Assembly: EmoTracker.Data, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: D41289DD-41C0-4478-8624-1DDBDED644F2
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.Data.dll

using EmoTracker.Core;
using EmoTracker.Data.JSON;
using EmoTracker.Data.Layout;
using EmoTracker.Data.Locations;
using EmoTracker.Data.Packages;
using EmoTracker.Data.Scripting;
using EmoTracker.Data.Settings;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.IO;
using System.Linq;

namespace EmoTracker.Data
{
  public class Tracker : ObservableSingleton<Tracker>, ICodeProvider
  {
    private static readonly string DefaultDisabledImageFilterSpec = "grayscale, dim";
    private string mDisabledImageFilterSpec = Tracker.DefaultDisabledImageFilterSpec;
    private bool mbSuspendPackReadyEvent;
    private bool mbMapEnabled = true;
    private bool mbSwapLeftRight;
    private bool mbAllowResize = true;
    private IGamePackage mActiveGamePackage;
    private IGamePackageVariant mActiveGamePackageVariant;
    private bool mbReloadInProgress;

    public event EventHandler<EventArgs> OnPackageLoadStarting;

    public event EventHandler<EventArgs> OnPackageLoadComplete;

    public bool SuspendPackReadyEvent
    {
      get => this.mbSuspendPackReadyEvent;
      protected set => this.SetProperty<bool>(ref this.mbSuspendPackReadyEvent, value, nameof (SuspendPackReadyEvent));
    }

    public string DisabledImageFilterSpec
    {
      get => this.mDisabledImageFilterSpec;
      set => this.SetProperty<string>(ref this.mDisabledImageFilterSpec, value, nameof (DisabledImageFilterSpec));
    }

    private void ValidatePackageSafety(IGamePackage package)
    {
    }

    public IGamePackage ActiveGamePackage
    {
      get => this.mActiveGamePackage;
      set
      {
        if (!this.SetProperty<IGamePackage>(ref this.mActiveGamePackage, value, nameof (ActiveGamePackage)))
          return;
        this.ValidatePackageSafety(this.mActiveGamePackage);
        ObservableSingleton<PackageManager>.Instance.RefreshActiveState();
        this.Reload();
        if (this.mActiveGamePackage != null)
        {
          ObservableSingleton<ApplicationSettings>.Instance.LastActivePackage = this.mActiveGamePackage.UniqueID;
          if (this.ActiveGamePackage.AvailableVariants.Contains<IGamePackageVariant>(this.ActiveGamePackageVariant))
            return;
          this.ActiveGamePackageVariant = (IGamePackageVariant) null;
        }
        else
          this.ActiveGamePackageVariant = (IGamePackageVariant) null;
      }
    }

    [DependentProperty("ActiveVariantUID")]
    public IGamePackageVariant ActiveGamePackageVariant
    {
      get => this.mActiveGamePackageVariant;
      set
      {
        if (!this.SetProperty<IGamePackageVariant>(ref this.mActiveGamePackageVariant, value, nameof (ActiveGamePackageVariant)))
          return;
        if (this.mActiveGamePackageVariant != null)
        {
          this.ActiveGamePackage = this.mActiveGamePackageVariant.Package;
          this.ActiveGamePackage.ActiveVariant = this.mActiveGamePackageVariant;
          ObservableSingleton<ApplicationSettings>.Instance.LastActivePackageVariant = this.mActiveGamePackageVariant.UniqueID;
        }
        else
          ObservableSingleton<ApplicationSettings>.Instance.LastActivePackageVariant = (string) null;
        ObservableSingleton<PackageManager>.Instance.RefreshActiveState();
        this.Reload();
      }
    }

    public string ActiveVariantUID => this.ActiveGamePackageVariant != null ? this.ActiveGamePackageVariant.UniqueID : "none";

    public bool SwapLeftRight
    {
      get => this.mbSwapLeftRight;
      set
      {
        if (!this.SetProperty<bool>(ref this.mbSwapLeftRight, value, nameof (SwapLeftRight)))
          return;
        this.Reload();
      }
    }

    public bool MapEnabled
    {
      get => this.mbMapEnabled;
      set
      {
        if (!this.SetProperty<bool>(ref this.mbMapEnabled, value, nameof (MapEnabled)))
          return;
        this.Reload();
      }
    }

    public bool AllowResize
    {
      get => this.mbAllowResize;
      set => this.SetProperty<bool>(ref this.mbAllowResize, value, nameof (AllowResize));
    }

    public bool IsActivePackage(string ID) => this.mActiveGamePackage != null && this.mActiveGamePackage.UniqueID == ID;

    public bool SaveProgress(string path, Action<JObject> dataAction = null)
    {
      JObject root = new JObject();
      if (this.ActiveGamePackage != null)
      {
        root["package_uid"] = (JToken) this.ActiveGamePackage.UniqueID;
        if (this.ActiveGamePackageVariant != null)
          root["package_variant_uid"] = (JToken) this.ActiveGamePackageVariant.UniqueID;
        root["package_version"] = (JToken) this.ActiveGamePackage.Version.ToString();
        root["creation_time"] = (JToken) DateTime.Now.ToString();
        root["ignore_all_logic"] = (JToken) ObservableSingleton<ApplicationSettings>.Instance.IgnoreAllLogic;
        root["display_all_locations"] = (JToken) ObservableSingleton<ApplicationSettings>.Instance.DisplayAllLocations;
        root["always_allow_chest_manipulation"] = (JToken) ObservableSingleton<ApplicationSettings>.Instance.AlwaysAllowClearing;
        root["auto_unpin_locations_on_clear"] = (JToken) ObservableSingleton<ApplicationSettings>.Instance.AutoUnpinLocationsOnClear;
        root["pin_locations_on_item_capture"] = (JToken) ObservableSingleton<ApplicationSettings>.Instance.PinLocationsOnItemCapture;
        Singleton<ItemDatabase>.Instance.Save(root);
        ObservableSingleton<LocationDatabase>.Instance.Save(root);
        if (dataAction != null)
          dataAction(root);
        try
        {
          using (TextWriter textWriter = (TextWriter) new StreamWriter(path))
          {
            using (JsonTextWriter writer = new JsonTextWriter(textWriter))
            {
              writer.Formatting = Formatting.Indented;
              root.WriteTo((JsonWriter) writer);
            }
          }
          return true;
        }
        catch
        {
          if (ObservableSingleton<ScriptManager>.Instance.NotificationService != null)
            ObservableSingleton<ScriptManager>.Instance.NotificationService.PushMarkdownNotification(NotificationType.Error, "### Couldn't Save Progress\r\n\r\nAn error occurred while saving. This may be due to anti-virus/malware software protecting your chosen save directory over-aggressively.\r\n* Ensure you have write permissions for the directory you chose\r\n* Saving to a different location may work");
        }
      }
      return false;
    }

    public bool LoadProgress(string path, Action<JObject> dataAction = null)
    {
      this.Reload();
      try
      {
        ObservableSingleton<ScriptManager>.Instance.Output("Loading save game \"{0}\"", (object) path);
        ObservableSingleton<ScriptManager>.Instance.InvokeStandardCallback(ScriptManager.StandardCallback.StartLoadingSaveFile);
        using (StreamReader reader = new StreamReader(path))
        {
          JObject jobject = (JObject) JToken.ReadFrom((JsonReader) new JsonTextReader((TextReader) reader));
          string uid1 = jobject.GetValue<string>("package_uid", (string) null);
          string uid2 = jobject.GetValue<string>("package_variant_uid", (string) null);
          Version result;
          Version.TryParse(jobject.GetValue<string>("package_version", (string) null), out result);
          IGamePackage installedPackage = (IGamePackage) ObservableSingleton<PackageManager>.Instance.FindInstalledPackage(uid1);
          if (installedPackage == null || installedPackage.Version != result)
            return false;
          IGamePackageVariant variant = installedPackage.FindVariant(uid2);
          if (!string.IsNullOrWhiteSpace(uid2) && variant == null)
            return false;
          this.SuspendPackReadyEvent = true;
          if (variant != null)
            this.ActiveGamePackageVariant = variant;
          else
            this.ActiveGamePackage = installedPackage;
          if (!Singleton<ItemDatabase>.Instance.Load(jobject) || !ObservableSingleton<LocationDatabase>.Instance.Load(jobject))
            return false;
          ObservableSingleton<ApplicationSettings>.Instance.IgnoreAllLogic = jobject.GetValue<bool>("ignore_all_logic");
          ObservableSingleton<ApplicationSettings>.Instance.DisplayAllLocations = jobject.GetValue<bool>("display_all_locations");
          ObservableSingleton<ApplicationSettings>.Instance.AlwaysAllowClearing = jobject.GetValue<bool>("always_allow_chest_manipulation");
          ObservableSingleton<ApplicationSettings>.Instance.AutoUnpinLocationsOnClear = jobject.GetValue<bool>("auto_unpin_locations_on_clear", true);
          ObservableSingleton<ApplicationSettings>.Instance.PinLocationsOnItemCapture = jobject.GetValue<bool>("pin_locations_on_item_capture", true);
          if (dataAction != null)
            dataAction(jobject);
        }
        return true;
      }
      catch (Exception ex)
      {
        ObservableSingleton<ScriptManager>.Instance.OutputError("Error encountered while loading save game");
        ObservableSingleton<ScriptManager>.Instance.OutputException(ex);
      }
      finally
      {
        ObservableSingleton<ScriptManager>.Instance.InvokeStandardCallback(ScriptManager.StandardCallback.FinishLoadingSaveFile);
        ObservableSingleton<ScriptManager>.Instance.InvokeStandardCallback(ScriptManager.StandardCallback.PackReady);
        ObservableSingleton<ScriptManager>.Instance.Output("Finished loading save game \"{0}\"", (object) path);
        this.SuspendPackReadyEvent = false;
      }
      return false;
    }

    private void GetFilteredCodeAndProvider(ref string code, out ICodeProvider provider)
    {
      provider = (ICodeProvider) Singleton<ItemDatabase>.Instance;
      if (code.StartsWith("@"))
      {
        code = code.Substring(1, code.Length - 1);
        provider = (ICodeProvider) ObservableSingleton<LocationDatabase>.Instance;
      }
      else
      {
        if (!code.StartsWith("$"))
          return;
        code = code.Substring(1, code.Length - 1);
        provider = (ICodeProvider) ObservableSingleton<ScriptManager>.Instance;
      }
    }

    public object FindObjectForCode(string code)
    {
      ICodeProvider provider;
      this.GetFilteredCodeAndProvider(ref code, out provider);
      return provider.FindObjectForCode(code);
    }

    public uint ProviderCountForCode(string code, out AccessibilityLevel maxAccessibility)
    {
      ICodeProvider provider;
      this.GetFilteredCodeAndProvider(ref code, out provider);
      return provider.ProviderCountForCode(code, out maxAccessibility);
    }

    public (bool, string) LoadDefaultPackage()
    {
      string uid = string.IsNullOrEmpty(ObservableSingleton<ApplicationSettings>.Instance.CommandLinePackage) ? ObservableSingleton<ApplicationSettings>.Instance.LastActivePackage : ObservableSingleton<ApplicationSettings>.Instance.CommandLinePackage;
      string b = string.IsNullOrEmpty(ObservableSingleton<ApplicationSettings>.Instance.CommandLinePackageVariant) ? ObservableSingleton<ApplicationSettings>.Instance.LastActivePackageVariant : ObservableSingleton<ApplicationSettings>.Instance.CommandLinePackageVariant;
      ObservableSingleton<Tracker>.Instance.ActiveGamePackage = (IGamePackage) ObservableSingleton<PackageManager>.Instance.FindInstalledPackage(uid);
      if (ObservableSingleton<Tracker>.Instance.ActiveGamePackage != null && ObservableSingleton<Tracker>.Instance.ActiveGamePackage.AvailableVariants != null)
      {
        bool flag = false;
        if (!string.IsNullOrWhiteSpace(b))
        {
          foreach (IGamePackageVariant availableVariant in ObservableSingleton<Tracker>.Instance.ActiveGamePackage.AvailableVariants)
          {
            if (string.Equals(availableVariant.UniqueID, b, StringComparison.Ordinal))
            {
              ObservableSingleton<Tracker>.Instance.ActiveGamePackageVariant = availableVariant;
              flag = true;
              break;
            }
          }
        }
        if (ObservableSingleton<Tracker>.Instance.ActiveGamePackageVariant == null && ObservableSingleton<Tracker>.Instance.ActiveGamePackage.AvailableVariants != null)
          ObservableSingleton<Tracker>.Instance.ActiveGamePackageVariant = ObservableSingleton<Tracker>.Instance.ActiveGamePackage.AvailableVariants.FirstOrDefault<IGamePackageVariant>();
        if (!flag)
        {
          if (ObservableSingleton<Tracker>.Instance.ActiveGamePackageVariant == null)
            return (false, "### Package Variant " + b + " not found, loading default pack for " + uid);
          string uniqueId = ObservableSingleton<Tracker>.Instance.ActiveGamePackageVariant.UniqueID;
          return (false, "### Package Variant " + b + " not found, loading default variant `" + uniqueId + "` for " + uid);
        }
      }
      return (true, string.Empty);
    }

    private void ResetPackageSettings()
    {
      this.AllowResize = true;
      this.DisabledImageFilterSpec = Tracker.DefaultDisabledImageFilterSpec;
    }

    private void LoadPackageSettings()
    {
      if (this.mActiveGamePackage == null)
        return;
      bool flag = false;
      using (Stream stream = this.mActiveGamePackage.Open("settings.json"))
      {
        if (stream != null)
        {
          ObservableSingleton<ScriptManager>.Instance.Output("Loading package settings");
          using (new LoggingBlock())
          {
            try
            {
              using (StreamReader reader = new StreamReader(stream))
              {
                JObject jobject = (JObject) JToken.ReadFrom((JsonReader) new JsonTextReader((TextReader) reader));
                this.AllowResize = jobject.GetValue<bool>("allow_resize", true);
                string str = jobject.GetValue<string>("disabled_image_filter", (string) null);
                if (str != null)
                  this.DisabledImageFilterSpec = str;
                ObservableSingleton<LocationDatabase>.Instance.ParseLocationVisualProperties(jobject, (LocationVisualProperties) ObservableSingleton<LocationDatabase>.Instance.Root, this.mActiveGamePackage);
                AccessibilityRule.EnableCache = jobject.GetValue<bool>("enable_accessibility_rule_caching", true);
                flag = true;
              }
            }
            catch (Exception ex)
            {
              ObservableSingleton<ScriptManager>.Instance.OutputException(ex);
            }
          }
        }
      }
      if (flag)
        return;
      ObservableSingleton<ScriptManager>.Instance.OutputWarning("Loading legacy package settings from tracker_layout.json");
      using (new LoggingBlock())
      {
        try
        {
          using (StreamReader reader = new StreamReader(this.mActiveGamePackage.Open("tracker_layout.json")))
          {
            JObject jobject = (JObject) JToken.ReadFrom((JsonReader) new JsonTextReader((TextReader) reader));
            this.AllowResize = jobject.GetValue<bool>("allow_resize", true);
            string str = jobject.GetValue<string>("disabled_image_filter", (string) null);
            if (str != null)
              this.DisabledImageFilterSpec = str;
            ObservableSingleton<LocationDatabase>.Instance.ParseLocationVisualProperties(jobject, (LocationVisualProperties) ObservableSingleton<LocationDatabase>.Instance.Root, this.mActiveGamePackage);
          }
        }
        catch (Exception ex)
        {
          ObservableSingleton<ScriptManager>.Instance.OutputException(ex);
        }
      }
    }

    public void Reload()
    {
      if (this.mbReloadInProgress)
        return;
      this.mbReloadInProgress = true;
      try
      {
        AccessibilityRule.ClearCaches();
        if (this.OnPackageLoadStarting != null)
          this.OnPackageLoadStarting((object) this, EventArgs.Empty);
        this.ResetPackageSettings();
        ObservableSingleton<LayoutManager>.Instance.Clear();
        ObservableSingleton<MapDatabase>.Instance.Reset();
        ObservableSingleton<LocationDatabase>.Instance.Reset();
        Singleton<ItemDatabase>.Instance.Reset();
        ObservableSingleton<ScriptManager>.Instance.Reset();
        ObservableSingleton<ApplicationColors>.Instance.LoadColors();
        if (this.mActiveGamePackage == null)
          return;
        ObservableSingleton<ScriptManager>.Instance.Output("Beginning Package Load");
        using (new LoggingBlock())
        {
          ObservableSingleton<ScriptManager>.Instance.Output(string.Format("Package: {0}", (object) this.ActiveGamePackage.UniqueID));
          if (this.ActiveGamePackageVariant != null)
            ObservableSingleton<ScriptManager>.Instance.Output(string.Format("Variant: {0}", (object) this.ActiveGamePackageVariant.UniqueID));
          this.LoadPackageSettings();
          ObservableSingleton<ScriptManager>.Instance.Load(this.mActiveGamePackage);
          Singleton<ItemDatabase>.Instance.LegacyLoad(this.mActiveGamePackage);
          ObservableSingleton<MapDatabase>.Instance.LegacyLoad(this.mActiveGamePackage);
          ObservableSingleton<LocationDatabase>.Instance.LegacyLoad(this.mActiveGamePackage);
        }
        ObservableSingleton<ScriptManager>.Instance.Output("Package Load Finished");
      }
      finally
      {
        AccessibilityRule.ClearCaches();
        this.mbReloadInProgress = false;
        if (this.OnPackageLoadComplete != null)
          this.OnPackageLoadComplete((object) this, EventArgs.Empty);
        if (!this.SuspendPackReadyEvent)
          ObservableSingleton<ScriptManager>.Instance.InvokeStandardCallback(ScriptManager.StandardCallback.PackReady);
      }
    }

    public void AddItems(string path)
    {
      if (this.ActiveGamePackage == null)
        return;
      Singleton<ItemDatabase>.Instance.IncrementalLoad(path, this.ActiveGamePackage);
    }

    public void AddMaps(string path)
    {
      if (this.ActiveGamePackage == null)
        return;
      ObservableSingleton<MapDatabase>.Instance.IncrementalLoad(path, this.ActiveGamePackage);
    }

    public void AddLocations(string path)
    {
      if (this.ActiveGamePackage == null)
        return;
      ObservableSingleton<LocationDatabase>.Instance.IncrementalLoad(path, this.ActiveGamePackage);
    }

    public void AddLayouts(string path)
    {
      if (this.ActiveGamePackage == null)
        return;
      ObservableSingleton<LayoutManager>.Instance.IncrementalLoad(path, this.ActiveGamePackage);
    }
  }
}
