﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.App
// Assembly: EmoTracker, Version=2.3.8.17, Culture=neutral, PublicKeyToken=null
// MVID: 0D1A2596-8086-4DA5-90AE-09EB4913AFBC
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.exe

using EmoTracker.Core;
using EmoTracker.Core.Services.Backends;
using EmoTracker.Data;
using EmoTracker.Data.Core.Transactions;
using EmoTracker.Data.Core.Transactions.Processors;
using EmoTracker.Extensions;
using EmoTracker.Services;
using Sentry;
using Sentry.Protocol;
using Sentry.Serilog;
using Serilog;
using Serilog.Events;
using System;
using System.CodeDom.Compiler;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Security;
using System.Net.Sockets;
using System.Runtime.InteropServices;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Windows;
using System.Windows.Media;
using System.Windows.Threading;

namespace EmoTracker
{
    public class App : Application
    {
        private IDisposable mSentrySDKHandle;
        private static readonly string kSaltString = "+69642a0110cb09ae7dc4";

        private IPAddress LocalIPAddress() => !NetworkInterface.GetIsNetworkAvailable() ? (IPAddress)null : ((IEnumerable<IPAddress>)Dns.GetHostEntry(Dns.GetHostName()).AddressList).FirstOrDefault<IPAddress>((Func<IPAddress, bool>)(ip => ip.AddressFamily == AddressFamily.InterNetwork));

        internal static string GetAnonymizedStringHash(string text)
        {
            if (string.IsNullOrEmpty(text))
                return string.Empty;
            using (MD5Cng md5Cng = new MD5Cng())
            {
                byte[] bytes = Encoding.UTF8.GetBytes(text + App.kSaltString);
                return BitConverter.ToString(md5Cng.ComputeHash(bytes)).Replace("-", string.Empty);
            }
        }

        internal static string GetAnonymizedUserIdentifier()
        {
            string userName = Environment.UserName;
            string machineName = Environment.MachineName;
            string text = "";
            if (!string.IsNullOrWhiteSpace(userName))
                text += userName;
            if (!string.IsNullOrWhiteSpace(machineName))
            {
                if (!string.IsNullOrWhiteSpace(text))
                    text += "+";
                text += machineName;
            }
            return !string.IsNullOrWhiteSpace(text) ? "anonymous_" + App.GetAnonymizedStringHash(text) : "anonymous_unknown";
        }

        private void ConfigureSentryScope() => SentrySdk.ConfigureScope((Action<Scope>)(scope =>
        {
            string anonymizedUserIdentifier = App.GetAnonymizedUserIdentifier();
            string str = "unknown";
            IPAddress ipAddress = this.LocalIPAddress();
            if (ipAddress != null)
                str = ipAddress.ToString();
            if (!string.IsNullOrWhiteSpace(anonymizedUserIdentifier) || !string.IsNullOrWhiteSpace(str))
                scope.User = new User()
                {
                    Username = anonymizedUserIdentifier,
                    IpAddress = str
                };
            if (Environment.Is64BitProcess)
                scope.SetTag("process-type", "x64");
            else
                scope.SetTag("process-type", "x86");
            if (ObservableSingleton<Tracker>.Instance.ActiveGamePackage != null)
            {
                scope.SetTag("active-package", ObservableSingleton<Tracker>.Instance.ActiveGamePackage.UniqueID);
                if (ObservableSingleton<Tracker>.Instance.ActiveGamePackageVariant != null)
                    scope.SetTag("active-variant", ObservableSingleton<Tracker>.Instance.ActiveGamePackageVariant.UniqueID);
                else
                    scope.UnsetTag("active-variant");
            }
            else
            {
                scope.UnsetTag("active-package");
                scope.UnsetTag("active-variant");
            }
        }));

        protected override void OnStartup(StartupEventArgs e)
        {
            this.mSentrySDKHandle = SentrySdk.Init((Action<SentryOptions>)(o =>
            {
                o.Dsn = new Dsn("https://ac8fef540fae4540990db03e8adfdf01@sentry.io/1401062");
                o.Environment = "production";
                o.SendDefaultPii = false;
                o.IsEnvironmentUser = false;
                o.Debug = true;
            }));
            this.ConfigureSentryScope();
            this.Dispatcher.UnhandledException += new DispatcherUnhandledExceptionEventHandler(this.Dispatcher_UnhandledException);
            this.ForceLoadLuaDLL();
            this.ConfigurePlatformDllPaths();
            TransactionProcessor.SetTransactionProcessor((ITransactionProcessor)new LocalTransactionProcessorWithUndo());
            EmoTracker.Core.Services.Backends.LogService.SetServiceBackend((ILogServiceBackend)new EmoTracker.Services.LogService());
            EmoTracker.Core.Services.Backends.DispatchService.SetServiceBackend((IDispatchServiceBackend)new EmoTracker.Services.DispatchService());
            try
            {
                string logDirectory = Path.Combine(UserDirectory.Path, "logs");
                Log.Logger = new LoggerConfiguration().MinimumLevel.Verbose().Enrich.FromLogContext().WriteTo.File(
                    Path.Combine(logDirectory, "emotracker_log.txt"),
                    LogEventLevel.Verbose,
                    "{Timestamp:yyyy-MM-dd HH:mm:ss.fff zzz} [{Level:u3}] {Message:lj}{NewLine}{Exception}",
                    null, new long?(1073741824L), null, true, false,
                    new TimeSpan?(TimeSpan.FromSeconds(5.0)), RollingInterval.Day,
                    false, new int?(31), null, null
                ).WriteTo.Console(
                    LogEventLevel.Information,
                    "[{Timestamp:HH:mm:ss} {Level:u3}] {Message:lj}{NewLine}{Exception}",
                    null, null, null, null
                ).WriteTo.DeveloperConsole(null).WriteTo.Sentry(delegate (SentrySerilogOptions o)
                {
                    o.MinimumBreadcrumbLevel = LogEventLevel.Debug;
                    o.MinimumEventLevel = LogEventLevel.Error;
                    o.AttachStacktrace = true;
                }).CreateLogger();
            }
            catch (Exception ex)
            {
                SentryEvent evt = new SentryEvent(ex);
                evt.Level = new SentryLevel?(SentryLevel.Info);
                evt.Message = "Failed to initialize Serilog, due to an exception";
                SentrySdk.CaptureEvent(evt);
            }
            if (Environment.OSVersion.Version.Major < 10)
            {
                ServicePointManager.Expect100Continue = true;
                ServicePointManager.SecurityProtocol |= SecurityProtocolType.Tls12;
            }
            ServicePointManager.ServerCertificateValidationCallback += new RemoteCertificateValidationCallback(this.ValidateRemoteCertificate);
            foreach (FontFamily fontFamily in (IEnumerable<FontFamily>)Fonts.GetFontFamilies(new Uri("pack://application:,,,/"), "./Resources/Fonts/"))
                this.Resources.Add((object)Path.GetFileName(fontFamily.ToString()).Replace("#", "").Replace(" ", ""), (object)fontFamily);
            ObservableSingleton<ApplicationSettings>.CreateInstance();
            MainWindow mainWindow = new MainWindow();
            this.MainWindow = (Window)mainWindow;
            mainWindow.Show();
            base.OnStartup(e);
        }

        private void Dispatcher_UnhandledException(
          object sender,
          DispatcherUnhandledExceptionEventArgs args)
        {
            Exception exception = args.Exception;
            if (exception == null)
                return;
            this.ConfigureSentryScope();
            SentryEvent evt = new SentryEvent(exception);
            evt.Level = new SentryLevel?(SentryLevel.Fatal);
            evt.Message = exception.Message;
            SentrySdk.CaptureEvent(evt);
            SentrySdk.FlushAsync(TimeSpan.FromMilliseconds(500.0));
        }

        [DllImport("kernel32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        private static extern IntPtr LoadLibrary(string libname);

        [DllImport("kernel32.dll", CharSet = CharSet.Unicode, SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        private static extern bool SetDllDirectory(string lpPathName);

        private void ForceLoadLuaDLL()
        {
            string str = "x64";
            if (!Environment.Is64BitProcess)
                str = "x86";
            string libname = Path.Combine(AppDomain.CurrentDomain.SetupInformation.ApplicationBase, string.Format("{0}\\lua53.dll", (object)str));
            try
            {
                if (!(App.LoadLibrary(libname) == IntPtr.Zero))
                    return;
                int num = (int)MessageBox.Show("EmoTracker was unable to load the Lua interpreter DLL, and will now close.\n\nSee the EmoTracker Discord for more help.", "Critical Error");
                SentrySdk.CaptureMessage(string.Format("Failed to force-load Lua interpreter DLL at path: {0}", (object)libname), SentryLevel.Fatal);
                this.Shutdown(-20);
            }
            catch (Exception ex)
            {
                int num = (int)MessageBox.Show("EmoTracker was unable to load the Lua interpreter DLL, and will now close.\n\nSee the EmoTracker Discord for more help.", "Critical Error");
                throw new InvalidOperationException(string.Format("Failed to force-load Lua interpreter DLL at path: {0}", (object)libname), ex);
            }
        }

        private void ConfigurePlatformDllPaths()
        {
            try
            {
                string str = "x64";
                if (!Environment.Is64BitProcess)
                    str = "x86";
                App.SetDllDirectory(Path.Combine(AppDomain.CurrentDomain.SetupInformation.ApplicationBase, string.Format("{0}\\", (object)str)));
            }
            catch
            {
                throw new InvalidOperationException("Failed to set platform DLL search directory.");
            }
        }

        private bool ValidateRemoteCertificate(
          object sender,
          X509Certificate certificate,
          X509Chain chain,
          SslPolicyErrors sslPolicyErrors)
        {
            if (!(sender is HttpWebRequest httpWebRequest) || !(httpWebRequest.Address.Host == "emotracker.net") || !httpWebRequest.Address.AbsoluteUri.Contains("supported_games.json"))
                return sslPolicyErrors == SslPolicyErrors.None;
            string certHashString = certificate.GetCertHashString();
            return certHashString == "4BF8AB9AB6CDDC510ADDBF26AA786D14C80C5179" || certHashString == "C55D11E139CE20D4D12F86BDD49E4AEAA65DD859" || certHashString == "111B70EC242C2B9D28E2AA02BE75A2CAEB562193" || certHashString == "12386C2837661FBAA80B60338EAB27EDBFE60963" || certHashString == "95EBC6BDF39FCB6FC3AF6712ED65DC278CD9DA4D";
        }

        protected override void OnExit(ExitEventArgs e)
        {
            try
            {
                if (e.ApplicationExitCode == 0)
                    ObservableSingleton<ExtensionManager>.Instance.OnApplicationClosing();
                if (!ObservableSingleton<ApplicationSettings>.Instance.EnableDiscordRichPresence)
                    return;
                try
                {
                    DiscordRpc.ClearPresence();
                    DiscordRpc.Shutdown();
                }
                catch
                {
                }
            }
            finally
            {
                if (this.mSentrySDKHandle != null)
                    this.mSentrySDKHandle.Dispose();
                base.OnExit(e);
            }
        }

        [STAThread]
        [DebuggerNonUserCode]
        [GeneratedCode("PresentationBuildTasks", "4.0.0.0")]
        public static void Main() => new App().Run();
    }
}
