﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.ApplicationModel
// Assembly: EmoTracker, Version=2.3.8.17, Culture=neutral, PublicKeyToken=null
// MVID: 0D1A2596-8086-4DA5-90AE-09EB4913AFBC
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.exe

using EmoTracker.Core;
using EmoTracker.Data;
using EmoTracker.Data.Core.Transactions;
using EmoTracker.Data.Core.Transactions.Processors;
using EmoTracker.Data.JSON;
using EmoTracker.Data.Layout;
using EmoTracker.Data.Locations;
using EmoTracker.Data.Packages;
using EmoTracker.Data.Scripting;
using EmoTracker.Extensions;
using EmoTracker.Notifications;
using EmoTracker.UI;
using EmoTracker.UI.Converters;
using EmoTracker.UI.Media;
using Microsoft.Win32;
using Newtonsoft.Json.Linq;
using Sentry;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Windows;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Threading;

namespace EmoTracker
{
    public class ApplicationModel :
      ObservableSingleton<ApplicationModel>,
      ICodeProvider,
      INotificationService
    {
        private EmoTracker.Data.Layout.Layout mBroadcastLayout = new EmoTracker.Data.Layout.Layout();
        private EmoTracker.Data.Layout.Layout mTrackerLayout;
        private EmoTracker.Data.Layout.Layout mTrackerHorizontalLayout;
        private EmoTracker.Data.Layout.Layout mTrackerVerticalLayout;
        private EmoTracker.Data.Layout.Layout mTrackerCaptureItemLayout;
        private int mMainLayoutScale = 100;
        private string mCurrentSavePath;
        private ApplicationModel.AvailablePackageViewFilterType mAvailablePackagesViewFilter;
        private DelegateCommand mSetAvailablePackageViewFilterCommand;
        private ListCollectionView mAvailablePackagesView;
        private ListCollectionView mInstalledPackagesView;
        private string mPackFilterText;
        private ObservableCollection<Notification> mPreviousNotifications = new ObservableCollection<Notification>();
        private ObservableCollection<Notification> mNotifications = new ObservableCollection<Notification>();
        private DispatcherTimer mNotificationUpdateTimer;

        public DelegateCommand RefreshCommand { get; private set; }

        public DelegateCommand ResetUserDataCommand { get; private set; }

        public DelegateCommand OpenPackOverrideFolderCommand { get; private set; }

        public DelegateCommand ActivatePackCommand { get; private set; }

        public DelegateCommand ShowPackageManagerCommand { get; private set; }

        public DelegateCommand ExportPackageOverrideCommand { get; private set; }

        public DelegateCommand CheckForUpdateCommand { get; private set; }

        public DelegateCommand ShowBroadcastViewCommand { get; private set; }

        public DelegateCommand ShowDeveloperConsoleCommand { get; private set; }

        public DelegateCommand SaveCommand { get; private set; }

        public DelegateCommand SaveAsCommand { get; private set; }

        public DelegateCommand OpenCommand { get; private set; }

        public DelegateCommand OpenPackageDocumentationCommand { get; private set; }

        public DelegateCommand ResetLayoutScaleCommand { get; private set; }

        public DelegateCommand InstallPackageCommand { get; private set; }

        public DelegateCommand UninstallPackageCommand { get; private set; }

        public string MainWindowTitle
        {
            get
            {
                string mainWindowTitle = string.Format("EmoTracker {0}", (object)ApplicationVersion.Current);
                if (ObservableSingleton<Tracker>.Instance.ActiveGamePackage != null && ObservableSingleton<Tracker>.Instance.ActiveGamePackageVariant != null)
                    mainWindowTitle = string.Format("{0}  ::  {1} | {2}", (object)mainWindowTitle, (object)ObservableSingleton<Tracker>.Instance.ActiveGamePackage.DisplayName, (object)ObservableSingleton<Tracker>.Instance.ActiveGamePackageVariant.DisplayName);
                else if (ObservableSingleton<Tracker>.Instance.ActiveGamePackage != null)
                    mainWindowTitle = string.Format("{0}  ::  {1}", (object)mainWindowTitle, (object)ObservableSingleton<Tracker>.Instance.ActiveGamePackage.DisplayName);
                return mainWindowTitle;
            }
        }

        public EmoTracker.Data.Layout.Layout BroadcastLayout
        {
            get => this.mBroadcastLayout;
            protected set => this.SetProperty<EmoTracker.Data.Layout.Layout>(ref this.mBroadcastLayout, value, nameof(BroadcastLayout));
        }

        public EmoTracker.Data.Layout.Layout TrackerLayout
        {
            get => this.mTrackerLayout;
            protected set
            {
                this.SetProperty<EmoTracker.Data.Layout.Layout>(ref this.mTrackerLayout, value, nameof(TrackerLayout));
                this.NotifyPropertyChanged("TrackerHorizontalLayout");
                this.NotifyPropertyChanged("TrackerVerticalLayout");
            }
        }

        public EmoTracker.Data.Layout.Layout TrackerHorizontalLayout
        {
            get => this.mTrackerHorizontalLayout != null ? this.mTrackerHorizontalLayout : this.mTrackerLayout;
            protected set => this.SetProperty<EmoTracker.Data.Layout.Layout>(ref this.mTrackerHorizontalLayout, value, nameof(TrackerHorizontalLayout));
        }

        public EmoTracker.Data.Layout.Layout TrackerVerticalLayout
        {
            get => this.mTrackerVerticalLayout != null ? this.mTrackerVerticalLayout : this.mTrackerLayout;
            protected set => this.SetProperty<EmoTracker.Data.Layout.Layout>(ref this.mTrackerVerticalLayout, value, nameof(TrackerVerticalLayout));
        }

        public EmoTracker.Data.Layout.Layout TrackerCaptureItemLayout
        {
            get => this.mTrackerCaptureItemLayout;
            protected set => this.SetProperty<EmoTracker.Data.Layout.Layout>(ref this.mTrackerCaptureItemLayout, value, nameof(TrackerCaptureItemLayout));
        }

        public ApplicationModel()
        {
            this.InitializeNotifications();
            if (Application.Current is App)
            {
                ObservableSingleton<PackageManager>.CreateInstance();
                ObservableSingleton<PackageManager>.Instance.Initialize();
                this.InitializePackageManagerViews();
            }
            ObservableSingleton<Tracker>.Instance.OnPackageLoadStarting += new EventHandler<EventArgs>(this.Tracker_OnPackageLoadStarting);
            ObservableSingleton<Tracker>.Instance.OnPackageLoadComplete += new EventHandler<EventArgs>(this.Tracker_OnPackageLoadComplete);
            this.RefreshCommand = new DelegateCommand(new Action<object>(this.RefreshHandler));
            this.ResetUserDataCommand = new DelegateCommand(new Action<object>(this.ResetUserDataHandler));
            this.OpenPackOverrideFolderCommand = new DelegateCommand(new Action<object>(this.OpenPackOverrideFolderHandler));
            this.ActivatePackCommand = new DelegateCommand(new Action<object>(this.ActivatePackHandler));
            this.ShowPackageManagerCommand = new DelegateCommand(new Action<object>(this.ShowPackManagerHandler));
            this.ExportPackageOverrideCommand = new DelegateCommand(new Action<object>(this.ExportPackageOverrideHandler));
            this.CheckForUpdateCommand = new DelegateCommand(new Action<object>(this.CheckForUpdateHandler));
            this.SaveCommand = new DelegateCommand(new Action<object>(this.SaveHandler), new Predicate<object>(this.CanSave));
            this.SaveAsCommand = new DelegateCommand(new Action<object>(this.SaveAsHandler), new Predicate<object>(this.CanSave));
            this.OpenCommand = new DelegateCommand(new Action<object>(this.OpenHandler));
            this.OpenPackageDocumentationCommand = new DelegateCommand(new Action<object>(this.OpenPackageDocumentation), new Predicate<object>(this.CanOpenPackageDocumentation));
            this.ResetLayoutScaleCommand = new DelegateCommand(new Action<object>(this.ResetLayoutScale));
            this.ShowBroadcastViewCommand = new DelegateCommand(new Action<object>(this.ShowBroadcastView));
            this.ShowDeveloperConsoleCommand = new DelegateCommand(new Action<object>(this.ShowDevleoperConsole));
            this.InstallPackageCommand = new DelegateCommand(new Action<object>(this.InstallPackage));
            this.UninstallPackageCommand = new DelegateCommand(new Action<object>(this.UninstallPackage), new Predicate<object>(this.CanUninstallPackage));
        }

        public void Initialize()
        {
            ObservableSingleton<ExtensionManager>.CreateInstance();
            ObservableSingleton<ExtensionManager>.Instance.Start();
            string markdown = ObservableSingleton<Tracker>.Instance.LoadDefaultPackage().Item2;
            if (!string.IsNullOrWhiteSpace(markdown))
                this.PushMarkdownNotification(NotificationType.Error, markdown, -1);
            this.NotifyPropertyChanged("MainWindowTitle");
        }

        private void ShowBroadcastView(object obj)
        {
            if (!(Application.Current.MainWindow is MainWindow mainWindow))
                return;
            mainWindow.ShowBroadcastView();
        }

        private void ShowDevleoperConsole(object obj)
        {
            if (!(Application.Current.MainWindow is MainWindow mainWindow))
                return;
            mainWindow.ShowDeveloperConsole();
        }

        private void InstallPackage(object obj)
        {
            PackageRepositoryEntry packageRepositoryEntry = (PackageRepositoryEntry)obj;
            if (packageRepositoryEntry.ExistingPackage != null && new DirectoryInfo(packageRepositoryEntry.ExistingPackage.OverridePath).Exists)
            {
                switch (MessageBox.Show("You have user overrides in place for " + packageRepositoryEntry.Name + " which may cause issues after updating. Do you want to backup and disable your overrides prior to updating?", "Uninstall Package", MessageBoxButton.YesNoCancel, MessageBoxImage.Exclamation))
                {
                    case MessageBoxResult.Cancel:
                        return;
                    case MessageBoxResult.Yes:
                        switch (packageRepositoryEntry.BackupOverride())
                        {
                            case BackupOverrideResult.Failed:
                                int num = (int)MessageBox.Show("Unable to backup " + packageRepositoryEntry.Name + " overrides. Check to make sure that no other application is using the folder or you do not have a backup instance already. Canceling update", "Backup Failed", MessageBoxButton.OK, MessageBoxImage.Hand);
                                return;
                        }
                        break;
                }
            }
            packageRepositoryEntry.Install();
        }

        private void UninstallPackage(object obj)
        {
            PackageRepositoryEntry packageRepositoryEntry = (PackageRepositoryEntry)obj;
            if (MessageBox.Show("You are about to uninstall \"" + packageRepositoryEntry.Name + "\". This will remove all the files associated with the package as well as the overrides. Do you wish to continue?", "Uninstall Package", MessageBoxButton.YesNo, MessageBoxImage.Exclamation) == MessageBoxResult.No)
                return;
            switch (packageRepositoryEntry.Uninstall())
            {
                case UninstallResult.FailedUninstall:
                    int num1 = (int)MessageBox.Show("Failed to uninstall \"" + packageRepositoryEntry.Name + "\"! Please ensure no other applications are using the file and try again.", "Failed to Uninstall", MessageBoxButton.OK, MessageBoxImage.Hand);
                    break;
                case UninstallResult.FailedOverrides:
                    int num2 = (int)MessageBox.Show("Failed to remove \"" + packageRepositoryEntry.Name + "\" overrides folder. You will need to remove it manually", "Failed to Remove Overrides", MessageBoxButton.OK, MessageBoxImage.Hand);
                    break;
            }
        }

        private bool CanUninstallPackage(object obj)
        {
            PackageRepositoryEntry packageRepositoryEntry = (PackageRepositoryEntry)obj;
            return packageRepositoryEntry != null && packageRepositoryEntry.ExistingPackage != null && packageRepositoryEntry.ExistingPackage.Source != null && packageRepositoryEntry.ExistingPackage.Source is ZipPackageSource;
        }

        private void OpenPackOverrideFolderHandler(object obj)
        {
            if (ObservableSingleton<Tracker>.Instance.ActiveGamePackage == null)
                return;
            if (string.IsNullOrWhiteSpace(ObservableSingleton<Tracker>.Instance.ActiveGamePackage.OverridePath))
                return;
            try
            {
                Directory.CreateDirectory(ObservableSingleton<Tracker>.Instance.ActiveGamePackage.OverridePath);
            }
            catch
            {
            }
            if (Directory.Exists(ObservableSingleton<Tracker>.Instance.ActiveGamePackage.OverridePath))
                Process.Start("explorer.exe", ObservableSingleton<Tracker>.Instance.ActiveGamePackage.OverridePath);
            else
                this.PushMarkdownNotification(NotificationType.Error, string.Format("### Cannot open override folder\r\nFailed to find or create the active pack's override folder at `{0}`.\r\n\r\nMake sure you have available disk space and permissions for the selected location.", (object)ObservableSingleton<Tracker>.Instance.ActiveGamePackage.OverridePath), -1);
        }

        private void ExportPackageOverrideHandler(object obj)
        {
            if (ObservableSingleton<Tracker>.Instance.ActiveGamePackage == null)
                return;
            string filename = obj as string;
            if (!string.IsNullOrWhiteSpace(filename))
            {
                if (!(ObservableSingleton<Tracker>.Instance.ActiveGamePackage is GamePackage activeGamePackage))
                    return;
                activeGamePackage.ExportUserOverride(filename);
            }
            else
            {
                OverrideExportDialog overrideExportDialog = new OverrideExportDialog();
                overrideExportDialog.Owner = Application.Current.MainWindow;
                overrideExportDialog.ShowDialog();
            }
        }

        private void ShowPackManagerHandler(object obj)
        {
            PackageManagerWindow packageManagerWindow = new PackageManagerWindow();
            packageManagerWindow.Owner = Application.Current.MainWindow;
            packageManagerWindow.ShowDialog();
            Keyboard.Focus((IInputElement)Application.Current.MainWindow);
        }

        private void CheckForUpdateHandler(object obj)
        {
            AppUpdateWindow appUpdateWindow = new AppUpdateWindow(false);
            appUpdateWindow.Owner = Application.Current.MainWindow;
            appUpdateWindow.ShowDialog();
            Keyboard.Focus((IInputElement)Application.Current.MainWindow);
        }

        private void RefreshHandler(object param)
        {
            if (ObservableSingleton<ApplicationSettings>.Instance.PromptOnRefreshClose && MessageBox.Show(Application.Current.MainWindow, "Refreshing will cause you to lose all unsaved progress. Are you sure you want to refresh?", "Warning!", MessageBoxButton.YesNo, MessageBoxImage.Exclamation, MessageBoxResult.No) != MessageBoxResult.Yes)
                return;
            this.Reload();
            Keyboard.Focus((IInputElement)Application.Current.MainWindow);
        }

        private void ResetUserDataHandler(object param)
        {
            if (ObservableSingleton<Tracker>.Instance.ActiveGamePackage != null)
            {
                if (ObservableSingleton<ApplicationSettings>.Instance.PromptOnRefreshClose && MessageBox.Show("Clearing overrides will cause you to lose all unsaved progress. Are you sure you want to continue?", "Warning!", MessageBoxButton.YesNo, MessageBoxImage.Exclamation, MessageBoxResult.No) != MessageBoxResult.Yes)
                    return;
                ObservableSingleton<Tracker>.Instance.ActiveGamePackage.ResetUserOverrides();
                this.Reload();
            }
            Keyboard.Focus((IInputElement)Application.Current.MainWindow);
        }

        private void ActivatePackHandler(object obj)
        {
            Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Background, new Action(delegate ()
            {
                IGamePackage package = obj as IGamePackage;
                IGamePackageVariant variant = obj as IGamePackageVariant;
                if (package != null)
                {
                    ObservableSingleton<Tracker>.Instance.ActiveGamePackageVariant = null;
                    ObservableSingleton<Tracker>.Instance.ActiveGamePackage = package;
                    return;
                }
                if (variant != null)
                {
                    ObservableSingleton<Tracker>.Instance.ActiveGamePackageVariant = variant;
                }
            }));
        }

        [DependentProperty("MainLayoutScaleFactor")]
        public int MainLayoutScale
        {
            get => this.mMainLayoutScale;
            set => this.SetProperty<int>(ref this.mMainLayoutScale, Math.Min(Math.Max(value, 100), 500), nameof(MainLayoutScale));
        }

        public double MainLayoutScaleFactor => (double)this.mMainLayoutScale / 100.0;

        public void IncrementMainLayoutScale(int steps) => this.MainLayoutScale += steps * 10;

        public void ResetLayoutScale(object obj = null) => this.MainLayoutScale = 100;

        private void OpenHandler(object obj)
        {
            string str = Path.Combine(UserDirectory.Path, "saves");
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = "EmoTracker Save File (*.json)|*.json";
            openFileDialog.InitialDirectory = str;
            bool? nullable = openFileDialog.ShowDialog();
            bool flag = true;
            if (!(nullable.GetValueOrDefault() == flag & nullable.HasValue))
                return;
            if (!this.LoadProgress(openFileDialog.FileName))
            {
                this.Reload();
                int num = (int)MessageBox.Show(Application.Current.MainWindow, "Failed to load the requested save file. Possible reasons include:\n\n• The original pack or variant no longer exists\n• The save data has been corruped\n• The pack version is different from the version used to save\n• The pack contents do not match the save data.\n\nNote that certain types of user overrides can affect this, if added/changed since saving.", "Failed to load save data...", MessageBoxButton.OK, MessageBoxImage.Hand);
            }
            else
                this.mCurrentSavePath = openFileDialog.FileName;
        }

        private bool CanSave(object obj) => ObservableSingleton<Tracker>.Instance.ActiveGamePackage != null;

        private void SaveHandler(object obj)
        {
            if (!string.IsNullOrWhiteSpace(this.mCurrentSavePath))
                this.SaveProgress(this.mCurrentSavePath);
            else
                this.SaveAsHandler(obj);
        }

        private void SaveAsHandler(object obj)
        {
            string path = Path.Combine(UserDirectory.Path, "saves");
            try
            {
                if (!Directory.Exists(path))
                    Directory.CreateDirectory(path);
            }
            catch
            {
                this.PushMarkdownNotification(NotificationType.Error, string.Format("### Failed to Save Progress\r\nFailed to create the default save directory at ```{0}```. Make sure you have available disk space and permissions for the default location.", (object)path), -1);
            }
            SaveFileDialog saveFileDialog = new SaveFileDialog();
            saveFileDialog.AddExtension = true;
            saveFileDialog.CheckPathExists = true;
            saveFileDialog.Filter = "EmoTracker Save File (*.json)|*.json";
            saveFileDialog.InitialDirectory = path;
            bool? nullable = saveFileDialog.ShowDialog();
            bool flag = true;
            if (!(nullable.GetValueOrDefault() == flag & nullable.HasValue))
                return;
            Directory.CreateDirectory(Path.GetDirectoryName(saveFileDialog.FileName));
            this.SaveProgress(saveFileDialog.FileName);
        }

        private bool SaveProgress(string path)
        {
            if (!this.CanSave((object)null))
                return false;
            try
            {
                if (ObservableSingleton<Tracker>.Instance.SaveProgress(path, (Action<JObject>)(root =>
                {
                    root["main_window_width"] = (JToken)Application.Current.MainWindow.Width;
                    root["main_window_height"] = (JToken)Application.Current.MainWindow.Height;
                    JObject jobject = new JObject();
                    bool flag = false;
                    foreach (Extension extension in ObservableSingleton<ExtensionManager>.Instance.Extensions)
                    {
                        JToken json = extension.SerializeToJson();
                        if (json != null)
                        {
                            jobject[extension.UID] = json;
                            flag = true;
                        }
                    }
                    if (!flag)
                        return;
                    root["extensions"] = (JToken)jobject;
                })))
                {
                    this.mCurrentSavePath = path;
                    this.PushMarkdownNotification(NotificationType.Message, string.Format("### Progress Saved\r\nSuccessfully saved progress to ```{0}```", (object)path), -1);
                    return true;
                }
            }
            catch
            {
            }
            this.PushMarkdownNotification(NotificationType.Error, string.Format("### Failed to Save Progress\r\nFailed to save progress to ```{0}```. Make sure you have available disk space and permissions for the selected location.", (object)path), -1);
            return false;
        }

        private bool LoadProgress(string path)
        {
            if (!ObservableSingleton<Tracker>.Instance.LoadProgress(path, (Action<JObject>)(root =>
            {
                Application.Current.MainWindow.Width = root.GetValue<double>("main_window_width", Application.Current.MainWindow.Width);
                Application.Current.MainWindow.Height = root.GetValue<double>("main_window_height", Application.Current.MainWindow.Height);
                JObject jobject = root.GetValue<JObject>("extensions");
                if (jobject == null)
                    return;
                foreach (JProperty property in jobject.Properties())
                    ObservableSingleton<ExtensionManager>.Instance.FindExtensionByUID(property.Name)?.DeserializeFromJson(property.Value);
            })))
                return false;
            this.AcquireLayouts();
            return true;
        }

        private bool CanOpenPackageDocumentation(object obj = null)
        {
            if (ObservableSingleton<Tracker>.Instance.ActiveGamePackage != null)
            {
                PackageRepositoryEntry repositoryEntry = ObservableSingleton<PackageManager>.Instance.FindRepositoryEntry(ObservableSingleton<Tracker>.Instance.ActiveGamePackage.UniqueID);
                if (repositoryEntry != null && !string.IsNullOrWhiteSpace(repositoryEntry.DocumentationURL))
                    return true;
            }
            return false;
        }

        private void OpenPackageDocumentation(object obj = null)
        {
            if (ObservableSingleton<Tracker>.Instance.ActiveGamePackage == null)
                return;
            PackageRepositoryEntry repositoryEntry = ObservableSingleton<PackageManager>.Instance.FindRepositoryEntry(ObservableSingleton<Tracker>.Instance.ActiveGamePackage.UniqueID);
            if (repositoryEntry == null || string.IsNullOrWhiteSpace(repositoryEntry.DocumentationURL))
                return;
            Process.Start(repositoryEntry.DocumentationURL);
        }

        private void GetFilteredCodeAndProvider(ref string code, out ICodeProvider provider)
        {
            provider = (ICodeProvider)Singleton<ItemDatabase>.Instance;
            if (code.StartsWith("@"))
            {
                code = code.Substring(1, code.Length - 1);
                provider = (ICodeProvider)ObservableSingleton<LocationDatabase>.Instance;
            }
            else
            {
                if (!code.StartsWith("$"))
                    return;
                code = code.Substring(1, code.Length - 1);
                provider = (ICodeProvider)ObservableSingleton<ScriptManager>.Instance;
            }
        }

        public object FindObjectForCode(string code)
        {
            ICodeProvider provider;
            this.GetFilteredCodeAndProvider(ref code, out provider);
            return provider.FindObjectForCode(code);
        }

        public uint ProviderCountForCode(string code, out AccessibilityLevel maxAccessibility)
        {
            ICodeProvider provider;
            this.GetFilteredCodeAndProvider(ref code, out provider);
            return provider.ProviderCountForCode(code, out maxAccessibility);
        }

        public void Reload()
        {
            this.ExpireAllNotifications();
            ObservableSingleton<Tracker>.Instance.Reload();
        }

        private void Tracker_OnPackageLoadStarting(object sender, EventArgs e)
        {
            SentrySdk.ConfigureScope((Action<Scope>)(scope =>
            {
                if (ObservableSingleton<Tracker>.Instance.ActiveGamePackage != null)
                {
                    scope.SetTag("active-package", ObservableSingleton<Tracker>.Instance.ActiveGamePackage.UniqueID);
                    if (ObservableSingleton<Tracker>.Instance.ActiveGamePackageVariant != null)
                        scope.SetTag("active-variant", ObservableSingleton<Tracker>.Instance.ActiveGamePackageVariant.UniqueID);
                    else
                        scope.SetTag("active-variant", (string)null);
                }
                else
                {
                    scope.SetTag("active-package", (string)null);
                    scope.SetTag("active-variant", (string)null);
                }
            }));
            if (TransactionProcessor.Current is IUndoableTransactionProcessor current)
                current.ClearUndoHistory();
            this.mCurrentSavePath = (string)null;
            ObservableSingleton<ExtensionManager>.Instance.OnPackageUnloaded();
            this.NotifyPropertyChanged("MainWindowTitle");
            ObservableSingleton<ImageReferenceService>.Instance.ClearImageCache();
            this.mPreviousNotifications.Clear();
            this.OpenPackageDocumentationCommand.RaiseCanExecuteChanged();
        }

        private void Tracker_OnPackageLoadComplete(object sender, EventArgs e)
        {
            ObservableSingleton<ExtensionManager>.Instance.OnPackageLoaded();
            this.AcquireLayouts();
            if (TransactionProcessor.Current is IUndoableTransactionProcessor current)
                current.ClearUndoHistory();
            this.OpenPackageDocumentationCommand.RaiseCanExecuteChanged();
            Keyboard.Focus((IInputElement)Application.Current.MainWindow);
        }

        public void AcquireLayouts()
        {
            try
            {
                this.BroadcastLayout = ObservableSingleton<LayoutManager>.Instance.FindLayout("tracker_broadcast");
                this.TrackerLayout = ObservableSingleton<LayoutManager>.Instance.FindLayout("tracker_default");
                this.TrackerHorizontalLayout = ObservableSingleton<LayoutManager>.Instance.FindLayout("tracker_horizontal");
                this.TrackerVerticalLayout = ObservableSingleton<LayoutManager>.Instance.FindLayout("tracker_vertical");
                this.TrackerCaptureItemLayout = ObservableSingleton<LayoutManager>.Instance.FindLayout("tracker_capture_item");
            }
            catch (Exception ex)
            {
                if (SentrySdk.IsEnabled)
                {
                    SentrySdk.AddBreadcrumb("Exception occurred while acquiring modern tracker layouts");
                    SentrySdk.CaptureException(ex);
                }
            }
            try
            {
                if (this.TrackerLayout == null)
                {
                    if (this.TrackerHorizontalLayout == null)
                    {
                        if (this.TrackerVerticalLayout == null)
                        {
                            ObservableSingleton<LayoutManager>.Instance.LegacyLoad(ObservableSingleton<Tracker>.Instance.ActiveGamePackage);
                            this.TrackerLayout = ObservableSingleton<LayoutManager>.Instance.FindLayout("tracker_default");
                            this.TrackerHorizontalLayout = ObservableSingleton<LayoutManager>.Instance.FindLayout("tracker_horizontal");
                            this.TrackerVerticalLayout = ObservableSingleton<LayoutManager>.Instance.FindLayout("tracker_vertical");
                            this.TrackerCaptureItemLayout = ObservableSingleton<LayoutManager>.Instance.FindLayout("tracker_capture_item");
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                if (SentrySdk.IsEnabled)
                {
                    SentrySdk.AddBreadcrumb("Exception occurred while acquiring legacy tracker layouts");
                    SentrySdk.CaptureException(ex);
                }
            }
            try
            {
                if (this.BroadcastLayout != null)
                    return;
                this.BroadcastLayout = new EmoTracker.Data.Layout.Layout();
                ObservableSingleton<ScriptManager>.Instance.OutputWarning("Loading legacy broadcast layout data");
                using (new LoggingBlock())
                {
                    if (ObservableSingleton<Tracker>.Instance.ActiveGamePackage == null)
                        return;
                    this.BroadcastLayout.Load(ObservableSingleton<Tracker>.Instance.ActiveGamePackage.Open("broadcast_layout.json"), ObservableSingleton<Tracker>.Instance.ActiveGamePackage);
                }
            }
            catch (Exception ex)
            {
                if (!SentrySdk.IsEnabled)
                    return;
                SentrySdk.AddBreadcrumb("Exception occurred while acquiring legacy broadcast layouts");
                SentrySdk.CaptureException(ex);
            }
        }

        public ApplicationModel.AvailablePackageViewFilterType AvailablePackageViewFilter
        {
            get => this.mAvailablePackagesViewFilter;
            set
            {
                if (!this.SetProperty<ApplicationModel.AvailablePackageViewFilterType>(ref this.mAvailablePackagesViewFilter, value, nameof(AvailablePackageViewFilter)))
                    return;
                this.AvailablePackagesView.Refresh();
            }
        }

        public DelegateCommand SetAvailablePackageViewFilterCommand => this.mSetAvailablePackageViewFilterCommand;

        public void SetAvailablePackageViewFilter(object param)
        {
            try
            {
                ApplicationModel.AvailablePackageViewFilterType result;
                if (!Enum.TryParse<ApplicationModel.AvailablePackageViewFilterType>(param.ToString(), out result))
                    return;
                this.AvailablePackageViewFilter = result;
            }
            catch
            {
            }
        }

        public CollectionView AvailablePackagesView => (CollectionView)this.mAvailablePackagesView;

        public CollectionView InstalledPackagesView => (CollectionView)this.mInstalledPackagesView;

        private void InitializePackageManagerViews()
        {
            this.mSetAvailablePackageViewFilterCommand = new DelegateCommand(new Action<object>(this.SetAvailablePackageViewFilter));
            ObservableSingleton<PackageManager>.Instance.OnGameListDownloaded += new EventHandler(this.PackageManager_OnGameListDownloaded);
            this.mAvailablePackagesView = new ListCollectionView(ObservableSingleton<PackageManager>.Instance.AvailablePackages as IList);
            this.mAvailablePackagesView.CustomSort = (IComparer)new ApplicationModel.RepoEntryGameNameSort();
            this.mAvailablePackagesView.Filter = new Predicate<object>(this.PackageFilter);
            this.mAvailablePackagesView.GroupDescriptions.Add((GroupDescription)new PropertyGroupDescription("Game", (IValueConverter)Singleton<GameNameToActualGameNameConverter>.Instance));
            this.mInstalledPackagesView = new ListCollectionView(ObservableSingleton<PackageManager>.Instance.InstalledPackages as IList);
            this.mInstalledPackagesView.CustomSort = (IComparer)new ApplicationModel.GameNameSort();
            this.mInstalledPackagesView.GroupDescriptions.Add((GroupDescription)new PropertyGroupDescription("Game", (IValueConverter)Singleton<GameNameToActualGameNameConverter>.Instance));
            this.AvailablePackagesView.Refresh();
            this.InstalledPackagesView.Refresh();
            DispatcherTimer dispatcherTimer = new DispatcherTimer();
            dispatcherTimer.Interval = TimeSpan.FromMinutes(30.0);
            dispatcherTimer.Tick += new EventHandler(this.OnRefreshPackageRepositoriesTimer);
            dispatcherTimer.Start();
            ObservableSingleton<PackageManager>.Instance.OnRepositoryUpdated += new EventHandler<PackageRepository>(this.PackageManager_OnRepositoryUpdated);
        }

        private void PackageManager_OnRepositoryUpdated(object sender, PackageRepository e) => this.OpenPackageDocumentationCommand.RaiseCanExecuteChanged();

        private void OnRefreshPackageRepositoriesTimer(object sender, EventArgs e) => ObservableSingleton<PackageManager>.Instance.RefreshRemoteRepositories();

        private void PackageManager_OnGameListDownloaded(object sender, EventArgs e)
        {
            this.AvailablePackagesView.Refresh();
            this.InstalledPackagesView.Refresh();
        }

        public string PackFilterText
        {
            get => this.mPackFilterText;
            set
            {
                if (!this.SetProperty<string>(ref this.mPackFilterText, value, nameof(PackFilterText)))
                    return;
                this.RefreshPackageCollectionView();
            }
        }

        private void RefreshPackageCollectionView() => this.mAvailablePackagesView.Refresh();

        private bool PackageFilter(object obj)
        {
            PackageRepositoryEntry packageRepositoryEntry = obj as PackageRepositoryEntry;
            bool flag = true;
            if (!string.IsNullOrWhiteSpace(this.PackFilterText))
            {
                string lower = this.PackFilterText.ToLower();
                flag = false;
                if (packageRepositoryEntry != null)
                {
                    PackageManager.Game game = ObservableSingleton<PackageManager>.Instance.FindGame(packageRepositoryEntry.Game);
                    flag = !string.IsNullOrWhiteSpace(packageRepositoryEntry.Name) && packageRepositoryEntry.Name.ToLower().Contains(lower) || !string.IsNullOrWhiteSpace(game.Name) && game.Name.ToLower().Contains(lower) || !string.IsNullOrWhiteSpace(game.Series) && game.Series.ToLower().Contains(lower) || !string.IsNullOrWhiteSpace(packageRepositoryEntry.Author) && packageRepositoryEntry.Author.ToLower().Contains(lower) || packageRepositoryEntry.Flags.ToString().ToLower().Contains(lower);
                }
            }
            if (flag)
            {
                switch (this.AvailablePackageViewFilter)
                {
                    case ApplicationModel.AvailablePackageViewFilterType.Installed:
                        switch (packageRepositoryEntry.Status)
                        {
                            case PackageRepositoryEntry.PackageStatus.Development:
                            case PackageRepositoryEntry.PackageStatus.Installed:
                            case PackageRepositoryEntry.PackageStatus.UpdateAvailable:
                                break;
                            default:
                                flag = false;
                                break;
                        }
                        break;
                    case ApplicationModel.AvailablePackageViewFilterType.InstalledAndHasUpdate:
                        if (packageRepositoryEntry.Status != PackageRepositoryEntry.PackageStatus.UpdateAvailable)
                        {
                            flag = false;
                            break;
                        }
                        break;
                }
            }
            return flag;
        }

        public IEnumerable<Notification> PreviousNotifications => (IEnumerable<Notification>)this.mPreviousNotifications;

        public IEnumerable<Notification> Notifications => (IEnumerable<Notification>)this.mNotifications;

        public bool HasPendingNotifications => this.mNotifications.Count > 0;

        private void InitializeNotifications()
        {
            this.mNotificationUpdateTimer = new DispatcherTimer(TimeSpan.FromMilliseconds(500.0), DispatcherPriority.Normal, new EventHandler(this.NotificationExpirationTimer_Tick), Application.Current.Dispatcher);
            this.mNotifications.CollectionChanged += new NotifyCollectionChangedEventHandler(this.Notifications_CollectionChanged);
            ObservableSingleton<ScriptManager>.Instance.SetNotificationService((INotificationService)this);
        }

        private void ExpireAllNotifications()
        {
            foreach (Notification mNotification in (Collection<Notification>)this.mNotifications)
                mNotification.Expired = true;
        }

        private void NotificationExpirationTimer_Tick(object sender, EventArgs e)
        {
            List<Notification> notificationList = new List<Notification>();
            DateTime now = DateTime.Now;
            foreach (Notification mNotification in (Collection<Notification>)this.mNotifications)
            {
                FrameworkElement frameworkElement = (FrameworkElement)null;
                if (Application.Current.MainWindow is MainWindow mainWindow)
                    frameworkElement = mainWindow.NotificationsHost.ItemContainerGenerator.ContainerFromItem((object)mNotification) as FrameworkElement;
                if (frameworkElement != null && frameworkElement.IsMouseOver)
                    mNotification.ExpirationTime = now;
                else if (mNotification.ExpirationTime <= now || mNotification.Expired)
                {
                    mNotification.Expired = true;
                    if (now - mNotification.ExpirationTime > TimeSpan.FromSeconds(2.0) && !notificationList.Contains(mNotification))
                        notificationList.Add(mNotification);
                    else if (frameworkElement != null && frameworkElement.RenderSize.Height == 0.0)
                        notificationList.Add(mNotification);
                }
            }
            foreach (Notification notification in notificationList)
                this.mNotifications.Remove(notification);
        }

        private void Notifications_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e) => this.NotifyPropertyChanged("HasPendingNotifications");

        public void PushMarkdownNotification(NotificationType type, string markdown, int timeout = -1)
        {
            if (!string.IsNullOrWhiteSpace(markdown))
            {
                Application.Current.Dispatcher.BeginInvoke(new Action(delegate ()
                {
                    MarkdownNotification notification = new MarkdownNotification(timeout)
                    {
                        Markdown = markdown,
                        Type = type
                    };
                    while (this.mPreviousNotifications.Count >= 10)
                    {
                        this.mPreviousNotifications.RemoveAt(9);
                    }
                    this.mPreviousNotifications.Insert(0, notification);
                    this.mNotifications.Insert(0, notification);
                }), Array.Empty<object>());
            }
        }

        public enum AvailablePackageViewFilterType
        {
            All,
            Installed,
            InstalledAndHasUpdate,
        }

        private class RepoEntryGameNameSort : IComparer
        {
            public int Compare(object _x, object _y)
            {
                PackageRepositoryEntry packageRepositoryEntry1 = (PackageRepositoryEntry)_x;
                PackageRepositoryEntry packageRepositoryEntry2 = (PackageRepositoryEntry)_y;
                PackageManager.Game game1 = ObservableSingleton<PackageManager>.Instance.FindGame(packageRepositoryEntry1.Game);
                PackageManager.Game game2 = ObservableSingleton<PackageManager>.Instance.FindGame(packageRepositoryEntry2.Game);
                if (game1.Key.Equals("Other", StringComparison.OrdinalIgnoreCase))
                    return 1;
                if (game2.Key.Equals("Other", StringComparison.OrdinalIgnoreCase))
                    return -1;
                int num1 = this.CompareInt(game1.SeriesPriority, game2.SeriesPriority);
                if (num1 != 0)
                    return num1;
                int num2 = this.CompareString(game1.Series, game2.Series);
                if (num2 != 0)
                    return num2;
                int num3 = this.CompareInt(game1.Priority, game2.Priority);
                if (num3 != 0)
                    return num3;
                int num4 = this.CompareString(game1.Name, game2.Name);
                if (num4 != 0)
                    return num4;
                int num5 = this.ComparePreferredBool(packageRepositoryEntry1.Flags.HasFlag((Enum)PackageFlags.Official), packageRepositoryEntry2.Flags.HasFlag((Enum)PackageFlags.Official));
                if (num5 != 0)
                    return num5;
                int num6 = this.ComparePreferredBool(packageRepositoryEntry1.Flags.HasFlag((Enum)PackageFlags.Featured), packageRepositoryEntry2.Flags.HasFlag((Enum)PackageFlags.Featured));
                if (num6 != 0)
                    return num6;
                int num7 = this.CompareString(packageRepositoryEntry1.Author, packageRepositoryEntry2.Author);
                return num7 != 0 ? num7 : 0;
            }

            private int ComparePreferredBool(bool x, bool y)
            {
                if (x && !y)
                    return -1;
                return !x & y ? 1 : 0;
            }

            private int CompareInt(int x, int y) => x.CompareTo(y);

            private int CompareString(string x, string y)
            {
                if (!string.IsNullOrWhiteSpace(x) && string.IsNullOrWhiteSpace(y))
                    return -1;
                return string.IsNullOrWhiteSpace(x) && !string.IsNullOrWhiteSpace(y) ? 1 : string.CompareOrdinal(x, y);
            }
        }

        private class GameNameSort : IComparer
        {
            public int Compare(object _x, object _y)
            {
                GamePackage gamePackage1 = (GamePackage)_x;
                GamePackage gamePackage2 = (GamePackage)_y;
                PackageManager.Game game1 = ObservableSingleton<PackageManager>.Instance.FindGame(gamePackage1.Game);
                PackageManager.Game game2 = ObservableSingleton<PackageManager>.Instance.FindGame(gamePackage2.Game);
                if (game1.Key.Equals("Other", StringComparison.OrdinalIgnoreCase))
                    return 1;
                if (game2.Key.Equals("Other", StringComparison.OrdinalIgnoreCase))
                    return -1;
                int num1 = this.CompareString(game1.Series, game2.Series);
                if (num1 != 0)
                    return num1;
                int num2 = this.CompareInt(game1.SeriesPriority, game2.SeriesPriority);
                if (num2 != 0)
                    return num2;
                int num3 = this.CompareInt(game1.Priority, game2.Priority);
                if (num3 != 0)
                    return num3;
                int num4 = this.CompareString(game1.Name, game2.Name);
                if (num4 != 0)
                    return num4;
                int num5 = this.CompareString(gamePackage1.Author, gamePackage2.Author);
                return num5 != 0 ? num5 : game1.Name.CompareTo(game2.Name);
            }

            private int CompareInt(int x, int y) => x.CompareTo(y);

            private int CompareString(string x, string y)
            {
                if (!string.IsNullOrWhiteSpace(x) && string.IsNullOrWhiteSpace(y))
                    return -1;
                return string.IsNullOrWhiteSpace(x) && !string.IsNullOrWhiteSpace(y) ? 1 : string.CompareOrdinal(x, y);
            }
        }
    }
}
