﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Extensions.AutoTracker.AutoTrackerExtension
// Assembly: EmoTracker, Version=2.3.8.17, Culture=neutral, PublicKeyToken=null
// MVID: 0D1A2596-8086-4DA5-90AE-09EB4913AFBC
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.exe

using ConnectorLib;
using EmoTracker.Core;
using EmoTracker.Data;
using EmoTracker.Data.Packages;
using EmoTracker.Data.Scripting;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Threading;

namespace EmoTracker.Extensions.AutoTracker
{
    public class AutoTrackerExtension : ObservableObject, Extension, IMemoryWatchService
    {
        private bool mbError;
        private bool mbConnected;
        private GamePlatform mActivePlatform;
        private ObservableCollection<AutoTrackerExtension.ConnectorTypeDesc> mApplicableConnectorTypes = new ObservableCollection<AutoTrackerExtension.ConnectorTypeDesc>();
        private AutoTrackerExtension.ConnectorTypeDesc mSelectedConnectorType;
        private IAddressableConnector mActiveConnector;
        private DelegateCommand mStartCommand;
        private DelegateCommand mStopCommand;
        private DelegateCommand mSetConnectorTypeCommand;
        private DispatcherTimer mUpdateTimer;
        private Task mActiveUpdateTask;
        private int mStartMemorySegmentIdx;
        private List<MemorySegment> mMemorySegments = new List<MemorySegment>();
        private List<MemoryTimer> mMemoryTimers = new List<MemoryTimer>();

        public string Name => "Auto Tracking";

        public string UID => "emotracker_auto_tracking";

        public int Priority => -100;

        public FrameworkElement StatusBarControl
        {
            get
            {
                AutoTrackerExtensionView statusBarControl = new AutoTrackerExtensionView();
                statusBarControl.DataContext = (object)this;
                return (FrameworkElement)statusBarControl;
            }
        }

        public void OnPackageUnloaded()
        {
            this.StopAutoTracking();
            this.Clear();
            this.Error = false;
        }

        public void OnPackageLoaded()
        {
            if (ObservableSingleton<Tracker>.Instance.ActiveGamePackage != null)
                this.ActivePlatform = ObservableSingleton<Tracker>.Instance.ActiveGamePackage.Platform;
            this.NotifyPropertyChanged("Active");
        }

        public bool Active => this.mApplicableConnectorTypes.Count > 0 && this.mMemorySegments.Count > 0;

        public bool Error
        {
            get => this.mbError;
            private set => this.SetProperty<bool>(ref this.mbError, value, nameof(Error));
        }

        public bool Connected
        {
            get => this.mbConnected;
            private set => this.SetProperty<bool>(ref this.mbConnected, value, nameof(Connected));
        }

        public GamePlatform ActivePlatform
        {
            get => this.mActivePlatform;
            private set
            {
                if (!this.SetProperty<GamePlatform>(ref this.mActivePlatform, value, nameof(ActivePlatform)))
                    return;
                this.mApplicableConnectorTypes.Clear();
                ConnectorType connectorType;
                if (!this.ConnectorTypeForGamePlatform(this.mActivePlatform, out connectorType))
                    return;
                ConnectorFactory.ConnectorProfile[] connectorProfileArray = ConnectorFactory.Available[(int)connectorType];
                if (connectorProfileArray == null || connectorProfileArray.Length == 0)
                    return;
                foreach (ConnectorFactory.ConnectorProfile connectorProfile in connectorProfileArray)
                {
                    if (connectorProfile.Visibility == ConnectorFactory.Visibility.Production)
                        this.mApplicableConnectorTypes.Add(new AutoTrackerExtension.ConnectorTypeDesc()
                        {
                            Name = connectorProfile.Name,
                            InstanceType = connectorProfile.Type
                        });
                }
            }
        }

        public IEnumerable<AutoTrackerExtension.ConnectorTypeDesc> ApplicableConnectorTypes => (IEnumerable<AutoTrackerExtension.ConnectorTypeDesc>)this.mApplicableConnectorTypes;

        private bool ConnectorTypeForGamePlatform(
          GamePlatform platform,
          out ConnectorType connectorType)
        {
            switch (platform)
            {
                case GamePlatform.NES:
                    connectorType = ConnectorType.NESConnector;
                    return true;
                case GamePlatform.SNES:
                    connectorType = ConnectorType.SNESConnector;
                    return true;
                case GamePlatform.N64:
                    connectorType = ConnectorType.N64Connector;
                    return true;
                case GamePlatform.Gameboy:
                    connectorType = ConnectorType.GBConnector;
                    return true;
                case GamePlatform.GBA:
                    connectorType = ConnectorType.GBAConnector;
                    return true;
                case GamePlatform.Genesis:
                    connectorType = ConnectorType.GenesisConnector;
                    return true;
                default:
                    connectorType = ConnectorType.ExternalConnector;
                    return false;
            }
        }

        public AutoTrackerExtension.ConnectorTypeDesc SelectedConnectorType
        {
            get => this.mSelectedConnectorType;
            private set
            {
                if (!this.SetProperty<AutoTrackerExtension.ConnectorTypeDesc>(ref this.mSelectedConnectorType, value, nameof(SelectedConnectorType)))
                    return;
                this.ActiveConnector = (IAddressableConnector)null;
                this.UpdateConnectorTypeDescActiveState(this.mSelectedConnectorType);
                this.InvalidateCommandAvailability();
                if (!this.CanStartAutoTracking())
                    return;
                this.StartAutoTracking();
            }
        }

        public IAddressableConnector ActiveConnector
        {
            get => this.mActiveConnector;
            private set
            {
                this.Connected = false;
                this.WaitForPendingMemoryUpdate();
                IAddressableConnector mActiveConnector1 = this.mActiveConnector;
                if (!this.SetProperty<IAddressableConnector>(ref this.mActiveConnector, value, nameof(ActiveConnector)))
                    return;
                mActiveConnector1?.Dispose();
                IGameConnector mActiveConnector2 = (IGameConnector)this.mActiveConnector;
                if (mActiveConnector2 != null)
                {
                    this.Connected = mActiveConnector2.Connected;
                    mActiveConnector2.ConnectionStatusChanged += new EventHandler<(ConnectionStatus, string)>(this.ActiveConnector_ConnectionStatusChanged);
                }
                this.InvalidateCommandAvailability();
            }
        }

        private void ActiveConnector_ConnectionStatusChanged(
          object sender,
          (ConnectionStatus status, string) e)
        {
            IGameConnector mActiveConnector = (IGameConnector)this.mActiveConnector;
            if (mActiveConnector == null)
                return;
            this.Connected = mActiveConnector.Connected || e.status == ConnectionStatus.Open;
        }

        private void UpdateConnectorTypeDescActiveState(AutoTrackerExtension.ConnectorTypeDesc desc)
        {
            foreach (AutoTrackerExtension.ConnectorTypeDesc applicableConnectorType in this.ApplicableConnectorTypes)
                applicableConnectorType.Active = applicableConnectorType == desc;
        }

        public byte ReadU8(ulong address, byte defaultVal = 0)
        {
            try
            {
                if (this.ActiveConnector != null)
                {
                    if (this.Connected)
                    {
                        if (this.ActiveConnector is I8BitConnector activeConnector)
                        {
                            byte num = defaultVal;
                            if (activeConnector.Read8(address, out num))
                                return num;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                ObservableSingleton<ScriptManager>.Instance.OutputError("Error occurred during raw byte read via AutoTracker");
                ObservableSingleton<ScriptManager>.Instance.OutputException(ex);
            }
            return defaultVal;
        }

        public sbyte Read8(ulong address, sbyte defaultVal = 0) => (sbyte)this.ReadU8(address, (byte)defaultVal);

        public ushort ReadU16(ulong address, ushort defaultVal = 0)
        {
            try
            {
                if (this.ActiveConnector != null)
                {
                    if (this.Connected)
                    {
                        if (this.ActiveConnector is I16BitConnector activeConnector)
                        {
                            ushort num = defaultVal;
                            if (activeConnector.Read16(address, out num))
                                return num;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                ObservableSingleton<ScriptManager>.Instance.OutputError("Error occurred during raw word read via AutoTracker");
                ObservableSingleton<ScriptManager>.Instance.OutputException(ex);
            }
            return defaultVal;
        }

        public short Read16(ulong address, short defaultVal = 0) => (short)this.ReadU16(address, (ushort)defaultVal);

        public DelegateCommand StartCommand
        {
            get => this.mStartCommand;
            set => this.SetProperty<DelegateCommand>(ref this.mStartCommand, value, nameof(StartCommand));
        }

        public DelegateCommand StopCommand
        {
            get => this.mStopCommand;
            set => this.SetProperty<DelegateCommand>(ref this.mStopCommand, value, nameof(StopCommand));
        }

        public DelegateCommand SetConnectorTypeCommand
        {
            get => this.mSetConnectorTypeCommand;
            set => this.SetProperty<DelegateCommand>(ref this.mSetConnectorTypeCommand, value, nameof(SetConnectorTypeCommand));
        }

        private void InvalidateCommandAvailability()
        {
            this.StartCommand.RaiseCanExecuteChanged();
            this.StopCommand.RaiseCanExecuteChanged();
        }

        private void SetConnectorType(object obj) => this.SelectedConnectorType = obj as AutoTrackerExtension.ConnectorTypeDesc;

        private bool CanStopAutoTracking(object obj = null) => this.ActiveConnector != null;

        private void StopAutoTracking(object obj = null)
        {
            this.WaitForPendingMemoryUpdate();
            int num = this.ActiveConnector != null ? 1 : 0;
            this.ActiveConnector = (IAddressableConnector)null;
            if (num == 0)
                return;
            ObservableSingleton<ScriptManager>.Instance.InvokeStandardCallback(ScriptManager.StandardCallback.AutoTrackerStopped);
        }

        private bool CanStartAutoTracking(object obj = null) => this.ActiveConnector == null && this.SelectedConnectorType != null;

        private void StartAutoTracking(object obj = null)
        {
            if (!this.CanStartAutoTracking(obj) || this.SelectedConnectorType == null)
                return;
            foreach (MemorySegment mMemorySegment in this.mMemorySegments)
                mMemorySegment.Dirty = true;
            try
            {
                this.ActiveConnector = Activator.CreateInstance(this.SelectedConnectorType.InstanceType) as IAddressableConnector;
                ObservableSingleton<ScriptManager>.Instance.InvokeStandardCallback(ScriptManager.StandardCallback.AutoTrackerStarted);
            }
            catch
            {
            }
        }

        public AutoTrackerExtension()
        {
            this.StopAutoTracking();
            sd2snesConnector.Usb2SnesApplicationName = string.Format("EmoTracker {0}", (object)ApplicationVersion.Current);
            this.StartCommand = new DelegateCommand(new Action<object>(this.StartAutoTracking), new Predicate<object>(this.CanStartAutoTracking));
            this.StopCommand = new DelegateCommand(new Action<object>(this.StopAutoTracking), new Predicate<object>(this.CanStopAutoTracking));
            this.SetConnectorTypeCommand = new DelegateCommand(new Action<object>(this.SetConnectorType));
        }

        public void Start()
        {
            ObservableSingleton<ScriptManager>.Instance.SetGlobalObject("AutoTracker", (object)this);
            ObservableSingleton<ScriptManager>.Instance.SetMemoryWatchService((IMemoryWatchService)this);
            this.mUpdateTimer = new DispatcherTimer();
            this.mUpdateTimer.Tick += new EventHandler(this.UpdateMemoryHooks);
            this.mUpdateTimer.Interval = new TimeSpan(0, 0, 0, 0, 30);
            this.mUpdateTimer.Start();
        }

        private bool HasPendingMemoryUpdate() => this.mActiveUpdateTask != null;

        public void WaitForPendingMemoryUpdate()
        {
            if (this.mActiveUpdateTask != null)
                this.mActiveUpdateTask.Wait(1000);
            this.mActiveUpdateTask = (Task)null;
        }

        private void UpdateMemoryHooks(object sender, EventArgs e)
        {
            if (this.HasPendingMemoryUpdate() || this.ActiveConnector == null || !this.Connected)
                return;
            PackageManager.Game game = (PackageManager.Game)null;
            if (ObservableSingleton<Tracker>.Instance.ActiveGamePackage != null)
            {
                PackageManager.Game game1 = ObservableSingleton<PackageManager>.Instance.FindGame(ObservableSingleton<Tracker>.Instance.ActiveGamePackage.Game);
                if (game1 != ObservableSingleton<PackageManager>.Instance.DefaultGame)
                    game = game1;
            }
            IAddressableConnector connectorInstance = this.ActiveConnector;
            IGameConnector gameConnector = (IGameConnector)connectorInstance;
            List<MemorySegment> activeSegments;
            List<MemoryTimer> activeTimers;
            lock (this)
            {
                activeSegments = new List<MemorySegment>((IEnumerable<MemorySegment>)this.mMemorySegments);
                activeTimers = new List<MemoryTimer>((IEnumerable<MemoryTimer>)this.mMemoryTimers);
            }
            if (gameConnector != null && !gameConnector.Connected)
            {
                foreach (MemorySegment memorySegment in activeSegments)
                    memorySegment.Dirty = true;
                this.Error = true;
            }
            else
                this.mActiveUpdateTask = Task.Run((Action)(() =>
                {
                    int num = this.mStartMemorySegmentIdx;
                    if (num >= activeSegments.Count)
                        num = 0;
                    int nextInitialSegmentIdx = num;
                    Stopwatch stopwatch = new Stopwatch();
                    stopwatch.Start();
                    bool bError = false;
                    for (int index = num; index < activeSegments.Count && stopwatch.ElapsedMilliseconds < 250L; ++index)
                    {
                        if (activeSegments[index].UpdateWithConnector(connectorInstance, game) != MemorySegment.UpdateResult.Success)
                            bError = true;
                        nextInitialSegmentIdx = index + 1;
                    }
                    for (int index = 0; index < num && stopwatch.ElapsedMilliseconds < 250L; ++index)
                    {
                        if (activeSegments[index].UpdateWithConnector(connectorInstance, game) != MemorySegment.UpdateResult.Success)
                            bError = true;
                        nextInitialSegmentIdx = index + 1;
                    }
                    foreach (MemoryTimer memoryTimer in activeTimers)
                    {
                        if (!memoryTimer.UpdateWithConnector(connectorInstance, game))
                            bError = true;
                    }
                    Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
                    {
                        this.Error = bError;
                        this.mActiveUpdateTask = null;
                        this.mStartMemorySegmentIdx = nextInitialSegmentIdx;
                    }));
                }));
        }

        public void Stop() => this.StopAutoTracking();

        public JToken SerializeToJson() => (JToken)null;

        public bool DeserializeFromJson(JToken token) => true;

        public IMemorySegment AddMemoryWatch(
          string name,
          ulong startAddress,
          ulong length,
          Func<IMemorySegment, bool> callback,
          Action<IMemorySegment> disposeCallback,
          int period)
        {
            lock (this)
            {
                MemorySegment memorySegment = new MemorySegment(name, startAddress, length, callback, disposeCallback, period);
                this.mMemorySegments.Add(memorySegment);
                this.NotifyPropertyChanged("Active");
                return (IMemorySegment)memorySegment;
            }
        }

        public void RemoveMemoryWatch(IMemorySegment segmentBase)
        {
            lock (this)
            {
                if (!(segmentBase is MemorySegment memorySegment) || !this.mMemorySegments.Contains(memorySegment))
                    return;
                this.mMemorySegments.Remove(memorySegment);
                memorySegment.Dispose();
            }
        }

        public MemoryTimer AddMemoryTimer(
          string name,
          Func<IAddressableConnector, PackageManager.Game, bool> callback,
          int period)
        {
            lock (this)
            {
                MemoryTimer memoryTimer = new MemoryTimer(name, callback, period);
                this.mMemoryTimers.Add(memoryTimer);
                this.NotifyPropertyChanged("Active");
                return memoryTimer;
            }
        }

        public void RemoveMemoryTimer(MemoryTimer timer)
        {
            lock (this)
            {
                if (timer == null || !this.mMemoryTimers.Contains(timer))
                    return;
                this.mMemoryTimers.Remove(timer);
                this.DisposeObject((object)timer);
            }
        }

        private void Clear()
        {
            lock (this)
            {
                this.DisposeCollection<MemorySegment>((IEnumerable<MemorySegment>)this.mMemorySegments);
                this.mMemorySegments.Clear();
                this.DisposeCollection<MemoryTimer>((IEnumerable<MemoryTimer>)this.mMemoryTimers);
                this.mMemoryTimers.Clear();
            }
        }

        public class ConnectorTypeDesc : ObservableObject
        {
            private bool mbActive;

            public string Name { get; set; }

            public Type InstanceType { get; set; }

            public bool Active
            {
                get => this.mbActive;
                set => this.SetProperty<bool>(ref this.mbActive, value, nameof(Active));
            }
        }

        private class ConnectorLibLogger
        {
            public void Debug(string msg)
            {
            }

            public void Error(string msg)
            {
            }

            public void Exception(Exception e, string msg)
            {
            }

            public void Info(string msg)
            {
            }

            public void Message(string msg)
            {
            }

            public void Warning(string msg)
            {
            }
        }
    }
}
