﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Extensions.AutoTracker.MemorySegment
// Assembly: EmoTracker, Version=2.3.8.17, Culture=neutral, PublicKeyToken=null
// MVID: 0D1A2596-8086-4DA5-90AE-09EB4913AFBC
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.exe

using ConnectorLib;
using EmoTracker.Core;
using EmoTracker.Data;
using EmoTracker.Data.Packages;
using EmoTracker.Data.Scripting;
using NLua;
using System;
using System.Windows;
using System.Windows.Threading;

namespace EmoTracker.Extensions.AutoTracker
{
    public class MemorySegment : IMemorySegment, IDisposable
    {
        private Func<IMemorySegment, bool> mCallback;
        private Action<IMemorySegment> mDisposeCallback;
        private string mName;
        private DateTime mLastUpdate;
        private int mPeriod = 500;
        private ulong mStartAddress;
        private ulong mLength;
        private bool mbDirty;
        private bool mbFrozen;
        private byte[][] mBuffers;

        public static event MemorySegment.MemorySegmentUpdatedHandler OnMemorySegmentModified;

        public static event MemorySegment.MemorySegmentUpdatedHandler OnMemorySegmentUpdated;

        public string Name => this.mName;

        public int Period => this.mPeriod;

        public ulong StartAddress => this.mStartAddress;

        public ulong EndAddress => this.mStartAddress + (this.Length - 1UL);

        public ulong Length => this.mLength;

        public bool Dirty
        {
            get => this.mbDirty;
            set => this.mbDirty = value;
        }

        public bool Frozen
        {
            get => this.mbFrozen;
            protected set => this.mbFrozen = value;
        }

        private byte[] ReadBuffer => this.mBuffers[0];

        private byte[] WriteBuffer => this.mBuffers[1];

        public bool ContainsAddress(ulong address) => address >= this.mStartAddress && address - this.mStartAddress < this.mLength;

        private ulong GetOffsetForAddress(ulong address)
        {
            if (address >= this.mStartAddress)
            {
                ulong offsetForAddress = address - this.mStartAddress;
                if (offsetForAddress < this.mLength)
                    return offsetForAddress;
            }
            throw new InvalidOperationException("Address is not contained within this segment");
        }

        public byte ReadUInt8(ulong address, bool bRawRead = false)
        {
            if (this.ReadBuffer != null)
            {
                try
                {
                    return this.ReadBuffer[this.GetOffsetForAddress(address)];
                }
                catch
                {
                    ObservableSingleton<ScriptManager>.Instance.OutputError("Address 0x{0:x} is out of range of segment '{3}' = [0x{1:x}:0x{2:x}]", (object)address, (object)this.StartAddress, (object)(ulong)((long)this.StartAddress + (long)this.Length), (object)this.Name);
                }
            }
            return 0;
        }

        public sbyte ReadInt8(ulong address, bool bRawRead = false) => (sbyte)this.ReadUInt8(address, false);

        public ushort ReadUInt16(ulong address, bool bRawRead = false)
        {
            if (this.ReadBuffer != null)
            {
                try
                {
                    long offsetForAddress = (long)this.GetOffsetForAddress(address);
                    byte num = this.ReadUInt8(address, false);
                    return (ushort)((uint)this.ReadUInt8(address + 1UL, false) << 8 | (uint)num);
                }
                catch
                {
                    ObservableSingleton<ScriptManager>.Instance.OutputError("Address 0x{0:x} is out of range of segment '{3}' = [0x{1:x}:0x{2:x}]", (object)address, (object)this.StartAddress, (object)(ulong)((long)this.StartAddress + (long)this.Length), (object)this.Name);
                }
            }
            return 0;
        }

        public short ReadInt16(ulong address, bool bRawRead = false) => 0;

        public MemorySegment(
          string name,
          ulong startAddress,
          ulong length,
          Func<IMemorySegment, bool> callback,
          Action<IMemorySegment> disposeCallback,
          int period = 500)
        {
            if (length == 0UL)
                throw new InvalidOperationException("Buffer must have non-zero size");
            this.mName = name;
            this.mbDirty = true;
            this.mCallback = callback;
            this.mDisposeCallback = disposeCallback;
            this.mStartAddress = startAddress;
            this.mLength = length;
            this.mPeriod = period;
            this.mBuffers = new byte[2][];
            this.mBuffers[0] = new byte[length];
            this.mBuffers[1] = new byte[length];
        }

        public void Freeze() => this.Frozen = true;

        public void Unfreeze() => this.Frozen = false;

        [LuaHide]
        public MemorySegment.UpdateResult UpdateWithConnector(
          IAddressableConnector connector,
          PackageManager.Game game)
        {
            if (this.Frozen)
                return MemorySegment.UpdateResult.Success;
            try
            {
                DateTime now = DateTime.Now;
                if (this.mLastUpdate.ToBinary() != 0L && (now - this.mLastUpdate).CompareTo(TimeSpan.FromMilliseconds((double)this.Period)) < 0)
                    return MemorySegment.UpdateResult.Success;
                this.mLastUpdate = now;
                bool flag1 = false;
                for (int index = 0; index < 5; ++index)
                {
                    if (!flag1)
                    {
                        try
                        {
                            flag1 = connector.Read(this.StartAddress, this.WriteBuffer);
                        }
                        catch
                        {
                        }
                    }
                    else
                        break;
                }
                if (flag1)
                {
                    bool flag2 = this.Dirty;
                    for (ulong index = 0; !flag2 && index < this.Length; ++index)
                    {
                        if ((int)this.ReadBuffer[index] != (int)this.WriteBuffer[index])
                        {
                            flag2 = true;
                            break;
                        }
                    }
                    if (flag2)
                    {
                        lock (this)
                        {
                            Buffer.BlockCopy((Array)this.WriteBuffer, 0, (Array)this.ReadBuffer, 0, (int)this.Length);
                            MemorySegment.MemorySegmentUpdatedHandler memorySegmentModified = MemorySegment.OnMemorySegmentModified;
                            if (memorySegmentModified != null)
                                memorySegmentModified(this, connector, game);
                            Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
                            {
                                bool bResult = true;
                                if (this.mCallback != null)
                                {
                                    bResult = this.mCallback(this);
                                }
                                if (bResult)
                                {
                                    this.Dirty = false;
                                }
                            }));
                        }
                    }
                    MemorySegment.MemorySegmentUpdatedHandler memorySegmentUpdated = MemorySegment.OnMemorySegmentUpdated;
                    if (memorySegmentUpdated != null)
                        memorySegmentUpdated(this, connector, game);
                    return MemorySegment.UpdateResult.Success;
                }
            }
            catch
            {
            }
            return MemorySegment.UpdateResult.Error;
        }

        public void Dispose()
        {
            lock (this)
            {
                if (this.mDisposeCallback == null)
                    return;
                this.mDisposeCallback((IMemorySegment)this);
            }
        }

        public delegate void MemorySegmentUpdatedHandler(
          MemorySegment segment,
          IAddressableConnector connector,
          PackageManager.Game game);

        public enum UpdateResult
        {
            Success,
            Error,
            MissingGameData,
            InvalidAccess,
        }
    }
}
