﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Extensions.Twitch.TwitchExtension
// Assembly: EmoTracker, Version=2.3.8.17, Culture=neutral, PublicKeyToken=null
// MVID: 0D1A2596-8086-4DA5-90AE-09EB4913AFBC
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.exe

using EmoTracker.Core;
using EmoTracker.Core.Services;
using EmoTracker.Data;
using EmoTracker.Data.Items;
using EmoTracker.Data.JSON;
using EmoTracker.UI.Media.Utility;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using System.Windows.Threading;
using TwitchLib.Client;
using TwitchLib.Client.Enums;
using TwitchLib.Client.Events;
using TwitchLib.Client.Models;
using TwitchLib.Communication.Events;

namespace EmoTracker.Extensions.Twitch
{
  public class TwitchExtension : ObservableObject, Extension
  {
    private DateTime mLastVFXCommandTime = DateTime.MinValue;
    private DefaultPermissions mDefaultPermissions = DefaultPermissions.Moderator;
    private ConnectionState mConnectionState;
    private TwitchExtension.DisconnectReason mDisconnectReason;
    private FrameworkElement mStatusControl;
    private TwitchClient mClient;
    private bool mbActive;
    private List<string> mUserBlacklist = new List<string>();
    private List<string> mUserWhitelist = new List<string>();
    private bool mbSuspendSave;

    public DelegateCommand ConnectCommand { get; private set; }

    public DelegateCommand DisconnectCommand { get; private set; }

    public ConnectionState ConnectionState
    {
      get => this.mConnectionState;
      set
      {
        if (!this.SetProperty<ConnectionState>(ref this.mConnectionState, value, nameof (ConnectionState)))
          return;
        Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate()
        {
          this.ConnectCommand.RaiseCanExecuteChanged();
          this.DisconnectCommand.RaiseCanExecuteChanged();
        }));
      }
    }

    public string Name => "Twitch Chat HUD";

    public string UID => "twitch_chat_hud";

    public int Priority => 0;

    public FrameworkElement StatusBarControl => !this.mbActive ? (FrameworkElement) null : this.mStatusControl;

    public void Start()
    {
      if (string.IsNullOrWhiteSpace(ObservableSingleton<ApplicationSettings>.Instance.TwitchChannelName))
        return;
      this.mbActive = true;
      this.LoadPermissions();
    }

    public void Stop() => this.Disconnect(TwitchExtension.DisconnectReason.Reset);

    public void OnPackageUnloaded()
    {
    }

    public void OnPackageLoaded()
    {
    }

    public JToken SerializeToJson() => (JToken) null;

    public bool DeserializeFromJson(JToken token) => true;

    public TwitchExtension()
    {
      this.ConnectCommand = new DelegateCommand(new Action<object>(this.ConnectExecute), new Predicate<object>(this.ConnectCanExecute));
      this.DisconnectCommand = new DelegateCommand(new Action<object>(this.DisconnectExecute), new Predicate<object>(this.DisconnectCanExecute));
      TwitchStatusIndicator twitchStatusIndicator = new TwitchStatusIndicator();
      twitchStatusIndicator.DataContext = (object) this;
      this.mStatusControl = (FrameworkElement) twitchStatusIndicator;
    }

    private bool DisconnectCanExecute(object obj) => this.ConnectionState == ConnectionState.Connected;

    private void DisconnectExecute(object obj) => this.Disconnect(TwitchExtension.DisconnectReason.User);

    private bool ConnectCanExecute(object obj) => this.ConnectionState == ConnectionState.Disconnected;

    private void ConnectExecute(object obj) => this.Connect();

    private void Connect()
    {
      this.CreatePermissionsFile();
      this.Disconnect(TwitchExtension.DisconnectReason.Reset);
      if (string.IsNullOrWhiteSpace(ObservableSingleton<ApplicationSettings>.Instance.TwitchChannelName))
        return;
      this.mbActive = true;
      this.mClient = new TwitchClient(protocol: ClientProtocol.TCP);
      this.mClient.OnConnected += new EventHandler<OnConnectedArgs>(this.OnConnected);
      this.mClient.OnConnectionError += new EventHandler<OnConnectionErrorArgs>(this.OnConnectionError);
      this.mClient.OnMessageReceived += new EventHandler<OnMessageReceivedArgs>(this.OnMessageReceived);
      this.mClient.OnChatCommandReceived += new EventHandler<OnChatCommandReceivedArgs>(this.OnChatCommandReceived);
      this.mClient.OnDisconnected += new EventHandler<OnDisconnectedEventArgs>(this.OnDisconnected);
      this.mClient.OnChannelStateChanged += new EventHandler<OnChannelStateChangedArgs>(this.OnChannelStateChanged);
      this.mClient.OnJoinedChannel += new EventHandler<OnJoinedChannelArgs>(this.OnJoinedChannel);
      this.mClient.OnLeftChannel += new EventHandler<OnLeftChannelArgs>(this.OnLeftChannel);
      this.mClient.OnReconnected += new EventHandler<OnReconnectedEventArgs>(this.OnReconnected);
      this.mClient.OnLog += new EventHandler<OnLogArgs>(this.OnClientLog);
      this.mClient.Initialize(new ConnectionCredentials("EmoTracker", "oauth:za7ac5ztaeecbvqzg5ssg7gm6acug8"), (string) null, '!', '!', true);
      this.mClient.AddChatCommandIdentifier('!');
      this.ConnectionState = ConnectionState.Connecting;
      this.mClient.Connect();
    }

    private void OnClientLog(object sender, OnLogArgs e) => Log.Debug("[TwitchLib] {0}", (object) e.Data);

    private void OnMessageReceived(object sender, OnMessageReceivedArgs e)
    {
    }

    private void OnChatCommandReceived(object sender, OnChatCommandReceivedArgs e) => this.HandleCommand(e.Command.ArgumentsAsList.ToArray(), e.Command.ChatMessage);

    private void OnReconnected(object sender, OnReconnectedEventArgs e)
    {
    }

    private void Disconnect(TwitchExtension.DisconnectReason reason)
    {
      this.ConnectionState = ConnectionState.Disconnected;
      this.mDisconnectReason = reason;
      TwitchClient mClient = this.mClient;
      this.mClient = (TwitchClient) null;
      try
      {
        mClient?.Disconnect();
      }
      catch
      {
      }
    }

    private void OnJoinedChannel(object sender, OnJoinedChannelArgs e)
    {
    }

    private void OnLeftChannel(object sender, OnLeftChannelArgs e) => Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, (Delegate)new Action(delegate () { this.Disconnect(TwitchExtension.DisconnectReason.Error); }));

    private void OnChannelStateChanged(object sender, OnChannelStateChangedArgs e)
    {
    }

    private void OnConnected(object sender, OnConnectedArgs e) => Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
    {
      this.ConnectionState = ConnectionState.Connected;
      this.mClient.JoinChannel(ObservableSingleton<ApplicationSettings>.Instance.TwitchChannelName, false);
    }));

    private void OnConnectionError(object sender, OnConnectionErrorArgs e) => Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
    {
      if (e.Error != null)
      {
        int num1 = (int) MessageBox.Show(e.Error.Message, "Twitch Connection Error");
      }
      int num2 = (int) MessageBox.Show(e.ToString(), "Twitch Connection Error");
      this.Disconnect(TwitchExtension.DisconnectReason.Error);
    }));

    private void OnDisconnected(object sender, OnDisconnectedEventArgs e) => Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
    {
      this.mClient = (TwitchClient) null;
      this.Disconnect(TwitchExtension.DisconnectReason.Unknown);
    }));

    private void HandleCommand(string[] args, ChatMessage src) => Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
    {
      try
      {
        if (this.mClient == null || !this.mClient.IsConnected || !this.mClient.JoinedChannels.Any<JoinedChannel>() || args == null || args.Length < 1)
          return;
        if (src.IsBroadcaster)
        {
          if (args[0].StartsWith("reset", StringComparison.OrdinalIgnoreCase))
          {
                Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate () { ObservableSingleton<ApplicationModel>.Instance.RefreshCommand.Execute((object)null); }));
            return;
          }
          if (args[0].StartsWith("add", StringComparison.OrdinalIgnoreCase) && args.Length >= 2)
          {
            this.WhitelistUser(args[1].Trim());
            return;
          }
          if (args[0].StartsWith("remove", StringComparison.OrdinalIgnoreCase) && args.Length >= 2)
          {
            this.RemoveWhitelistUser(args[1].Trim());
            return;
          }
          if (args[0].StartsWith("ban", StringComparison.OrdinalIgnoreCase) && args.Length >= 2)
          {
            this.BanUser(args[1].Trim());
            return;
          }
          if (args[0].StartsWith("unban", StringComparison.OrdinalIgnoreCase) && args.Length >= 2)
          {
            this.UnBanUser(args[1].Trim());
            return;
          }
        }
        if (!this.UserIsAllowed(src))
          return;
        Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
        {
          foreach (ITrackableItem trackableItem in Singleton<ItemDatabase>.Instance.FindProvidingItemsForCode(args[0]))
          {
            if (trackableItem is ToggleItem toggleItem2)
              toggleItem2.Active = args.Length < 2 || !args[1].StartsWith("off", StringComparison.OrdinalIgnoreCase);
            if (trackableItem is ProgressiveItem progressiveItem2)
            {
              if (args.Length >= 2)
              {
                if (args[1].StartsWith("down", StringComparison.OrdinalIgnoreCase))
                  progressiveItem2.Downgrade();
                else
                  progressiveItem2.AdvanceToCode(args[1]);
              }
              else
                progressiveItem2.AdvanceToCode(args[0]);
            }
            if (trackableItem is ToggleBadgedItem toggleBadgedItem2)
            {
              if (args.Length >= 2)
              {
                if (args[1].StartsWith("off", StringComparison.OrdinalIgnoreCase))
                  toggleBadgedItem2.Active = false;
                else
                  toggleBadgedItem2.AdvanceToCode(args[1]);
              }
              else
                toggleBadgedItem2.AdvanceToCode(args[0]);
            }
            if (trackableItem is ConsumableItem consumableItem2)
            {
              if (!consumableItem2.SwapActions)
              {
                if (args.Length >= 2 && args[1].StartsWith("down", StringComparison.OrdinalIgnoreCase))
                  consumableItem2.Decrement();
                else
                  consumableItem2.Increment();
              }
              else if (args.Length >= 2 && args[1].StartsWith("up", StringComparison.OrdinalIgnoreCase))
                consumableItem2.Increment();
              else
                consumableItem2.Decrement();
            }
            if (trackableItem is ProgressiveToggleItem progressiveToggleItem2)
            {
              if (args.Length == 1)
                progressiveToggleItem2.Active = true;
              else if (args.Length >= 2)
              {
                if (args[1].StartsWith("off", StringComparison.OrdinalIgnoreCase))
                  progressiveToggleItem2.Active = false;
                else
                  progressiveToggleItem2.AdvanceToPrivateCode(args[1].Trim());
              }
            }
          }
        }));
        if (args[0].StartsWith("flush", StringComparison.OrdinalIgnoreCase))
        {
          Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
          {
            if (!src.IsBroadcaster && DateTime.Now - this.mLastVFXCommandTime < new TimeSpan(0, 0, 0, 30))
            {
              this.mClient.SendMessage(ObservableSingleton<ApplicationSettings>.Instance.TwitchChannelName, "That command has a cooldown... Wait a few seconds. Kappa", false);
            }
            else
            {
              this.mLastVFXCommandTime = DateTime.Now;
              if (!(Application.Current.MainWindow is MainWindow mainWindow2) || mainWindow2.BroadcastView == null)
                return;
              mainWindow2.BroadcastView.Flush();
            }
          }));
        }
        else
        {
          if (!args[0].StartsWith("rain", StringComparison.OrdinalIgnoreCase))
            return;
          Application.Current.Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action(delegate ()
          {
            if (!src.IsBroadcaster && DateTime.Now - this.mLastVFXCommandTime < new TimeSpan(0, 0, 0, 30))
            {
              this.mClient.SendMessage(ObservableSingleton<ApplicationSettings>.Instance.TwitchChannelName, "That command has a cooldown... Wait a few seconds. Kappa", false);
            }
            else
            {
              this.mLastVFXCommandTime = DateTime.Now;
              if (!(Application.Current.MainWindow is MainWindow mainWindow4) || mainWindow4.BroadcastView == null || args.Length <= 1 || string.IsNullOrWhiteSpace(args[1]))
                return;
              ImageSource image = IconUtility.GetImage(new Uri(Path.Combine(ExtensionManager.GetExtensionPath((Extension) this), string.Format("images/{0}.png", (object) args[1])))) ?? IconUtility.GetImage(new Uri(string.Format("pack://application:,,,/EmoTracker;component/Resources/{0}.png", (object) args[1])));
              if (image == null)
                return;
              mainWindow4.BroadcastView.Rain(image);
            }
          }));
        }
      }
      catch
      {
      }
    }));

    private string FindUserInCollection(string user, List<string> collection) => collection.Find((Predicate<string>) (value => string.Equals(value, user, StringComparison.OrdinalIgnoreCase) || string.Equals(value, user, StringComparison.OrdinalIgnoreCase)));

    private bool IsUserWhitelisted(string user) => this.FindUserInCollection(user, this.mUserWhitelist) != null;

    private void WhitelistUser(string user)
    {
      this.UnBanUser(user);
      if (this.IsUserWhitelisted(user))
        return;
      this.mUserWhitelist.Add(user);
      this.SavePermissions();
    }

    private void RemoveWhitelistUser(string user)
    {
      string userInCollection = this.FindUserInCollection(user, this.mUserWhitelist);
      if (userInCollection == null)
        return;
      this.mUserWhitelist.Remove(userInCollection);
      this.SavePermissions();
    }

    private bool IsUserBanned(string user) => this.FindUserInCollection(user, this.mUserBlacklist) != null;

    private void BanUser(string user)
    {
      this.RemoveWhitelistUser(user);
      if (this.IsUserBanned(user))
        return;
      this.mUserBlacklist.Add(user);
      this.SavePermissions();
    }

    private void UnBanUser(string user)
    {
      string userInCollection = this.FindUserInCollection(user, this.mUserBlacklist);
      if (userInCollection == null)
        return;
      this.mUserBlacklist.Remove(userInCollection);
      this.SavePermissions();
    }

    private bool UserIsAllowed(ChatMessage chatMessage)
    {
      if (this.IsUserWhitelisted(chatMessage.Username) || this.IsUserWhitelisted(chatMessage.DisplayName))
        return true;
      if (this.IsUserBanned(chatMessage.Username) || this.IsUserBanned(chatMessage.DisplayName))
        return false;
      if (chatMessage.IsBroadcaster || this.mDefaultPermissions.HasFlag((Enum) DefaultPermissions.All) || chatMessage.IsModerator && this.mDefaultPermissions.HasFlag((Enum) DefaultPermissions.Moderator) || chatMessage.IsVip && this.mDefaultPermissions.HasFlag((Enum) DefaultPermissions.VIP))
        return true;
      return chatMessage.IsSubscriber && this.mDefaultPermissions.HasFlag((Enum) DefaultPermissions.Subscriber);
    }

    private void LoadPermissions()
    {
      lock (this)
      {
        try
        {
          this.mbSuspendSave = true;
          string path = Path.Combine(ExtensionManager.GetExtensionPath((Extension) this), "user_permissions.json");
          if (!File.Exists(path))
            return;
          using (StreamReader reader = new StreamReader((Stream) File.OpenRead(path)))
          {
            JObject token = (JObject) JToken.ReadFrom((JsonReader) new JsonTextReader((TextReader) reader));
            this.mDefaultPermissions = token.GetEnumValue<DefaultPermissions>("default_permissions", this.mDefaultPermissions);
            JArray jarray1 = token.GetValue<JArray>("whitelist");
            if (jarray1 != null)
            {
              foreach (JToken jtoken in jarray1)
              {
                string user = (string) jtoken;
                if (!string.IsNullOrWhiteSpace(user))
                  this.WhitelistUser(user);
              }
            }
            JArray jarray2 = token.GetValue<JArray>("blacklist");
            if (jarray2 == null)
              return;
            foreach (JToken jtoken in jarray2)
            {
              string user = (string) jtoken;
              if (!string.IsNullOrWhiteSpace(user))
                this.BanUser(user);
            }
          }
        }
        catch
        {
          int num = (int) MessageBox.Show("Your Documents\\EmoTracker\\extensions\\twitch_chat_hud\\user_permissions.json file has invalid JSON. Please correct it and restart the tracker.\n\n", "JSON Load Error", MessageBoxButton.OK);
        }
        finally
        {
          this.mbSuspendSave = false;
        }
      }
    }

    private void CreatePermissionsFile()
    {
      if (File.Exists(Path.Combine(ExtensionManager.GetExtensionPath((Extension) this), "user_permissions.json")))
        return;
      this.SavePermissions();
    }

    private void SavePermissions()
    {
      try
      {
        lock (this)
        {
          if (this.mbSuspendSave)
            return;
          Directory.CreateDirectory(ExtensionManager.GetExtensionPath((Extension) this));
          using (StreamWriter streamWriter = new StreamWriter((Stream) File.Open(Path.Combine(ExtensionManager.GetExtensionPath((Extension) this), "user_permissions.json"), FileMode.Create)))
          {
            using (JsonTextWriter jsonTextWriter = new JsonTextWriter((TextWriter) streamWriter))
            {
              jsonTextWriter.AutoCompleteOnClose = true;
              jsonTextWriter.Formatting = Formatting.Indented;
              JObject jobject = new JObject();
              jobject.Add("default_permissions", JToken.FromObject((object) this.mDefaultPermissions.ToString()));
              JArray jarray1 = JArray.FromObject((object) this.mUserWhitelist);
              if (jarray1 != null)
                jobject.Add("whitelist", (JToken) jarray1);
              JArray jarray2 = JArray.FromObject((object) this.mUserBlacklist);
              if (jarray2 != null)
                jobject.Add("blacklist", (JToken) jarray2);
              jsonTextWriter.WriteToken(jobject.CreateReader());
            }
          }
        }
      }
      catch
      {
      }
    }

    private enum DisconnectReason
    {
      Unknown,
      Reset,
      User,
      Error,
    }
  }
}
