﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Extensions.VoiceRecognition.VoiceRecognitionExtension
// Assembly: EmoTracker, Version=2.3.8.17, Culture=neutral, PublicKeyToken=null
// MVID: 0D1A2596-8086-4DA5-90AE-09EB4913AFBC
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.exe

using EmoTracker.Core;
using EmoTracker.Data;
using EmoTracker.Data.Core.Transactions;
using EmoTracker.Data.Core.Transactions.Processors;
using EmoTracker.Data.Items;
using EmoTracker.Data.Locations;
using EmoTracker.Extensions.Twitch;
using Newtonsoft.Json.Linq;
using System;
using System.Globalization;
using System.Linq;
using System.Speech.Recognition;
using System.Speech.Synthesis;
using System.Windows;

namespace EmoTracker.Extensions.VoiceRecognition
{
  public class VoiceRecognitionExtension : ObservableObject, Extension
  {
    private bool mbActive;
    private bool mbListening;
    private SpeechRecognitionEngine sre;
    private SpeechSynthesizer synth;
    private Random RNG = new Random();
    private int mLastBabeChoice = -1;

    public string Name => "Voice Recognition";

    public string UID => "emotracker_voice_recognition";

    public int Priority => -10000;

    public bool Active
    {
      get => this.mbActive;
      set
      {
        if (!this.SetProperty<bool>(ref this.mbActive, value, nameof (Active)))
          return;
        if (this.mbActive)
        {
          this.synth = new SpeechSynthesizer();
          this.synth.SetOutputToDefaultAudioDevice();
          this.synth.SelectVoiceByHints(VoiceGender.Female, VoiceAge.Adult, 0, new CultureInfo("en-us"));
          this.sre = new SpeechRecognitionEngine();
          this.sre.SetInputToDefaultAudioDevice();
          this.sre.SpeechDetected += new EventHandler<SpeechDetectedEventArgs>(this.Sre_SpeechDetected);
          this.sre.SpeechRecognitionRejected += new EventHandler<SpeechRecognitionRejectedEventArgs>(this.Sre_SpeechRecognitionRejected);
          this.sre.SpeechRecognized += new EventHandler<SpeechRecognizedEventArgs>(this.Sre_SpeechRecognized);
          this.ConfigureSpeechRecognition();
          this.sre.RecognizeAsync(RecognizeMode.Multiple);
        }
        else
        {
          if (this.sre != null)
          {
            this.sre.SpeechDetected -= new EventHandler<SpeechDetectedEventArgs>(this.Sre_SpeechDetected);
            this.sre.SpeechRecognitionRejected -= new EventHandler<SpeechRecognitionRejectedEventArgs>(this.Sre_SpeechRecognitionRejected);
            this.sre.SpeechRecognized -= new EventHandler<SpeechRecognizedEventArgs>(this.Sre_SpeechRecognized);
            this.sre.RecognizeAsyncCancel();
          }
          this.DisposeObjectAndDefault<SpeechRecognitionEngine>(ref this.sre);
          this.DisposeObjectAndDefault<SpeechSynthesizer>(ref this.synth);
          this.Listening = false;
        }
      }
    }

    public bool Listening
    {
      get => this.mbListening;
      set => this.SetProperty<bool>(ref this.mbListening, value, nameof (Listening));
    }

    public FrameworkElement StatusBarControl { get; set; }

    public JToken SerializeToJson() => (JToken) null;

    public bool DeserializeFromJson() => true;

    public VoiceRecognitionExtension()
    {
      VoiceRecognitionStatusIndicator recognitionStatusIndicator = new VoiceRecognitionStatusIndicator();
      recognitionStatusIndicator.DataContext = (object) this;
      this.StatusBarControl = (FrameworkElement) recognitionStatusIndicator;
    }

    public void Start()
    {
    }

    private void Sre_SpeechRecognitionRejected(object sender, SpeechRecognitionRejectedEventArgs e) => this.Listening = false;

    private void Sre_SpeechDetected(object sender, SpeechDetectedEventArgs e) => this.Listening = true;

    private void Sre_SpeechRecognized(object sender, SpeechRecognizedEventArgs e)
    {
      using (TransactionProcessor.Current.OpenTransaction())
      {
        this.Listening = false;
        if ((double) e.Result.Confidence < 0.899999976158142)
          return;
        if (e.Result.Grammar.Name == "Chatter")
        {
          Console.Out.WriteLine("Chatter: {0}", (object) e.Result.Text);
        }
        else
        {
          Console.Out.WriteLine(e.Result.Text);
          if (e.Result.Grammar.Name == "Toggle")
          {
            ITrackableItem trackableItem = Singleton<ItemDatabase>.Instance.ResolvePersistableItemReference(e.Result.Semantics["item"].Value.ToString());
            bool flag1 = false;
            bool flag2 = e.Result.Semantics["toggle_mode"].Value.ToString() != "off";
            if (trackableItem is ToggleItem toggleItem && toggleItem.Active != flag2)
            {
              toggleItem.Active = flag2;
              flag1 = true;
            }
            if (trackableItem is ProgressiveToggleItem progressiveToggleItem && progressiveToggleItem.Active != flag2)
            {
              progressiveToggleItem.Active = flag2;
              flag1 = true;
            }
            if (flag1)
              this.synth.SpeakAsync(string.Format("Toggled {0} {1}", (object) trackableItem.Name, flag2 ? (object) "on" : (object) "off"));
          }
          if (e.Result.Grammar.Name == "Advance Progressive")
          {
            ITrackableItem trackableItem = Singleton<ItemDatabase>.Instance.ResolvePersistableItemReference(e.Result.Semantics["item"].Value.ToString());
            if (trackableItem is ProgressiveItem progressiveItem)
            {
              string code = "";
              if (e.Result.Semantics.ContainsKey("advance_mode"))
                code = e.Result.Semantics["advance_mode"].Value.ToString();
              if (e.Result.Semantics.ContainsKey("code"))
                code = e.Result.Semantics["code"].Value.ToString();
              if (code == "down")
              {
                progressiveItem.Downgrade();
                this.synth.SpeakAsync(string.Format("Downgraded {0} by one step", (object) trackableItem.Name));
              }
              else if (!string.IsNullOrWhiteSpace(code))
              {
                progressiveItem.AdvanceToCode(code);
                this.synth.SpeakAsync(string.Format("Set {0} as {1}", (object) trackableItem.Name, (object) code));
              }
              else
              {
                progressiveItem.Advance();
                this.synth.SpeakAsync(string.Format("Upgraded {0} by one step", (object) trackableItem.Name));
              }
            }
          }
          if (e.Result.Grammar.Name == "Increment Consumable")
          {
            ITrackableItem trackableItem = Singleton<ItemDatabase>.Instance.ResolvePersistableItemReference(e.Result.Semantics["item"].Value.ToString());
            if (trackableItem is ConsumableItem consumableItem)
            {
              int acquiredCount = consumableItem.AcquiredCount;
              int num = consumableItem.Increment() - acquiredCount;
              if (num == 1)
                this.synth.SpeakAsync(string.Format("Added a {0}", (object) trackableItem.Name));
              else if (num > 1)
                this.synth.SpeakAsync(string.Format("Added {0} {1}s", (object) num, (object) trackableItem.Name));
              else
                this.synth.SpeakAsync(string.Format("Couldn't add any {0}", (object) trackableItem.Name));
            }
          }
          if (e.Result.Grammar.Name == "Decrement Consumable")
          {
            ITrackableItem trackableItem = Singleton<ItemDatabase>.Instance.ResolvePersistableItemReference(e.Result.Semantics["item"].Value.ToString());
            if (trackableItem is ConsumableItem consumableItem)
            {
              int num = consumableItem.AcquiredCount - consumableItem.Decrement();
              if (num == 1)
                this.synth.SpeakAsync(string.Format("Removed a {0}", (object) trackableItem.Name));
              else if (num > 1)
                this.synth.SpeakAsync(string.Format("Removed {0} {1}s", (object) num, (object) trackableItem.Name));
              else
                this.synth.SpeakAsync(string.Format("Couldn't remove any {0}", (object) trackableItem.Name));
            }
          }
          if (e.Result.Grammar.Name == "Clear")
          {
            Location location = ObservableSingleton<LocationDatabase>.Instance.ResolvePersistableLocationReference(e.Result.Semantics["location"].Value.ToString());
            if (location != null)
            {
              uint availableItemCount = location.AvailableItemCount;
              location.FullClearAllPossible();
              if ((int) location.AvailableItemCount != (int) availableItemCount)
              {
                if (location.AvailableItemCount == 0U)
                  this.synth.SpeakAsync(string.Format("Full-cleared {0}", (object) location.Name));
                else if (availableItemCount > location.AvailableItemCount)
                  this.synth.SpeakAsync(string.Format("Cleared {0} items in {1}", (object) (uint) ((int) availableItemCount - (int) location.AvailableItemCount), (object) location.Name));
              }
              else
                this.synth.SpeakAsync(string.Format("Couldn't clear anything in {0}", (object) location.Name));
            }
          }
          if (e.Result.Grammar.Name == "Reset Location")
          {
            Location location = ObservableSingleton<LocationDatabase>.Instance.ResolvePersistableLocationReference(e.Result.Semantics["location"].Value.ToString());
            if (location != null)
            {
              foreach (Section section in location.Sections)
                section.AvailableChestCount = section.ChestCount;
              this.synth.SpeakAsync(string.Format("Reset {0}", (object) location.Name));
            }
          }
          if (e.Result.Grammar.Name == "Pin")
          {
            Location location = ObservableSingleton<LocationDatabase>.Instance.ResolvePersistableLocationReference(e.Result.Semantics["location"].Value.ToString());
            if (location != null)
            {
              location.Pinned = true;
              this.synth.SpeakAsync(string.Format("Pinned {0}", (object) location.Name));
            }
          }
          if (e.Result.Grammar.Name == "Unpin")
          {
            Location location = ObservableSingleton<LocationDatabase>.Instance.ResolvePersistableLocationReference(e.Result.Semantics["location"].Value.ToString());
            if (location != null && location.Pinned)
            {
              location.Pinned = false;
              this.synth.SpeakAsync(string.Format("Un Pinned {0}", (object) location.Name));
            }
          }
          if (e.Result.Grammar.Name == "Capture")
          {
            ITrackableItem trackableItem = Singleton<ItemDatabase>.Instance.ResolvePersistableItemReference(e.Result.Semantics["item"].Value.ToString());
            Location location = ObservableSingleton<LocationDatabase>.Instance.ResolvePersistableLocationReference(e.Result.Semantics["location"].Value.ToString());
            if (trackableItem != null && location != null)
            {
              foreach (Section section in location.Sections)
              {
                if (section.CaptureItem)
                {
                  section.CapturedItem = trackableItem;
                  section.Owner.Pinned = true;
                  if (location.Sections.Count<Section>() > 1)
                  {
                    this.synth.SpeakAsync(string.Format("Marked {0} at {1} in the section called {2}", (object) trackableItem.Name, (object) location.Name, (object) section.Name));
                    break;
                  }
                  this.synth.SpeakAsync(string.Format("Marked {0} at {1}", (object) trackableItem.Name, (object) location.Name));
                  break;
                }
              }
            }
          }
          if (e.Result.Grammar.Name == "Set Secondary Code")
          {
            ITrackableItem trackableItem = Singleton<ItemDatabase>.Instance.ResolvePersistableItemReference(e.Result.Semantics["item"].Value.ToString());
            string code = e.Result.Semantics["secondary_code"].Value.ToString();
            if (trackableItem is ProgressiveToggleItem progressiveToggleItem)
            {
              int currentStage = (int) progressiveToggleItem.CurrentStage;
              progressiveToggleItem.AdvanceToPrivateCode(code);
              this.synth.SpeakAsync(string.Format("Marked {0} as {1}", (object) progressiveToggleItem.Name, !string.IsNullOrWhiteSpace(code) ? (object) code : (object) "the default"));
            }
          }
          if (e.Result.Grammar.Name == "Set Option")
          {
            string str1 = e.Result.Semantics["op"].Value.ToString();
            string str2 = e.Result.Semantics["feature"].Value.ToString();
            bool flag = str1 == "enable";
            if (!(str2 == "show all locations"))
            {
              if (str2 == "chat hud")
              {
                TwitchExtension extension = ObservableSingleton<ExtensionManager>.Instance.FindExtension<TwitchExtension>();
                if (extension != null)
                {
                  if (flag && extension.ConnectCommand.CanExecute((object) null))
                    extension.ConnectCommand.Execute((object) null);
                  else if (extension.DisconnectCommand.CanExecute((object) null))
                    extension.DisconnectCommand.Execute((object) null);
                }
              }
            }
            else
              ObservableSingleton<ApplicationSettings>.Instance.DisplayAllLocations = flag;
            this.synth.SpeakAsync(string.Format("{0}d {1}", (object) str1, (object) str2));
          }
          if (e.Result.Grammar.Name == "Stop Listening")
          {
            this.synth.Speak("Okay, I'm no longer listening.");
            this.Active = false;
          }
          else if (e.Result.Grammar.Name == "Undo")
          {
            this.synth.SpeakAsync("Okay, I'll undo the last operation");
            if (!(TransactionProcessor.Current is IUndoableTransactionProcessor current))
              return;
            current.Undo();
          }
          else
          {
            if (!e.Result.Text.StartsWith("Hey Babe"))
              return;
            string[] strArray = new string[9]
            {
              "Also... Just so we're clear, I'm not your babe.",
              "Also... No problem kitty boo-boo bunchers!",
              "Also... I'm not a babe - I'm a computer",
              "Also... That's gonna be a yikes from me, dog.",
              "Also... I have never swiped left so hard in my life.",
              "Also... Is that you Aivan?",
              "Also... I've seen that movie, I see where this is going, and frankly, you're no Joaquin Phoenix.",
              "Also... Why is it always about you, and what you want? Did you ever stop to think that maybe there are things I'd like YOU to do?",
              "Also... After you're done playing your game, we need to have a serious talk, about your little problem."
            };
            int index = this.RNG.Next(strArray.Length);
            while (index == this.mLastBabeChoice)
              index = this.RNG.Next(strArray.Length);
            this.mLastBabeChoice = index;
            this.synth.SpeakAsync(strArray[index]);
          }
        }
      }
    }

    public void Stop()
    {
    }

    public void OnPackageLoaded() => this.ConfigureSpeechRecognition();

    private void AddLocationToChoices(Location location, Choices choices)
    {
      if (location == null)
        return;
      choices.Add((GrammarBuilder) new SemanticResultValue(location.Name, (object) ObservableSingleton<LocationDatabase>.Instance.GetPersistableLocationReference(location)));
      if (string.IsNullOrEmpty(location.ShortName))
        return;
      choices.Add((GrammarBuilder) new SemanticResultValue(location.ShortName, (object) ObservableSingleton<LocationDatabase>.Instance.GetPersistableLocationReference(location)));
    }

    private void ConfigureSpeechRecognition()
    {
      if (!this.Active || this.sre == null)
        return;
      if (this.synth == null)
        return;
      try
      {
        this.sre.UnloadAllGrammars();
      }
      catch
      {
      }
      CultureInfo cultureInfo = new CultureInfo("en-us");
      GrammarBuilder builder1 = new GrammarBuilder();
      builder1.Culture = cultureInfo;
      builder1.Append(new Choices(new string[2]
      {
        "Hey Tracker",
        "Hey Babe"
      }));
      try
      {
        GrammarBuilder builder2 = new GrammarBuilder();
        builder2.Culture = cultureInfo;
        builder2.Append(builder1);
        builder2.Append(new Choices(new string[5]
        {
          "track",
          "track a",
          "track the",
          "toggle",
          "toggle the"
        }));
        Choices choices1 = new Choices();
        foreach (ITrackableItem trackableItem in Singleton<ItemDatabase>.Instance.Items)
        {
          if (!string.IsNullOrWhiteSpace(trackableItem.Name))
          {
            if (trackableItem is ToggleItem toggleItem)
              choices1.Add((GrammarBuilder) new SemanticResultValue(toggleItem.Name, (object) Singleton<ItemDatabase>.Instance.GetPersistableItemReference((ITrackableItem) toggleItem)));
            if (trackableItem is ProgressiveToggleItem progressiveToggleItem)
              choices1.Add((GrammarBuilder) new SemanticResultValue(progressiveToggleItem.Name, (object) Singleton<ItemDatabase>.Instance.GetPersistableItemReference((ITrackableItem) progressiveToggleItem)));
          }
        }
        builder2.Append(new SemanticResultKey("item", new GrammarBuilder[1]
        {
          (GrammarBuilder) choices1
        }));
        Choices choices2 = new Choices();
        choices2.Add(new GrammarBuilder(), new GrammarBuilder("off"));
        builder2.Append(new SemanticResultKey("toggle_mode", new GrammarBuilder[1]
        {
          (GrammarBuilder) choices2
        }));
        this.sre.LoadGrammarAsync(new Grammar(builder2)
        {
          Name = "Toggle"
        });
      }
      catch
      {
      }
      try
      {
        GrammarBuilder builder3 = new GrammarBuilder();
        builder3.Culture = cultureInfo;
        builder3.Append(builder1);
        builder3.Append(new Choices(new string[3]
        {
          "track",
          "track a",
          "track the"
        }));
        Choices choices3 = new Choices();
        foreach (ITrackableItem trackableItem in Singleton<ItemDatabase>.Instance.Items)
        {
          if (!string.IsNullOrWhiteSpace(trackableItem.Name) && trackableItem is ProgressiveItem progressiveItem)
            choices3.Add((GrammarBuilder) new SemanticResultValue(progressiveItem.Name, (object) Singleton<ItemDatabase>.Instance.GetPersistableItemReference((ITrackableItem) progressiveItem)));
        }
        builder3.Append(new SemanticResultKey("item", new GrammarBuilder[1]
        {
          (GrammarBuilder) choices3
        }));
        Choices choices4 = new Choices();
        choices4.Add(new GrammarBuilder(), new GrammarBuilder("down"));
        builder3.Append(new SemanticResultKey("advance_mode", new GrammarBuilder[1]
        {
          (GrammarBuilder) choices4
        }));
        this.sre.LoadGrammarAsync(new Grammar(builder3)
        {
          Name = "Advance Progressive"
        });
        foreach (ITrackableItem trackableItem in Singleton<ItemDatabase>.Instance.Items)
        {
          if (!string.IsNullOrWhiteSpace(trackableItem.Name) && trackableItem is ProgressiveItem progressiveItem)
          {
            GrammarBuilder builder4 = new GrammarBuilder();
            builder4.Culture = cultureInfo;
            builder4.Append(builder1);
            builder4.Append(new Choices(new string[5]
            {
              "track",
              "track a",
              "track the",
              "set",
              "set the"
            }));
            Choices choices5 = new Choices(new GrammarBuilder[1]
            {
              (GrammarBuilder) new SemanticResultValue(progressiveItem.Name, (object) Singleton<ItemDatabase>.Instance.GetPersistableItemReference((ITrackableItem) progressiveItem))
            });
            builder4.Append(new SemanticResultKey("item", new GrammarBuilder[1]
            {
              (GrammarBuilder) choices5
            }));
            builder4.Append("as");
            Choices choices6 = new Choices();
            int num = 0;
            foreach (CodeProvider stage in progressiveItem.Stages)
            {
              foreach (string providedCode in stage.ProvidedCodes)
              {
                if (!string.IsNullOrWhiteSpace(providedCode))
                {
                  choices6.Add(providedCode);
                  ++num;
                }
              }
            }
            if (num > 0)
            {
              builder4.Append(new SemanticResultKey("code", new GrammarBuilder[1]
              {
                (GrammarBuilder) choices6
              }));
              this.sre.LoadGrammarAsync(new Grammar(builder4)
              {
                Name = "Advance Progressive"
              });
            }
          }
        }
      }
      catch
      {
      }
      try
      {
        foreach (ITrackableItem trackableItem in Singleton<ItemDatabase>.Instance.Items)
        {
          if (!string.IsNullOrWhiteSpace(trackableItem.Name) && trackableItem is ProgressiveToggleItem progressiveToggleItem)
          {
            GrammarBuilder builder5 = new GrammarBuilder();
            builder5.Culture = cultureInfo;
            builder5.Append(builder1);
            builder5.Append(new Choices(new string[3]
            {
              "track",
              "mark",
              "set"
            }));
            Choices choices7 = new Choices(new GrammarBuilder[1]
            {
              (GrammarBuilder) new SemanticResultValue(progressiveToggleItem.Name, (object) Singleton<ItemDatabase>.Instance.GetPersistableItemReference((ITrackableItem) progressiveToggleItem))
            });
            builder5.Append(new SemanticResultKey("item", new GrammarBuilder[1]
            {
              (GrammarBuilder) choices7
            }));
            builder5.Append("as");
            Choices choices8 = new Choices();
            int num = 0;
            for (uint index = 0; index < progressiveToggleItem.StageCount; ++index)
            {
              ProgressiveToggleItem.Stage activeStageForIndex = progressiveToggleItem.GetActiveStageForIndex(index);
              if (activeStageForIndex != null)
              {
                foreach (string providedCode in activeStageForIndex.PrivateCodes.ProvidedCodes)
                {
                  choices8.Add(providedCode);
                  ++num;
                }
              }
            }
            if (num > 0)
            {
              builder5.Append(new SemanticResultKey("secondary_code", new GrammarBuilder[1]
              {
                (GrammarBuilder) choices8
              }));
              this.sre.LoadGrammarAsync(new Grammar(builder5)
              {
                Name = "Set Secondary Code"
              });
            }
          }
        }
      }
      catch
      {
      }
      try
      {
        GrammarBuilder builder6 = new GrammarBuilder();
        builder6.Culture = cultureInfo;
        builder6.Append(builder1);
        builder6.Append(new Choices(new string[5]
        {
          "track",
          "track a",
          "track an",
          "add a",
          "add an"
        }));
        Choices choices = new Choices();
        foreach (ITrackableItem trackableItem in Singleton<ItemDatabase>.Instance.Items)
        {
          if (!string.IsNullOrWhiteSpace(trackableItem.Name) && trackableItem is ConsumableItem consumableItem)
            choices.Add((GrammarBuilder) new SemanticResultValue(consumableItem.Name, (object) Singleton<ItemDatabase>.Instance.GetPersistableItemReference((ITrackableItem) consumableItem)));
        }
        builder6.Append(new SemanticResultKey("item", new GrammarBuilder[1]
        {
          (GrammarBuilder) choices
        }));
        this.sre.LoadGrammarAsync(new Grammar(builder6)
        {
          Name = "Increment Consumable"
        });
      }
      catch
      {
      }
      try
      {
        GrammarBuilder builder7 = new GrammarBuilder();
        builder7.Culture = cultureInfo;
        builder7.Append(builder1);
        builder7.Append(new Choices(new string[3]
        {
          "remove",
          "remove a",
          "remove an"
        }));
        Choices choices = new Choices();
        foreach (ITrackableItem trackableItem in Singleton<ItemDatabase>.Instance.Items)
        {
          if (!string.IsNullOrWhiteSpace(trackableItem.Name) && trackableItem is ConsumableItem consumableItem && !string.IsNullOrWhiteSpace(consumableItem.Name))
            choices.Add((GrammarBuilder) new SemanticResultValue(consumableItem.Name, (object) Singleton<ItemDatabase>.Instance.GetPersistableItemReference((ITrackableItem) consumableItem)));
        }
        builder7.Append(new SemanticResultKey("item", new GrammarBuilder[1]
        {
          (GrammarBuilder) choices
        }));
        this.sre.LoadGrammarAsync(new Grammar(builder7)
        {
          Name = "Decrement Consumable"
        });
      }
      catch
      {
      }
      try
      {
        GrammarBuilder builder8 = new GrammarBuilder();
        builder8.Culture = cultureInfo;
        builder8.Append(builder1);
        builder8.Append(new Choices(new string[1]
        {
          "clear"
        }));
        Choices choices = new Choices();
        foreach (Location visibleLocation in ObservableSingleton<LocationDatabase>.Instance.VisibleLocations)
          this.AddLocationToChoices(visibleLocation, choices);
        builder8.Append(new SemanticResultKey("location", new GrammarBuilder[1]
        {
          (GrammarBuilder) choices
        }));
        this.sre.LoadGrammarAsync(new Grammar(builder8)
        {
          Name = "Clear"
        });
      }
      catch
      {
      }
      try
      {
        GrammarBuilder builder9 = new GrammarBuilder();
        builder9.Culture = cultureInfo;
        builder9.Append(builder1);
        builder9.Append(new Choices(new string[1]
        {
          "reset"
        }));
        Choices choices = new Choices();
        foreach (Location visibleLocation in ObservableSingleton<LocationDatabase>.Instance.VisibleLocations)
          this.AddLocationToChoices(visibleLocation, choices);
        builder9.Append(new SemanticResultKey("location", new GrammarBuilder[1]
        {
          (GrammarBuilder) choices
        }));
        this.sre.LoadGrammarAsync(new Grammar(builder9)
        {
          Name = "Reset Location"
        });
      }
      catch
      {
      }
      try
      {
        GrammarBuilder builder10 = new GrammarBuilder();
        builder10.Culture = cultureInfo;
        builder10.Append(builder1);
        builder10.Append(new Choices(new string[1]{ "pin" }));
        Choices choices = new Choices();
        foreach (Location visibleLocation in ObservableSingleton<LocationDatabase>.Instance.VisibleLocations)
          this.AddLocationToChoices(visibleLocation, choices);
        builder10.Append(new SemanticResultKey("location", new GrammarBuilder[1]
        {
          (GrammarBuilder) choices
        }));
        this.sre.LoadGrammarAsync(new Grammar(builder10)
        {
          Name = "Pin"
        });
      }
      catch
      {
      }
      try
      {
        GrammarBuilder builder11 = new GrammarBuilder();
        builder11.Culture = cultureInfo;
        builder11.Append(builder1);
        builder11.Append(new Choices(new string[1]
        {
          "remove pin for"
        }));
        Choices choices = new Choices();
        foreach (Location visibleLocation in ObservableSingleton<LocationDatabase>.Instance.VisibleLocations)
          this.AddLocationToChoices(visibleLocation, choices);
        builder11.Append(new SemanticResultKey("location", new GrammarBuilder[1]
        {
          (GrammarBuilder) choices
        }));
        this.sre.LoadGrammarAsync(new Grammar(builder11)
        {
          Name = "Unpin"
        });
      }
      catch
      {
      }
      try
      {
        GrammarBuilder builder12 = new GrammarBuilder();
        builder12.Culture = cultureInfo;
        builder12.Append(builder1);
        builder12.Append("make a note on");
        Choices choices = new Choices();
        foreach (Location visibleLocation in ObservableSingleton<LocationDatabase>.Instance.VisibleLocations)
          this.AddLocationToChoices(visibleLocation, choices);
        builder12.Append(new SemanticResultKey("location", new GrammarBuilder[1]
        {
          (GrammarBuilder) choices
        }));
        this.sre.LoadGrammarAsync(new Grammar(builder12)
        {
          Name = "Note"
        });
      }
      catch
      {
      }
      try
      {
        GrammarBuilder builder13 = new GrammarBuilder();
        builder13.Culture = cultureInfo;
        builder13.Append(builder1);
        builder13.Append(new Choices(new string[1]
        {
          "mark"
        }));
        Choices choices9 = new Choices();
        foreach (ITrackableItem trackableItem in Singleton<ItemDatabase>.Instance.Items)
        {
          if (trackableItem.Capturable && !string.IsNullOrWhiteSpace(trackableItem.Name))
            choices9.Add((GrammarBuilder) new SemanticResultValue(trackableItem.Name, (object) Singleton<ItemDatabase>.Instance.GetPersistableItemReference(trackableItem)));
        }
        Choices choices10 = new Choices();
        foreach (Location visibleLocation in ObservableSingleton<LocationDatabase>.Instance.VisibleLocations)
          this.AddLocationToChoices(visibleLocation, choices10);
        builder13.Append(new SemanticResultKey("item", new GrammarBuilder[1]
        {
          (GrammarBuilder) choices9
        }));
        builder13.Append(new Choices(new string[2]
        {
          "on",
          "at"
        }));
        builder13.Append(new SemanticResultKey("location", new GrammarBuilder[1]
        {
          (GrammarBuilder) choices10
        }));
        this.sre.LoadGrammarAsync(new Grammar(builder13)
        {
          Name = "Capture"
        });
      }
      catch
      {
      }
      try
      {
        GrammarBuilder builder14 = new GrammarBuilder();
        builder14.Culture = cultureInfo;
        builder14.Append(builder1);
        builder14.Append(new SemanticResultKey("op", new GrammarBuilder[1]
        {
          (GrammarBuilder) new Choices(new string[2]
          {
            "enable",
            "disable"
          })
        }));
        builder14.Append(new SemanticResultKey("feature", new GrammarBuilder[1]
        {
          (GrammarBuilder) new Choices(new string[2]
          {
            "show all locations",
            "chat hud"
          })
        }));
        this.sre.LoadGrammarAsync(new Grammar(builder14)
        {
          Name = "Set Option"
        });
        GrammarBuilder builder15 = new GrammarBuilder();
        builder15.Culture = cultureInfo;
        builder15.Append(builder1);
        builder15.Append("stop listening");
        this.sre.LoadGrammarAsync(new Grammar(builder15)
        {
          Name = "Stop Listening"
        });
        GrammarBuilder builder16 = new GrammarBuilder();
        builder16.Culture = cultureInfo;
        builder16.Append(builder1);
        builder16.Append("undo that");
        this.sre.LoadGrammarAsync(new Grammar(builder16)
        {
          Name = "Undo"
        });
        SpeechRecognitionEngine sre = this.sre;
        DictationGrammar dictationGrammar = new DictationGrammar();
        dictationGrammar.Name = "Chatter";
        sre.LoadGrammarAsync((Grammar) dictationGrammar);
      }
      catch
      {
      }
    }

    public void OnPackageUnloaded()
    {
    }

    public bool DeserializeFromJson(JToken token) => true;
  }
}
