﻿// Decompiled with JetBrains decompiler
// Type: EmoTracker.Update.AppUpdate
// Assembly: EmoTracker, Version=2.3.8.17, Culture=neutral, PublicKeyToken=null
// MVID: 0D1A2596-8086-4DA5-90AE-09EB4913AFBC
// Assembly location: C:\Program Files (x86)\EmoTracker\EmoTracker.exe

using EmoTracker.Core;
using EmoTracker.Data.JSON;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Net.Cache;
using System.Reflection;
using System.Windows;

namespace EmoTracker.Update
{
  public class AppUpdate : ObservableSingleton<AppUpdate>
  {
    private static string AppUpdateURL = "https://emotracker.net/service/application_version.json";
    private AppUpdate.UpdateStatus mStatus;
    private int mDownloadPercentage;
    private WebClient mWebClient;
    private Version mCurrentAppVersion = Assembly.GetExecutingAssembly().GetName().Version;
    private Version mAvailableAppVersion;
    private string mAvailableVersionURL;
    private DelegateCommand mDownloadAndInstallUpdateCommand;

    public AppUpdate.UpdateStatus Status
    {
      get => this.mStatus;
      set => this.SetProperty<AppUpdate.UpdateStatus>(ref this.mStatus, value, nameof (Status));
    }

    public int DownloadPercentage
    {
      get => this.mDownloadPercentage;
      set => this.SetProperty<int>(ref this.mDownloadPercentage, value, nameof (DownloadPercentage));
    }

    public Version CurrentVersion => this.mCurrentAppVersion;

    public Version AvailableVersion
    {
      get => this.mAvailableAppVersion;
      protected set => this.SetProperty<Version>(ref this.mAvailableAppVersion, value, nameof (AvailableVersion));
    }

    public bool UpdateAvailable => this.AvailableVersion != (Version) null && this.AvailableVersion > this.CurrentVersion;

    public DelegateCommand DownloadAndInstallUpdateCommand
    {
      get => this.mDownloadAndInstallUpdateCommand;
      private set => this.SetProperty<DelegateCommand>(ref this.mDownloadAndInstallUpdateCommand, value, nameof (DownloadAndInstallUpdateCommand));
    }

    public AppUpdate()
    {
      this.mWebClient = new WebClient();
      this.mWebClient.Headers.Add("User-Agent", string.Format("EmoTracker/{0} (Windows)", (object) this.CurrentVersion));
      this.mWebClient.CachePolicy = new RequestCachePolicy(RequestCacheLevel.NoCacheNoStore);
      this.mWebClient.DownloadDataCompleted += new DownloadDataCompletedEventHandler(this.MWebClient_DownloadDataCompleted);
      this.mWebClient.DownloadFileCompleted += new AsyncCompletedEventHandler(this.MWebClient_DownloadFileCompleted);
      this.mWebClient.DownloadProgressChanged += new DownloadProgressChangedEventHandler(this.MWebClient_DownloadProgressChanged);
      this.DownloadAndInstallUpdateCommand = new DelegateCommand(new Action<object>(this.DownloadAndInstallUpdate));
    }

    private void DownloadAndInstallUpdate(object obj)
    {
      if (string.IsNullOrWhiteSpace(this.mAvailableVersionURL))
        return;
      this.Status = AppUpdate.UpdateStatus.DownloadingUpdate;
      string str = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), "EmoTracker", "emotracker_setup.exe");
      Directory.CreateDirectory(Path.GetDirectoryName(str));
      this.mWebClient.DownloadFileAsync(new Uri(this.mAvailableVersionURL), str, (object) str);
    }

    private void MWebClient_DownloadDataCompleted(object sender, DownloadDataCompletedEventArgs e)
    {
      if (e.Error == null && e.Result != null)
      {
        if (e.Result.Length != 0)
        {
          try
          {
            using (Stream stream = (Stream) new MemoryStream(e.Result))
            {
              using (StreamReader reader = new StreamReader(stream))
              {
                JObject token = (JObject) JToken.ReadFrom((JsonReader) new JsonTextReader((TextReader) reader));
                if (token != null)
                {
                  string input = token.GetValue<string>("version", (string) null);
                  if (!string.IsNullOrWhiteSpace(input))
                  {
                    Version result = new Version();
                    if (Version.TryParse(input, out result))
                    {
                      this.AvailableVersion = result;
                      this.mAvailableVersionURL = token.GetValue<string>("link", (string) null);
                    }
                  }
                }
              }
            }
            if (this.UpdateAvailable)
            {
              if (this.CurrentVersion.Major == 0)
              {
                this.Status = AppUpdate.UpdateStatus.RunningLegacyApp;
                return;
              }
              this.Status = AppUpdate.UpdateStatus.UpdateAvailable;
              return;
            }
            this.Status = AppUpdate.UpdateStatus.NoUpdateAvailable;
            return;
          }
          catch
          {
            this.Status = AppUpdate.UpdateStatus.Error;
            return;
          }
        }
      }
      this.Status = AppUpdate.UpdateStatus.Error;
    }

    private void MWebClient_DownloadProgressChanged(
      object sender,
      DownloadProgressChangedEventArgs e)
    {
      this.DownloadPercentage = (int) ((double) e.BytesReceived / (double) e.TotalBytesToReceive * 100.0 + 0.5);
    }

    private void MWebClient_DownloadFileCompleted(object sender, AsyncCompletedEventArgs e)
    {
      if (e.Error != null)
      {
        this.Status = AppUpdate.UpdateStatus.Error;
      }
      else
      {
        try
        {
          this.Status = AppUpdate.UpdateStatus.InstallingUpdate;
          Process.Start((string) e.UserState);
          Application.Current.Shutdown();
        }
        catch
        {
          this.Status = AppUpdate.UpdateStatus.Error;
        }
      }
    }

    public void CheckForUpdates()
    {
      try
      {
        this.Status = AppUpdate.UpdateStatus.CheckingForUpdate;
        this.mWebClient.DownloadDataAsync(new Uri(AppUpdate.AppUpdateURL));
      }
      catch
      {
        this.Status = AppUpdate.UpdateStatus.Error;
      }
    }

    public enum UpdateStatus
    {
      Idle,
      RunningLegacyApp,
      CheckingForUpdate,
      UpdateAvailable,
      NoUpdateAvailable,
      DownloadingUpdate,
      InstallingUpdate,
      Error,
    }
  }
}
