﻿// Decompiled with JetBrains decompiler
// Type: NewTek.NDI.Router
// Assembly: NDILibDotNet2, Version=1.0.0.0, Culture=neutral, PublicKeyToken=null
// MVID: A7218F26-32E7-4B46-B496-253D3CB5BE4C
// Assembly location: C:\Program Files (x86)\EmoTracker\NDILibDotNet2.dll

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;

namespace NewTek.NDI
{
  public class Router : IDisposable, INotifyPropertyChanged
  {
    private bool _disposed;
    private string[] _groups;
    private IntPtr _routingInstancePtr = IntPtr.Zero;
    private Source _selectedSource = new Source();
    private string _routingName = "Routing";

    [Category("NewTek NDI")]
    [Description("The NDI source to route elsewhere. An empty new Source() or a Source with no Name will disconnect.")]
    public Source SelectedSource
    {
      get => this._selectedSource;
      set
      {
        if (!(value.Name != this._selectedSource.Name))
          return;
        this._selectedSource = value;
        this.UpdateRouting();
        this.NotifyPropertyChanged("FromSource");
      }
    }

    [Category("NewTek NDI")]
    [Description("The name that will be given to the routed source. If empty it will default to 'Routing'.")]
    public string RoutingName
    {
      get => !string.IsNullOrWhiteSpace(this._routingName) ? this._routingName : "Routing";
      set
      {
        if (!(value != this._routingName))
          return;
        this._routingName = string.IsNullOrWhiteSpace(value) ? "Routing" : value;
        this.CreateRouting();
        this.NotifyPropertyChanged(nameof (RoutingName));
      }
    }

    public Router(string routingName = "Routing", string[] groups = null)
    {
      this._groups = groups;
      this._routingName = routingName;
      if (!NDIlib.initialize())
      {
        if (!NDIlib.is_supported_CPU())
          throw new InvalidOperationException("CPU incompatible with NDI.");
        throw new InvalidOperationException("Unable to initialize NDI.");
      }
      this.CreateRouting();
    }

    public void Clear()
    {
      if (!(this._routingInstancePtr != IntPtr.Zero))
        return;
      NDIlib.routing_clear(this._routingInstancePtr);
    }

    public void UpdateRouting()
    {
      if (this._routingInstancePtr == IntPtr.Zero)
        this.CreateRouting();
      else if (this._selectedSource == null || string.IsNullOrEmpty(this._selectedSource.Name))
      {
        this.Clear();
      }
      else
      {
        NDIlib.source_t p_source = new NDIlib.source_t()
        {
          p_ndi_name = UTF.StringToUtf8(this._selectedSource.Name)
        };
        if (!NDIlib.routing_change(this._routingInstancePtr, ref p_source))
        {
          Marshal.FreeHGlobal(p_source.p_ndi_name);
          throw new InvalidOperationException("Failed to change routing.");
        }
        Marshal.FreeHGlobal(p_source.p_ndi_name);
      }
    }

    public event PropertyChangedEventHandler PropertyChanged;

    private void NotifyPropertyChanged(string info)
    {
      if (this.PropertyChanged == null)
        return;
      this.PropertyChanged((object) this, new PropertyChangedEventArgs(info));
    }

    public void Dispose()
    {
      this.Dispose(true);
      GC.SuppressFinalize((object) this);
    }

    ~Router() => this.Dispose(false);

    protected virtual void Dispose(bool disposing)
    {
      if (this._disposed)
        return;
      if (disposing)
        this.Clear();
      if (this._routingInstancePtr != IntPtr.Zero)
      {
        NDIlib.routing_destroy(this._routingInstancePtr);
        this._routingInstancePtr = IntPtr.Zero;
      }
      NDIlib.destroy();
      this._disposed = true;
    }

    private void CreateRouting()
    {
      if (this._routingInstancePtr != IntPtr.Zero)
      {
        NDIlib.routing_destroy(this._routingInstancePtr);
        this._routingInstancePtr = IntPtr.Zero;
      }
      if (this._selectedSource == null || string.IsNullOrEmpty(this._selectedSource.Name))
        return;
      IntPtr utf8 = UTF.StringToUtf8(this._routingName);
      IntPtr hglobal = IntPtr.Zero;
      if (this._groups != null)
      {
        StringBuilder stringBuilder = new StringBuilder();
        foreach (string group in this._groups)
        {
          stringBuilder.Append(group);
          if (group != ((IEnumerable<string>) this._groups).Last<string>())
            stringBuilder.Append(',');
        }
        hglobal = UTF.StringToUtf8(stringBuilder.ToString());
      }
      NDIlib.routing_create_t p_create_settings = new NDIlib.routing_create_t()
      {
        p_ndi_name = utf8,
        p_groups = hglobal
      };
      this._routingInstancePtr = NDIlib.routing_create(ref p_create_settings);
      Marshal.FreeHGlobal(utf8);
      Marshal.FreeHGlobal(hglobal);
      if (this._routingInstancePtr == IntPtr.Zero)
        throw new InvalidOperationException("Failed to create routing instance.");
      this.UpdateRouting();
    }
  }
}
